@@ -7,6 +7,7 @@ agentclient.c
         Markku-Juhani Saarinen <mjos@ssh.com>
         Timo J. Rinne <tri@ssh.com>
         Sami Lehtinen <sjl@ssh.com>
+        Firdaus Mah <firdaus@ssh.com>
 
   Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
@@ -16,13 +17,13 @@ agentclient.c
 */
 
 #include "ssh2includes.h"
-#include "sshcross.h"
 #include "sshencode.h"
 #include "sshtcp.h"
 #include "sshagent.h"
 #include "sshagentint.h"
 #include "sshtimeouts.h"
 #include "sshcrypt.h"
+#include "sshpacketstream.h"
 
 #define SSH_DEBUG_MODULE "SshAgentClient"
 
@@ -30,7 +31,7 @@ agentclient.c
 #define SSH_AGENT_TIMEOUT       (30L) /* seconds */
 
 /* In this module, we assume that we never send so big requests that
-   SshCrossDown couldn't handle them.  Thus, we don't need can_send
+   SshPacketWrapper couldn't handle them.  Thus, we don't need can_send
    callbacks.  */
 
 /* Internal status of the agent client. */
@@ -43,14 +44,12 @@ typedef enum {
 } SshAgentClientState;
 
 struct SshAgentRec {
-  /* The agent uses a packet format identical to the cross layer protocol.
-     Thus, we can use the SshCrossDown and SshCrossUp objects for handling
-     the packets.  In the client, we use SshCrossDown. */
-  SshCrossDown down;
+  /* The binary packet protocol implementation. */
+  SshPacketWrapper wrapper;
 
   /* Agent version, as returned by agent request. */
   unsigned long version;
-  
+
   /* State of the ssh-agent client. */
   SshAgentClientState state;
 
@@ -62,9 +61,11 @@ struct SshAgentRec {
   SshAgentCompletion completion_callback;
   SshAgentListCallback list_callback;
   SshAgentOpCallback op_callback;
+  SshAgentRandomCallback random_callback;
   void *context;
 };
 
+
 /* This is called if a request times out. */
 
 void ssh_agent_timeout(void *context)
@@ -78,7 +79,8 @@ void ssh_agent_timeout(void *context)
       break;
     case SSH_AC_WAITING_SUCCESS:
       if (agent->completion_callback)
-        (*agent->completion_callback)(SSH_AGENT_ERROR_TIMEOUT, agent->context);
+        (*agent->completion_callback)(SSH_AGENT_ERROR_TIMEOUT,
+                                      agent->context);
       break;
     case SSH_AC_WAITING_LIST:
       if (agent->list_callback)
@@ -102,7 +104,7 @@ void ssh_agent_timeout(void *context)
 
 /* This is called when a packet is received from the agent. */
 
-void ssh_agent_received_packet(SshCrossPacketType type,
+void ssh_agent_received_packet(SshPacketType type,
                                const unsigned char *data,
                                size_t len,
                                void *context)
@@ -118,7 +120,8 @@ void ssh_agent_received_packet(SshCrossPacketType type,
   switch ((int)type)
     {
     case 2: /* Version response from old (version 1.x) ssh-agent. */
-      ssh_debug("ssh_agent_received_packet: packet number 2 (version response from 1.x agent)");
+      ssh_debug("ssh_agent_received_packet: packet number 2 (version "
+                "response from 1.x agent)");
       agent->version = 1;
 
       /* We don't support the 1.x agent yet.  Nor will we ever. */
@@ -141,19 +144,19 @@ void ssh_agent_received_packet(SshCrossPacketType type,
       if (agent->completion_callback)
         (*agent->completion_callback)(SSH_AGENT_ERROR_OK, agent->context);
       break;
-      
+
     case SSH_AGENT_FAILURE:
       if (agent->state == SSH_AC_WAITING_VERSION)
         {
           /* This may happen, if we have agent from versions
              ssh-2.0.{7-10} and our client is newer.  We now send
              the version request again without identifying our own
-             version and leaving the state waiting for version 
+             version and leaving the state waiting for version
              response. */
           agent->broken_agent = TRUE;
-          ssh_cross_down_send_encode(agent->down,
-                                (SshCrossPacketType)SSH_AGENT_REQUEST_VERSION,
-                                 SSH_FORMAT_END);
+          ssh_packet_wrapper_send_encode(agent->wrapper,
+                                         SSH_AGENT_REQUEST_VERSION,
+                                         SSH_FORMAT_END);
           return;
         }
       if (agent->state != SSH_AC_WAITING_SUCCESS &&
@@ -164,7 +167,7 @@ void ssh_agent_received_packet(SshCrossPacketType type,
         }
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32, &code,
-                           SSH_FORMAT_END) != len)
+                           SSH_FORMAT_END) != len || len == 0)
         {
           ssh_debug("ssh_agent_received_packet: FAILURE bad data");
           err = SSH_AGENT_ERROR_FAILURE;
@@ -196,7 +199,7 @@ void ssh_agent_received_packet(SshCrossPacketType type,
       ssh_cancel_timeouts(ssh_agent_timeout, (void *)agent);
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32, &temp,
-                           SSH_FORMAT_END) != len)
+                           SSH_FORMAT_END) != len || len == 0)
         {
           ssh_debug("ssh_agent_received_packet: VERSION_RESPONSE bad data");
           return;
@@ -206,7 +209,7 @@ void ssh_agent_received_packet(SshCrossPacketType type,
       if (agent->open_callback)
         (*agent->open_callback)(agent, agent->context);
       break;
-      
+
     case SSH_AGENT_KEY_LIST:
       if (agent->state != SSH_AC_WAITING_LIST)
         {
@@ -233,9 +236,9 @@ void ssh_agent_received_packet(SshCrossPacketType type,
         {
           bytes = ssh_decode_array(data, len,
                                    SSH_FORMAT_UINT32_STR_NOCOPY,
-                                     &keys[i].certs, &keys[i].certs_len,
+                                   &keys[i].certs, &keys[i].certs_len,
                                    SSH_FORMAT_UINT32_STR,
-                                     &keys[i].description, NULL,
+                                   &keys[i].description, NULL,
                                    SSH_FORMAT_END);
           if (bytes == 0)
             {
@@ -265,7 +268,7 @@ void ssh_agent_received_packet(SshCrossPacketType type,
         ssh_xfree(keys[i].description);
       ssh_xfree(keys);
       break;
-      
+
     case SSH_AGENT_OPERATION_COMPLETE:
       if (agent->state != SSH_AC_WAITING_OPERATION_COMPLETE)
         {
@@ -274,7 +277,7 @@ void ssh_agent_received_packet(SshCrossPacketType type,
         }
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32_STR_NOCOPY, &result, &result_len,
-                           SSH_FORMAT_END) != len)
+                           SSH_FORMAT_END) != len || len == 0)
         {
           ssh_debug("ssh_agent_received_packet: OP_COMPLETE bad data");
           return;
@@ -286,6 +289,22 @@ void ssh_agent_received_packet(SshCrossPacketType type,
                               agent->context);
       break;
 
+      /* generated random data from authentication agent */
+    case SSH_AGENT_RANDOM_DATA:
+      if (ssh_decode_array(data, len,
+                           SSH_FORMAT_UINT32_STR, &result, &result_len,
+                           SSH_FORMAT_END) != len || len == 0)
+        {
+          ssh_debug("ssh_agent_received_packet: RANDOM_COMPLETE bad data");
+          return;
+        }
+      ssh_cancel_timeouts(ssh_agent_timeout, (void *)agent);
+      agent->state = SSH_AC_IDLE;
+      if (agent->random_callback)
+        (*agent->random_callback)(SSH_AGENT_ERROR_OK, result, result_len,
+                                  agent->context);
+      break;
+
     default:
       ssh_debug("ssh_agent_received_packet: received unknown packet %d",
                 (int)type);
@@ -299,14 +318,15 @@ void ssh_agent_received_eof(void *context)
   SshAgent agent = (SshAgent)context;
 
   ssh_cancel_timeouts(ssh_agent_timeout, (void *)agent);
-  
+
   switch (agent->state)
     {
     case SSH_AC_IDLE:
       break;
     case SSH_AC_WAITING_SUCCESS:
       if (agent->completion_callback)
-        (*agent->completion_callback)(SSH_AGENT_ERROR_FAILURE, agent->context);
+        (*agent->completion_callback)(SSH_AGENT_ERROR_FAILURE,
+                                      agent->context);
       break;
     case SSH_AC_WAITING_LIST:
       if (agent->list_callback)
@@ -361,14 +381,15 @@ void ssh_agent_open_complete(SshStream stream, void *context)
 
   /* Initialize the agent connection object. */
   agent = ssh_xcalloc(1, sizeof(*agent));
-  agent->down = ssh_cross_down_create(stream,
-                                      ssh_agent_received_packet,
-                                      ssh_agent_received_eof,
-                                      NULL,
-                                      (void *)agent);
-  ssh_cross_down_can_receive(agent->down, TRUE);
+  agent->wrapper = ssh_packet_wrap(stream,
+                                   ssh_agent_received_packet,
+                                   ssh_agent_received_eof,
+                                   NULL,
+                                   (void *)agent);
+  ssh_packet_wrapper_can_receive(agent->wrapper, TRUE);
 
   /* Prepare to send version request. */
+
   agent->state = SSH_AC_WAITING_VERSION;
   agent->open_callback = c->callback;
   agent->context = c->context;
@@ -376,12 +397,25 @@ void ssh_agent_open_complete(SshStream stream, void *context)
   ssh_register_timeout(SSH_AGENT_TIMEOUT, 0L,
                        ssh_agent_timeout, (void *)agent);
 
+
+#ifdef SSH_AGENT_PKCS11
   /* Send a version request message. */
-  ssh_cross_down_send_encode(agent->down,
-                             (SshCrossPacketType)SSH_AGENT_REQUEST_VERSION,
-                             SSH_FORMAT_UINT32_STR,
-                             SSH2_VERSION_STRING, strlen(SSH2_VERSION_STRING),
-                             SSH_FORMAT_END);
+  ssh_packet_wrapper_send_encode(agent->wrapper,
+                                 SSH_AGENT_REQUEST_VERSION,
+                                 SSH_FORMAT_UINT32_STR,
+                                 SSH_AGENT_PKCS11_VERSION_STRING,
+                                 strlen(SSH_AGENT_PKCS11_VERSION_STRING),
+                                 SSH_FORMAT_END);
+#else /* SSH_AGENT_PKCS11 */
+  /* Send a version request message. */
+  ssh_packet_wrapper_send_encode(agent->wrapper,
+                                 SSH_AGENT_REQUEST_VERSION,
+                                 SSH_FORMAT_UINT32_STR,
+                                 SSH2_VERSION_STRING,
+                                 strlen(SSH2_VERSION_STRING),
+                                 SSH_FORMAT_END);
+
+#endif /* SSH_AGENT_PKCS11 */
 
   /* The callback will be called when a version number has been received
      or the request times out. */
@@ -400,11 +434,11 @@ void ssh_agent_open(SshAgentOpenCallback callback, void *context)
   c->context = context;
 
   /* Connect to the agent socket. */
-
+#ifndef WIN32
   ssh_agenti_connect(ssh_agent_open_complete, FALSE, (void *)c);
-
-
-
+#else /* WIN32 */
+  ssh_local_connect("SSH Accession", ssh_agent_open_complete, (void *)c);
+#endif /* WIN32 */
 }
 
 /* Closes the connection to the authentication agent.  If a command is
@@ -412,7 +446,7 @@ void ssh_agent_open(SshAgentOpenCallback callback, void *context)
 
 void ssh_agent_close(SshAgent agent)
 {
-  ssh_cross_down_destroy(agent->down);
+  ssh_packet_wrapper_destroy(agent->wrapper);
   ssh_cancel_timeouts(ssh_agent_timeout, (void *)agent);
   memset(agent, 'F', sizeof(*agent));
   ssh_xfree(agent);
@@ -425,7 +459,7 @@ void ssh_agent_init_key_attrs(SshAgentKeyAttrs attrs)
   attrs->path_len_limit = 0xffffffff;
   attrs->path_constraint = NULL;
   attrs->timeout_time = (SshTime) 0;
-  attrs->compat_allowed = TRUE; 
+  attrs->compat_allowed = TRUE;
 }
 
 void ssh_agent_add_generic(SshAgent agent,
@@ -448,10 +482,10 @@ void ssh_agent_add_generic(SshAgent agent,
         (*callback)(SSH_AGENT_ERROR_BUSY, context);
       return;
     }
-  
+
   if (key)
     {
-      if (ssh_private_key_export(key, "none", (unsigned char *)"", 
+      if (ssh_private_key_export(key, "none", (unsigned char *)"",
                                  0, &blob,
                                  &blob_len) != SSH_CRYPTO_OK)
         {
@@ -473,62 +507,62 @@ void ssh_agent_add_generic(SshAgent agent,
   agent->state = SSH_AC_WAITING_SUCCESS;
   agent->completion_callback = callback;
   agent->context = context;
-  
+
   /* Send the request. */
   if (attrs == NULL)
     {
-      ssh_cross_down_send_encode(agent->down,
-                                 (SshCrossPacketType)SSH_AGENT_ADD_KEY,
-                                 SSH_FORMAT_UINT32_STR, blob, blob_len,
-                                 SSH_FORMAT_UINT32_STR, certs, certs_len,
-                                 SSH_FORMAT_UINT32_STR,
-                                 description, strlen(description),
-                                 SSH_FORMAT_END);
+      ssh_packet_wrapper_send_encode(agent->wrapper,
+                                     SSH_AGENT_ADD_KEY,
+                                     SSH_FORMAT_UINT32_STR, blob, blob_len,
+                                     SSH_FORMAT_UINT32_STR, certs, certs_len,
+                                     SSH_FORMAT_UINT32_STR,
+                                     description, strlen(description),
+                                     SSH_FORMAT_END);
     }
   else
     {
       ssh_buffer_init(&buffer);
       if (attrs->timeout_time != 0)
-          ssh_encode_buffer(&buffer,
-                            SSH_FORMAT_CHAR, 
-                            (unsigned int)SSH_AGENT_CONSTRAINT_TIMEOUT,
-                            SSH_FORMAT_UINT32, (SshUInt32) attrs->timeout_time,
-                            SSH_FORMAT_END);
+        ssh_encode_buffer(&buffer,
+                          SSH_FORMAT_CHAR,
+                          (unsigned int)SSH_AGENT_CONSTRAINT_TIMEOUT,
+                          SSH_FORMAT_UINT32, (SshUInt32) attrs->timeout_time,
+                          SSH_FORMAT_END);
       if (attrs->use_limit != 0xffffffff)
-          ssh_encode_buffer(&buffer,
-                            SSH_FORMAT_CHAR, 
-                            (unsigned int)SSH_AGENT_CONSTRAINT_USE_LIMIT,
-                            SSH_FORMAT_UINT32, attrs->use_limit,
-                            SSH_FORMAT_END);
+        ssh_encode_buffer(&buffer,
+                          SSH_FORMAT_CHAR,
+                          (unsigned int)SSH_AGENT_CONSTRAINT_USE_LIMIT,
+                          SSH_FORMAT_UINT32, attrs->use_limit,
+                          SSH_FORMAT_END);
       if (attrs->path_len_limit != 0xffffffff)
-          ssh_encode_buffer(&buffer,
-                            SSH_FORMAT_CHAR, 
-                           (unsigned int)SSH_AGENT_CONSTRAINT_FORWARDING_STEPS,
-                            SSH_FORMAT_UINT32, attrs->path_len_limit,
-                            SSH_FORMAT_END);
+        ssh_encode_buffer(&buffer,
+                          SSH_FORMAT_CHAR,
+                          (unsigned int)SSH_AGENT_CONSTRAINT_FORWARDING_STEPS,
+                          SSH_FORMAT_UINT32, attrs->path_len_limit,
+                          SSH_FORMAT_END);
       if (attrs->path_constraint != NULL)
-          ssh_encode_buffer(&buffer,
-                            SSH_FORMAT_CHAR, 
-                            (unsigned int)SSH_AGENT_CONSTRAINT_FORWARDING_PATH,
-                            SSH_FORMAT_UINT32_STR, 
-                            attrs->path_constraint, 
-                            strlen(attrs->path_constraint),
-                            SSH_FORMAT_END);
+        ssh_encode_buffer(&buffer,
+                          SSH_FORMAT_CHAR,
+                          (unsigned int)SSH_AGENT_CONSTRAINT_FORWARDING_PATH,
+                          SSH_FORMAT_UINT32_STR,
+                          attrs->path_constraint,
+                          strlen(attrs->path_constraint),
+                          SSH_FORMAT_END);
       ssh_encode_buffer(&buffer,
-                        SSH_FORMAT_CHAR, 
+                        SSH_FORMAT_CHAR,
                         (unsigned int)SSH_AGENT_CONSTRAINT_COMPAT,
                         SSH_FORMAT_BOOLEAN, attrs->compat_allowed,
                         SSH_FORMAT_END);
-      ssh_cross_down_send_encode(agent->down,
-                                 (SshCrossPacketType)SSH_AGENT_ADD_KEY,
-                                 SSH_FORMAT_UINT32_STR, blob, blob_len,
-                                 SSH_FORMAT_UINT32_STR, certs, certs_len,
-                                 SSH_FORMAT_UINT32_STR,
-                                 description, strlen(description),
-                                 SSH_FORMAT_DATA,
-                                 ssh_buffer_ptr(&buffer),
-                                 ssh_buffer_len(&buffer),
-                                 SSH_FORMAT_END);
+      ssh_packet_wrapper_send_encode(agent->wrapper,
+                                     SSH_AGENT_ADD_KEY,
+                                     SSH_FORMAT_UINT32_STR, blob, blob_len,
+                                     SSH_FORMAT_UINT32_STR, certs, certs_len,
+                                     SSH_FORMAT_UINT32_STR,
+                                     description, strlen(description),
+                                     SSH_FORMAT_DATA,
+                                     ssh_buffer_ptr(&buffer),
+                                     ssh_buffer_len(&buffer),
+                                     SSH_FORMAT_END);
       ssh_buffer_uninit(&buffer);
     }
 
@@ -557,16 +591,16 @@ void ssh_agent_add_with_attrs(SshAgent agent,
                               const unsigned char *certs,
                               size_t certs_len,
                               const char *description,
-                              SshUInt32 path_len_limit, 
+                              SshUInt32 path_len_limit,
                               char *path_constraint,
-                              SshUInt32 use_limit, 
-                              Boolean compat_forbidden, 
+                              SshUInt32 use_limit,
+                              Boolean compat_forbidden,
                               SshTime timeout_time,
                               SshAgentCompletion callback,
                               void *context)
 {
   struct SshAgentKeyAttrsRec attrs;
-  
+
   if (agent->broken_agent)
     {
       ssh_debug("ssh_agent_add: remote agent broken");
@@ -578,7 +612,7 @@ void ssh_agent_add_with_attrs(SshAgent agent,
   attrs.compat_allowed = !compat_forbidden;
   attrs.timeout_time = timeout_time;
   attrs.use_limit = use_limit;
-  attrs.path_constraint = (path_constraint ? 
+  attrs.path_constraint = (path_constraint ?
                            ssh_xstrdup(path_constraint) : NULL);
   ssh_agent_add_generic(agent, key, certs, certs_len, description,
                         &attrs, callback, context);
@@ -605,13 +639,13 @@ void ssh_agent_delete_all(SshAgent agent, SshAgentCompletion callback,
   agent->context = context;
 
   /* Send the request. */
-  ssh_cross_down_send_encode(agent->down,
-                             (SshCrossPacketType)SSH_AGENT_DELETE_ALL_KEYS,
-                             SSH_FORMAT_END);
+  ssh_packet_wrapper_send_encode(agent->wrapper,
+                                 SSH_AGENT_DELETE_ALL_KEYS,
+                                 SSH_FORMAT_END);
 }
 
 /* Deletes the given key from the agent. */
-void ssh_agent_delete(SshAgent agent, 
+void ssh_agent_delete(SshAgent agent,
                       const unsigned char *certs, size_t certs_len,
                       const char *description,
                       SshAgentCompletion callback, void *context)
@@ -632,12 +666,12 @@ void ssh_agent_delete(SshAgent agent,
   agent->context = context;
 
   /* Send the request. */
-  ssh_cross_down_send_encode(agent->down,
-                             (SshCrossPacketType)SSH_AGENT_DELETE_KEY,
-                             SSH_FORMAT_UINT32_STR, certs, certs_len,
-                             SSH_FORMAT_UINT32_STR, description, 
-                             (description ? strlen(description) : 0),
-                             SSH_FORMAT_END);
+  ssh_packet_wrapper_send_encode(agent->wrapper,
+                                 SSH_AGENT_DELETE_KEY,
+                                 SSH_FORMAT_UINT32_STR, certs, certs_len,
+                                 SSH_FORMAT_UINT32_STR, description,
+                                 (description ? strlen(description) : 0),
+                                 SSH_FORMAT_END);
 }
 
 /* Returns the public keys for all private keys in possession of the agent.
@@ -654,7 +688,7 @@ void ssh_agent_list(SshAgent agent, SshAgentListCallback callback,
         (*callback)(SSH_AGENT_ERROR_BUSY, 0, NULL, context);
       return;
     }
-  
+
   /* Prepare to send the request. */
   ssh_register_timeout(SSH_AGENT_TIMEOUT, 0L,
                        ssh_agent_timeout, (void *)agent);
@@ -663,9 +697,9 @@ void ssh_agent_list(SshAgent agent, SshAgentListCallback callback,
   agent->context = context;
 
   /* Send the request. */
-  ssh_cross_down_send_encode(agent->down,
-                             (SshCrossPacketType)SSH_AGENT_LIST_KEYS,
-                             SSH_FORMAT_END);
+  ssh_packet_wrapper_send_encode(agent->wrapper,
+                                 SSH_AGENT_LIST_KEYS,
+                                 SSH_FORMAT_END);
 }
 
 /* Performs a private-key operation using the agent.  Calls the given
@@ -693,9 +727,24 @@ void ssh_agent_op(SshAgent agent, SshAgentOp op,
     case SSH_AGENT_SIGN:
       name = "sign";
       break;
+    case SSH_AGENT_SIGN_PKCS1_SHA1:
+      name = "sign[pkcs1,sha1]";
+      break;
+    case SSH_AGENT_SIGN_PKCS1_MD5:
+      name = "sign[pkcs1,md5]";
+      break;
+    case SSH_AGENT_SIGN_PKCS1V2_OAEP:
+      name = "sign[pkcs1v2,oaep]";
+      break;
     case SSH_AGENT_HASH_AND_SIGN:
       name = "hash-and-sign";
       break;
+    case SSH_AGENT_HASH_AND_SIGN_PKCS1_SHA1:
+      name = "hash-and-sign[pkcs1,sha1]";
+      break;
+    case SSH_AGENT_HASH_AND_SIGN_PKCS1_MD5:
+      name = "hash-and-sign[pkcs1,md5]";
+      break;
     case SSH_AGENT_DECRYPT:
       name = "decrypt";
       break;
@@ -717,21 +766,21 @@ void ssh_agent_op(SshAgent agent, SshAgentOp op,
   if (op != SSH_AGENT_SSH1_CHALLENGE_RESPONSE)
     {
       /* Send the request packet. */
-      ssh_cross_down_send_encode(agent->down,
-                                 (SshCrossPacketType)SSH_AGENT_PRIVATE_KEY_OP,
-                                 SSH_FORMAT_UINT32_STR, name, strlen(name),
-                                 SSH_FORMAT_UINT32_STR, certs, certs_len,
-                                 SSH_FORMAT_UINT32_STR, data, len,
-                                 SSH_FORMAT_END);
+      ssh_packet_wrapper_send_encode(agent->wrapper,
+                                     SSH_AGENT_PRIVATE_KEY_OP,
+                                     SSH_FORMAT_UINT32_STR, name, strlen(name),
+                                     SSH_FORMAT_UINT32_STR, certs, certs_len,
+                                     SSH_FORMAT_UINT32_STR, data, len,
+                                     SSH_FORMAT_END);
     }
   else
     {
-      ssh_cross_down_send_encode(agent->down,
-                                 (SshCrossPacketType)SSH_AGENT_PRIVATE_KEY_OP,
-                                 SSH_FORMAT_UINT32_STR, name, strlen(name),
-                                 SSH_FORMAT_UINT32_STR, certs, certs_len,
-                                 SSH_FORMAT_DATA, data, len,
-                                 SSH_FORMAT_END);
+      ssh_packet_wrapper_send_encode(agent->wrapper,
+                                     SSH_AGENT_PRIVATE_KEY_OP,
+                                     SSH_FORMAT_UINT32_STR, name, strlen(name),
+                                     SSH_FORMAT_UINT32_STR, certs, certs_len,
+                                     SSH_FORMAT_DATA, data, len,
+                                     SSH_FORMAT_END);
     }
 }
 
@@ -756,11 +805,11 @@ void ssh_agent_lock(SshAgent agent, const char *password,
   agent->context = context;
 
   /* Send the request. */
-  ssh_cross_down_send_encode(agent->down,
-                             (SshCrossPacketType)SSH_AGENT_LOCK,
-                             SSH_FORMAT_UINT32_STR, 
-                             password, (password ? strlen(password) : 0),
-                             SSH_FORMAT_END);
+  ssh_packet_wrapper_send_encode(agent->wrapper,
+                                 SSH_AGENT_LOCK,
+                                 SSH_FORMAT_UINT32_STR,
+                                 password, (password ? strlen(password) : 0),
+                                 SSH_FORMAT_END);
 }
 
 /* Attempts to unlock the agent with given password */
@@ -783,9 +832,51 @@ void ssh_agent_unlock(SshAgent agent, const char *password,
   agent->context = context;
 
   /* Send the request. */
-  ssh_cross_down_send_encode(agent->down,
-                             (SshCrossPacketType)SSH_AGENT_UNLOCK,
-                             SSH_FORMAT_UINT32_STR, 
-                             password, (password ? strlen(password) : 0),
-                             SSH_FORMAT_END);
+  ssh_packet_wrapper_send_encode(agent->wrapper,
+                                 SSH_AGENT_UNLOCK,
+                                 SSH_FORMAT_UINT32_STR,
+                                 password, (password ? strlen(password) : 0),
+                                 SSH_FORMAT_END);
 }
+
+/* generate random from authentication agent */
+void ssh_agent_random(SshAgent agent, size_t rnd_blob_len,
+                      SshAgentRandomCallback callback, void *context)
+{
+
+
+  if (agent->state != SSH_AC_IDLE)
+    {
+      ssh_debug("ssh_agent_random: busy");
+      if (callback)
+        (*callback)(SSH_AGENT_ERROR_BUSY, NULL, 0, context);
+      return;
+    }
+
+
+  /* Prepare to send the request. */
+
+  ssh_register_timeout(SSH_AGENT_TIMEOUT, 0L,
+                       ssh_agent_timeout, (void *)agent);
+  agent->state = SSH_AC_WAITING_LIST;
+  agent->random_callback = callback;
+  agent->context = context;
+
+
+  /* Send the request packet. */
+  ssh_packet_wrapper_send_encode(agent->wrapper,
+                                 SSH_AGENT_RANDOM,
+                                 SSH_FORMAT_UINT32, (SshUInt32)rnd_blob_len,
+                                 SSH_FORMAT_END);
+}
+
+
+
+
+
+
+
+
+
+
+
