@@ -4,9 +4,9 @@
 
   Author: Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1997-2001 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
-                  
+
   Hostbased authentication, client-side.
 
 */
@@ -16,21 +16,23 @@
 #include "sshpacketstream.h"
 #include "sshencode.h"
 #include "sshmsgs.h"
+#include "sshkeyfile.h"
 #include "sshclient.h"
 #include "sshunixpipestream.h"
 #include "sshtcp.h"
+#include "sshuserfiles.h"
+#include "authc-hostbased.h"
+#include "sshconfig.h"
+#include "ssholdfsm.h"
+#include "sshtimeouts.h"
+#include "sshdsprintf.h"
+#include "sshhostkeyio.h"
+#include "sshcipherlist.h"
+#include "ssh2pubkeyencode.h"
+#include "sshsnlist.h"
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif /* HAVE_SYS_PARAM_H */
-#include "sshuserfiles.h"
-#include "ssh2pubkeyencode.h"
-#include "authc-hostbased.h"
-#include "sshconfig.h"
-#include "sshdsprintf.h"
-
-#include "sshfsm.h"
-#include "sshtimeouts.h"
-#include "sshdsprintf.h"
 
 #define SSH_DEBUG_MODULE "Ssh2AuthHostBasedClient"
 
@@ -41,46 +43,105 @@ typedef struct SshClientHostbasedAuthContextRec
   const unsigned char *session_id;
   size_t session_id_len;
   const char *user;
+  char *pk_algorithms;
   char *pubkey_algorithm;
-  unsigned char *pubkeyblob;
-  size_t pubkeyblob_len;
+  SshBuffer pubkeyblob;
   const char *local_user_name;
   char *local_host_name;
   SshAuthClientCompletionProc completion;
   void *completion_context;
   void **state_placeholder;
-  
+
   unsigned char *packet;
   size_t packet_len;
   SshPacketType packet_type;
   Boolean packet_pending;
-  
+
   SshConfig server_conf;
 
+
+
+
   SshTransportCompat compat_flags;
-  
-  SshFSM fsm;
-  SshFSMThread main_thread;
-  
+
+
+  SshOldFSM fsm;
+  SshOldFSMThread main_thread;
+
 } *SshClientHostbasedAuth;
 
 /* Forward declarations. */
 void auth_hostbased_can_send(void *context);
 
-SSH_FSM_STEP(hostbased_suspend)
+SSH_OLDFSM_STEP(hostbased_suspend)
 {
-  return SSH_FSM_SUSPENDED;
+  return SSH_OLDFSM_SUSPENDED;
 }
 
-SSH_FSM_STEP(hostbased_send_compat_flags)
+SSH_OLDFSM_STEP(hostbased_init_pubkey)
+{
+  SSH_OLDFSM_GDATA(SshClientHostbasedAuth);
+
+  SSH_OLDFSM_SET_NEXT("hostbased_send_packet");
+
+  ssh_xfree(gdata->pubkey_algorithm);
+  gdata->pubkey_algorithm = NULL;
+
+  if (!gdata->pk_algorithms)
+    {
+      *gdata->state_placeholder = NULL;
+      (*gdata->completion)(SSH_AUTH_CLIENT_FAIL_AND_DISABLE_METHOD,
+                           gdata->user, NULL,
+                           gdata->completion_context);
+      SSH_TRACE(1, ("No more keys to try."));
+      SSH_OLDFSM_SET_NEXT("hostbased_send_end");
+      return SSH_OLDFSM_CONTINUE;
+    }
+
+  gdata->pubkey_algorithm = ssh_snlist_get_name(gdata->pk_algorithms);
+
+  SSH_ASSERT(gdata->pubkey_algorithm);
+
+  {
+    char *to_be_deleted = gdata->pk_algorithms;
+    const char *new_pk_list = ssh_snlist_step_forward(gdata->pk_algorithms);
+
+    if (new_pk_list)
+      gdata->pk_algorithms = ssh_xstrdup(new_pk_list);
+    else
+      gdata->pk_algorithms = NULL;
+
+    SSH_DEBUG(4, ("New pk list: %s", new_pk_list ? new_pk_list : "NULL"));
+    ssh_xfree(to_be_deleted);
+  }
+
+  SSH_TRACE(1, ("Trying \"hostbased\" authentication with `%s' key.",
+                gdata->pubkey_algorithm));
+
+  gdata->pubkeyblob =
+    ssh_host_key_get_pubkey_by_algname(gdata->pubkey_algorithm,
+                                       gdata->server_conf->host_keys_ctx);
+
+  SSH_ASSERT(gdata->pubkeyblob);
+
+  return SSH_OLDFSM_CONTINUE;
+}
+
+SSH_OLDFSM_STEP(hostbased_send_compat_flags)
 {
   char *compat_flag_string = NULL, *to_be_deleted;
-  SSH_FSM_GDATA(SshClientHostbasedAuth);
-  
-  SSH_FSM_SET_NEXT("hostbased_send_packet");
+  SSH_OLDFSM_GDATA(SshClientHostbasedAuth);
 
-  if (*gdata->compat_flags->hostbased_service_name_draft_incompatibility)
-    { 
+  SSH_OLDFSM_SET_NEXT("hostbased_init_pubkey");
+
+  if (
+
+
+
+      *gdata->compat_flags->hostbased_service_name_draft_incompatibility
+
+      )
+    {
       to_be_deleted = compat_flag_string;
       ssh_dsprintf(&compat_flag_string, "%s%s%s",
                    compat_flag_string ? compat_flag_string : "",
@@ -88,9 +149,15 @@ SSH_FSM_STEP(hostbased_send_compat_flags)
                    HOSTBASED_REQUESTED_SERVICE_DRAFT_INCOMPAT);
       ssh_xfree(to_be_deleted);
     }
-  
-  if (*gdata->compat_flags->malformed_signatures_draft_incompatibility)
-    { 
+
+  if (
+
+
+
+      *gdata->compat_flags->malformed_signatures_draft_incompatibility
+
+      )
+    {
       to_be_deleted = compat_flag_string;
       ssh_dsprintf(&compat_flag_string, "%s%s%s",
                    compat_flag_string ? compat_flag_string : "",
@@ -99,9 +166,25 @@ SSH_FSM_STEP(hostbased_send_compat_flags)
       ssh_xfree(to_be_deleted);
     }
 
+  if (
+
+
+
+      *gdata->compat_flags->rsa_hash_scheme_draft_incompatibility
+
+      )
+    {
+      to_be_deleted = compat_flag_string;
+      ssh_dsprintf(&compat_flag_string, "%s%s%s",
+                   compat_flag_string ? compat_flag_string : "",
+                   compat_flag_string ? "," : "",
+                   RSA_HASH_SCHEME_DRAFT_INCOMPAT);
+      ssh_xfree(to_be_deleted);
+    }
+
   if (!compat_flag_string)
     {
-      return SSH_FSM_CONTINUE;
+      return SSH_OLDFSM_CONTINUE;
     }
   else
     {
@@ -109,38 +192,38 @@ SSH_FSM_STEP(hostbased_send_compat_flags)
 
       gdata->packet_pending = TRUE;
       gdata->packet_type = SSH_AUTH_HOSTBASED_COMPAT;
-  
+
       gdata->packet_len =
         ssh_encode_array_alloc(&gdata->packet,
                                SSH_FORMAT_UINT32_STR, compat_flag_string,
                                strlen(compat_flag_string),
                                SSH_FORMAT_END);
-      
+
       ssh_xfree(compat_flag_string);
 
       if (ssh_packet_wrapper_can_send(gdata->wrapper))
         ssh_register_timeout(0L, 0L, auth_hostbased_can_send, gdata);
 
-      return SSH_FSM_SUSPENDED;
+      return SSH_OLDFSM_SUSPENDED;
     }
 
   SSH_NOTREACHED;
 }
 
-SSH_FSM_STEP(hostbased_process_error)
+SSH_OLDFSM_STEP(hostbased_process_error)
 {
   char *error_message = NULL;
   unsigned int error_code = 0;
   size_t len;
-  
-  SSH_FSM_GDATA(SshClientHostbasedAuth);
-  
+
+  SSH_OLDFSM_GDATA(SshClientHostbasedAuth);
+
   len = ssh_decode_array(gdata->packet, gdata->packet_len,
                          SSH_FORMAT_CHAR, &error_code,
                          SSH_FORMAT_UINT32_STR, &error_message, NULL,
                          SSH_FORMAT_UINT32_STR, NULL, NULL,
                          SSH_FORMAT_END);
-  
+
   if (len == 0 || len != gdata->packet_len)
     {
       ssh_warning("Received malformed error packet from ssh-signer.");
@@ -153,44 +236,50 @@ SSH_FSM_STEP(hostbased_process_error)
 
   ssh_packet_wrapper_can_receive(gdata->wrapper, FALSE);
 
-  SSH_FSM_SET_NEXT("hostbased_finish");
-  return SSH_FSM_CONTINUE;
+  SSH_OLDFSM_SET_NEXT("hostbased_finish");
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(hostbased_send_end)
+SSH_OLDFSM_STEP(hostbased_send_end)
 {
-  SSH_FSM_GDATA(SshClientHostbasedAuth);
-  
+  SSH_OLDFSM_GDATA(SshClientHostbasedAuth);
+
   ssh_packet_wrapper_send_encode(gdata->wrapper,
                                  SSH_AUTH_HOSTBASED_END,
                                  SSH_FORMAT_END);
-  
+
   if (gdata->wrapper)
     {
       ssh_packet_wrapper_destroy(gdata->wrapper);
       gdata->wrapper = NULL;
     }
-  
-  SSH_FSM_SET_NEXT("hostbased_finish");
-  
-  return SSH_FSM_CONTINUE;
+
+  SSH_OLDFSM_SET_NEXT("hostbased_finish");
+
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(hostbased_send_packet)
+SSH_OLDFSM_STEP(hostbased_send_packet)
 {
   char *service;
-  SSH_FSM_GDATA(SshClientHostbasedAuth);
-  
-  SSH_FSM_SET_NEXT("hostbased_suspend");
-  
-  if (!(*gdata->compat_flags->hostbased_service_name_draft_incompatibility))
+  SSH_OLDFSM_GDATA(SshClientHostbasedAuth);
+
+  SSH_OLDFSM_SET_NEXT("hostbased_suspend");
+
+  if (
+
+
+
+      !(*gdata->compat_flags->hostbased_service_name_draft_incompatibility)
+
+      )
     service = SSH_CONNECTION_SERVICE;
   else
     service = SSH_USERAUTH_SERVICE;
 
   gdata->packet_pending = TRUE;
   gdata->packet_type = SSH_AUTH_HOSTBASED_PACKET;
-  
+
   gdata->packet_len =
     ssh_encode_array_alloc(&gdata->packet,
                            /* session_id*/
@@ -214,8 +303,9 @@ SSH_FSM_STEP(hostbased_send_packet)
                            gdata->pubkey_algorithm,
                            strlen(gdata->pubkey_algorithm),
                            /* public key (string) */
-                           SSH_FORMAT_UINT32_STR, gdata->pubkeyblob,
-                           gdata->pubkeyblob_len,
+                           SSH_FORMAT_UINT32_STR,
+                           ssh_buffer_ptr(gdata->pubkeyblob),
+                           ssh_buffer_len(gdata->pubkeyblob),
                            /* client host name (FQDN, string) */
                            SSH_FORMAT_UINT32_STR, gdata->local_host_name,
                            strlen(gdata->local_host_name),
@@ -225,29 +315,29 @@ SSH_FSM_STEP(hostbased_send_packet)
                            SSH_FORMAT_END);
 
   SSH_DEBUG(3, ("Sending packet..."));
-  
+
   if (ssh_packet_wrapper_can_send(gdata->wrapper))
     ssh_register_timeout(0L, 0L, auth_hostbased_can_send, gdata);
-  
-  return SSH_FSM_SUSPENDED;
+
+  return SSH_OLDFSM_SUSPENDED;
 }
 
-SSH_FSM_STEP(hostbased_process_signature)
+SSH_OLDFSM_STEP(hostbased_process_signature)
 {
   SshBuffer buffer;
-  SSH_FSM_GDATA(SshClientHostbasedAuth);
-  
+  SSH_OLDFSM_GDATA(SshClientHostbasedAuth);
+
   /* We've got a signature. */
-  buffer = ssh_buffer_allocate();
-  
+  buffer = ssh_xbuffer_allocate();
+
   ssh_encode_buffer(buffer,
                     /* public key algorithm (string) */
                     SSH_FORMAT_UINT32_STR,
                     gdata->pubkey_algorithm,
                     strlen(gdata->pubkey_algorithm),
                     /* public key (string) */
-                    SSH_FORMAT_UINT32_STR, gdata->pubkeyblob,
-                    gdata->pubkeyblob_len,
+                    SSH_FORMAT_UINT32_STR, ssh_buffer_ptr(gdata->pubkeyblob),
+                    ssh_buffer_len(gdata->pubkeyblob),
                     /* client host name (FQDN, string) */
                     SSH_FORMAT_UINT32_STR, gdata->local_host_name,
                     strlen(gdata->local_host_name),
@@ -259,26 +349,28 @@ SSH_FSM_STEP(hostbased_process_signature)
                     SSH_FORMAT_END);
 
   /* Call the authentication method completion procedure. */
-  *gdata->state_placeholder = NULL;
-  (*gdata->completion)(SSH_AUTH_CLIENT_SEND, gdata->user, buffer,
+  (*gdata->completion)(SSH_AUTH_CLIENT_SEND_AND_CONTINUE_MULTIPLE,
+                       gdata->user, buffer,
                        gdata->completion_context);
-  
+
   /* Free the buffer */
   ssh_buffer_free(buffer);
 
-  SSH_FSM_SET_NEXT("hostbased_send_end");
-  
-  return SSH_FSM_CONTINUE;  
+  SSH_OLDFSM_SET_NEXT("hostbased_suspend");
+
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(hostbased_finish)
+SSH_OLDFSM_STEP(hostbased_finish)
 {
-  SSH_FSM_GDATA(SshClientHostbasedAuth);
+  SSH_OLDFSM_GDATA(SshClientHostbasedAuth);
 
   if (gdata->wrapper)
     ssh_packet_wrapper_can_receive(gdata->wrapper, FALSE);
 
-  return SSH_FSM_FINISH;
+  ssh_oldfsm_destroy(gdata->fsm);
+  gdata->fsm = NULL;
+  return SSH_OLDFSM_FINISH;
 }
 
 /* Callback, which is used to notify that our packetstream has a
@@ -291,7 +383,7 @@ void auth_hostbased_received_packet(SshPacketType type,
   SshClientHostbasedAuth state = (SshClientHostbasedAuth) context;
 
   ssh_packet_wrapper_can_receive(state->wrapper, FALSE);
-  
+
   switch (type)
     {
     case SSH_AUTH_HOSTBASED_PACKET:
@@ -305,7 +397,7 @@ void auth_hostbased_received_packet(SshPacketType type,
       SSH_TRACE(2, ("ssh-signer returned SSH_AUTH_HOSTBASED_COMPAT "\
                     "(this is an error)"));
       goto send_error_to_server;
-      break;      
+      break;
     case SSH_AUTH_HOSTBASED_END:
       /* signer shouldn't send this to us, so this is an error.*/
       SSH_TRACE(2, ("ssh-signer returned SSH_AUTH_HOSTBASED_END "\
@@ -313,9 +405,9 @@ void auth_hostbased_received_packet(SshPacketType type,
     send_error_to_server:
       ssh_warning("Error communicating with ssh-signer.");
 
-      ssh_fsm_set_next(state->main_thread, "hostbased_finish");
-      ssh_fsm_continue(state->main_thread);
-      
+      ssh_oldfsm_set_next(state->main_thread, "hostbased_finish");
+      ssh_oldfsm_continue(state->main_thread);
+
       *state->state_placeholder = NULL;
       (*state->completion)(SSH_AUTH_CLIENT_FAIL_AND_DISABLE_METHOD,
                            state->user, NULL,
@@ -324,8 +416,8 @@ void auth_hostbased_received_packet(SshPacketType type,
     case SSH_AUTH_HOSTBASED_SIGNATURE:
       SSH_TRACE(2, ("ssh-signer returned SSH_AUTH_HOSTBASED_SIGNATURE"));
 
-      ssh_fsm_set_next(state->main_thread, "hostbased_process_signature");
-      ssh_fsm_continue(state->main_thread);
+      ssh_oldfsm_set_next(state->main_thread, "hostbased_process_signature");
+      ssh_oldfsm_continue(state->main_thread);
       break;
     case SSH_AUTH_HOSTBASED_ERROR:
       SSH_TRACE(0, ("ssh-signer returned SSH_AUTH_HOSTBASED_ERROR"));
@@ -335,13 +427,13 @@ void auth_hostbased_received_packet(SshPacketType type,
                            state->user, NULL,
                            state->completion_context);
 
-      ssh_fsm_set_next(state->main_thread, "hostbased_process_error");
-      ssh_fsm_continue(state->main_thread);
+      ssh_oldfsm_set_next(state->main_thread, "hostbased_process_error");
+      ssh_oldfsm_continue(state->main_thread);
       break;
     }
   if (state->packet)
     ssh_xfree(state->packet);
-  
+
   state->packet = ssh_xmemdup(packet, packet_len);
   state->packet_len = packet_len;
 }
@@ -351,10 +443,11 @@ void auth_hostbased_received_packet(SshPacketType type,
 void auth_hostbased_received_eof(void *context)
 {
   SshClientHostbasedAuth state = (SshClientHostbasedAuth) context;
-  
+
   SSH_TRACE(0, ("received EOF from ssh-signer2."));
-  
-  ssh_packet_wrapper_send_eof(state->wrapper);
+
+  ssh_packet_wrapper_destroy(state->wrapper);
+  state->wrapper = NULL;
 
   /* Send failure message up, and return. */
   *state->state_placeholder = NULL;
@@ -362,8 +455,8 @@ void auth_hostbased_received_eof(void *context)
                        state->user, NULL,
                        state->completion_context);
 
-  ssh_fsm_set_next(state->main_thread, "hostbased_finish");
-  ssh_fsm_continue(state->main_thread);
+  ssh_oldfsm_set_next(state->main_thread, "hostbased_finish");
+  ssh_oldfsm_continue(state->main_thread);
 }
 
 /* Callback, which notifies that packetstream is ready for sending. */
@@ -375,12 +468,12 @@ void auth_hostbased_can_send(void *context)
     return;
 
   state->packet_pending = FALSE;
-  
+
   SSH_PRECOND(state);
   SSH_PRECOND(state->packet_type);
   SSH_PRECOND(state->packet);
   SSH_PRECOND(state->packet_len);
-    
+
   ssh_packet_wrapper_send(state->wrapper,
                           state->packet_type,
                           state->packet,
@@ -392,22 +485,23 @@ void auth_hostbased_can_send(void *context)
   state->packet_len = 0L;
 
   ssh_packet_wrapper_can_receive(state->wrapper, TRUE);
-  
-  ssh_fsm_continue(state->main_thread);  
+
+  ssh_oldfsm_continue(state->main_thread);
 }
 
 void hostbased_destructor(void *gdata)
 {
   SshClientHostbasedAuth state = (SshClientHostbasedAuth)gdata;
-  
+
   if (state->wrapper)
     {
       ssh_packet_wrapper_destroy(state->wrapper);
       state->wrapper = NULL;
     }
-  
+
+  ssh_xfree(state->pk_algorithms);
   ssh_xfree(state->pubkey_algorithm);
-  ssh_xfree(state->pubkeyblob);
+
   ssh_xfree(state->local_host_name);
 
   ssh_xfree(state->packet);
@@ -416,19 +510,22 @@ void hostbased_destructor(void *gdata)
   memset(state, 'F', sizeof(*state));
 }
 
-SshFSMStateMapItemStruct hostbased_states[] =
+SshOldFSMStateMapItemStruct hostbased_states[] =
 {
   { "hostbased_suspend", "Suspend thread", hostbased_suspend },
-  
+
+  { "hostbased_init_pubkey", "Initialize next pubkey",
+    hostbased_init_pubkey },
+
   { "hostbased_send_compat_flags", "Send compatibility flags",
     hostbased_send_compat_flags },
   { "hostbased_process_error", "Process received error packet",
-    hostbased_process_error },  
+    hostbased_process_error },
   { "hostbased_process_signature", "Process received signature",
-    hostbased_process_signature },  
+    hostbased_process_signature },
   { "hostbased_send_end", "Send END packet",
     hostbased_send_end },
-  
+
   { "hostbased_send_packet", "Send hostbased-authentication packet",
     hostbased_send_packet },
 
@@ -449,14 +546,14 @@ void ssh_client_auth_hostbased(SshAuthClientOperation op,
   SshClientHostbasedAuth state;
   SshClient client;
   SshStream stdio_stream;
-  char hostkeyfile[512];
-  /*  char *keytype;
-  SshPublicKey pubkey;*/
   char **signer_argv;
-  char config_filename[512];
   size_t hostname_len;
-  SshFSM fsm;
-  
+  char *config_filename;
+  SshOldFSM fsm;
+  Boolean partial_success = FALSE;
+  size_t bytes = 0L;
+              
+
   SSH_DEBUG(6, ("auth_hostbased op = %d  user = %s", op, user));
 
   client = (SshClient)method_context;
@@ -474,76 +571,87 @@ void ssh_client_auth_hostbased(SshAuthClientOperation op,
          authentication method before checking.*/
       (*completion)(SSH_AUTH_CLIENT_FAIL, user, NULL, completion_context);
       break;
-      
+
     case SSH_AUTH_CLIENT_OP_START:
       /* This is the first operation for doing hostbased authentication.
          We should not have any previous saved state when we come here. */
       SSH_ASSERT(*state_placeholder == NULL);
 
-      fsm = ssh_fsm_allocate(sizeof(*state),
-                             hostbased_states,
-                             SSH_FSM_NUM_STATES(hostbased_states),
-                             hostbased_destructor);
+      fsm = ssh_oldfsm_allocate(sizeof(*state),
+                                hostbased_states,
+                                SSH_OLDFSM_NUM_STATES(hostbased_states),
+                                hostbased_destructor);
 
-      state = ssh_fsm_get_gdata_fsm(fsm);
+      state = ssh_oldfsm_get_gdata_fsm(fsm);
 
       memset(state, 0, sizeof(*state));
-      
+
       state->fsm = fsm;
-      
+
       state->session_id = session_id;
       state->session_id_len = session_id_len;
       state->user = ssh_xstrdup(user);
       state->state_placeholder = state_placeholder;
 
       state->compat_flags = client->common->compat_flags;
-      
+
       /* We have to dig up the server configuration to get the place
          for the client host's publickey. This is a very kludgeish
          solution. XXX*/
       state->server_conf = ssh_server_create_config();
-      
-      /* Dig up hosts publickey. */
-      snprintf(config_filename, sizeof(config_filename), "%s/%s",
-               SSH_SERVER_DIR, SSH_SERVER_CONFIG_FILE);
-      
+
+      ssh_dsprintf(&config_filename, "%s/%s", SSH_SERVER_DIR,
+                   SSH_SERVER_CONFIG_FILE);
+
       if (!ssh_config_read_file(client->user_data, state->server_conf,
-                                NULL, config_filename, NULL))
+                                NULL, config_filename))
         SSH_TRACE(2, ("Failed to read config file %s", \
                       config_filename));
 
+      ssh_xfree(config_filename);
 
-      if(state->server_conf->public_host_key_file[0] != '/')
+      /* Dig up hosts publickey(s). */
+      if (!ssh_host_key_read_keys(state->server_conf->host_keys_ctx,
+                                  state->server_conf,
+                                  client->user_data, FALSE, TRUE))
         {
-          snprintf(hostkeyfile, sizeof(hostkeyfile), "%s/%s", SSH_SERVER_DIR,
-                   state->server_conf->public_host_key_file);
-        }
-      else
-        {
-          snprintf(hostkeyfile, sizeof(hostkeyfile), "%s",
-                   state->server_conf->public_host_key_file);  
-        }
-
-      /* This pubkey*-stuff is for the client _host's_ public
-         hostkey. */
-      SSH_DEBUG(4, ("Reading pubkey-blob from %s...", hostkeyfile));
-      if (ssh2_key_blob_read(client->user_data, hostkeyfile, TRUE, NULL,
-                             &state->pubkeyblob,
-                             &state->pubkeyblob_len, NULL) 
-          != SSH_KEY_MAGIC_PUBLIC)
-        {         
+          SSH_TRACE(0, ("Unable to load public hostkeys."));
           goto error;
         }
-      
-      SSH_DEBUG(4, ("done."));
-      if ((state->pubkey_algorithm =
-           ssh_pubkeyblob_type(state->pubkeyblob,
-                               state->pubkeyblob_len))
-          == NULL)
+
+      state->pk_algorithms =
+        ssh_host_key_get_algorithmlist(state->server_conf->host_keys_ctx);
+
+      if (
+
+
+
+          *state->compat_flags->hostbased_single_key_bug
+
+          )
         {
-          goto error;
+          /* Compat code with older (<2.4.1) clients. */
+          /* Prefer "ssh-dss" over others. */
+          if (ssh_cipher_list_contains(state->pk_algorithms, SSH_SSH_DSS))
+            {
+              ssh_xfree(state->pk_algorithms);
+              state->pk_algorithms = ssh_xstrdup(SSH_SSH_DSS);
+            }
+          /* If "ssh-dss" is not supported, try "ssh-rsa". */
+          else if (ssh_cipher_list_contains(state->pk_algorithms, SSH_SSH_RSA))
+            {
+              ssh_xfree(state->pk_algorithms);
+              state->pk_algorithms = ssh_xstrdup(SSH_SSH_DSS);
+            }
+          /* Otherwise, just try the first in the list. */
+          else
+            {
+              char *cp = ssh_snlist_get_name(state->pk_algorithms);
+              ssh_xfree(state->pk_algorithms);
+              state->pk_algorithms = cp;
+            }
         }
-      
+
       state->local_user_name = ssh_user_name(client->user_data);
       state->local_host_name = ssh_xcalloc(1, MAXHOSTNAMELEN + 1);
       ssh_tcp_get_host_name(state->local_host_name, MAXHOSTNAMELEN + 1);
@@ -555,9 +663,12 @@ void ssh_client_auth_hostbased(SshAuthClientOperation op,
           if (!client->config->default_domain)
             {
               ssh_warning("Hostbased authentication is DISABLED.");
-              ssh_warning("Hostname \"%s\" is not a fully qualified domain name (FQDN),", state->local_host_name);
-              ssh_warning("and DefaultDomain configuration parameter is not set. Ask your ");
-              ssh_warning("sysadmin to set it to the systemwide configuration file.");
+              ssh_warning("Hostname \"%s\" is not a fully qualified domain "
+                          "name (FQDN),", state->local_host_name);
+              ssh_warning("and DefaultDomain configuration parameter is not "
+                          "set. Ask your ");
+              ssh_warning("sysadmin to set it to the systemwide "
+                          "configuration file.");
               ssh_warning("(which is %s .)", SSH_CLIENT_GLOBAL_CONFIG_FILE);
               goto error;
             }
@@ -568,16 +679,16 @@ void ssh_client_auth_hostbased(SshAuthClientOperation op,
           state->local_host_name = temp_name;
           hostname_len = strlen(state->local_host_name);
         }
-      
+
       /* Sanity check */
       SSH_ASSERT(hostname_len + 2 < MAXHOSTNAMELEN);
       /* We want FQDN. */
       state->local_host_name[hostname_len] = '.';
       state->local_host_name[hostname_len + 1] = '\0';
-      
+
       state->completion = completion;
       state->completion_context = completion_context;
-      
+
       /* Assign the state to the placeholder that survives across
          calls.  (this is needed only for detecting error conditions.)  */
       *state_placeholder = state;
@@ -599,7 +710,7 @@ void ssh_client_auth_hostbased(SshAuthClientOperation op,
           signer_argv = ssh_xcalloc(2, sizeof(char *));
           signer_argv[0] = client->config->signer_path;
           signer_argv[1] = NULL;
-          
+
           execvp(client->config->signer_path, signer_argv);
           fprintf(stderr, "Couldn't exec '%s' (System error message: %s)",
                   client->config->signer_path, strerror(errno));
@@ -620,34 +731,64 @@ void ssh_client_auth_hostbased(SshAuthClientOperation op,
 
       ssh_packet_wrapper_can_receive(state->wrapper, FALSE);
 
-      state->main_thread = ssh_fsm_spawn(state->fsm,
-                                         0, "hostbased_send_compat_flags",
-                                         NULL, NULL);
-      
+      state->main_thread = ssh_oldfsm_spawn(state->fsm,
+                                            0, "hostbased_send_compat_flags",
+                                            NULL, NULL);
       /* Rest is done in callbacks. */
       break;
-      
-    case SSH_AUTH_CLIENT_OP_CONTINUE:          
-      SSH_TRACE(2, ("Invalid message. We didn't return " \
-                    "SSH_AUTH_CLIENT_SEND_AND_CONTINUE at any stage!"));
-      /* Send failure message.*/
-      (*completion)(SSH_AUTH_CLIENT_FAIL_AND_DISABLE_METHOD,
-                    user, NULL, completion_context);
 
-      if (state && state->main_thread)
+    case SSH_AUTH_CLIENT_OP_CONTINUE:
+      SSH_ASSERT(state);
+
+      switch (packet_type)
         {
-          ssh_fsm_set_next(state->main_thread, "hostbased_finish");
-          ssh_fsm_continue(state->main_thread);
+        case SSH_MSG_USERAUTH_FAILURE:
+          /* Decode the packet to get the `partial success'
+             field. */
+          bytes = ssh_decode_buffer(packet_in,
+                                    /* We don't care about the
+                                       continuations. */
+                                    SSH_FORMAT_UINT32_STR, NULL, NULL,
+                                    SSH_FORMAT_BOOLEAN, &partial_success,
+                                    SSH_FORMAT_END);
+
+          /* The auth layer has already sanity checked the
+             packet. (XXX we can in principle fail here because
+             of memory allocation errors) */
+          SSH_ASSERT(bytes);
+          if (partial_success == FALSE )
+            {
+              SSH_TRACE(0, ("Server rejected the signature."));
+              ssh_oldfsm_set_next(state->main_thread,
+                                  "hostbased_init_pubkey");
+              ssh_oldfsm_continue(state->main_thread);
+              break;
+            }
+          /* FALLTHROUGH */
+        case SSH_MSG_USERAUTH_SUCCESS:
+          *state_placeholder = NULL;
+          if (packet_type == SSH_MSG_USERAUTH_SUCCESS)
+            SSH_TRACE(2, ("Received SSH_MSG_USERAUTH_SUCCESS"));
+          else
+            SSH_TRACE(2, ("Received partial success (\"hostbased\" auth was "
+                          "successful, but more needed)."));
+            
+          ssh_oldfsm_set_next(state->main_thread, "hostbased_send_end");
+          ssh_oldfsm_continue(state->main_thread);
+          break;
+        default:
+          ssh_fatal("ssh_client_auth_hostbased: Received invalid "
+                    "message %d from server.", packet_type);
+          break;
         }
-      
       return;
 
     case SSH_AUTH_CLIENT_OP_ABORT:
       /* Abort the authentication operation immediately. */
       if (state && state->main_thread)
         {
-          ssh_fsm_set_next(state->main_thread, "hostbased_finish");
-          ssh_fsm_continue(state->main_thread);
+          ssh_oldfsm_set_next(state->main_thread, "hostbased_finish");
+          ssh_oldfsm_continue(state->main_thread);
         }
       *state_placeholder = NULL;
       break;
@@ -660,9 +801,9 @@ void ssh_client_auth_hostbased(SshAuthClientOperation op,
 
  error:
   /* Destroy state. */
-  ssh_fsm_destroy(fsm);
+  ssh_oldfsm_destroy(fsm);
   *state_placeholder = NULL;
-  
+
   /* Send failure message.*/
   (*completion)(SSH_AUTH_CLIENT_FAIL_AND_DISABLE_METHOD, user,
                 NULL, completion_context);
