@@ -21,6 +21,8 @@ auths-passwd.c
 #include "sshconfig.h"
 #include "auths-common.h"
 #include "auths-passwd.h"
+#include "sshdsprintf.h"
+#include "sshuserfiles.h"
 
 #define SSH_DEBUG_MODULE "Ssh2AuthPasswdServer"
 
@@ -34,7 +36,7 @@ Boolean is_root_and_disabled(SshConfig config, SshUser uc,
                              SshAuthServerCompletionProc completion_proc,
                              void *completion_context)
 {
-  if(ssh_user_uid(uc) == SSH_UID_ROOT &&
+  if(ssh_user_uid(uc) == UID_ROOT &&
      (config->permit_root_login == SSH_ROOTLOGIN_FALSE ||
       config->permit_root_login == SSH_ROOTLOGIN_NOPWD))
     {
@@ -81,7 +83,7 @@ void ssh_server_auth_passwd(SshAuthServerOperation op,
   SshConfig config = server->config;
   Boolean change_request;
   char *password, *prompt;
-  int disable_method = 0;
+  Boolean disable_method = FALSE;
   size_t pass_len = 0L;
   
   SSH_TRACE(6, ("password auth."));
@@ -98,25 +100,23 @@ void ssh_server_auth_passwd(SshAuthServerOperation op,
           if (config->password_guesses <= 0)
             {
               /* If this attempt is not succesful, disable this method. */
-              disable_method = 1;
+              disable_method = TRUE;
             }
           goto password_bad;
         }
       
-      {
-        config->password_guesses--;
-        if (config->password_guesses <= 0)
-          {
-            /* If this attempt is not succesful, disable this method. */
-            disable_method = 1;
-          }
+      config->password_guesses--;
+      if (config->password_guesses <= 0)
+        {
+          /* If this attempt is not succesful, disable this method. */
+          disable_method = TRUE;
+        }
 
 
-        if (is_root_and_disabled(config, uc, packet,
-          completion_proc, completion_context))
-          return;
+      if (is_root_and_disabled(config, uc, packet,
+                               completion_proc, completion_context))
+        return;
 
-      }
 
       /* Parse the password authentication request. */
       if (ssh_decode_buffer(packet,
@@ -154,7 +154,7 @@ void ssh_server_auth_passwd(SshAuthServerOperation op,
           ssh_xfree(password);
           goto password_bad;
         }
-      
+
 
       /* Try SECURE RPC passwords.  We do this first, as this might be
          needed to access disks. */
@@ -184,17 +184,9 @@ void ssh_server_auth_passwd(SshAuthServerOperation op,
 
 
       /* Try a local password (either normal or shadow). */
-#ifdef SSHDIST_SESSION_SIA
-#ifdef HAVE_SIA
       if (ssh_user_validate_local_password(uc, 
                                            password,
                                            server->common->remote_host))
-#else /* HAVE_SIA */
-      if (ssh_user_validate_local_password(uc, password))
-#endif /* HAVE_SIA */
-
-
-#endif /* SSHDIST_SESSION_SIA */
         {
 
 
@@ -223,6 +215,9 @@ void ssh_server_auth_passwd(SshAuthServerOperation op,
       goto password_bad;
 
     password_bad:
+      if (uc)
+        ssh_user_record_login_failure(uc, server->common->remote_host);
+      
       (*completion_proc)((disable_method ?
                           SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED :
                           SSH_AUTH_SERVER_REJECTED),
@@ -240,16 +235,25 @@ void ssh_server_auth_passwd(SshAuthServerOperation op,
       /* Check if the user's password needs to be changed. */
       if (ssh_user_password_must_be_changed(uc, &prompt))
         {
-          ssh_buffer_clear(packet);
-          ssh_encode_buffer(packet,
-                            SSH_FORMAT_CHAR,
-                            (unsigned int) SSH_MSG_USERAUTH_PASSWD_CHANGEREQ,
-                            SSH_FORMAT_UINT32_STR, prompt, strlen(prompt),
-                            SSH_FORMAT_END);
-          ssh_xfree(prompt);
-          (*completion_proc)(SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK,
-                             packet, completion_context);
-          return;
+          ssh_log_event(config->log_facility, SSH_LOG_INFORMATIONAL,
+                        "User %s forced to change password.",
+                        ssh_user_name(uc));
+          if (server->common->publickey_options)
+            ssh_xfree(server->common->publickey_options);
+          else
+            server->common->publickey_options =
+              ssh_xcalloc(1, sizeof(*server->common->publickey_options));
+
+          ssh_xfree(server->common->publickey_options->command);
+
+          /* XXX This is a problem, if the same uid is shared by many
+             user's. In that case, invoking "passwd" without arguments
+             will only change the _first_ password in the passwd
+             db. OTOH, things are made trickier with RH Linux, which
+             doesn't like a <username> argument with passwd when
+             executer is non-root... Fantastic. */
+          server->common->publickey_options->command =
+            ssh_xstrdup(PASSWD_PATH);
         }
 
       (*completion_proc)(SSH_AUTH_SERVER_ACCEPTED, packet, completion_context);
