@@ -4,7 +4,7 @@ auths-securid.c
 
   Author: Graeme Ahokas <gahokas@ssh.com>
 
-  Copyright (C) 2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 2000, 2001 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   SecurID authentication, server-side. This calls the SecurID library
@@ -19,7 +19,7 @@ auths-securid.c
 #include "sshserver.h"
 #include "sshconfig.h"
 #include "auths-common.h"
-#include "sshfsm.h"
+#include "ssholdfsm.h"
 
 #ifdef SSH_SERVER_WITH_SECURID
 
@@ -35,8 +35,8 @@ union config_record configure;
 
 typedef struct SshServerSecurIDAuthRec
 {
-  SshFSM fsm;
-  SshFSMThread main_thread;
+  SshOldFSM fsm;
+  SshOldFSMThread main_thread;
   SshUser uc;
 
   SshBuffer client_packet;
@@ -83,19 +83,19 @@ int ssh_server_securid_init( struct SD_CLIENT *sd_data )
 
 /* begins server communcations with the securid server and
    asks the client for the user's passphrase */
-SSH_FSM_STEP(server_securid_start)
+SSH_OLDFSM_STEP(server_securid_start)
 {
-  SSH_FSM_GDATA(SshServerSecurIDAuth);
+  SSH_OLDFSM_GDATA(SshServerSecurIDAuth);
   char *passcode;
-  int  length;
+  size_t length;
   int ret_code;
 
   if ( ssh_server_securid_init( gdata->sd_data ) )
     {
-      ssh_buffer_free(gdata->client_packet); 
+      ssh_buffer_free(gdata->client_packet);
       SSH_DEBUG(2,("Error connecting to SecurID server."));
-      SSH_FSM_SET_NEXT("server_securid_reject_and_disable");
-      return SSH_FSM_CONTINUE;
+      SSH_OLDFSM_SET_NEXT("server_securid_reject_and_disable");
+      return SSH_OLDFSM_CONTINUE;
     }
 
   /* communications with securid server established
@@ -112,8 +112,8 @@ SSH_FSM_STEP(server_securid_start)
     {
       ssh_buffer_free(gdata->client_packet);
       SSH_DEBUG(4,("Client packet decode failed.\r\n"));
-      SSH_FSM_SET_NEXT("server_securid_reject_and_disable");
-      return SSH_FSM_CONTINUE; 
+      SSH_OLDFSM_SET_NEXT("server_securid_reject_and_disable");
+      return SSH_OLDFSM_CONTINUE;
     }
 
   /* XXX could check now for valid length of passcode */
@@ -128,48 +128,48 @@ SSH_FSM_STEP(server_securid_start)
       case ACM_OK:
 
         SSH_DEBUG(2,("User AUTHENTICATED.\r\n"));
-        SSH_FSM_SET_NEXT("server_securid_auth_success");
-        return SSH_FSM_CONTINUE; 
+        SSH_OLDFSM_SET_NEXT("server_securid_auth_success");
+        return SSH_OLDFSM_CONTINUE;
 
       case ACM_ACCESS_DENIED:
 
         gdata->config->securid_guesses--;
         if (gdata->config->securid_guesses <= 0)
           {
-            SSH_FSM_SET_NEXT("server_securid_reject_and_disable");
-            return SSH_FSM_CONTINUE;
+            SSH_OLDFSM_SET_NEXT("server_securid_reject_and_disable");
+            return SSH_OLDFSM_CONTINUE;
           }
 
         SSH_DEBUG(2,("Access denied from SecurID server"));
-        SSH_FSM_SET_NEXT("server_securid_reject");
-        return SSH_FSM_CONTINUE;
+        SSH_OLDFSM_SET_NEXT("server_securid_reject");
+        return SSH_OLDFSM_CONTINUE;
 
       case ACM_NEXT_CODE_REQUIRED:
 
         SSH_DEBUG(2,("Next code required"));
-        SSH_FSM_SET_NEXT("server_securid_get_next_code");
-        return SSH_FSM_CONTINUE;
+        SSH_OLDFSM_SET_NEXT("server_securid_get_next_code");
+        return SSH_OLDFSM_CONTINUE;
 
       case ACM_NEW_PIN_REQUIRED:
 
         SSH_DEBUG(2,("New PIN required."));
-        SSH_FSM_SET_NEXT("server_securid_get_new_pin");
-        return SSH_FSM_CONTINUE;
+        SSH_OLDFSM_SET_NEXT("server_securid_get_new_pin");
+        return SSH_OLDFSM_CONTINUE;
 
       default:
         SSH_DEBUG(4,("Unknown return code from SecurID server sd_check"));
-        SSH_FSM_SET_NEXT("server_securid_reject_and_disable");
-        return SSH_FSM_CONTINUE;
+        SSH_OLDFSM_SET_NEXT("server_securid_reject_and_disable");
+        return SSH_OLDFSM_CONTINUE;
     }
 
   /* not reached */
 }
 
 /* Successful authentication, send ACCEPTED msg and quit */
-SSH_FSM_STEP(server_securid_auth_success)
+SSH_OLDFSM_STEP(server_securid_auth_success)
 {
 
-  SSH_FSM_GDATA(SshServerSecurIDAuth);
+  SSH_OLDFSM_GDATA(SshServerSecurIDAuth);
 
   *gdata->state_placeholder = NULL;
 
@@ -177,17 +177,17 @@ SSH_FSM_STEP(server_securid_auth_success)
                             NULL,
                             gdata->completion_context);
 
-  SSH_FSM_SET_NEXT("server_securid_finish");
-  return SSH_FSM_CONTINUE;
+  SSH_OLDFSM_SET_NEXT("server_securid_finish");
+  return SSH_OLDFSM_CONTINUE;
 }
 
 /* Sends a CHALLENGE msg to client for next token */
-SSH_FSM_STEP(server_securid_get_next_code)
+SSH_OLDFSM_STEP(server_securid_get_next_code)
 {
 
-  SSH_FSM_GDATA(SshServerSecurIDAuth);
+  SSH_OLDFSM_GDATA(SshServerSecurIDAuth);
 
-  gdata->response_packet = ssh_buffer_allocate();
+  gdata->response_packet = ssh_xbuffer_allocate();
 
   ssh_encode_buffer(gdata->response_packet,
                     SSH_FORMAT_CHAR,
@@ -200,33 +200,33 @@ SSH_FSM_STEP(server_securid_get_next_code)
                             gdata->response_packet,
                             gdata->completion_context);
 
-  SSH_FSM_SET_NEXT("server_securid_process_next_code");
-  return SSH_FSM_SUSPENDED;
+  SSH_OLDFSM_SET_NEXT("server_securid_process_next_code");
+  return SSH_OLDFSM_SUSPENDED;
 }
 
 
 /* Finishes the fsm */
-SSH_FSM_STEP(server_securid_finish)
+SSH_OLDFSM_STEP(server_securid_finish)
 {
-  SSH_FSM_GDATA(SshServerSecurIDAuth);
+  SSH_OLDFSM_GDATA(SshServerSecurIDAuth);
 
   /* close communications with securid server */
   sd_close();
 
-  ssh_fsm_destroy(gdata->fsm);
+  ssh_oldfsm_destroy(gdata->fsm);
   gdata->fsm = NULL;
 
   *gdata->state_placeholder = NULL;
 
-  return SSH_FSM_FINISH;
+  return SSH_OLDFSM_FINISH;
 }
 
 /* receives challenge response from client and tests*/
-SSH_FSM_STEP(server_securid_process_next_code)
+SSH_OLDFSM_STEP(server_securid_process_next_code)
 {
-  SSH_FSM_GDATA(SshServerSecurIDAuth);
+  SSH_OLDFSM_GDATA(SshServerSecurIDAuth);
   char *passcode;
-  int length;
+  size_t length;
   int ret_code;
 
   if (ssh_decode_buffer(gdata->client_packet,
@@ -238,11 +238,11 @@ SSH_FSM_STEP(server_securid_process_next_code)
     {
       ssh_xfree(gdata->client_packet);
       SSH_DEBUG(4,("Client packet decode failed."));
-      SSH_FSM_SET_NEXT("server_securid_reject_and_disable");
-      return SSH_FSM_CONTINUE; 
+      SSH_OLDFSM_SET_NEXT("server_securid_reject_and_disable");
+      return SSH_OLDFSM_CONTINUE;
     }
 
-  /* we're verifying the challenge from the server for the next token 
+  /* we're verifying the challenge from the server for the next token
      or making user authenticate with new pin */
 
   /* XXX could check now for valid length of passcode */
@@ -262,14 +262,14 @@ SSH_FSM_STEP(server_securid_process_next_code)
       case ACM_OK:
 
         SSH_DEBUG(2,("User AUTHENTICATED by challenge."));
-        SSH_FSM_SET_NEXT("server_securid_auth_success");
-        return SSH_FSM_CONTINUE;
+        SSH_OLDFSM_SET_NEXT("server_securid_auth_success");
+        return SSH_OLDFSM_CONTINUE;
 
       case ACM_ACCESS_DENIED:
 
         SSH_DEBUG(2,("User access denied from challenge."));
-        SSH_FSM_SET_NEXT("server_securid_reject");
-        return SSH_FSM_CONTINUE;
+        SSH_OLDFSM_SET_NEXT("server_securid_reject");
+        return SSH_OLDFSM_CONTINUE;
 
       case ACM_NEXT_CODE_REQUIRED:
 
@@ -277,22 +277,22 @@ SSH_FSM_STEP(server_securid_process_next_code)
            a next code req msg after a successful authentication with
            a new pin, but just to be safe */
         SSH_DEBUG(2,("Next code required"));
-        SSH_FSM_SET_NEXT("server_securid_get_next_code");
-        return SSH_FSM_CONTINUE;
+        SSH_OLDFSM_SET_NEXT("server_securid_get_next_code");
+        return SSH_OLDFSM_CONTINUE;
 
       default:
 
         SSH_DEBUG(4,("Unknown return code from securid server sd_next"));
-        SSH_FSM_SET_NEXT("server_securid_reject_and_disable");
-        return SSH_FSM_CONTINUE;
+        SSH_OLDFSM_SET_NEXT("server_securid_reject_and_disable");
+        return SSH_OLDFSM_CONTINUE;
     }
 }
 
 /* Sends a reject authentication msg to client and stops fsm */
-SSH_FSM_STEP(server_securid_reject)
+SSH_OLDFSM_STEP(server_securid_reject)
 {
 
-  SSH_FSM_GDATA(SshServerSecurIDAuth);
+  SSH_OLDFSM_GDATA(SshServerSecurIDAuth);
   SSH_PRECOND(gdata);
 
   *gdata->state_placeholder = NULL;
@@ -301,15 +301,15 @@ SSH_FSM_STEP(server_securid_reject)
                             NULL,
                             gdata->completion_context);
 
-  SSH_FSM_SET_NEXT("server_securid_finish");
+  SSH_OLDFSM_SET_NEXT("server_securid_finish");
 
-  return SSH_FSM_CONTINUE;
+  return SSH_OLDFSM_CONTINUE;
 }
 
 /* sends reject and method disable msg and stops fsm */
-SSH_FSM_STEP(server_securid_reject_and_disable)
+SSH_OLDFSM_STEP(server_securid_reject_and_disable)
 {
-  SSH_FSM_GDATA(SshServerSecurIDAuth);
+  SSH_OLDFSM_GDATA(SshServerSecurIDAuth);
   SSH_PRECOND(gdata);
 
   *gdata->state_placeholder = NULL;
@@ -318,23 +318,23 @@ SSH_FSM_STEP(server_securid_reject_and_disable)
                             NULL,
                             gdata->completion_context);
 
-  SSH_FSM_SET_NEXT("server_securid_finish");
-  return SSH_FSM_CONTINUE;
+  SSH_OLDFSM_SET_NEXT("server_securid_finish");
+  return SSH_OLDFSM_CONTINUE;
 }
 
 
 /* sends the client a new pin request, complete with information
    on the limitations of the pin, and a server assigned pin if
    the user is not permitted to choose */
-SSH_FSM_STEP(server_securid_get_new_pin)
+SSH_OLDFSM_STEP(server_securid_get_new_pin)
 {
-  SSH_FSM_GDATA(SshServerSecurIDAuth);
+  SSH_OLDFSM_GDATA(SshServerSecurIDAuth);
   unsigned int user_selectable;
   Boolean alphanumeric;
   char *system_pin;
 
 
-  gdata->response_packet = ssh_buffer_allocate();
+  gdata->response_packet = ssh_xbuffer_allocate();
 
   /* assign the user_selectable field to inform
      the user about their limitations in pin selection*/
@@ -379,19 +379,19 @@ SSH_FSM_STEP(server_securid_get_new_pin)
 
   ssh_xfree(system_pin);
 
-  SSH_FSM_SET_NEXT("server_securid_process_new_pin");
-  return SSH_FSM_SUSPENDED;
+  SSH_OLDFSM_SET_NEXT("server_securid_process_new_pin");
+  return SSH_OLDFSM_SUSPENDED;
 }
 
 
 /* takes the pin from the client and informs the securid server */
-SSH_FSM_STEP(server_securid_process_new_pin)
+SSH_OLDFSM_STEP(server_securid_process_new_pin)
 {
-  SSH_FSM_GDATA(SshServerSecurIDAuth);
+  SSH_OLDFSM_GDATA(SshServerSecurIDAuth);
 
   int ret_code;
   char *new_pin;
-  int new_pin_len;
+  size_t new_pin_len;
   char cancel_new_pin;
 
 
@@ -402,8 +402,8 @@ SSH_FSM_STEP(server_securid_process_new_pin)
                         SSH_FORMAT_END) == 0)
     {
       ssh_buffer_free(gdata->client_packet);
-      SSH_FSM_SET_NEXT("server_securid_reject_and_disable");
-      return SSH_FSM_CONTINUE;
+      SSH_OLDFSM_SET_NEXT("server_securid_reject_and_disable");
+      return SSH_OLDFSM_CONTINUE;
     }
 
   ssh_buffer_free(gdata->client_packet);
@@ -412,7 +412,7 @@ SSH_FSM_STEP(server_securid_process_new_pin)
     cancel_new_pin = 1;
   else
     cancel_new_pin = 0;
- 
+
   /* verify the new pin with the securid server */
   ret_code = sd_pin(new_pin, cancel_new_pin, gdata->sd_data);
 
@@ -423,18 +423,18 @@ SSH_FSM_STEP(server_securid_process_new_pin)
       SSH_DEBUG(2,("New PIN operation successful"));
       gdata->pin_change_success = TRUE;
       /* new pin is ok so send a passphrase next token msg to client */
-      SSH_FSM_SET_NEXT("server_securid_get_next_code");
-      return SSH_FSM_CONTINUE;
+      SSH_OLDFSM_SET_NEXT("server_securid_get_next_code");
+      return SSH_OLDFSM_CONTINUE;
     }
 
   SSH_DEBUG(2,("New PIN operation failed."));
   /* new pin operation failed */
-  SSH_FSM_SET_NEXT("server_securid_reject");
-  return SSH_FSM_CONTINUE;
+  SSH_OLDFSM_SET_NEXT("server_securid_reject");
+  return SSH_OLDFSM_CONTINUE;
 }
 
 
-SshFSMStateMapItemStruct server_securid_states[] =
+SshOldFSMStateMapItemStruct server_securid_states[] =
 {
   { "server_securid_start", "Starting securid authentication",
      server_securid_start },
@@ -463,7 +463,7 @@ void server_securid_destructor( void *gdata_ctx )
   SSH_PRECOND(gdata);
 
   if ( gdata->fsm )
-    ssh_fsm_destroy( gdata->fsm );
+    ssh_oldfsm_destroy( gdata->fsm );
   gdata->fsm = NULL;
   *gdata->state_placeholder = NULL;
 
@@ -488,9 +488,9 @@ void ssh_server_auth_securid(SshAuthServerOperation op,
 {
   SshServer server = (SshServer)method_context;
   SshConfig config = server->config;
-  SshServerSecurIDAuth state = *state_placeholder; 
+  SshServerSecurIDAuth state = *state_placeholder;
   int packet_type;
-  SshFSM fsm;
+  SshOldFSM fsm;
 
   SSH_TRACE(6, ("securid auth."));
 
@@ -514,7 +514,7 @@ void ssh_server_auth_securid(SshAuthServerOperation op,
         }
 
 
-      if(ssh_user_uid(uc) == SSH_UID_ROOT &&
+      if(ssh_user_uid(uc) == UID_ROOT &&
          (config->permit_root_login == SSH_ROOTLOGIN_FALSE ||
           config->permit_root_login == SSH_ROOTLOGIN_NOPWD))
         {
@@ -535,15 +535,15 @@ void ssh_server_auth_securid(SshAuthServerOperation op,
       /* start up the fsm */
 
       SSH_ASSERT(*state_placeholder == NULL);
-      fsm = ssh_fsm_allocate(sizeof(*state),
+      fsm = ssh_oldfsm_allocate(sizeof(*state),
                              server_securid_states,
-                             SSH_FSM_NUM_STATES(server_securid_states),
+                             SSH_OLDFSM_NUM_STATES(server_securid_states),
                              server_securid_destructor );
 
-      state = ssh_fsm_get_gdata_fsm(fsm);
+      state = ssh_oldfsm_get_gdata_fsm(fsm);
       memset(state, 0, sizeof(*state));
       state->fsm = fsm;
- 
+
       state->uc = uc;
       state->completion_proc = completion_proc;
       state->completion_context = completion_context;
@@ -554,8 +554,8 @@ void ssh_server_auth_securid(SshAuthServerOperation op,
       state->pin_change_success = FALSE;
       state->config = config;
       *state_placeholder = state;
- 
-      state->main_thread = ssh_fsm_spawn(state->fsm,
+
+      state->main_thread = ssh_oldfsm_spawn(state->fsm,
                                          0, "server_securid_start",
                                          NULL, NULL);
 
@@ -569,14 +569,14 @@ void ssh_server_auth_securid(SshAuthServerOperation op,
       state->completion_proc = completion_proc;
       state->completion_context = completion_context;
 
-      ssh_fsm_continue(state->main_thread);
+      ssh_oldfsm_continue(state->main_thread);
       return;
 
     case SSH_AUTH_SERVER_OP_ABORT:
 
       SSH_ASSERT(state && state->main_thread);
-      ssh_fsm_set_next(state->main_thread, "server_securid_finish");
-      ssh_fsm_continue(state->main_thread);
+      ssh_oldfsm_set_next(state->main_thread, "server_securid_finish");
+      ssh_oldfsm_continue(state->main_thread);
       (*completion_proc)(SSH_AUTH_SERVER_REJECTED, packet, completion_context);
       return;
 
@@ -586,9 +586,9 @@ void ssh_server_auth_securid(SshAuthServerOperation op,
       if (state)
         if (state->main_thread)
           {
-            ssh_fsm_set_next(state->main_thread,
+            ssh_oldfsm_set_next(state->main_thread,
                              "server_securid_finish");
-            ssh_fsm_continue(state->main_thread);
+            ssh_oldfsm_continue(state->main_thread);
           }
 
       *state_placeholder = NULL;
@@ -599,7 +599,7 @@ void ssh_server_auth_securid(SshAuthServerOperation op,
     default:
       ssh_fatal("ssh_server_auth_securid: unknown op %d", (int)op);
     }
- 
+
   SSH_NOTREACHED;
 }
 
