@@ -18,6 +18,8 @@ readpass.c
 
 #define SSH_DEBUG_MODULE "SshReadPass"
 
+
+#ifndef VXWORKS
 /* Saved old terminal mode for read_passphrase. */
 #ifdef USING_TERMIOS
 static struct termios saved_tio;
@@ -110,14 +112,14 @@ char *ssh_read_passphrase(const char *prompt, Boolean from_stdin)
                 }
               quoted_prompt[i] = '\0';
   
-              snprintf(command, sizeof(command),
-                       "ssh-askpass '%.400s'", quoted_prompt);
+              ssh_snprintf(command, sizeof(command),
+                           SSH_ASKPASS_PATH " '%.400s'", quoted_prompt);
               
               f = popen(command, "r");
               if (f == NULL)
                 {
-                  fprintf(stderr, "Could not query passphrase: '%.200s' failed.\n",
-                          command);
+                  fprintf(stderr, "Could not query passphrase: '%.200s' "
+                          "failed.\n", command);
                   return NULL;
                 }
               if (!fgets(buf, sizeof(buf), f))
@@ -133,7 +135,8 @@ char *ssh_read_passphrase(const char *prompt, Boolean from_stdin)
             }
 
           /* No controlling terminal and no DISPLAY.  Nowhere to read. */
-          fprintf(stderr, "You have no controlling tty and no DISPLAY.  Cannot read passphrase.\n");
+          fprintf(stderr, "You have no controlling tty and no DISPLAY.  "
+                  "Cannot read passphrase.\n");
           return NULL;
         }
     }
@@ -235,33 +238,10 @@ char *ssh_read_passphrase(const char *prompt, Boolean from_stdin)
         /* Close the file. */
         if (f != stdin)
           fclose(f);
-        return NULL;
-        
+        return NULL;        
       }
   }
 
-#ifdef OLDXXX
-  /* Read the passphrase from the terminal. */
-  if (fgets(buf, sizeof(buf), f) == NULL)
-    {
-      /* Got EOF.  Just return NULL. */
-      /* Restore terminal modes. */
-#ifdef USING_TERMIOS
-      tcsetattr(fileno(f), TCSANOW, &saved_tio);
-#endif
-#ifdef USING_SGTTY
-      ioctl(fileno(f), TIOCSETP, &saved_tio);
-#endif
-      /* Restore the signal handler. */
-      signal(SIGINT, old_handler);
-      /* Print a newline (the prompt probably didn\'t have one). */
-      fprintf(stderr, "\r\n");
-      /* Close the file. */
-      if (f != stdin)
-        fclose(f);
-      return NULL;
-    }
-#endif /* OLDXXX */
   /* Restore terminal modes. */
 #ifdef USING_TERMIOS
   tcsetattr(fileno(f), TCSANOW, &saved_tio);
@@ -353,3 +333,7 @@ Boolean ssh_read_confirmation(const char *prompt, Boolean from_stdin)
     fclose(f);
   return TRUE;
 }
+
+
+
+#endif /* VXWORKS */
