@@ -23,7 +23,12 @@ char *main_help =
 "lcd\t\tpwd\t\tlpwd\t\tls\t\tlls\r\n"
 "get\t\tmget\t\tput\t\tmput\t\trm\r\n"
 "lrm\t\tmkdir\t\tlmkdir\t\trmdir\t\tlrmdir\r\n"
-"rename\t\tlrename\t\thelp\t\t\r\n"
+"rename\t\tlrename\t\tascii\t\tbinary\t\tauto\r\n"
+"setext\t\tgetext\t\t"
+
+
+
+"help\t\t\r\n"
 "\r\n";
 
 char *helps[][2] = {
@@ -99,6 +104,27 @@ char *helps[][2] = {
     "exists, the files are left intact.\r\n" },
   { "lrename",
     "Same as rename, but operates on the local files.\r\n" },
+  { "ascii",
+    "ascii\r\n"
+    "Files will be transfered in ascii mode.\r\n" },
+  { "binary",
+    "binary\r\n"
+    "Files will be transfered in binary mode.\r\n" },
+  { "auto",
+    "auto\r\n"
+    "File transfer mode will be selected automatically from the file\r\n"
+    "extension.\r\n" },
+  { "setext",
+    "setext [ <extension> ... ]\r\n"
+    "Sets the file extensions that will be ascii in the auto transfer\r\n"
+    "mode. Wildcards ? and * can be used in the file extensions.\r\n" },
+  { "getext",
+    "getext\r\n"
+    "Displays the extensions that will be ascii in the auto transfer\r\n"
+    "mode.\r\n" },
+  { "setmask",
+    "setmask <umask>\r\n"
+    "Set the default file permission mask for upload.\r\n" },
   { "help",
     "help [ <topic> ]\r\n"
     "If <topic> is not given, lists the available topics. If <topic> is\r\n"
