@@ -1,20 +1,20 @@
 /*
 
-  sftppager.h
+  sftppager.c
 
   Author: Tomi Salo <ttsalo@ssh.com>
 
-  Copyright (c) 1999 SSH Communications Security, Finland
+  Copyright (c) 1999, 2001 SSH Communications Security, Finland
   All rights reserved.
 
   Created Wed Jan 19 15:37:52 2000.
 
   Paging for Sftp2.
-  
+
   */
 #include "sshincludes.h"
 #include "sftppager.h"
-#include "sshfsm.h"
+#include "ssholdfsm.h"
 #include "sshbuffer.h"
 
 
@@ -26,8 +26,8 @@
 struct SshSftpPagerCtxRec
 {
   SshBuffer buffer;
-  SshFSMCondition buffer_grown;
-  SshFSMThread thread;
+  SshOldFSMCondition buffer_grown;
+  SshOldFSMThread thread;
   SshStream stream;
   unsigned int screen_width;
   unsigned int screen_height;
@@ -42,8 +42,8 @@ struct SshSftpPagerCtxRec
   SshSftpPagerCb user_callback;
   void *user_context;
 
-  SshFSMCondition input_avail;
-  SshFSMThread input_availability_notifier;
+  SshOldFSMCondition input_avail;
+  SshOldFSMThread input_availability_notifier;
 
   int received, printed;
 
@@ -52,7 +52,7 @@ struct SshSftpPagerCtxRec
 
 struct SshSftpPagerIANotifierCtxRec
 {
-  SshFSMCondition input_avail;
+  SshOldFSMCondition input_avail;
   SshStream stream;
   Boolean want_input_signal, callback_set;
 };
@@ -66,39 +66,39 @@ ssh_sftp_pager_thread_destructor(void *tdata)
 
   ssh_buffer_free(ctx->buffer);
   ssh_xfree(ctx->screen_buffer);
-  ssh_fsm_condition_destroy(ctx->input_avail);
+  ssh_oldfsm_condition_destroy(ctx->input_avail);
 }
 
 /* Initializes the pager module. Requires the SshStream for
    input/output. */
 SshSftpPagerCtx
-ssh_sftp_pager_init(SshStream stream, SshFSM fsm,
+ssh_sftp_pager_init(SshStream stream, SshOldFSM fsm,
                     unsigned int screen_width,
                     unsigned int screen_height)
 {
   SshSftpPagerCtx ctx;
   SshSftpPagerIANotifierCtx ia_notifier_ctx;
-  SshFSMThread thread = ssh_fsm_spawn(fsm, sizeof(struct SshSftpPagerCtxRec),
+  SshOldFSMThread thread = ssh_oldfsm_spawn(fsm, sizeof(struct SshSftpPagerCtxRec),
                                       "pager_buf_to_out", NULL,
                                       ssh_sftp_pager_thread_destructor);
 
-  ctx = ssh_fsm_get_tdata(thread);
+  ctx = ssh_oldfsm_get_tdata(thread);
 
   ctx->thread = thread;
   ctx->stream = stream;
 
-  ctx->buffer = ssh_buffer_allocate();
-  ctx->buffer_grown = ssh_fsm_condition_create(fsm);
+  ctx->buffer = ssh_xbuffer_allocate();
+  ctx->buffer_grown = ssh_oldfsm_condition_create(fsm);
   ctx->screen_width = screen_width;
   ctx->screen_height = screen_height;
   ctx->screen_buffer = ssh_xmalloc(screen_width * screen_height + 128);
   ctx->want_exit = FALSE;
   ctx->disabled = FALSE;
 
-  ctx->input_avail = ssh_fsm_condition_create(fsm);
+  ctx->input_avail = ssh_oldfsm_condition_create(fsm);
 
   ctx->input_availability_notifier =
-    ssh_fsm_spawn(fsm, sizeof(struct SshSftpPagerIANotifierCtxRec),
+    ssh_oldfsm_spawn(fsm, sizeof(struct SshSftpPagerIANotifierCtxRec),
                   "pager_input_availability_notify", NULL, NULL);
 
   ctx->received = 0;
@@ -107,12 +107,12 @@ ssh_sftp_pager_init(SshStream stream, SshFSM fsm,
   ctx->bytes_written = 0;
   ctx->current_column = 0;
 
-  ia_notifier_ctx = ssh_fsm_get_tdata(ctx->input_availability_notifier);
+  ia_notifier_ctx = ssh_oldfsm_get_tdata(ctx->input_availability_notifier);
   ia_notifier_ctx->input_avail = ctx->input_avail;
   ia_notifier_ctx->stream = ctx->stream;
   ia_notifier_ctx->want_input_signal = FALSE;
   ia_notifier_ctx->callback_set = FALSE;
-  
+
   return ctx;
 }
 
@@ -137,15 +137,15 @@ ssh_sftp_pager_write_string(SshSftpPagerCtx pager_ctx,
   if (!pager_ctx->disabled)
     {
       pager_ctx->received += strlen(line);
-      ssh_buffer_append(pager_ctx->buffer, (unsigned char *)line,
+      ssh_xbuffer_append(pager_ctx->buffer, (unsigned char *)line,
                         strlen(line));
     }
 }
 
 void
-ssh_sftp_pager_wakeup(SshSftpPagerCtx pager_ctx, SshFSMThread thread)
+ssh_sftp_pager_wakeup(SshSftpPagerCtx pager_ctx, SshOldFSMThread thread)
 {
-  SSH_FSM_CONDITION_SIGNAL(pager_ctx->buffer_grown);
+  SSH_OLDFSM_CONDITION_SIGNAL(pager_ctx->buffer_grown);
 }
 
 /* Changes the current screen size */
@@ -160,13 +160,13 @@ ssh_sftp_pager_change_screen_size(SshSftpPagerCtx pager_ctx,
   pager_ctx->screen_buffer = ssh_xmalloc(screen_width * screen_height + 128);
 }
 
-SSH_FSM_STEP(sftp_pager_buf_to_out)
+SSH_OLDFSM_STEP(sftp_pager_buf_to_out)
 {
-  SSH_FSM_TDATA(SshSftpPagerCtx);
+  SSH_OLDFSM_TDATA(SshSftpPagerCtx);
   int i, len;
   char *buffer_ptr;
 
-  SSH_FSM_SET_NEXT("pager_buf_to_out");
+  SSH_OLDFSM_SET_NEXT("pager_buf_to_out");
 
 #if defined(O_NONBLOCK) && !defined(O_NONBLOCK_BROKEN)
       fcntl(1, F_SETFL, fcntl(1, F_GETFL, 0) & ~(O_NONBLOCK));
@@ -178,15 +178,15 @@ SSH_FSM_STEP(sftp_pager_buf_to_out)
     {
       ssh_buffer_consume(tdata->buffer, ssh_buffer_len(tdata->buffer));
     }
-  
+
   if (ssh_buffer_len(tdata->buffer) == 0)
     {
       if (tdata->want_exit == TRUE)
         {
-          SSH_FSM_SET_NEXT("pager_finish");
-          return SSH_FSM_CONTINUE;
+          SSH_OLDFSM_SET_NEXT("pager_finish");
+          return SSH_OLDFSM_CONTINUE;
         }
-      SSH_FSM_CONDITION_WAIT(tdata->buffer_grown);
+      SSH_OLDFSM_CONDITION_WAIT(tdata->buffer_grown);
     }
 
   /* lines_written tells us how much text there is already on the
@@ -216,20 +216,20 @@ SSH_FSM_STEP(sftp_pager_buf_to_out)
           /* Screen full. */
           write(1, tdata->screen_buffer, i);
           ssh_buffer_consume(tdata->buffer, i);
-          SSH_FSM_SET_NEXT("pager_empty_input");
-          return SSH_FSM_CONTINUE;
+          SSH_OLDFSM_SET_NEXT("pager_empty_input");
+          return SSH_OLDFSM_CONTINUE;
         }
     }
 
   write(1, tdata->screen_buffer, i);
   ssh_buffer_consume(tdata->buffer, i);
-  
-  return SSH_FSM_CONTINUE;
+
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(sftp_pager_empty_input)
+SSH_OLDFSM_STEP(sftp_pager_empty_input)
 {
-  SSH_FSM_TDATA(SshSftpPagerCtx);
+  SSH_OLDFSM_TDATA(SshSftpPagerCtx);
   char bunch_of_bytes[256];
   int ret;
 
@@ -244,11 +244,11 @@ SSH_FSM_STEP(sftp_pager_empty_input)
     ret = ssh_stream_read(tdata->stream, (unsigned char *)bunch_of_bytes, 256);
   } while (ret > 0);
 
-  SSH_FSM_SET_NEXT("pager_write_prompt");
-  return SSH_FSM_CONTINUE;
+  SSH_OLDFSM_SET_NEXT("pager_write_prompt");
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(sftp_pager_write_prompt)
+SSH_OLDFSM_STEP(sftp_pager_write_prompt)
 {
 
 #if defined(O_NONBLOCK) && !defined(O_NONBLOCK_BROKEN)
@@ -257,15 +257,15 @@ SSH_FSM_STEP(sftp_pager_write_prompt)
       fcntl(1, F_SETFL, fcntl(1, F_GETFL, 0) & ~(O_NDELAY));
 #endif /* O_NONBLOCK && !O_NONBLOCK_BROKEN */
 
-  write(1, "<Press any key for more or q to quit>\r", 38); 
+  write(1, "<Press any key for more or q to quit>\r", 38);
 
-  SSH_FSM_SET_NEXT("pager_read_a_char");
-  return SSH_FSM_CONTINUE;
+  SSH_OLDFSM_SET_NEXT("pager_read_a_char");
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(sftp_pager_read_a_char)
+SSH_OLDFSM_STEP(sftp_pager_read_a_char)
 {
-  SSH_FSM_TDATA(SshSftpPagerCtx);
+  SSH_OLDFSM_TDATA(SshSftpPagerCtx);
   int ret;
   char one_char;
 
@@ -284,17 +284,17 @@ SSH_FSM_STEP(sftp_pager_read_a_char)
 
   if (ret == -1)
     {
-      SSH_FSM_CONDITION_WAIT(tdata->input_avail);
+      SSH_OLDFSM_CONDITION_WAIT(tdata->input_avail);
     }
   if (one_char == 'q')
     tdata->disabled = TRUE;
-  SSH_FSM_SET_NEXT("pager_reset");
-  return SSH_FSM_CONTINUE;
+  SSH_OLDFSM_SET_NEXT("pager_reset");
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(sftp_pager_reset)
+SSH_OLDFSM_STEP(sftp_pager_reset)
 {
-  SSH_FSM_TDATA(SshSftpPagerCtx);
+  SSH_OLDFSM_TDATA(SshSftpPagerCtx);
 
   tdata->lines_written = 0;
   tdata->current_column = 0;
@@ -308,39 +308,39 @@ SSH_FSM_STEP(sftp_pager_reset)
   memset(tdata->screen_buffer, ' ', tdata->screen_width);
   write(1, tdata->screen_buffer, tdata->screen_width);
   write(1, "\r", 1);
-  
-  SSH_FSM_SET_NEXT("pager_buf_to_out");
-  return SSH_FSM_CONTINUE;
+
+  SSH_OLDFSM_SET_NEXT("pager_buf_to_out");
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(sftp_pager_finish)
+SSH_OLDFSM_STEP(sftp_pager_finish)
 {
-  SSH_FSM_TDATA(SshSftpPagerCtx);
-  ssh_fsm_kill_thread(tdata->input_availability_notifier);
+  SSH_OLDFSM_TDATA(SshSftpPagerCtx);
+  ssh_oldfsm_kill_thread(tdata->input_availability_notifier);
   tdata->user_callback(SSH_SFTP_PAGER_OK, tdata->user_context);
-  return SSH_FSM_FINISH;
+  return SSH_OLDFSM_FINISH;
 }
 
 void
 sftp_pager_ia_notifier_stream_cb(SshStreamNotification notification,
                                  void *context)
 {
-  SshFSMThread thread = (SshFSMThread)context;
+  SshOldFSMThread thread = (SshOldFSMThread)context;
   SshSftpPagerIANotifierCtx ctx =
-    (SshSftpPagerIANotifierCtx)ssh_fsm_get_tdata(thread);
+    (SshSftpPagerIANotifierCtx)ssh_oldfsm_get_tdata(thread);
   if (notification == SSH_STREAM_INPUT_AVAILABLE)
     {
       ctx->want_input_signal = TRUE;
-      ssh_fsm_continue(thread);
+      ssh_oldfsm_continue(thread);
     }
 }
 
-SSH_FSM_STEP(sftp_pager_input_availability_notify)
+SSH_OLDFSM_STEP(sftp_pager_input_availability_notify)
 {
-  SSH_FSM_TDATA(SshSftpPagerIANotifierCtx);
+  SSH_OLDFSM_TDATA(SshSftpPagerIANotifierCtx);
   if (tdata->want_input_signal == TRUE)
     {
-      SSH_FSM_CONDITION_SIGNAL(tdata->input_avail);
+      SSH_OLDFSM_CONDITION_SIGNAL(tdata->input_avail);
       tdata->want_input_signal = FALSE;
     }
   if (tdata->callback_set == FALSE)
@@ -349,6 +349,5 @@ SSH_FSM_STEP(sftp_pager_input_availability_notify)
                               thread);
       tdata->callback_set = TRUE;
     }
-  return SSH_FSM_SUSPENDED;
+  return SSH_OLDFSM_SUSPENDED;
 }
-
