@@ -4,13 +4,13 @@
 
   Author: Tomi Salo <ttsalo@ssh.com>
 
-  Copyright (c) 1999 SSH Communications Security, Finland
+  Copyright (c) 1999, 2001 SSH Communications Security, Finland
   All rights reserved.
 
   Created Wed Jan 19 15:37:52 2000.
 
   Paging for Sftp2.
-  
+
   */
 
 #ifndef SFTPPAGER_H
@@ -18,10 +18,10 @@
 
 #include "sshincludes.h"
 #include "sshstream.h"
-#include "sshfsmstreams.h"
-#include "sshfsm.h"
+#include "ssholdfsmstreams.h"
+#include "ssholdfsm.h"
 
-#define SSH_FSM_PROTOTYPES
+#define SSH_OLDFSM_PROTOTYPES
 #include "sftppager_states.h"
 
 typedef struct SshSftpPagerCtxRec *SshSftpPagerCtx;
@@ -39,7 +39,7 @@ typedef enum
    when the pager module is active, unless you are using them
    on different streams. */
 SshSftpPagerCtx
-ssh_sftp_pager_init(SshStream stream, SshFSM fsm, 
+ssh_sftp_pager_init(SshStream stream, SshOldFSM fsm,
                     unsigned int screen_width,
                     unsigned int screen_height);
 
@@ -61,7 +61,7 @@ ssh_sftp_pager_write_string(SshSftpPagerCtx pager_ctx,
    wake the pager up. Also, this HAS to be called from a step
    function. */
 void
-ssh_sftp_pager_wakeup(SshSftpPagerCtx pager_ctx, SshFSMThread thread);
+ssh_sftp_pager_wakeup(SshSftpPagerCtx pager_ctx, SshOldFSMThread thread);
 
 /* Changes the current screen size */
 void
