@@ -15,16 +15,258 @@ ssh-add.c
 
 */
 
+
+
+
+
+
+
+
 /*
- * $Id: ssh-add2.c,v 1.32.2.1 2000/12/04 20:23:52 sjl Exp $
- * $Log: ssh-add2.c,v $
- * $EndLog$
- */
+ * $Id: ssh-add2.c,v 1.49.2.2 2001/11/12 06:46:55 sjl Exp $
+ * $Log: ssh-add2.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * $EndLog$ */
 
 #include "sshincludes.h"
 #include "sshcrypt.h"
 #include "sshtimeouts.h"
 #include "sshuserfiles.h"
+#include "sshkeyfile.h"
 #include "sshagent.h"
 #include "sshuser.h"
 #include "readpass.h"
@@ -37,18 +279,13 @@ ssh-add.c
 #include "sshfilebuffer.h"
 #include "sshpgp.h"
 #include "ssh2pgp.h"
-#include "ssh2pubkeyencode.h"
-#include "ssh1keydecode.h"
 #endif /* WITH_PGP */
 #include "sshdsprintf.h"
+#include "sshfileio.h"
+#include "ssh2includes.h"
 
 #define SSH_DEBUG_MODULE "SshAdd"
 
-#ifdef HAVE_LIBWRAP
-int allow_severity = SSH_LOG_INFORMATIONAL;
-int deny_severity = SSH_LOG_WARNING;
-#endif /* HAVE_LIBWRAP */
-
 #define EXIT_STATUS_OK          0
 #define EXIT_STATUS_NOAGENT     1
 #define EXIT_STATUS_BADPASS     2
@@ -56,9 +293,8 @@ int deny_severity = SSH_LOG_WARNING;
 #define EXIT_STATUS_NOIDENTITY  4
 #define EXIT_STATUS_ERROR       5
 
-typedef enum 
-{ 
-  LIST, 
+typedef enum { 
+  LISTA, 
   ADD, 
   ADD_URL,
   DELETE,
@@ -69,8 +305,7 @@ typedef enum
 } SshAgentAction;
 
 #ifdef WITH_PGP
-typedef enum
-{
+typedef enum {
   PGP_KEY_NONE = 0,
   PGP_KEY_NAME,
   PGP_KEY_FINGERPRINT,
@@ -97,10 +332,6 @@ SshTime key_timeout = 0;
 Boolean have_attrs = FALSE;
 SshUInt32 use_limit = 0xffffffff;
 
-unsigned char *user_cert = NULL;
-size_t user_cert_len = 0;
-#define SSH_ADD_USER_CERT_MAX_SIZE 0x8000
-
 /* Force to read passphrases from stdin. */
 int use_stdin = FALSE;
 
@@ -112,18 +343,19 @@ void agent_completion(SshAgentError result, void *context);
 void add_file(SshAgent agent, const char *filename)
 {
   SshPrivateKey key = NULL;
-  char *saved_comment, *comment = NULL, *pass;
+  char *saved_comment = NULL, *comment = NULL, *pass;
   int query_cnt;
   unsigned char *certs = NULL;
   size_t certs_len;
-  char privname[500], pubname[500];
+  char privname[500], pubname[500], certname[500];
   unsigned long magic;
   struct stat st;
+  Boolean pub_key_exists = TRUE;
 
   if (action == ADD_URL)
     {
       printf("Adding URL identity: %s\n", filename);
-      snprintf(privname, sizeof(privname), "%s", filename);
+      ssh_snprintf(privname, sizeof(privname), "%s", filename);
       if (have_attrs)
         ssh_agent_add_with_attrs(agent, NULL, NULL, 0, privname,
                                  path_limit, path_constraint, use_limit,
@@ -137,7 +369,7 @@ void add_file(SshAgent agent, const char *filename)
   else if (action == DELETE_URL)
     {
       printf("Deleting URL identity: %s\n", filename);
-      snprintf(privname, sizeof(privname), "%s", filename);
+      ssh_snprintf(privname, sizeof(privname), "%s", filename);
       ssh_agent_delete(agent, NULL, 0, privname,
                        agent_completion, (void *)agent);
       return;
@@ -150,70 +382,111 @@ void add_file(SshAgent agent, const char *filename)
       if (strlen(filename) > 4 &&
           strcmp(filename + strlen(filename) - 4, ".pub") == 0)
         {
-          snprintf(pubname, sizeof(pubname), "%s", filename);
-          snprintf(privname, sizeof(privname), "%s", filename);
+          ssh_snprintf(pubname, sizeof(pubname), "%s", filename);
+          ssh_snprintf(privname, sizeof(privname), "%s", filename);
           privname[strlen(privname) - 4] = '\0';
+          ssh_snprintf(certname, sizeof(certname), "%s.crt", privname);
         }
       else
         {
-          snprintf(pubname, sizeof(pubname), "%s.pub", filename);
-          snprintf(privname, sizeof(privname), "%s", filename);
+          ssh_snprintf(pubname, sizeof(pubname), "%s.pub", filename);
+          ssh_snprintf(privname, sizeof(privname), "%s", filename);
+          ssh_snprintf(certname, sizeof(certname), "%s.crt", filename);
         }
-    
+
       if (action == ADD)
         printf("Adding identity: %s\n", pubname);
       else if (action == DELETE)
         printf("Deleting identity: %s\n", pubname);
-    
+
       if (stat(pubname, &st) < 0)
         {
           printf("Public key file %s does not exist.\n", pubname);
+          pub_key_exists = FALSE;
+        }
+
+      if (!pub_key_exists)
+        {
+
+
+
+
+
+
+
+
           (*agent_completion)(SSH_AGENT_ERROR_OK, (void *)agent);
           return;
         }
-    
+
       if (stat(privname, &st) < 0)
         {
           printf("Private key file %s does not exist.\n", privname);
           (*agent_completion)(SSH_AGENT_ERROR_OK, (void *)agent);
           return;
         }
-    
-      /* Read the public key blob. */
-      magic = ssh2_key_blob_read(user, 
-                                 pubname,
-                                 TRUE,
-                                 &saved_comment,
-                                 &certs, 
-                                 &certs_len,
-                                 NULL);
-      if (magic != SSH_KEY_MAGIC_PUBLIC)
+
+      if (pub_key_exists) 
         {
-          printf("Bad public key file %s\n", pubname);
-          ssh_xfree(certs);
-          (*agent_completion)(SSH_AGENT_ERROR_OK, (void *)agent);
-          return;
+          /* Read the public key blob. */
+          magic = ssh2_key_blob_read(user, 
+                                     pubname,
+                                     TRUE,
+                                     &saved_comment,
+                                     &certs, 
+                                     &certs_len,
+                                     NULL);
+          if (magic != SSH_KEY_MAGIC_PUBLIC)
+            {
+              printf("Bad public key file %s\n", pubname);
+              ssh_xfree(certs);
+              (*agent_completion)(SSH_AGENT_ERROR_OK, (void *)agent);
+              return;
+            }
         }
-    
+
+
+
+
+
+
+
+
+
+
+
+
+      if (!saved_comment)
+        saved_comment = ssh_xstrdup("");
+
       if (action == ADD)
         {
+          SshPrivKeyResult ret;
+          
           /* Loop until we manage to load the file, or a maximum number of
              attempts have been made.  First try with an empty passphrase. */
           pass = ssh_xstrdup("");
           query_cnt = 0;
-          while ((key = ssh_privkey_read(user, 
+          while ((ret = ssh_privkey_read(user, 
                                          privname,
                                          pass, 
                                          &comment, 
-                                         NULL)) == NULL)
+                                         &key)) != SSH_PRIVKEY_OK)
             {
               char buf[1024];
               FILE *f;
-            
+
               /* Free the old passphrase. */
               memset(pass, 0, strlen(pass));
               ssh_xfree(pass);
-            
+
+              if (ret == SSH_PRIVKEY_KEY_UNREADABLE)
+                {
+                  fprintf(stderr,
+                          "Private key is unreadable.\n");
+                  exit(EXIT_STATUS_NOFILE);
+                }              
+
               query_cnt++;
               if (query_cnt > 5)
                 {
@@ -225,12 +498,13 @@ void add_file(SshAgent agent, const char *filename)
               /* Ask for a passphrase. */
               if (!use_stdin && getenv("DISPLAY") && !isatty(fileno(stdin)))
                 {
-                  snprintf(buf, sizeof(buf),
-                           "ssh-askpass2 '%sEnter passphrase for %.100s'", 
-                           ((query_cnt <= 1) ? 
-                            "" : 
-                            "You entered wrong passphrase.  "), 
-                           saved_comment);
+                  ssh_snprintf(buf, sizeof(buf),
+                               SSH_ASKPASS_PATH " '%sEnter passphrase "
+                               "for %.100s'",
+                               ((query_cnt <= 1) ? 
+                                "" : 
+                                "You entered wrong passphrase.  "), 
+                               saved_comment);
                   f = popen(buf, "r");
                   if (!fgets(buf, sizeof(buf), f))
                     {
@@ -306,8 +580,8 @@ void add_file(SshAgent agent, const char *filename)
           if (have_attrs)
             ssh_agent_add_with_attrs(agent,
                                      key, 
-                                     (user_cert ? user_cert : certs), 
-                                     (user_cert ? user_cert_len : certs_len),
+                                     certs, 
+                                     certs_len,
                                      saved_comment,
                                      path_limit, 
                                      path_constraint,
@@ -319,8 +593,8 @@ void add_file(SshAgent agent, const char *filename)
           else
             ssh_agent_add(agent, 
                           key, 
-                          (user_cert ? user_cert : certs), 
-                          (user_cert ? user_cert_len : certs_len),
+                          certs, 
+                          certs_len,
                           saved_comment,
                           agent_completion, 
                           (void *)agent);
@@ -329,8 +603,8 @@ void add_file(SshAgent agent, const char *filename)
       else if (action == DELETE)
         {
           ssh_agent_delete(agent, 
-                           (user_cert ? user_cert : certs), 
-                           (user_cert ? user_cert_len : certs_len),
+                           certs, 
+                           certs_len,
                            saved_comment,
                            agent_completion, 
                            (void *)agent);
@@ -437,12 +711,13 @@ void add_file(SshAgent agent, const char *filename)
               /* Ask for a passphrase. */
               if (!use_stdin && getenv("DISPLAY") && !isatty(fileno(stdin)))
                 {
-                  snprintf(buf, sizeof(buf),
-                           "ssh-askpass2 '%sEnter passphrase for \"%.100s\"'", 
-                           ((query_cnt <= 1) ? 
-                            "" : 
-                            "You entered wrong passphrase.  "), 
-                           comment);
+                  ssh_snprintf(buf, sizeof(buf),
+                               SSH_ASKPASS_PATH " '%sEnter passphrase "
+                               "for \"%.100s\"'", 
+                               ((query_cnt <= 1) ? 
+                                "" : 
+                                "You entered wrong passphrase.  "), 
+                               comment);
                   f = popen(buf, "r");
                   if (!fgets(buf, sizeof(buf), f))
                     {
@@ -609,7 +884,7 @@ void agent_completion(SshAgentError result, void *context)
 }
 
 void agent_list_callback(SshAgentError error, unsigned int num_keys,
-                             SshAgentKeyInfo keys, void *context)
+                         SshAgentKeyInfo keys, void *context)
 {
   SshAgent agent = (SshAgent)context;
   int i;
@@ -643,7 +918,7 @@ void agent_open_callback(SshAgent agent, void *context)
   if (!agent)
     {
       fprintf(stderr,
-        "Failed to connect to authentication agent - agent not running?\n");
+              "Failed to connect to authentication agent - agent not running?\n");
       exit(EXIT_STATUS_NOAGENT);
     }
 
@@ -654,7 +929,7 @@ void agent_open_callback(SshAgent agent, void *context)
       ssh_agent_delete_all(agent, agent_completion, (void *)agent);
       break;
       
-    case LIST:
+    case LISTA:
       fprintf(stderr, "Listing identities.\n");
       ssh_agent_list(agent, agent_list_callback, (void *)agent);
       break;
@@ -662,8 +937,8 @@ void agent_open_callback(SshAgent agent, void *context)
     case LOCK:
       if (!use_stdin && getenv("DISPLAY") && !isatty(fileno(stdin)))
         {
-          snprintf(buf, sizeof (buf),
-                   "ssh-askpass2 'Enter lock passphrase'");
+          ssh_snprintf(buf, sizeof (buf),
+                       SSH_ASKPASS_PATH " 'Enter lock passphrase'");
           f = popen(buf, "r");
           if (! fgets(buf, sizeof (buf), f))
             {
@@ -681,7 +956,7 @@ void agent_open_callback(SshAgent agent, void *context)
           if (password == NULL)
             {
               fprintf(stderr, "Unable to read password.\n");
-                exit(EXIT_STATUS_BADPASS);
+              exit(EXIT_STATUS_BADPASS);
             }
           password_vrfy = ssh_read_passphrase("Again: ", use_stdin);
           if (password_vrfy == NULL)
@@ -701,8 +976,8 @@ void agent_open_callback(SshAgent agent, void *context)
     case UNLOCK:
       if (!use_stdin && getenv("DISPLAY") && !isatty(fileno(stdin)))
         {
-          snprintf(buf, sizeof (buf),
-                   "ssh-askpass2 'Enter unlock passphrase'");
+          ssh_snprintf(buf, sizeof (buf),
+                       SSH_ASKPASS_PATH " 'Enter unlock passphrase'");
           f = popen(buf, "r");
           if (! fgets(buf, sizeof (buf), f))
             {
@@ -753,13 +1028,14 @@ int main(int ac, char **av)
   char *ssh2dirname = NULL;
   Boolean dynamic_array = FALSE;
   struct dirent * cand;
-
+  struct stat st;
+  
   /* ptrs for the list of private keys, and a config to open ident file */
   struct SshConfigPrivateKey **priv_key_list;
   struct SshConfigPrivateKey **tmp_priv_key_list;
   struct SshConfigPrivateKey *tmp_key_ptr;
   SshConfig config;
-  char temp_s[1024];
+  char *temp_s = NULL;
 
 
   /* Save program name. */
@@ -770,12 +1046,21 @@ int main(int ac, char **av)
 
   user = ssh_user_initialize(NULL, FALSE);
 
+#ifdef SSHDIST_CRYPT_RSA
+  ssh_pk_provider_register(&ssh_pk_if_modn);
+  ssh_pk_provider_register(&ssh_pk_if_modn_generator);
+#endif /* SSHDIST_CRYPT_RSA */
+#ifdef SSHDIST_CRYPT_DSA
+  ssh_pk_provider_register(&ssh_pk_dl_modp);
+  ssh_pk_provider_register(&ssh_pk_dl_modp_generator);
+#endif /* SSHDIST_CRYPT_DSA */
+
 #ifdef WITH_PGP
   pgp_keyring = ssh_xstrdup(SSH_PGP_SECRET_KEY_FILE);
 #endif /* WITH_PGP */
 
   action = ADD;
-  while ((opt = ssh_getopt(ac, av, "ldDput:f:F:1LUNPIC:", NULL)) != EOF)
+  while ((opt = ssh_getopt(ac, av, "ldDput:f:F:1LUNPIR:C:", NULL)) != EOF)
     {
       if (!ssh_optval)
         {
@@ -822,48 +1107,13 @@ int main(int ac, char **av)
           break;
 
         case 'l':
-          action = LIST;
+          action = LISTA;
           break;
 
         case 'p':
           use_stdin = TRUE;
           break;
 
-        case 'C':
-          {
-            FILE *f;
-
-            f = fopen(ssh_optarg, "r");
-            if (f == NULL)
-              {
-                fprintf(stderr, "%s: Can't open certificate file \"%s\".\n", 
-                        ssh_optarg,
-                        av0);
-                exit(EXIT_STATUS_ERROR);
-              }
-            user_cert = ssh_xmalloc(SSH_ADD_USER_CERT_MAX_SIZE + 1);
-            user_cert_len = fread(user_cert, 
-                                  1,
-                                  SSH_ADD_USER_CERT_MAX_SIZE + 1, 
-                                  f);
-            fclose(f);
-            if (user_cert_len == 0) 
-              {
-                fprintf(stderr, "%s: Can't read certificate file \"%s\".\n", 
-                        ssh_optarg,
-                        av0);
-                exit(EXIT_STATUS_ERROR);
-              }
-            if (user_cert_len > SSH_ADD_USER_CERT_MAX_SIZE)
-              {
-                fprintf(stderr, "%s: Certificate file \"%s\" too big.\n", 
-                        ssh_optarg,
-                        av0);
-                exit(EXIT_STATUS_ERROR);
-              }
-          }
-          break;
-
         case 'd':
           if (action == ADD_URL)
             action = DELETE_URL;
@@ -936,10 +1186,10 @@ int main(int ac, char **av)
   if (pgp_keyring[0] != '/')
     {
       char buf[1024];
-      snprintf(buf, sizeof (buf), "%s/%s/%s", 
-               ssh_user_dir(user), 
-               SSH_USER_DIR,
-               pgp_keyring);
+      ssh_snprintf(buf, sizeof (buf), "%s/%s/%s", 
+                   ssh_user_dir(user), 
+                   SSH_USER_DIR,
+                   pgp_keyring);
       ssh_xfree(pgp_keyring);
       pgp_keyring = ssh_xstrdup(buf);
     }
@@ -948,9 +1198,10 @@ int main(int ac, char **av)
   files = &av[ssh_optind];
   num_files = ac - ssh_optind;
 
+  ssh_event_loop_initialize();  
 
   /* no key supplied on command line so look in identification file */
-  if (num_files == 0 && action != LIST && action != DELETE_ALL &&
+  if (num_files == 0 && action != LISTA && action != DELETE_ALL &&
       action != LOCK && action != UNLOCK)
     {
 
@@ -962,44 +1213,42 @@ int main(int ac, char **av)
         }
 #endif /* WITH_PGP */
 
-      config = NULL;
-      config = ssh_xcalloc( 1, sizeof(*config));
-
-      if (config == NULL)
-        {
-          ssh_fatal("Failed to allocate enough memory\n");
-        }
-
       config = ssh_client_create_config();
 
       /* Try to read the global configuration file */
-      ssh_config_read_file(user, config, NULL,
-                           SSH_CLIENT_GLOBAL_CONFIG_FILE, NULL);
-
+      
+      if ((stat(SSH_CLIENT_GLOBAL_CONFIG_FILE, &st) == 0) &&
+          !ssh_config_read_file(user, config, NULL,
+                                SSH_CLIENT_GLOBAL_CONFIG_FILE))
+        ssh_fatal("Error reading config file %s.",
+                  SSH_CLIENT_GLOBAL_CONFIG_FILE);
 
       /* Try to read in the user configuration file. */
       ssh2dirname = ssh_userdir(user, config, TRUE);
       if (ssh2dirname == NULL)
-        {
-          ssh_fatal("Failed to create user ssh directory.");
-        }
+        ssh_fatal("Failed to create user ssh directory.");
 
-      snprintf(temp_s, sizeof (temp_s), "%s/%s",
-               ssh2dirname, SSH_CLIENT_CONFIG_FILE);
+      ssh_dsprintf(&temp_s, "%s/%s",
+                   ssh2dirname, SSH_CLIENT_CONFIG_FILE);
       ssh_xfree(ssh2dirname);
 
-      ssh_config_read_file(user, config, config->host_to_connect,
-                           temp_s, NULL);
+      if ((stat(temp_s, &st) == 0) &&
+          !ssh_config_read_file(user, config, config->host_to_connect,
+                                temp_s))
+        ssh_fatal("Error reading config file %s.",
+                  temp_s);
 
+      ssh_xfree(temp_s);
+      
       /* get list of private keys from identification file */
       priv_key_list = NULL;
-      priv_key_list = ssh_privkey_list( user, NULL, config );
+      priv_key_list = ssh_privkey_list(user, NULL, config);
 
       num_files = 0;
 
       if (priv_key_list != NULL)
         {
-
+          int skipped = 0;
           /* count the number of private key files */
 
           tmp_priv_key_list = priv_key_list;
@@ -1008,41 +1257,52 @@ int main(int ac, char **av)
               num_files++;
               tmp_priv_key_list++;
             }
- 
+
+          SSH_DEBUG(4, ("There are %d private keys in the identification "
+                        "file.", num_files));
+          
           /* put the file names into the files array
              and free the ptr list at the same time */
 
-          files = ssh_xcalloc( num_files + 1, sizeof(char*) );
+          files = ssh_xcalloc(num_files + 1, sizeof(char*));
           dynamic_array = TRUE;
  
-          for (i = 0; i < num_files; i++)
-            {
-              files[i] = ssh_xstrdup((*priv_key_list)->keyfile);
+          for (i = 0; i + skipped < num_files; )
+            {              
+              /* The `keyfile' member is dynamically allocated. */
+              files[i] = (*priv_key_list)->keyfile;
 
+              /* XXX PGP-keys can't yet be added like this
+                 (automatically from an identification file) */
+              
               /* we've copied the file name, now delete the
                  SshConfigPrivateKey struct */
-
-              ssh_xfree((*priv_key_list)->keyfile);
+              (*priv_key_list)->keyfile = NULL;
 
 #ifdef WITH_PGP
-             if ((*priv_key_list)->pgp_keyring)
-               ssh_xfree((*priv_key_list)->pgp_keyring);
+              if ((*priv_key_list)->pgp_keyring)
+                ssh_xfree((*priv_key_list)->pgp_keyring);
 
-             if ((*priv_key_list)->pgp_name)
-               ssh_xfree((*priv_key_list)->pgp_name);
+              if ((*priv_key_list)->pgp_name)
+                ssh_xfree((*priv_key_list)->pgp_name);
 
-             if ((*priv_key_list)->pgp_fingerprint)
-               ssh_xfree((*priv_key_list)->pgp_fingerprint);
+              if ((*priv_key_list)->pgp_fingerprint)
+                ssh_xfree((*priv_key_list)->pgp_fingerprint);
 #endif /* WITH_PGP */
-
               tmp_key_ptr = *priv_key_list;
               priv_key_list++;
               ssh_xfree(tmp_key_ptr);
+              if (files[i] == NULL)
+                {
+                  /* Skip this entry (probably a PGP-key definition. */
+                  skipped++;
+                  continue;
+                }
+              i++;
             }
-          }
-
+          num_files = i;
+        }
       ssh_xfree(config);
-
     }
 
   /* Fetch default from ~/.ssh2/id_* (the first that we happen to get)
@@ -1050,7 +1310,7 @@ int main(int ac, char **av)
 
 #define ID_PREFIX "id"
   
-  if (num_files == 0 && action != LIST && action != DELETE_ALL &&
+  if (num_files == 0 && action != LISTA && action != DELETE_ALL &&
       action != LOCK && action != UNLOCK)
     {
 #ifdef WITH_PGP
@@ -1097,8 +1357,6 @@ int main(int ac, char **av)
   
   signal(SIGPIPE, SIG_IGN);
   
-  ssh_event_loop_initialize();
-  
   ssh_agent_open(agent_open_callback, NULL);
 
   ssh_event_loop_run();
