@@ -7,21 +7,15 @@
         Timo J. Rinne <tri@ssh.com>
         Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1997-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1997-2001 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   A tool for generating and manipulating private keys.
 */
 
-/* 
- *  $Log: ssh-keygen2.c,v $
- *  $EndLog$
- */
-
-
-
 #include "sshincludes.h"
 #include "ssh2includes.h"
+#include "sshkeyfile.h"
 #include "sshuserfiles.h"
 #include "sshreadline.h"
 #include "sshfileio.h"
@@ -29,7 +23,6 @@
 #include "sshuser.h"
 #include "sshcrypt.h"
 #include "sshcipherlist.h"
-#include "ssh2pubkeyencode.h"
 #include "sshgetopt.h"
 #include "sshappcommon.h"
 #include <sys/types.h>
@@ -39,7 +32,16 @@
 #include "sshsignals.h"
 
 
+
+
+
+
+
+
+
+#ifndef VXWORKS
 #include <pwd.h>
+#endif /* VXWORKS */
 
 
 
@@ -64,11 +66,6 @@
 #define KEYGEN_ASSUMED_PUBKEY_LEN 1024
 #endif /* KEYGEN_ASSUMED_PUBKEY_LEN */
 
-#ifdef HAVE_LIBWRAP
-int allow_severity = SSH_LOG_INFORMATIONAL;
-int deny_severity = SSH_LOG_WARNING;
-#endif /* HAVE_LIBWRAP */
-
 /* helptext */
 
 const char keygen_helptext[] =
@@ -76,15 +73,15 @@ const char keygen_helptext[] =
   "\n"
   "Where `options' are:\n"
   " -b nnn         Specify key strength in bits (e.g. 1024)\n"
-
-
-
-
-
-
-
+#ifdef SSHDIST_CRYPT_RSA
+#ifdef WITH_RSA
+  " -t dsa | rsa   Choose the key type.\n"
+#else /* WITH_RSA */
   " -t dsa         Choose the key type (only dsa available).\n"
+#endif /* WITH_RSA */
 
+
+#endif /* SSHDIST_CRYPT_RSA */
   " -c comment     Provide the comment.\n"
   " -e file        Edit the comment/passphrase of the key.\n"
   " -p passphrase  Provide passphrase.\n"
@@ -94,22 +91,33 @@ const char keygen_helptext[] =
   " -q             Suppress the progress indicator.\n"
   " -1             Convert a SSH 1.x key.\n"
   " -i file        Load and display information on `file'.\n"
-  " -D file        Derive the private key given in 'file' to public key.\n"
+  " -D file        Derive the public key from the private key 'file'.\n"
   " -B number      The number base for displaying key information (default 10).\n"
   " -V             Print ssh-keygen version number.\n"
   " -r file        Stir data from file to random pool.\n"
+
+
+
+
+
+
   " -F file        Dump fingerprint of file.\n\n";
 
 /* A context structure -- we don't like global variables. */
 
-typedef struct 
+typedef struct
 {
   int keybits;
   Boolean newkey;
   Boolean convert;
+
+
+
+
+
   Boolean status;
   Boolean edit_key;
-  
+
   char *keytype;
   char *keytypecommon;
   char *comment;
@@ -121,7 +129,7 @@ typedef struct
   Boolean pass_empty;
 
   unsigned int base;
-  
+
   SshPrivateKey private_key;
   SshPublicKey public_key;
   SshUser user;
@@ -141,18 +149,18 @@ typedef struct
    The first entry in this list will be the preferred (standard)
    choice. */
 
-const char *keygen_common_names[][2] = 
+const char *keygen_common_names[][2] =
 {
   /* Digital Signature Standard */
   { "dsa", SSH_CRYPTO_DSS },
   { "dss", SSH_CRYPTO_DSS },
 
-
-
-
-
-
-
+#ifdef SSHDIST_CRYPT_RSA
+#ifdef WITH_RSA
+  /* RSA */
+  { "rsa", SSH_CRYPTO_RSA },
+#endif /* WITH_RSA */
+#endif /* SSHDIST_CRYPT_RSA */
 
   /* Last entry */
   { NULL, NULL }
@@ -168,10 +176,15 @@ KeyGenCtx *keygen_init_ctx(void)
   kgc->keybits = FALSE;
   kgc->newkey = FALSE;
   kgc->convert = FALSE;
+
+
+
+
+
   kgc->status = FALSE;
   kgc->read_file = FALSE;
   kgc->edit_key = FALSE;
-  
+
   kgc->keytype = NULL;
   kgc->keytypecommon = NULL;
   kgc->comment = NULL;
@@ -183,10 +196,10 @@ KeyGenCtx *keygen_init_ctx(void)
   kgc->derive_file = NULL;
   kgc->dump_fingerprint = FALSE;
   kgc->fingerprint_type = SSH_FINGERPRINT_BABBLE;
-  
+
   kgc->base = 10;
 
-  kgc->user = ssh_user_initialize(NULL, FALSE);  
+  kgc->user = ssh_user_initialize(NULL, FALSE);
   kgc->public_key = NULL;
   kgc->private_key = NULL;
 
@@ -277,12 +290,12 @@ char *ssh_askpass_read_stdin(char *prompt)
   if (fgets(buf, sizeof (buf)-1, stdin) == NULL)
     return ssh_xstrdup("");
 
-  for(p = buf; *p >= 32 || *p < 0; p++); 
+  for(p = buf; *p >= 32 || *p < 0; p++);
   *p = '\0';
 
   p = ssh_xstrdup(buf);
   memset(buf, 0, sizeof (buf));
-  
+
   return p;
 }
 
@@ -310,12 +323,12 @@ void keygen_prog_ind(SshCryptoProgressID id, unsigned int time_value,
   if (i % 13 == 0)
     {
       printf("\r %3d ", i / 13 + 1);
-    } 
+    }
   else
     {
       switch( i % 4 )
         {
-        case 0:   
+        case 0:
           printf(".");
           break;
         case 1:
@@ -350,8 +363,8 @@ void keygen_choose_filename(KeyGenCtx *kgc)
 
   for (i = 'a'; i <= 'z'; i++)
     {
-      snprintf(buf, sizeof (buf), "%s/id_%s_%d_%c",
-               udir, kgc->keytypecommon, kgc->keybits, i);
+      ssh_snprintf(buf, sizeof (buf), "%s/id_%s_%d_%c",
+                   udir, kgc->keytypecommon, kgc->keybits, i);
       if (stat(buf, &st) == 0)
         continue;
       kgc->out_filename = ssh_xstrdup(buf);
@@ -369,26 +382,39 @@ SshPrivateKey keygen_read_privkey(KeyGenCtx *kgc, char *filename)
 {
   SshPrivateKey prvkey = NULL;
   /* Guess that it is a private key. */
-  
+
   if (kgc->passphrase)
     {
-      prvkey = ssh_privkey_read(kgc->user,
-                                filename,
-                                kgc->passphrase, NULL, NULL);
+      if (ssh_privkey_read(kgc->user, filename, kgc->passphrase, NULL,
+                           &prvkey) == SSH_PRIVKEY_KEY_UNREADABLE)
+        {
+          ssh_warning("Private key %s is unreadable.", filename);
+          return NULL;
+        }
     }
 
   if (prvkey == NULL)
     {
-      prvkey = ssh_privkey_read(kgc->user, filename,
-                                "", NULL, NULL);
+      if (ssh_privkey_read(kgc->user, filename,
+                           "", NULL, &prvkey) == SSH_PRIVKEY_KEY_UNREADABLE)
+        {
+          ssh_warning("Private key %s is unreadable.", filename);
+          return NULL;
+        }
+
       if (prvkey == NULL &&
           !kgc->passphrase && kgc->pass_empty == FALSE)
         {
           char *pass;
           pass = ssh_read_passphrase("Passphrase : ", FALSE);
 
-          prvkey = ssh_privkey_read(kgc->user, filename,
-                                    pass, NULL, NULL);
+          if (ssh_privkey_read(kgc->user, filename, pass, NULL,
+                               &prvkey) == SSH_PRIVKEY_KEY_UNREADABLE)
+            {
+              ssh_warning("Private key %s is unreadable.", filename);
+              return NULL;
+            }
+
           ssh_xfree(pass);
         }
     }
@@ -451,10 +477,10 @@ static const char *compute_years(int e)
 
   if (e < 0)
     {
-      snprintf(buf, 20, "<1");
+      ssh_snprintf(buf, 20, "<1");
       return buf;
     }
-  
+
   ssh_mp_init(&g);
 
   /* Compute quickly 2^e. */
@@ -470,7 +496,7 @@ static const char *compute_years(int e)
       b10 += 2;
       neg += 2;
     }
-  
+
   for (i = 2; i < b10; i++)
     ssh_mp_div_ui(&g, &g, 10);
 
@@ -478,8 +504,8 @@ static const char *compute_years(int e)
   buf[1] = '.';
   buf[0] = '0' + ssh_mp_div_ui(&g, &g, 10);
 
-  snprintf(&buf[3], 17, " * 10^%d", ((int)b10 - (int)neg));
-  
+  ssh_snprintf(&buf[3], 17, " * 10^%d", ((int)b10 - (int)neg));
+
   ssh_mp_clear(&g);
 
   return buf;
@@ -523,7 +549,7 @@ static void pubkey_print_estimate(SshPublicKey pub)
         {
           remarks = "\n  Use of the key is not recommended for encryption.";
         }
-      
+
       ssh_mp_clear(&n);
       ssh_mp_clear(&e);
     }
@@ -560,7 +586,7 @@ static void pubkey_print_estimate(SshPublicKey pub)
     }
 
   years = compute_years(log2 - FAST_MACHINE_OPS_LOG2);
-  
+
   printf("[Strength estimation as of %s considering %s:"
          " Attack requires O(2^%u) steps, which is roughly"
          " equivalent to %s years of effort with %uGHz machine.%s]\n",
@@ -634,7 +660,7 @@ static void prvkey_print_estimate(SshPrivateKey prv)
       log2 = dsa_strength(ssh_mp_get_size(&q, 2),
                           ssh_mp_get_size(&p, 2));
       method = "NFS and Pollard rho";
-      
+
       ssh_mp_clear(&p);
       ssh_mp_clear(&q);
     }
@@ -646,7 +672,7 @@ static void prvkey_print_estimate(SshPrivateKey prv)
     }
 
   years = compute_years(log2 - FAST_MACHINE_OPS_LOG2);
-  
+
   printf("[Strength estimation as of %s considering %s:"
          " Attack requires O(2^%u) steps, which is roughly"
          " equivalent to %s years of effort with %uGHz machine.%s]\n",
@@ -702,11 +728,11 @@ Boolean keygen_view_prvkey(KeyGenCtx *kgc, SshPrivateKey prv)
 
       printf("RSA Private Key\n");
       prvkey_print_estimate(prv);
-      
+
       printf("  e = [Encryption exponent]\n");
       keygen_out_number(kgc, &e);
 
-      printf("  n = [Large composite moduli]\n");
+      printf("  n = [Large composite modulus]\n");
       keygen_out_number(kgc, &n);
 
       printf("  d = [Inverse of e modulo (p-1)*(q-1) (Euler's phi function of n)]\n");
@@ -839,7 +865,7 @@ Boolean keygen_view_pubkey(KeyGenCtx *kgc, SshPublicKey pub)
       printf("  e = [Encryption exponent]\n");
       keygen_out_number(kgc, &e);
 
-      printf("  n = [Large composite moduli]\n");
+      printf("  n = [Large composite modulus]\n");
       keygen_out_number(kgc, &n);
 
       ssh_mp_clear(&n);
@@ -952,8 +978,8 @@ int keygen_keygen(KeyGenCtx *kgc)
   if ((! kgc->passphrase) || (! (*(kgc->passphrase)))) {
     if (!(kgc->pass_empty))
       {
-        pass = ssh_read_passphrase_twice("Passphrase : ", 
-                                         "Again      : ", 
+        pass = ssh_read_passphrase_twice("Passphrase : ",
+                                         "Again      : ",
                                          FALSE);
       }
     else
@@ -979,29 +1005,29 @@ int keygen_keygen(KeyGenCtx *kgc)
     }
 
   if (ssh_privkey_write(kgc->user,
-                        kgc->out_filename, 
-                        kgc->passphrase, 
+                        kgc->out_filename,
+                        kgc->passphrase,
                         kgc->comment,
                         kgc->private_key, NULL))
     {
       ssh_warning("Private key not written !");
       r++;
     }
-  else 
+  else
     {
       printf("Private key saved to %s\n", kgc->out_filename);
     }
 
   /* Save the public key */
 
-  snprintf(buf, sizeof (buf), "%s.pub", kgc->out_filename);
+  ssh_snprintf(buf, sizeof (buf), "%s.pub", kgc->out_filename);
   kgc->public_key = ssh_private_key_derive_public_key(kgc->private_key);
   if (kgc->public_key == NULL)
     {
       ssh_warning("Could not derive public key from private key.");
       return r + 1;
     }
-  
+
   if (ssh_pubkey_write(kgc->user, buf, kgc->comment, kgc->public_key, NULL))
     {
       ssh_warning("Public key not written !");
@@ -1034,218 +1060,685 @@ void stir_from_file(KeyGenCtx *kgc)
     printf("Stirred in %lu bytes.\n", (unsigned long) buffer_len);
 }
 
-/* converts SSH1 keys to SSH2 format */
-int keygen_convert_key(KeyGenCtx *kgc)
+/* Saves a private key with an optional extention */
+int keygen_write_private_key(KeyGenCtx *kgc, SshPrivateKey key,
+                             char *comment, char *ext)
 {
-  SshPrivateKey prvkey = NULL;
-  SshPublicKey  pubkey = NULL;
-  char *comment = NULL;
+  char newfilename[127];
+  int retval;
+  char *passphrase;
+  char *empty_string = "";
+
+  if (ext == NULL)
+    ext = empty_string;
+
+  /* make passphrase empty if none supplied */
+  if (kgc->passphrase != NULL)
+    passphrase = kgc->passphrase;
+  else
+    passphrase = empty_string;
+
+  ssh_snprintf(newfilename, sizeof(newfilename),
+               "%s%s",kgc->in_filename, ext);
+
+  if (ssh_privkey_write(kgc->user, newfilename,
+                        passphrase, comment, key,
+                        NULL) == TRUE)
+    {
+      printf("Unable to write private key.\r\n");
+      retval = 1;
+    }
+  else
+    {
+      printf("Successfully saved private key to %s\r\n",newfilename);
+      retval = 0;
+    }
+
+  return retval;
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/* converts and saves public key to SSH2 format
+   returns 1 on error, 0 otherwise */
+int keygen_convert_public_key(KeyGenCtx *kgc)
+{
+  char *base_name;
   char newfilename[1024];
+  char *comment = NULL;
+  SshPublicKey pubkey = NULL;
+  Boolean retval = TRUE;
 
   pubkey = ssh_pubkey_read(kgc->user, kgc->in_filename,
                            &comment, NULL);
 
-  if (pubkey != NULL)
+  if (pubkey == NULL)
     {
-      /*It's a public key */
+      printf("Cannot open key %s\r\n",kgc->in_filename);
+      return 1;
+    }
 
-      snprintf( newfilename, sizeof(newfilename),
-                "%s_ssh2",kgc->in_filename);
+  /* remove the ".pub" extension if it is there */
+  if ((base_name = strstr(kgc->in_filename, ".pub")) != NULL)
+    {
+      base_name[0] = '\0';
+    }
 
-      if ( ssh_pubkey_write( kgc->user, newfilename,
-                             comment, pubkey, NULL ) == TRUE)
-        {
-          /* pubkey write failed for some reason */
-          printf("Unable to convert public key.\r\n");
-        }
-      else
-        {
-          printf("Successfully converted public key to %s\r\n",newfilename);
-        }
+  /* name new key name_ssh2.pub */
 
+  ssh_snprintf( newfilename, sizeof(newfilename),
+                "%s_ssh2.pub",kgc->in_filename);
+
+  if (ssh_pubkey_write(kgc->user, newfilename,
+                       comment, pubkey, NULL) == TRUE)
+    {
+      /* pubkey write failed for some reason */
+      printf("Unable to convert public key.\r\n");
+      retval = 1;
     }
   else
     {
-      /* maybe it's a private key */
-      kgc->passphrase = ssh_read_passphrase("Passphrase : ", FALSE);
-
-      prvkey = ssh_privkey_read(kgc->user, kgc->in_filename,
-                                kgc->passphrase, &comment, NULL );
-
-      if (prvkey != NULL)
-        {
-          snprintf( newfilename, sizeof(newfilename),
-                    "%s_ssh2",kgc->in_filename );
-
-          if (ssh_privkey_write( kgc->user, newfilename,
-                             kgc->passphrase, comment, prvkey, NULL ) == TRUE)
-            {
-              printf("Unable to convert private key.\r\n");
-            }
-          else
-            {
-              printf("Successfully converted private key to %s\r\n",newfilename);
-            }
-        }
-      else
-        {
-          printf("Cannot determine the type of the key or "
-                  "passphrase incorrect.\r\n");
-        }
+      printf("Successfully converted public key to %s\r\n",newfilename);
+      retval = 0;
     }
 
+  if (pubkey)
+    ssh_public_key_free( pubkey );
+
+  if (comment)
+    ssh_xfree(comment);
+
+  return retval;
+}
+
+
+int keygen_convert_private_key(KeyGenCtx *kgc)
+{
+
+  char *comment = NULL;
+  SshPrivateKey prvkey = NULL;
+  int retval = 1;
+
+  /* maybe it's a private key */
+  kgc->passphrase = ssh_read_passphrase("Passphrase : ", FALSE);
+  if (ssh_privkey_read(kgc->user, kgc->in_filename, kgc->passphrase,
+                       &comment, &prvkey) == SSH_PRIVKEY_KEY_UNREADABLE)
+    {
+      printf("Private key is unreadable.\r\n");
+      retval = 1;
+    }
+
+  else if (prvkey != NULL)
+    {
+      retval = keygen_write_private_key(kgc, prvkey, comment, "_ssh2");
+    }
+  else
+    {
+      printf("Cannot determine the type of the key or "
+             "passphrase incorrect.\r\n");
+      retval = 1;
+    }
+
+  if (kgc->passphrase)
+    ssh_xfree(kgc->passphrase);
+
+  kgc->passphrase = NULL;
+
   if (comment)
     ssh_xfree(comment);
 
   if (prvkey)
     ssh_private_key_free( prvkey );
 
-  if (pubkey)
-    ssh_public_key_free( pubkey );
+  return retval;
+}
+
+
+/* converts SSH1 keys to SSH2 format */
+int keygen_convert_key(KeyGenCtx *kgc)
+{
+  Boolean ret;
+  char *base_name;
+  char newfilename[1024];
+
+  if ((base_name = strstr(kgc->in_filename, ".pub")))
+    {
+      /* It's a public key */
+      if ((ret = keygen_convert_public_key(kgc)) == 0)
+        {
+          /* pub key convert worked so try priv key as well */
+
+          base_name[0] = '\0';
+          keygen_convert_private_key(kgc);
+        }
+    }
+
+  else
+    {
+      /* user supplied private key */
+      if ((ret = keygen_convert_private_key(kgc)) == 0)
+        {
+          /* priv key convert worked so add .pub to name and
+             try to convert pub key */
+          ssh_snprintf(newfilename, sizeof(newfilename),
+                       "%s.pub", kgc->in_filename);
+          ssh_xfree(kgc->in_filename);
+
+          kgc->in_filename = ssh_xstrdup(newfilename);
+          keygen_convert_public_key(kgc);
+        }
+    }
 
   return 0;
 }
 
-int keygen_edit_key(KeyGenCtx *kgc)
+/* Returns FALSE when successful, TRUE otherwise. */
+Boolean keygen_edit_key(KeyGenCtx *kgc)
 {
   SshPrivateKey seckey = NULL;
   SshPublicKey pubkey = NULL;
-  char pubfn[1024];
-  char outpubfn[1024];
-  char pubbu[1024];
-  char secbu[1024];
-  char *secfn;
-  char *outsecfn;
-  int ok;
-  char *oldcomment = NULL;
-  char *newcomment = NULL;
+  char pubfn[1024], outpubfn[1024], pubbu[1024], secbu[1024];
+  char *secfn, *outsecfn, *oldcomment = NULL, *newcomment = NULL;
   char *newpass = NULL;
+  Boolean ok;
+  Boolean no_public_key = FALSE;
 
-  if (!(*(kgc->in_filename))) 
+  if (!(*(kgc->in_filename)))
     {
       ssh_warning("Invalid keyfile.");
-      return 1;
+      return TRUE;
     }
   else
     {
       secfn = kgc->in_filename;
-      snprintf(pubfn, sizeof (pubfn), "%s.pub", secfn);
+      ssh_snprintf(pubfn, sizeof (pubfn), "%s.pub", secfn);
     }
-  
+
   if (!(kgc->out_filename))
     {
       kgc->out_filename = ssh_xstrdup(kgc->in_filename);
     }
 
   outsecfn = kgc->out_filename;
-  snprintf(outpubfn, sizeof (outpubfn), "%s.pub", outsecfn);
+  ssh_snprintf(outpubfn, sizeof (outpubfn), "%s.pub", outsecfn);
 
-  snprintf(secbu, sizeof (secbu), "%s~", kgc->in_filename);
-  snprintf(pubbu, sizeof (pubbu), "%s~", pubfn);
+  ssh_snprintf(secbu, sizeof (secbu), "%s~", kgc->in_filename);
+  ssh_snprintf(pubbu, sizeof (pubbu), "%s~", pubfn);
   (void)unlink(secbu);
   (void)unlink(pubbu);
 
   pubkey = ssh_pubkey_read(kgc->user, pubfn, &oldcomment, NULL);
 
-  if (! pubkey)
+  if (!pubkey)
     {
+      /* no public key exists, but we can continue editing the private
+         key anyway */
       ssh_warning("Cannot read public keyfile %s.", pubfn);
-      return 1;
+      no_public_key  = TRUE;
     }
 
-  if (! oldcomment)
-    oldcomment = ssh_xstrdup("");
 
   if (kgc->passphrase)
     {
-      seckey = ssh_privkey_read(kgc->user, secfn, kgc->passphrase, NULL, NULL);
+      if (ssh_privkey_read(kgc->user, secfn, kgc->passphrase,
+                           oldcomment ? NULL : &oldcomment,
+                           &seckey) == SSH_PRIVKEY_KEY_UNREADABLE)
+        {
+          ssh_warning("Cannot read private keyfile %s.", secfn);
+          return TRUE;
+        }
     }
 
-  if (! seckey)
+  if (!seckey)
     {
-      seckey = ssh_privkey_read(kgc->user, secfn, "", NULL, NULL);
+      if (ssh_privkey_read(kgc->user, secfn, "",
+                           oldcomment ? NULL : &oldcomment,
+                           &seckey) == SSH_PRIVKEY_KEY_UNREADABLE)
+        {
+          ssh_warning("Cannot read private keyfile %s.", secfn);
+          return TRUE;
+        }
+
       if (seckey)
         {
           keygen_free_str(kgc->passphrase);
           kgc->passphrase = ssh_xstrdup("");
         }
     }
-    
-  if (! kgc->pass_empty)
+
+  if (!oldcomment)
+    oldcomment = ssh_xstrdup("");
+
+  if (!kgc->pass_empty)
     {
-      if (! seckey)
+      if (!seckey)
         {
           keygen_free_str(kgc->passphrase);
           printf("Passphrase needed for key \"%s\".\n", oldcomment);
           kgc->passphrase = ssh_read_passphrase("Passphrase : ", FALSE);
           if (kgc->passphrase)
             {
-              seckey = ssh_privkey_read(kgc->user, secfn, kgc->passphrase, 
-                                        NULL, NULL);
+              if (ssh_privkey_read(kgc->user, secfn, kgc->passphrase, NULL,
+                                   &seckey) == SSH_PRIVKEY_KEY_UNREADABLE)
+                {
+                  ssh_warning("Cannot read private keyfile %s.", secfn);
+                  return TRUE;
+                }
             }
         }
     }
 
-  if (! seckey)
+  if (!seckey)
     {
       ssh_warning("Cannot read private keyfile %s.", secfn);
-      return 1;
+      return TRUE;
     }
 
+
   printf("Do you want to edit key \"%s\" ", oldcomment);
-  if (! ssh_read_confirmation("(yes or no)? ", FALSE))
+  if (!ssh_read_confirmation("(yes or no)? ", FALSE))
     {
       printf("Key unedited and unsaved.\n");
       keygen_free_str(oldcomment);
-      return 0;
+      return FALSE;
     }
 
-  ok = 0;
+  ok = FALSE;
 
-  while (! ok)
+  while (!ok)
     {
-      if (! newcomment)
+      if (!newcomment)
         newcomment = ssh_xstrdup(oldcomment);
       printf("Your key comment is \"%s\". ", newcomment);
       if (ssh_read_confirmation("Do you want to edit it (yes or no)? ", FALSE))
         {
-          if (ssh_readline((unsigned char *)"New key comment: ", 
-                           (unsigned char **)&newcomment, 
+          if (ssh_readline("New key comment: ", &newcomment,
                            fileno(stdin), fileno(stdout)) < 0)
             {
               fprintf(stderr, "Abort!  Key unedited and unsaved.\n");
               keygen_free_str(newpass);
               keygen_free_str(newcomment);
               keygen_free_str(oldcomment);
-              return 1;
+              return TRUE;
             }
           putchar('\n');
         }
-      if (! newpass)
+      if (!newpass)
         newpass = ssh_xstrdup(kgc->passphrase);
-      if (! kgc->pass_empty)
+      if (!kgc->pass_empty)
         {
-          if (ssh_read_confirmation("Do you want to edit passphrase (yes or no)? ", FALSE))
+          if (ssh_read_confirmation("Do you want to edit passphrase (yes "
+                                    "or no)? ", FALSE))
             {
               keygen_free_str(newpass);
-              newpass = ssh_read_passphrase_twice("New passphrase : ", 
-                                                  "Again          : ", 
+              newpass = ssh_read_passphrase_twice("New passphrase : ",
+                                                  "Again          : ",
                                                   FALSE);
-              if (! newpass)
+              if (!newpass)
                 {
                   fprintf(stderr, "Abort!  Key unedited and unsaved.\n");
                   keygen_free_str(newcomment);
                   keygen_free_str(oldcomment);
-                  return 0;
+                  return FALSE;
                 }
             }
         }
       printf("Do you want to continue editing key \"%s\" ", newcomment);
       if (ssh_read_confirmation("(yes or no)? ", FALSE))
-        ok = 0;
+        ok = FALSE;
       else
-        ok = 1;
+        ok = TRUE;
     }
+
   if ((strcmp(newpass, kgc->passphrase) == 0) &&
       (strcmp(newcomment, oldcomment) == 0))
     {
@@ -1253,7 +1746,7 @@ int keygen_edit_key(KeyGenCtx *kgc)
       keygen_free_str(newpass);
       keygen_free_str(newcomment);
       keygen_free_str(oldcomment);
-      return 0;
+      return FALSE;
     }
   printf("Do you want to save key \"%s\" to file %s ", newcomment, outsecfn);
   if (! ssh_read_confirmation("(yes or no)? ", FALSE))
@@ -1262,9 +1755,9 @@ int keygen_edit_key(KeyGenCtx *kgc)
       keygen_free_str(newpass);
       keygen_free_str(newcomment);
       keygen_free_str(oldcomment);
-      return 0;
+      return FALSE;
     }
-  
+
   if (strcmp(outsecfn, kgc->in_filename) == 0)
     {
       if (rename(outsecfn, secbu) != 0)
@@ -1274,30 +1767,33 @@ int keygen_edit_key(KeyGenCtx *kgc)
           keygen_free_str(newcomment);
           keygen_free_str(oldcomment);
           fprintf(stderr, "Abort!\n");
-          return 1;
+          return TRUE;
         }
     }
 
-  if (strcmp(outpubfn, pubfn) == 0)
+  if (no_public_key == FALSE)
     {
-      if (rename(outpubfn, pubbu) != 0)
+      if (strcmp(outpubfn, pubfn) == 0)
         {
-          ssh_warning("Unable to backup private key.");
-          if (strcmp(outsecfn, secfn) == 0)
-            (void)rename(secbu, outsecfn);
-          keygen_free_str(newpass);
-          keygen_free_str(newcomment);
-          keygen_free_str(oldcomment);
-          fprintf(stderr, "Abort!\n");
-          return 1;
+          if (rename(outpubfn, pubbu) != 0)
+            {
+              ssh_warning("Unable to backup public key.");
+              if (strcmp(outsecfn, secfn) == 0)
+                (void)rename(secbu, outsecfn);
+              keygen_free_str(newpass);
+              keygen_free_str(newcomment);
+              keygen_free_str(oldcomment);
+              fprintf(stderr, "Abort!\n");
+              return TRUE;
+            }
         }
     }
 
   if (ssh_privkey_write(kgc->user,
-                        outsecfn, 
-                        newpass, 
+                        outsecfn,
+                        newpass,
                         newcomment,
-                        seckey, 
+                        seckey,
                         NULL))
     {
       ssh_warning("Unable to write private key.!");
@@ -1309,33 +1805,36 @@ int keygen_edit_key(KeyGenCtx *kgc)
       keygen_free_str(newcomment);
       keygen_free_str(oldcomment);
       fprintf(stderr, "Abort!\n");
-      return 1;
+      return TRUE;
     }
 
-  if (ssh_pubkey_write(kgc->user,
-                       outpubfn,
-                       newcomment,
-                       pubkey, 
-                       NULL))
+  if (no_public_key == FALSE)
     {
-      ssh_warning("Unable to write public key.!");
-      unlink(outsecfn);
-      if (strcmp(outsecfn, secfn) == 0)
-        (void)rename(secbu, outsecfn);
-      if (strcmp(outpubfn, pubfn) == 0)
-        (void)rename(pubbu, outpubfn);
-      keygen_free_str(newpass);
-      keygen_free_str(newcomment);
-      keygen_free_str(oldcomment);
-      fprintf(stderr, "Abort!\n");
-      return 1;
+      if (ssh_pubkey_write(kgc->user,
+                           outpubfn,
+                           newcomment,
+                           pubkey,
+                           NULL))
+        {
+          ssh_warning("Unable to write public key.!");
+          unlink(outsecfn);
+          if (strcmp(outsecfn, secfn) == 0)
+            (void)rename(secbu, outsecfn);
+          if (strcmp(outpubfn, pubfn) == 0)
+            (void)rename(pubbu, outpubfn);
+          keygen_free_str(newpass);
+          keygen_free_str(newcomment);
+          keygen_free_str(oldcomment);
+          fprintf(stderr, "Abort!\n");
+          return TRUE;
+        }
     }
 
   keygen_free_str(newpass);
   keygen_free_str(newcomment);
   keygen_free_str(oldcomment);
 
-  return 0;
+  return FALSE;
 }
 
 Boolean keygen_dump_pubkey_fingerprint(KeyGenCtx *kgc, SshPublicKey pubkey)
@@ -1347,14 +1846,14 @@ Boolean keygen_dump_pubkey_fingerprint(KeyGenCtx *kgc, SshPublicKey pubkey)
   unsigned char *blob;
   size_t blob_len;
   char *fingerprint;
-  
+
   if (!(blob_len = ssh_encode_pubkeyblob(pubkey, &blob)))
     {
       ssh_warning("keygen_dump_pubkey_fingerprint: Couldn't encode publickey "
                   "to blob.");
       return FALSE;
     }
-  
+
   if (!ssh_hash_supported(hash_name))
     {
       ssh_warning("keygen_dump_pubkey_fingerprint: Hash \"%s\" not supported, "
@@ -1368,7 +1867,7 @@ Boolean keygen_dump_pubkey_fingerprint(KeyGenCtx *kgc, SshPublicKey pubkey)
       ssh_warning("keygen_dump_pubkey_fingerprint: Couldn't initialize hash "
                   "function.", hash_name);
       return FALSE;
-    }  
+    }
 
   digest_len = ssh_hash_digest_length(hash_ctx);
   digest = ssh_xcalloc(digest_len, sizeof(unsigned char));
@@ -1388,6 +1887,12 @@ Boolean keygen_dump_pubkey_fingerprint(KeyGenCtx *kgc, SshPublicKey pubkey)
   return TRUE;
 }
 
+void keygen_fatal_cb(const char *message, void *context)
+{
+  fprintf(stderr, "%s: FATAL: %s\r\n", ssh_get_program_name(), message);
+  exit(255);
+}
+
 /* main for ssh-keygen */
 
 int main(int argc, char **argv)
@@ -1399,23 +1904,44 @@ int main(int argc, char **argv)
   int r = 0, rr = 0;
   char *progname;
 
+  ssh_debug_register_callbacks(keygen_fatal_cb, NULL, NULL, NULL);
+
+  /* check for --help before anything */
+  if (argc > 1)
+    if (strcmp(argv[1], "--help") == 0)
+      {
+        printf("%s", keygen_helptext);
+        exit(0);
+      }
+
   if (strchr(argv[0], '/'))
     progname = strrchr(argv[0], '/') + 1;
   else
     progname = argv[0];
 
+#ifdef SSHDIST_CRYPT_RSA
+  ssh_pk_provider_register(&ssh_pk_if_modn_generator);
+#endif /* SSHDIST_CRYPT_RSA */
+#ifdef SSHDIST_CRYPT_DSA
+  ssh_pk_provider_register(&ssh_pk_dl_modp_generator);
+#endif /* SSHDIST_CRYPT_DSA */
+
   /* Initialize the context */
   kgc = keygen_init_ctx();
 
 
-  ssh_signals_prevent_core(TRUE, kgc);
+  ssh_signals_prevent_core(FALSE, kgc);
 
 
 
 
   /* try to find the user's home directory */
 
-  while ((ch = ssh_getopt(argc, argv, "vhqr:?P1:t:b:p:i:D:B:e:c:VF:", NULL))
+  ssh_register_program_name(progname);
+  
+  while ((ch = ssh_getopt(argc, argv,
+                          "vhqr:?P1:7:x:t:b:p:i:D:B:e:c:VF:k:",
+                          NULL))
          != EOF)
     {
       if (!ssh_optval)
@@ -1424,7 +1950,7 @@ int main(int argc, char **argv)
           keygen_free_ctx(kgc);
           exit(1);
         }
-      switch (ch) 
+      switch (ch)
         {
           /* -b: specify the strength of the key */
 
@@ -1439,7 +1965,7 @@ int main(int argc, char **argv)
           /* -t: specify the key type. */
         case 't':
           /* the kgc->keytype gets the crypto library name of the alg. */
-          
+
           if (kgc->keytype != NULL)
             keygen_error(kgc, "multiple key types specified.");
 
@@ -1484,11 +2010,32 @@ int main(int argc, char **argv)
           kgc->in_filename = ssh_xstrdup(ssh_optarg);
           break;
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
           /* -V: print the version number */
         case 'V':
           printf("%s version " SSH2_VERSION
                  ", compiled "__DATE__".\n", progname);
-          /* XXX more stuff here possibly */ 
+
+
+
+          /* XXX more stuff here possibly */
           keygen_free_ctx(kgc);
           exit(0);
 
@@ -1510,7 +2057,7 @@ int main(int argc, char **argv)
           break;
 
           /* XXX Following are experimental options. */
-          
+
           /* -i: display (all) information about a key */
         case 'i':
           kgc->info_file = ssh_xstrdup(ssh_optarg);
@@ -1573,7 +2120,7 @@ int main(int argc, char **argv)
 
       if (!kgc->info_file)
         ssh_fatal("-F option needs public key file as argument.");
-      
+
       pubkey = ssh_pubkey_read(kgc->user, kgc->info_file,
                                NULL, NULL);
       if (pubkey != NULL)
@@ -1582,7 +2129,7 @@ int main(int argc, char **argv)
             r = 0;
           else
             r = 1;
-          
+
           goto finish;
         }
       else
@@ -1590,7 +2137,7 @@ int main(int argc, char **argv)
           ssh_fatal("Couldn't read public key \"%s\".", kgc->info_file);
         }
     }
-  
+
   /* Perhaps we should derive a key? */
   if (kgc->derive_file)
     {
@@ -1612,17 +2159,17 @@ int main(int argc, char **argv)
                  "   cryptographically strong keys cannot be broken!\n");
 
           pubkey_print_estimate(pubkey);
-          
+
           goto finish;
         }
 
       /* Perhaps its a private key? */
       prvkey = keygen_read_privkey(kgc, kgc->derive_file);
-      
+
       if (prvkey != NULL)
         {
           /* Derive the appropriate public key. */
-          snprintf(buf, sizeof (buf), "%s.pub", kgc->derive_file);
+          ssh_snprintf(buf, sizeof (buf), "%s.pub", kgc->derive_file);
           pubkey = ssh_private_key_derive_public_key(prvkey);
           if (pubkey == NULL)
             {
@@ -1658,7 +2205,7 @@ int main(int argc, char **argv)
 
       /* Perhaps its a private key? */
       prvkey = keygen_read_privkey(kgc, kgc->info_file);
-      
+
       if (prvkey != NULL)
         {
           keygen_view_prvkey(kgc, prvkey);
@@ -1667,11 +2214,25 @@ int main(int argc, char **argv)
 
       keygen_error(kgc, "Cannot determine the type of the key.");
     }
-        
+
   if (kgc->convert)
     {
       r = keygen_convert_key(kgc);
     }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   else if (kgc->edit_key)
     {
       r = keygen_edit_key(kgc);
@@ -1711,9 +2272,9 @@ int main(int argc, char **argv)
           kgc->comment = ssh_xmalloc(256);
           now = ssh_time();
           time_str = ssh_readable_time_string(now, TRUE);
-          snprintf(kgc->comment, 256, "%d-bit %s, %s@%s, %s", 
-                   kgc->keybits, kgc->keytypecommon,
-                   username, t, time_str);
+          ssh_snprintf(kgc->comment, 256, "%d-bit %s, %s@%s, %s",
+                       kgc->keybits, kgc->keytypecommon,
+                       username, t, time_str);
           ssh_xfree(time_str);
           ssh_xfree(t);
         }
@@ -1726,7 +2287,7 @@ int main(int argc, char **argv)
           r = keygen_keygen(kgc);
         }
       else
-        { 
+        {
           if (kgc->out_filename != NULL)
             keygen_keygen(kgc);
 
@@ -1750,7 +2311,7 @@ int main(int argc, char **argv)
     }
 
 finish:
-  
+
   keygen_free_ctx(kgc);
 
   return r;
@@ -1801,3 +2362,339 @@ finish:
 
 
 
+
+/*
+ *  $Log: ssh-keygen2.c,v $ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  
+ *  $EndLog$
+ */
