@@ -4,7 +4,7 @@
 
   Author: Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1999-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1999-2001 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Hostbased authentication, client-side. This program is supposed to
@@ -39,12 +39,12 @@
     This MUST be sent first, if at all. Packet payload is a
     comma-separated list of compatibility flags needed to interoperate
     with older versions of servers.
-    
+
   #define SSH_AUTH_HOSTBASED_END       (SshPacketType)2
 
     This notifies ssh-signer2 that ssh2 client no longer needs ssh-signer2.
     This message has no payload.
-    
+
   Possible messages from ssh-signer2 to ssh2:
 
   #define SSH_AUTH_HOSTBASED_SIGNATURE (SshPacketType)3
@@ -53,7 +53,7 @@
     the packet and signed it. Payload is the signature.
 
   Common messages:
-  
+
   #define SSH_AUTH_HOSTBASED_ERROR     (SshPacketType)4
 
     Payload:
@@ -70,23 +70,24 @@
 #include "ssheloop.h"
 #include "sshfdstream.h"
 #include "sshencode.h"
+#include "sshhostkeyio.h"
 #include "ssh-signer2.h"
 #include "sshmsgs.h"
 #include "sshauth.h"
-#include "sshconn.h"
 #include "sshconfig.h"
 #include "sshuserfiles.h"
 #include "sshuserfile.h"
-#include "ssh2pubkeyencode.h"
 #include "sshtcp.h"
 #include "sshconfig.h"
 #include "sshtimeouts.h"
 #include "sshdsprintf.h"
 #include "sshappcommon.h"
+#include "sshsnlist.h"
+#include "ssholdfsm.h"
+#include "ssh2compat.h"
 #ifdef HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
 #endif /* HAVE_SYS_UTSNAME_H */
-#include "sshfsm.h"
 
 #define SSH_DEBUG_MODULE "SshSigner2"
 
@@ -101,7 +102,9 @@
 /* #define SLEEP_AFTER_STARTUP */
 
 /* Define this to make ssh-signer2 really quiet. */
+#ifndef SIGNER_DEBUG
 #define SIGNER_QUIET
+#endif /* SIGNER_DEBUG */
 
 /* Stored program name for ssh-signer (taken from argv[0]). */
 static char *progname;
@@ -118,6 +121,7 @@ typedef struct SignerCompatRec
 {
   Boolean hostbased_requested_service_draft_incompat;
   Boolean signature_encode_draft_incompat;
+  Boolean rsa_hash_scheme_draft_incompat;
 } *SignerCompat;
 
 typedef struct SshSignerRec
@@ -129,7 +133,7 @@ typedef struct SshSignerRec
   SshPacketType packet_type;
 
   Boolean packet_pending;
-  
+
   /* Internal stuff */
   SshConfig serv_config;
   SshConfig global_config;
@@ -137,19 +141,19 @@ typedef struct SshSignerRec
   SshUser real_user;
   Boolean quiet;
   SshPacketWrapper wrapper;
-  SshFSM fsm;
-  SshFSMThread main_thread;
+  SshOldFSM fsm;
+  SshOldFSMThread main_thread;
   SignerState state;
   SshAuthHostBasedError error_code;
 
   /* Operation handle for the async signing operation. */
   SshOperationHandle op_handle;
-  
+
   /* compat flags. */
   SignerCompat compat;
 
   char *pubkeytype;
-  
+
   /* If we need to send an error message, this field will contain the
      "human readable error string". */
   char *error_message_to_ssh2;
@@ -160,25 +164,25 @@ typedef struct SshSignerRec
 /* Forward declarations. */
 void signer_can_send(void *context);
 
-SSH_FSM_STEP(signer_suspend)
+SSH_OLDFSM_STEP(signer_suspend)
 {
-  return SSH_FSM_SUSPENDED;
+  return SSH_OLDFSM_SUSPENDED;
 }
 
-SSH_FSM_STEP(signer_process_error_packet)
+SSH_OLDFSM_STEP(signer_process_error_packet)
 {
   unsigned int error_code;
   char *error_message = NULL, *lang_tag = NULL;
   size_t parsed_len = 0L;
-  
-  SSH_FSM_GDATA(SshSigner);
 
-  SSH_FSM_SET_NEXT("signer_finish");
+  SSH_OLDFSM_GDATA(SshSigner);
+
+  SSH_OLDFSM_SET_NEXT("signer_finish");
 
   /* Tell packet wrapper we don't want more packets, and output EOF to
      the stream. */
   ssh_packet_wrapper_send_eof(gdata->wrapper);
-  
+
   if ((parsed_len = ssh_decode_array(gdata->packet_payload,
                                      gdata->packet_payload_len,
                                      SSH_FORMAT_CHAR, &error_code,
@@ -187,7 +191,7 @@ SSH_FSM_STEP(signer_process_error_packet)
                                      SSH_FORMAT_UINT32_STR, &lang_tag, NULL,
                                      SSH_FORMAT_END)) == 0)
     {
-      ssh_warning("Invalid error packet received (what an irony...).");      
+      ssh_warning("Invalid error packet received (what an irony...).");
       gdata->error_code = SIGNER_ERROR_PROTOCOL_ERROR;
     }
 
@@ -198,7 +202,7 @@ SSH_FSM_STEP(signer_process_error_packet)
 
   ssh_warning("Received error %d, message \"%s\", lang_tag \"%s\".",
               error_code, error_message, lang_tag);
-              
+
   switch (error_code)
     {
     case SIGNER_ERROR_GENERIC:
@@ -212,19 +216,19 @@ SSH_FSM_STEP(signer_process_error_packet)
 
   ssh_xfree(error_message);
   ssh_xfree(lang_tag);
-  
-  return SSH_FSM_CONTINUE;
+
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(signer_process_end)
+SSH_OLDFSM_STEP(signer_process_end)
 {
-  SSH_FSM_SET_NEXT("signer_finish");
-  return SSH_FSM_CONTINUE;
+  SSH_OLDFSM_SET_NEXT("signer_finish");
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(signer_send_error)
+SSH_OLDFSM_STEP(signer_send_error)
 {
-  SSH_FSM_GDATA(SshSigner);
+  SSH_OLDFSM_GDATA(SshSigner);
 
   gdata->state = SIGNER_DEAD;
 
@@ -235,7 +239,7 @@ SSH_FSM_STEP(signer_send_error)
     }
 
   gdata->packet_type = SSH_AUTH_HOSTBASED_ERROR;
-  
+
   gdata->packet_payload_len =
     ssh_encode_array_alloc(&gdata->packet_payload,
                            SSH_FORMAT_CHAR,
@@ -247,17 +251,17 @@ SSH_FSM_STEP(signer_send_error)
 
   ssh_packet_wrapper_can_receive(gdata->wrapper, FALSE);
 
-  SSH_FSM_SET_NEXT("signer_finish");
+  SSH_OLDFSM_SET_NEXT("signer_finish");
 
   gdata->packet_pending = TRUE;
-  
+
   if (ssh_packet_wrapper_can_send(gdata->wrapper))
     ssh_register_timeout(0L, 0L, signer_can_send, gdata);
-  
-  return SSH_FSM_SUSPENDED;
+
+  return SSH_OLDFSM_SUSPENDED;
 }
 
-SSH_FSM_STEP(signer_check_hostbased_packet)
+SSH_OLDFSM_STEP(signer_check_hostbased_packet)
 {
   /* Received. */
   unsigned int msg_byte; /* This is unsigned int because
@@ -272,21 +276,20 @@ SSH_FSM_STEP(signer_check_hostbased_packet)
   unsigned char *recv_pubkeyblob = NULL;
   size_t recv_pubkeyblob_len;
   /* Dug up by us. */
-  char *pubkey_alg = NULL, *hostname = NULL, *username = NULL;
-  unsigned char *pubkeyblob = NULL;
+  char *hostname = NULL, *username = NULL;
+  SshBuffer pubkeyblob_buf = NULL;
   char *used_service;
-  size_t pubkeyblob_len;
   size_t hostname_len;
 
   /* Internal stuff*/
   char *hostkeyfile = NULL;
-  
-  SSH_FSM_GDATA(SshSigner);
 
-  SSH_PRECOND(gdata->packet_payload);  
+  SSH_OLDFSM_GDATA(SshSigner);
+
+  SSH_PRECOND(gdata->packet_payload);
 
   /* If check is failed, go to this state. */
-  SSH_FSM_SET_NEXT("signer_send_error");
+  SSH_OLDFSM_SET_NEXT("signer_send_error");
 
   if (ssh_decode_array(gdata->packet_payload, gdata->packet_payload_len,
                        /* session id */
@@ -321,83 +324,50 @@ SSH_FSM_STEP(signer_check_hostbased_packet)
       goto error;
     }
 
-  /* Get pubkeyblob, pubkeyblob_len, pubkey_alg, hostname and
-     username. */
-
-  /* Dig up hosts publickey. */
-  if(gdata->serv_config->public_host_key_file[0] != '/')
+  /* Get hostkeys. */
+  if (!ssh_host_key_read_keys(gdata->serv_config->host_keys_ctx,
+                              gdata->serv_config,
+                              gdata->effective_user_data, TRUE, TRUE))
     {
-      ssh_dsprintf(&hostkeyfile, "%s/%s", SSH_SERVER_DIR,
-                   gdata->serv_config->public_host_key_file);
-    }
-  else
-    {
-      hostkeyfile = ssh_xstrdup(gdata->serv_config->public_host_key_file);
-    }
-
-  SSH_TRACE(2, ("place to look for public key: %s", hostkeyfile));
-
-  /* This pubkey*-stuff is for the client _host's_ public
-     hostkey. */
-  /* Getting pubkeyblob, pubkeyblob_len */
-  SSH_DEBUG(4, ("Reading pubkey-blob from %s...", hostkeyfile));
-  if (ssh2_key_blob_read(gdata->effective_user_data, hostkeyfile, TRUE,
-                         NULL,
-                         &pubkeyblob,
-                         &pubkeyblob_len, NULL)
-      != SSH_KEY_MAGIC_PUBLIC)
-    {
-      SSH_TRACE(1, ("Reading public key failed."));
+      SSH_TRACE(0, ("Unable to load server hostkeys."));
       gdata->error_code_to_ssh2 = SIGNER_ERROR_GENERIC;
-      ssh_dsprintf(&gdata->error_message_to_ssh2, "Error reading public host "
-                   "key from '%s'.", hostkeyfile);
-      goto error;
-    }
-
-  SSH_DEBUG(4, ("done."));
-
-  if ((pubkey_alg =
-       ssh_pubkeyblob_type(pubkeyblob, pubkeyblob_len))
-      == NULL)
-    {
-      SSH_TRACE(1, ("Couldn't figure out public key algorithm."));
-      gdata->error_code_to_ssh2 = SIGNER_ERROR_GENERIC;
-      gdata->error_message_to_ssh2 =
-        ssh_xstrdup("Couldn't determine public key algorithm from "
-                    "public key.");
+      gdata->error_message_to_ssh2 = ssh_xstrdup("Unable to load host keys.");
       goto error;
     }
 
   /* Getting hostname. */
   hostname = ssh_xmalloc(MAXHOSTNAMELEN*2);
-  
+
   ssh_tcp_get_host_name(hostname, MAXHOSTNAMELEN*2);
   hostname_len = strlen(hostname);
   /* Sanity check */
-  SSH_ASSERT(hostname_len + 2 < MAXHOSTNAMELEN);
-  if (!strchr(hostname, '.'))
+  if (hostname_len + 2 >= MAXHOSTNAMELEN)
+    ssh_fatal("Local hostname too long!");
+  if (strchr(hostname, '.'))
+    {
+      /* Append trailing period to the name. */
+      hostname[hostname_len] = '.';
+      hostname[hostname_len + 1] = '\0';
+    }
+  else
     {
       char *temp_name;
       if (!gdata->global_config->default_domain)
         {
-          SSH_TRACE(2, ("hostname doesn't contain '.' and DefaultDomain " \
-                        "isn't set. Hostbased authentication will "       \
+          SSH_TRACE(2, ("hostname doesn't contain '.' and DefaultDomain "
+                        "isn't set. Hostbased authentication will "
                         "very likely fail."));
         }
       else
         {
-          ssh_dsprintf(&temp_name, "%s.%s", hostname,
+          /* Append domain name and trailing period in the name. */
+          ssh_dsprintf(&temp_name, "%s.%s.", hostname,
                        gdata->global_config->default_domain);
           ssh_xfree(hostname);
           hostname = temp_name;
-          hostname_len = strlen(hostname);
         }
     }
 
-  /* We want FQDN. */
-  hostname[hostname_len] = '.';
-  hostname[hostname_len + 1] = '\0';
-
   /* Getting username. */
   username = ssh_xstrdup(ssh_user_name(gdata->real_user));
 
@@ -405,7 +375,7 @@ SSH_FSM_STEP(signer_check_hostbased_packet)
   if (msg_byte != SSH_MSG_USERAUTH_REQUEST)
     {
       SSH_TRACE(1, ("Invalid packet."));
-      SSH_DEBUG(1, ("msg_byte != SSH_MSG_USERAUTH_REQUEST " \
+      SSH_DEBUG(1, ("msg_byte != SSH_MSG_USERAUTH_REQUEST "
                     "(msg_byte = %d)", msg_byte));
       gdata->error_code_to_ssh2 = SIGNER_ERROR_DATA_VERIFICATION_FAILED;
       gdata->error_message_to_ssh2 =
@@ -417,11 +387,11 @@ SSH_FSM_STEP(signer_check_hostbased_packet)
     used_service = SSH_USERAUTH_SERVICE;
   else
     used_service = SSH_CONNECTION_SERVICE;
-  
+
   if (strcmp(service_str, used_service) != 0)
     {
       SSH_TRACE(1, ("Invalid packet."));
-      SSH_DEBUG(1, ("service_str != \"%s\" (it was '%s')", \
+      SSH_DEBUG(1, ("service_str != \"%s\" (it was '%s')",
                     used_service, service_str));
       gdata->error_code_to_ssh2 = SIGNER_ERROR_DATA_VERIFICATION_FAILED;
       ssh_dsprintf(&gdata->error_message_to_ssh2, "Supplied packet is not "
@@ -431,52 +401,72 @@ SSH_FSM_STEP(signer_check_hostbased_packet)
   if (strcmp(hostbased_str, SSH_AUTH_HOSTBASED) != 0)
     {
       SSH_TRACE(1, ("Invalid packet."));
-      SSH_DEBUG(1, ("hostbased_str != \"hostbased\" (it was '%s')", \
+      SSH_DEBUG(1, ("hostbased_str != \"hostbased\" (it was '%s')",
                     hostbased_str));
       gdata->error_code_to_ssh2 = SIGNER_ERROR_DATA_VERIFICATION_FAILED;
       gdata->error_message_to_ssh2 =
-        ssh_xstrdup("Supplied packet is not for \"hostbased\" authentication.");
-      goto error;
-    }
-  /* XXX has to be changed when adding support for multiple hostkeys */
-  if (strcmp(recv_pubkey_alg, pubkey_alg) != 0)
-    {
-      SSH_TRACE(1, ("Invalid packet."));
-      SSH_DEBUG(1, ("Client gave us invalid pubkey-algorithms for our " \
-                    "hostkey."));
-      gdata->error_code_to_ssh2 = SIGNER_ERROR_DATA_VERIFICATION_FAILED;
-      gdata->error_message_to_ssh2 =
-        ssh_xstrdup("Supplied packet contains wrong public key algorithm(s).");
+        ssh_xstrdup("Supplied packet is not for \"hostbased\" "
+                    "authentication.");
       goto error;
     }
 
-  gdata->pubkeytype = pubkey_alg;
-  
-  if (recv_pubkeyblob_len == pubkeyblob_len)
-    {
-      if (memcmp(recv_pubkeyblob, pubkeyblob, pubkeyblob_len) != 0)
+  {
+    char *alg_list = ssh_host_key_get_algorithmlist
+      (gdata->serv_config->host_keys_ctx);
+    const char *temp_list = alg_list;
+    Boolean match = FALSE;
+
+    do {
+      char *next_alg = ssh_snlist_get_name(temp_list);
+      if (strcmp(next_alg, recv_pubkey_alg) == 0)
         {
-          SSH_TRACE(1, ("Invalid packet."));
-          SSH_DEBUG(1, ("client gave us wrong (or corrupted) " \
-                        "public key."));
-#ifdef HEXDUMPS
-          SSH_DEBUG_HEXDUMP(3, ("client gave us:"), \
-                            recv_pubkeyblob, pubkeyblob_len);
-          SSH_DEBUG_HEXDUMP(3, ("our pubkey:"), \
-                            recv_pubkeyblob, pubkeyblob_len);
-#endif /* HEXDUMPS */
-          gdata->error_code_to_ssh2 = SIGNER_ERROR_DATA_VERIFICATION_FAILED;
-          gdata->error_message_to_ssh2 =
-            ssh_xstrdup("Supplied packet contains malformed public key.");
-          goto error;
+          match = TRUE;
+          ssh_xfree(next_alg);
+          break;
         }
-    }
-  else
+      ssh_xfree(next_alg);
+      temp_list = ssh_snlist_step_forward(temp_list);
+    } while (temp_list);
+
+    ssh_xfree(alg_list);
+
+    if (!match)
+      {
+        SSH_TRACE(1, ("Invalid packet."));
+        SSH_DEBUG(1, ("Client gave us invalid pubkey-algorithms for our "
+                      "hostkey."));
+        gdata->error_code_to_ssh2 = SIGNER_ERROR_DATA_VERIFICATION_FAILED;
+        ssh_dsprintf(&gdata->error_message_to_ssh2,
+                     "Supplied packet contains wrong public "
+                     "key algorithm (%s).",recv_pubkey_alg);
+        goto error;
+      }
+  }
+
+  gdata->pubkeytype = ssh_xstrdup(recv_pubkey_alg);
+
+  pubkeyblob_buf = ssh_host_key_get_pubkey_by_algname
+    (gdata->pubkeytype, gdata->serv_config->host_keys_ctx);
+
+  if (!pubkeyblob_buf)
+    ssh_fatal("Internal error fetching publickey.");
+
+  if (recv_pubkeyblob_len != ssh_buffer_len(pubkeyblob_buf) ||
+      memcmp(recv_pubkeyblob, ssh_buffer_ptr(pubkeyblob_buf),
+             ssh_buffer_len(pubkeyblob_buf)) != 0)
     {
       SSH_TRACE(1, ("Invalid packet."));
-      SSH_DEBUG(1, ("Client gave us wrong (or corrupted) public key. " \
-                    "Lengths differ (received: %d ; ours: %d)", \
-                    recv_pubkeyblob_len, pubkeyblob_len));
+      SSH_DEBUG(1, ("Client gave us wrong (or corrupted) "
+                    "public key."));
+      if (recv_pubkeyblob_len != ssh_buffer_len(pubkeyblob_buf))
+        SSH_DEBUG(1, ("Lengths differ (received: %d ; ours: %d)",
+                      recv_pubkeyblob_len, ssh_buffer_len(pubkeyblob_buf)));
+#ifdef HEXDUMPS
+      SSH_DEBUG_HEXDUMP(3, ("client gave us:"),
+                        recv_pubkeyblob, pubkeyblob_len);
+      SSH_DEBUG_HEXDUMP(3, ("our pubkey:"),
+                        recv_pubkeyblob, pubkeyblob_len);
+#endif /* HEXDUMPS */
       gdata->error_code_to_ssh2 = SIGNER_ERROR_DATA_VERIFICATION_FAILED;
       gdata->error_message_to_ssh2 =
         ssh_xstrdup("Supplied packet contains malformed public key.");
@@ -486,8 +476,8 @@ SSH_FSM_STEP(signer_check_hostbased_packet)
   if (strcasecmp(recv_hostname, hostname) != 0)
     {
       SSH_TRACE(1, ("Invalid packet."));
-      SSH_DEBUG(1, ("Wethinks the client gave us the wrong hostname. " \
-                    "(client's opinion: '%s' ours: '%s'", \
+      SSH_DEBUG(1, ("Wethinks the client gave us the wrong hostname. "
+                    "(client's opinion: '%s' ours: '%s'",
                     recv_hostname, hostname));
       gdata->error_code_to_ssh2 = SIGNER_ERROR_DATA_VERIFICATION_FAILED;
       ssh_dsprintf(&gdata->error_message_to_ssh2, "Supplied packet had the "
@@ -498,19 +488,19 @@ SSH_FSM_STEP(signer_check_hostbased_packet)
   if (strcmp(recv_username, username) != 0)
     {
       SSH_TRACE(1, ("Invalid packet."));
-      SSH_DEBUG(1, ("Client definitely gave us the wrong user name. " \
-                    "(it says: '%s' we know: '%s')", recv_username, \
+      SSH_DEBUG(1, ("Client definitely gave us the wrong user name. "
+                    "(it says: '%s' we know: '%s')", recv_username,
                     username));
       gdata->error_code_to_ssh2 = SIGNER_ERROR_DATA_VERIFICATION_FAILED;
       ssh_dsprintf(&gdata->error_message_to_ssh2, "Supplied packet had the "
-                   "wrong client username (packet contained \"%s\", ssh-signer "
-                   "got \"%s\").", recv_username, username);
+                   "wrong client username (packet contained \"%s\", "
+                   "ssh-signer got \"%s\").", recv_username, username);
       goto error;
     }
 
   /* Success. */
   SSH_TRACE(0, ("Received packet was OK."));
-  SSH_FSM_SET_NEXT("signer_sign_hostbased_packet");
+  SSH_OLDFSM_SET_NEXT("signer_sign_hostbased_packet");
  error:
   ssh_xfree(service_str);
   ssh_xfree(hostbased_str);
@@ -520,9 +510,8 @@ SSH_FSM_STEP(signer_check_hostbased_packet)
   ssh_xfree(recv_pubkeyblob);
   ssh_xfree(hostname);
   ssh_xfree(username);
-  ssh_xfree(pubkeyblob);
   ssh_xfree(hostkeyfile);
-  return SSH_FSM_CONTINUE;
+  return SSH_OLDFSM_CONTINUE;
 }
 
 void signer_signature_cb(SshCryptoStatus result,
@@ -530,12 +519,12 @@ void signer_signature_cb(SshCryptoStatus result,
                          size_t sig_len,
                          void *context)
 {
-  SshFSMThread thread = (SshFSMThread)context;
-  SSH_FSM_GDATA(SshSigner);
+  SshOldFSMThread thread = (SshOldFSMThread)context;
+  SSH_OLDFSM_GDATA(SshSigner);
   SSH_PRECOND(gdata);
 
   gdata->op_handle = NULL;
-  
+
   if (result != SSH_CRYPTO_OK)
     {
       SSH_TRACE(0, ("ssh_private_key_sign() returned %d.", result));
@@ -559,7 +548,7 @@ void signer_signature_cb(SshCryptoStatus result,
                                SSH_FORMAT_UINT32_STR,
                                signature_buffer,
                                sig_len,
-                               SSH_FORMAT_END);      
+                               SSH_FORMAT_END);
     }
   else
     {
@@ -568,61 +557,44 @@ void signer_signature_cb(SshCryptoStatus result,
                                SSH_FORMAT_DATA,
                                signature_buffer,
                                sig_len,
-                               SSH_FORMAT_END);                         
+                               SSH_FORMAT_END);
     }
-  
-  SSH_FSM_SET_NEXT("signer_send_signature");
+
+  SSH_OLDFSM_SET_NEXT("signer_send_signature");
  error:
-  SSH_FSM_CONTINUE_AFTER_CALLBACK(thread);
+  SSH_OLDFSM_CONTINUE_AFTER_CALLBACK(thread);
 }
 
 /* Sign the packet and send it to client. */
-SSH_FSM_STEP(signer_sign_hostbased_packet)
+SSH_OLDFSM_STEP(signer_sign_hostbased_packet)
 {
-  SSH_FSM_GDATA(SshSigner);
+  SSH_OLDFSM_GDATA(SshSigner);
   char *comment = NULL;
   size_t sig_len;
   char *hostkeyfile = NULL;
   SshPrivateKey privkey;
-  
+
   /* If signature op fails, go to this state. */
-  SSH_FSM_SET_NEXT("signer_send_error");
+  SSH_OLDFSM_SET_NEXT("signer_send_error");
 
   /* If we've gotten this far, the packet is ok, and it can be
      signed. */
 
-  if(gdata->serv_config->public_host_key_file[0] != '/')
-    {
-      ssh_dsprintf(&hostkeyfile, "%s/%s", SSH_SERVER_DIR,
-                   gdata->serv_config->host_key_file);
-    }
-  else
-    {
-      hostkeyfile = ssh_xstrdup(gdata->serv_config->host_key_file);
-    }
+  privkey = ssh_host_key_get_privkey_by_algname
+    (gdata->pubkeytype, gdata->serv_config->host_keys_ctx);
 
-  SSH_TRACE(2, ("place to look for private key: %s", hostkeyfile));
-
-  if ((privkey = ssh_privkey_read(gdata->effective_user_data, hostkeyfile, "",
-                                  &comment, NULL)) == NULL)
-    {
-      SSH_TRACE(0, ("ssh_privkey_read from %s failed.", hostkeyfile));
-      gdata->error_code_to_ssh2 = SIGNER_ERROR_SIGNING_OPERATION_FAILED;
-      gdata->error_message_to_ssh2 = ssh_xstrdup("Couldn't read host key.");
-      goto error;
-    }
-  
+  SSH_ASSERT(privkey);
 
   /* Check how big a chunk our private key can sign (this is
      purely a sanity check, as both of our signature schemas do
      their own hashing) */
   sig_len = ssh_private_key_max_signature_input_len(privkey);
 
-  SSH_TRACE(2, ("max input length for signing: %d", sig_len));
+  SSH_TRACE(2, ("Max input length for signing: %d", sig_len));
 
   if (sig_len == 0)
     {
-      SSH_TRACE(0, ("private key not capable of signing! " \
+      SSH_TRACE(0, ("Private key not capable of signing! "
                     "(definitely an error)"));
       gdata->error_code_to_ssh2 = SIGNER_ERROR_SIGNING_OPERATION_FAILED;
       gdata->error_message_to_ssh2 = ssh_xstrdup("Private host key not capable "
@@ -631,8 +603,8 @@ SSH_FSM_STEP(signer_sign_hostbased_packet)
     }
   else if (sig_len != -1 && sig_len < gdata->packet_payload_len)
     {
-      SSH_TRACE(0, ("private key can't sign our data. (too much " \
-                    "data (data_len %d, max input len for signing " \
+      SSH_TRACE(0, ("Private key can't sign our data. (too much "
+                    "data (data_len %d, max input len for signing "
                     "%d))", gdata->packet_payload_len, sig_len));
       gdata->error_code_to_ssh2 = SIGNER_ERROR_SIGNING_OPERATION_FAILED;
       gdata->error_message_to_ssh2 = ssh_xstrdup("Private host key can't sign "
@@ -640,6 +612,18 @@ SSH_FSM_STEP(signer_sign_hostbased_packet)
       goto error;
     }
 
+  /* RSA hash scheme compatibility. */
+  if (!strncmp(gdata->pubkeytype, "ssh-rsa", 7) &&
+      !ssh_compat_rsa_private_key_change_scheme
+      (privkey, !gdata->compat->rsa_hash_scheme_draft_incompat))
+    {
+      SSH_TRACE(0, ("Changing private key scheme failed."));
+      gdata->error_code_to_ssh2 = SIGNER_ERROR_SIGNING_OPERATION_FAILED;
+      gdata->error_message_to_ssh2 =
+        ssh_xstrdup("Changing private host key hash scheme failed.");
+      goto error;
+    }
+
   /* Do the actual signing. */
 
 #ifdef HEXDUMPS
@@ -650,7 +634,7 @@ SSH_FSM_STEP(signer_sign_hostbased_packet)
   ssh_xfree(hostkeyfile);
   ssh_xfree(comment);
 
-  SSH_FSM_ASYNC_CALL((gdata->op_handle =
+  SSH_OLDFSM_ASYNC_CALL((gdata->op_handle =
                       ssh_private_key_sign_async(privkey,
                                                  gdata->packet_payload,
                                                  gdata->packet_payload_len,
@@ -659,18 +643,18 @@ SSH_FSM_STEP(signer_sign_hostbased_packet)
  error:
   ssh_xfree(hostkeyfile);
   ssh_xfree(comment);
-  return SSH_FSM_CONTINUE;
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(signer_process_compat_flags)
+SSH_OLDFSM_STEP(signer_process_compat_flags)
 {
-  SSH_FSM_GDATA(SshSigner);
+  SSH_OLDFSM_GDATA(SshSigner);
   size_t len = 0;
   char *compat_flags = NULL;
-  
+
   SSH_PRECOND(gdata->packet_payload);
   SSH_PRECOND(gdata->packet_payload_len);
-  
+
   len = ssh_decode_array(gdata->packet_payload, gdata->packet_payload_len,
                          SSH_FORMAT_UINT32_STR, &compat_flags, NULL,
                          SSH_FORMAT_END);
@@ -682,8 +666,8 @@ SSH_FSM_STEP(signer_process_compat_flags)
       gdata->error_code_to_ssh2 = SIGNER_ERROR_GENERIC;
       gdata->error_message_to_ssh2 =
         ssh_xstrdup("Malformed SSH_AUTH_HOSTBASED_COMPAT packet.");
-      SSH_FSM_SET_NEXT("signer_error");
-      return SSH_FSM_CONTINUE;
+      SSH_OLDFSM_SET_NEXT("signer_error");
+      return SSH_OLDFSM_CONTINUE;
     }
 
   /* Do the actual processing. */
@@ -691,18 +675,20 @@ SSH_FSM_STEP(signer_process_compat_flags)
     char *rest, *current;
 
     rest = compat_flags;
-    
+
     while (strlen(rest) > 0 &&
            (current = ssh_app_param_list_get_next(rest)) != NULL)
       {
         rest += strlen(current);
         if (*rest == ',')
           rest++;
-        
+
         if (strcmp(current, HOSTBASED_REQUESTED_SERVICE_DRAFT_INCOMPAT) == 0)
           gdata->compat->hostbased_requested_service_draft_incompat = TRUE;
         else if (strcmp(current, SIGNATURE_ENCODE_DRAFT_INCOMPAT) == 0)
           gdata->compat->signature_encode_draft_incompat = TRUE;
+        else if (strcmp(current, RSA_HASH_SCHEME_DRAFT_INCOMPAT) == 0)
+          gdata->compat->rsa_hash_scheme_draft_incompat = TRUE;
         /* If more compat-flags are needed, add them here. */
         ssh_xfree(current);
       }
@@ -712,48 +698,50 @@ SSH_FSM_STEP(signer_process_compat_flags)
 
   /* We are now prepared to receive more packets. */
   ssh_packet_wrapper_can_receive(gdata->wrapper, TRUE);
-  
-  return SSH_FSM_SUSPENDED;
+
+  return SSH_OLDFSM_SUSPENDED;
 }
 
-SSH_FSM_STEP(signer_send_signature)
+SSH_OLDFSM_STEP(signer_send_signature)
 {
-  SSH_FSM_GDATA(SshSigner);
+  SSH_OLDFSM_GDATA(SshSigner);
+
+  SSH_OLDFSM_SET_NEXT("signer_suspend");
 
-  SSH_FSM_SET_NEXT("signer_suspend");
-  
   gdata->packet_pending = TRUE;
-  
+
   if (ssh_packet_wrapper_can_send(gdata->wrapper))
     ssh_register_timeout(0L, 0L, signer_can_send, gdata);
-  
-  return SSH_FSM_SUSPENDED;
+
+  return SSH_OLDFSM_SUSPENDED;
 }
 
-SSH_FSM_STEP(signer_finish)
+SSH_OLDFSM_STEP(signer_finish)
 {
-  return SSH_FSM_FINISH;
+  SSH_OLDFSM_GDATA(SshSigner);
+  ssh_oldfsm_destroy(gdata->fsm);
+  return SSH_OLDFSM_FINISH;
 }
 
-SshFSMStateMapItemStruct signer_states[] =
+SshOldFSMStateMapItemStruct signer_states[] =
 {
   { "signer_suspend", "Suspend thread", signer_suspend },
-  
+
   { "signer_process_compat_flags", "Process compatibility flags",
     signer_process_compat_flags },
   { "signer_process_error_packet", "Process received error packet",
-    signer_process_error_packet },  
+    signer_process_error_packet },
   { "signer_process_end", "Process received END packet",
     signer_process_end },
   { "signer_send_error", "Send error packet", signer_send_error },
-  
+
   { "signer_check_hostbased_packet",
     "Check hostbased-authentication packet validity",
     signer_check_hostbased_packet },
   { "signer_sign_hostbased_packet", "Sign the hostbased packet",
     signer_sign_hostbased_packet },
   { "signer_send_signature", "Send the signature to the application",
-    signer_send_signature },  
+    signer_send_signature },
 
   { "signer_finish", "We're ready", signer_finish }
 };
@@ -772,20 +760,21 @@ void signer_received_packet(SshPacketType type,
   ssh_packet_wrapper_can_receive(signer->wrapper, FALSE);
 
   SSH_DEBUG(3, ("Previous state: %d", signer->state));
-  
+
   switch (type)
     {
     case SSH_AUTH_HOSTBASED_PACKET:
       if (signer->state != SIGNER_START &&
-          signer->state != SIGNER_COMPAT_FLAGS_RECEIVED)
+          signer->state != SIGNER_COMPAT_FLAGS_RECEIVED &&
+          signer->state != SIGNER_HOSTBASED_PACKET_RECEIVED)
         {
           /* This must be sent first or after compat flags. */
           /* XXX send error */
           SSH_NOTREACHED;
         }
-        
+
       signer->state = SIGNER_HOSTBASED_PACKET_RECEIVED;
-      ssh_fsm_set_next(signer->main_thread, "signer_check_hostbased_packet");
+      ssh_oldfsm_set_next(signer->main_thread, "signer_check_hostbased_packet");
       break;
     case SSH_AUTH_HOSTBASED_END:
       if (signer->state == SIGNER_DEAD)
@@ -795,7 +784,7 @@ void signer_received_packet(SshPacketType type,
           SSH_NOTREACHED;
         }
       signer->state = SIGNER_DEAD;
-      ssh_fsm_set_next(signer->main_thread, "signer_process_end");
+      ssh_oldfsm_set_next(signer->main_thread, "signer_process_end");
       break;
     case SSH_AUTH_HOSTBASED_ERROR:
       if (signer->state == SIGNER_DEAD)
@@ -805,7 +794,7 @@ void signer_received_packet(SshPacketType type,
           SSH_NOTREACHED;
         }
       signer->state = SIGNER_DEAD;
-      ssh_fsm_set_next(signer->main_thread, "signer_process_error");
+      ssh_oldfsm_set_next(signer->main_thread, "signer_process_error");
       break;
     case SSH_AUTH_HOSTBASED_COMPAT:
       if (signer->state != SIGNER_START)
@@ -815,7 +804,7 @@ void signer_received_packet(SshPacketType type,
           SSH_NOTREACHED;
         }
       signer->state = SIGNER_COMPAT_FLAGS_RECEIVED;
-      ssh_fsm_set_next(signer->main_thread, "signer_process_compat_flags");
+      ssh_oldfsm_set_next(signer->main_thread, "signer_process_compat_flags");
       break;
     default:
       SSH_TRACE(0, ("Invalid packet type %d received from ssh2-client.",
@@ -824,12 +813,12 @@ void signer_received_packet(SshPacketType type,
       SSH_NOTREACHED;
     }
 
-  ssh_fsm_continue(signer->main_thread);
+  ssh_oldfsm_continue(signer->main_thread);
 
   if (signer->packet_payload != NULL)
     ssh_xfree(signer->packet_payload);
 
-  signer->packet_payload = ssh_xmemdup(data, len);  
+  signer->packet_payload = ssh_xmemdup(data, len);
   signer->packet_payload_len = len;
 }
 
@@ -838,16 +827,16 @@ void signer_received_eof(void *context)
   SshSigner signer = (SshSigner) context;
 
   SSH_DEBUG(3, ("Received EOF from packetstream."));
-  
+
   if (signer->state != SIGNER_DEAD)
     {
       ssh_warning("Received EOF from packetstream before transactions were "
                   "complete.");
       signer->state = SIGNER_DEAD;
     }
-  
-  ssh_fsm_set_next(signer->main_thread, "signer_finish");
-  ssh_fsm_continue(signer->main_thread);
+
+  ssh_oldfsm_set_next(signer->main_thread, "signer_finish");
+  ssh_oldfsm_continue(signer->main_thread);
 }
 
 void signer_can_send(void *context)
@@ -858,12 +847,12 @@ void signer_can_send(void *context)
     return;
 
   signer->packet_pending = FALSE;
-  
+
   SSH_PRECOND(signer);
   SSH_PRECOND(signer->packet_type);
   SSH_PRECOND(signer->packet_payload);
   SSH_PRECOND(signer->packet_payload_len);
-    
+
   ssh_packet_wrapper_send(signer->wrapper,
                           signer->packet_type,
                           signer->packet_payload,
@@ -875,8 +864,8 @@ void signer_can_send(void *context)
   signer->packet_payload_len = 0L;
 
   ssh_packet_wrapper_can_receive(signer->wrapper, TRUE);
-  
-  ssh_fsm_continue(signer->main_thread);  
+
+  ssh_oldfsm_continue(signer->main_thread);
 }
 
 
@@ -936,7 +925,7 @@ void signer_thread_destructor(void *tdata)
 void signer_init_context(SshSigner signer)
 {
   memset(signer, 0, sizeof(*signer));
-  
+
   signer->compat = ssh_xcalloc(1, sizeof(*signer->compat));
 }
 
@@ -944,14 +933,20 @@ int main(int argc, char **argv)
 {
   SshStream stdio_stream;
   SshSigner signer;
-  SshFSM fsm;  
+  SshOldFSM fsm;
   char *config_filename;
   char *temp_name;
-  
+
 #ifdef SLEEP_AFTER_STARTUP
   sleep(30);
 #endif /* SLEEP_AFTER_STARTUP */
 
+  {
+    int fd;
+    for (fd=3; fd < 1000; fd++)
+      close(fd);
+  }
+
   /* Get program name (without path). */
   if ((temp_name = strrchr(argv[0], '/')) != NULL)
     progname = ssh_xstrdup(temp_name + 1);
@@ -963,17 +958,24 @@ int main(int argc, char **argv)
      euid. */
   ssh_event_loop_initialize();
 
-  fsm = ssh_fsm_allocate(sizeof(*signer),
+#ifdef SSHDIST_CRYPT_RSA
+  ssh_pk_provider_register(&ssh_pk_if_modn_generator);
+#endif /* SSHDIST_CRYPT_RSA */
+#ifdef SSHDIST_CRYPT_DSA
+  ssh_pk_provider_register(&ssh_pk_dl_modp_generator);
+#endif /* SSHDIST_CRYPT_DSA */
+
+  fsm = ssh_oldfsm_allocate(sizeof(*signer),
                          signer_states,
-                         SSH_FSM_NUM_STATES(signer_states),
+                         SSH_OLDFSM_NUM_STATES(signer_states),
                          signer_destructor);
 
-  signer = ssh_fsm_get_gdata_fsm(fsm);
+  signer = ssh_oldfsm_get_gdata_fsm(fsm);
 
   signer_init_context(signer);
-  
+
   signer->fsm = fsm;
-  
+
 #ifdef SIGNER_QUIET
   signer->quiet = TRUE;
 #else /* SIGNER_QUIET */
@@ -987,7 +989,7 @@ int main(int argc, char **argv)
 
   /* Act as server. */
   signer->serv_config = ssh_server_create_config();
- 
+
   signer->global_config = ssh_client_create_config();
   /* Initialize user context with euid. This is used to dig up the
      hostkey and such. */
@@ -1003,25 +1005,19 @@ int main(int argc, char **argv)
      idea. */
   ssh_dsprintf(&config_filename, "%s/%s",
                SSH_SERVER_DIR, SSH_SERVER_CONFIG_FILE);
-  
+
   if (!ssh_config_read_file(signer->effective_user_data, signer->serv_config,
-                            NULL, config_filename, NULL))
+                            NULL, config_filename))
     ssh_warning("%s: Failed to read config file %s", argv[0],
                 config_filename);
 
   ssh_xfree(config_filename);
 
-  ssh_userfile_init(ssh_user_name(signer->real_user),
-                    ssh_user_uid(signer->real_user),
-                    ssh_user_gid(signer->real_user), NULL, NULL);
-  
   if (!ssh_config_read_file(signer->real_user, signer->global_config,
-                            NULL, SSH_CLIENT_GLOBAL_CONFIG_FILE, NULL))
+                            NULL, SSH_CLIENT_GLOBAL_CONFIG_FILE))
     ssh_warning("%s: Failed to read config file %s", argv[0],
                 SSH_CLIENT_GLOBAL_CONFIG_FILE);
 
-  SSH_TRACE(2, ("public key file: %s", signer->serv_config->public_host_key_file));
-  SSH_TRACE(2, ("private key file: %s", signer->serv_config->host_key_file));
   SSH_TRACE(2, ("randomseed file: %s", signer->serv_config->random_seed_file));
 
   stdio_stream = ssh_stream_fd_wrap2(fileno(stdin), fileno(stdout),
@@ -1033,19 +1029,18 @@ int main(int argc, char **argv)
                                     signer_can_send,
                                     signer);
 
-  signer->main_thread = ssh_fsm_spawn(signer->fsm,
+  signer->main_thread = ssh_oldfsm_spawn(signer->fsm,
                                       0,
                                       "signer_suspend",
                                       NULL,
                                       signer_thread_destructor);
-  ssh_fsm_set_thread_name(signer->main_thread, "main_thread");
-  
+  ssh_oldfsm_set_thread_name(signer->main_thread, "main_thread");
+
   ssh_packet_wrapper_can_receive(signer->wrapper, TRUE);
-  
+
   ssh_event_loop_run();
 
   ssh_event_loop_uninitialize();
-  
+
   return 0;
 }
-
