@@ -17,7 +17,11 @@ ssh2includes.h
 #define SSH2INCLUDES_H
 
 #include "sshincludes.h"
+
+
 #include "sshsessionincludes.h"
+
+
 #include "ssh2version.h"
 
 
@@ -39,6 +43,25 @@ ssh2includes.h
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -64,29 +87,49 @@ ssh2includes.h
 #define SSH_USER_ENV_FILE       "environment"
 #define SSH_SYSTEM_RC           ETCDIR "/sshrc"
 
+#ifndef SSH_SFTP_LOG_FACILITY
+#define SSH_SFTP_LOG_FACILITY "SSH2_SFTP_LOG_FACILITY"
+#endif /* !SSH_SFTP_LOG_FACILITY */
+
 /* Default tcp/udp port */
 #define SSH_DEFAULT_PORT        "22"
 
 /* Definitions for authentication method names. */
-#define SSH_AUTH_PUBKEY       "publickey"
-#define SSH_AUTH_PASSWD       "password"
-#define SSH_AUTH_HOSTBASED    "hostbased"
-#define SSH_AUTH_PAM          "pam-1@ssh.com"
-#define SSH_AUTH_KERBEROS     "kerberos-1@ssh.com"
-#define SSH_AUTH_KERBEROS_TGT "kerberos-tgt-1@ssh.com"
-#define SSH_AUTH_SECURID      "securid-1@ssh.com"
+#define SSH_AUTH_PUBKEY            "publickey"
+#define SSH_AUTH_PASSWD            "password"
+#define SSH_AUTH_HOSTBASED         "hostbased"
+#define SSH_AUTH_PAM               "pam-1@ssh.com"
+#define SSH_AUTH_KERBEROS          "kerberos-2@ssh.com"
+#define SSH_AUTH_KERBEROS_TGT      "kerberos-tgt-2@ssh.com"
+#define SSH_AUTH_SECURID           "securid-1@ssh.com"
+
+
+
 
 /* Path to ssh-signer2 */
-#define SSH_SIGNER_PATH       "ssh-signer2"
+#define SSH_SIGNER_PATH       SSH_BINDIR "/ssh-signer2"
 
 /* Path to ssh-pam-client */
-#define SSH_PAM_CLIENT_PATH   "ssh-pam-client"
+#define SSH_PAM_CLIENT_PATH   SSH_BINDIR "/ssh-pam-client"
 
-/* Default idle timeout in minutes for the server. 0 = disabled */
+/* Path to ssh2 */
+#define SSH_SSH2_PATH         SSH_BINDIR "/ssh2"
+
+/* Path to ssh-askpass2 */
+#define SSH_ASKPASS_PATH      SSH_BINDIR "/ssh-askpass2"
+
+/* Path to libsocks5.conf */
+#define SSH_LIBSOCKS5_CONF_PATH    "/etc/libsocks5.conf"
+
+/* Default idle timeout in seconds for the server. 0 = disabled */
 #define SSH_SERVER_DEFAULT_IDLE_TIMEOUT   0
 
 /* arguments to ssh2 */
-#define SSH2_GETOPT_ARGUMENTS "ac:C::vd:e:f::F:hi:l:L:no:p:PqR:s:Stx8gVkw1:m:"
+
+
+
+#define SSH2_GETOPT_ARGUMENTS "ac:C::vd:e:f::F:hi:l:L:no:p:PqR:s:Stx8gVkw1:m:64"
+
 
 /* protocol version numbers */
 
@@ -116,6 +159,17 @@ ssh2includes.h
 
 
 
+
+
+
+
+
+
+
+
+
+
+
 
 
 
