@@ -17,10 +17,11 @@
 
 #ifdef WITH_PGP
 
-/* Search pgp keyring with given user credentials.  If key is find, convert
-   it to ssh2 public key blob and return it to caller in blob pointer.
-   Blob must be freed by the caller with ssh_xfree.  If return value is
-   FALSE, key was not found and blob pointer is not altered. */
+/* Search pgp keyring with given user credentials.  If key is found,
+   convert it to ssh2 public key blob and return it to caller in blob
+   pointer.  Blob must be freed by the caller with ssh_xfree.  If
+   return value is FALSE, key was not found and blob pointer is not
+   altered. */
 
 Boolean ssh2_find_pgp_public_key_with_fingerprint(SshUser uc,
                                                   const char *fn,
