@@ -3,9 +3,9 @@
   ssh2pgppub.c
 
   Authors:
-        Timo J. Rinne <tri@ssh.fi>
+        Timo J. Rinne <tri@ssh.com>
 
-  Copyright (C) 1999 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1999-2000 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
                   
   Retrieve ssh2 keyblobs from pgp keyring.
@@ -22,6 +22,7 @@
 #include "sshfilebuffer.h"
 #include "sshuserfilebuf.h"
 #include "ssh2pubkeyencode.h"
+#include "sshconfig.h"
 #include "ssh2pgp.h"
 
 #define SSH_DEBUG_MODULE "Ssh2PgpPublic"
@@ -51,7 +52,7 @@ Boolean ssh2_find_pgp_public_key_internal(SshUser uc,
 {
   struct stat st;
   SshUserFile uf;
-  SshFileBuffer fb;
+  SshFileBufferStruct fb;
   SshPgpPacket pp;
   SshPgpPublicKey pgpkey;
   Boolean found;
@@ -133,6 +134,8 @@ Boolean ssh2_find_pgp_public_key_internal(SshUser uc,
   ssh_pgp_public_key_free(pgpkey);
   if (comment)
     *comment = key_comment;
+  else
+    ssh_xfree(key_comment);
   return TRUE;
 }
 
