@@ -53,7 +53,7 @@ Boolean ssh2_find_pgp_secret_key_internal(SshUser uc,
 {
   struct stat st;
   SshUserFile uf;
-  SshFileBuffer fb;
+  SshFileBufferStruct fb;
   SshPgpPacket pp;
   Boolean found;
   char *key_comment = NULL;
