@@ -31,12 +31,15 @@ sshagentint.h
 #define SSH_AGENT_LOCK                  208
 #define SSH_AGENT_UNLOCK                209
 #define SSH_AGENT_PING                  212
+#define SSH_AGENT_RANDOM                213
+
 /* Responses from the agent. */
 #define SSH_AGENT_SUCCESS               101
 #define SSH_AGENT_FAILURE               102
 #define SSH_AGENT_VERSION_RESPONSE      103
 #define SSH_AGENT_KEY_LIST              104
 #define SSH_AGENT_OPERATION_COMPLETE    105
+#define SSH_AGENT_RANDOM_DATA           106
 #define SSH_AGENT_ALIVE                 150
 
 /* SSH1 agent request codes. */
