@@ -23,7 +23,6 @@ sshchssh1agent.c
 #include "sshtcp.h"
 #include "sshencode.h"
 #include "sshmsgs.h"
-#include "sshconn.h"
 #include "sshcommon.h"
 #include "sshagentint.h"
 
@@ -169,12 +168,21 @@ void ssh_channel_ssh1_agent_connection(SshStream stream, void *context)
 
   /* Send a channel open message.  If this fails, the stream will be
      automatically closed. */
+
+
+
+
+
+
+
+
   ssh_conn_send_channel_open(common->conn, "auth-ssh1-agent",
                              stream, TRUE, TRUE,
                              AGENT_WINDOW_SIZE, AGENT_PACKET_SIZE,
                              NULL, 0, NULL,
                              ssh_channel_ssh1_agent_connection_destroy,
                              (void *)common, NULL, NULL);
+
 }
 
 /***********************************************************************
@@ -186,10 +194,62 @@ typedef struct SshAgentConnectionRec
 {
   SshCommon common;
   int channel_id;
+
+
+
   SshConnOpenCompletionProc completion;
+
   void *context;
 } *SshAgentConnection;
 
+/* This filter function will insert a SSH_AGENT_FORWARDING_NOTICE
+   message into the channel to the agent.  */
+
+void
+ssh_agent_ssh1_forwarding_notice_insert_filter(void *context,
+                                               SshFilterGetCB get_data,
+                                               SshFilterCompletionCB completed,
+                                               void *internal_context)
+{
+  SshBuffer msg_buffer = (SshBuffer)context; /* Message is given in this buffer */
+  SshBuffer data_buf;
+  size_t offset, orig_data_len;
+  Boolean eof_received;
+  unsigned char *append_ptr;
+
+  if (ssh_buffer_len(msg_buffer) == 0)
+    {
+      (*completed)(internal_context, SSH_FILTER_SHORTCIRCUIT);
+      return;
+    }
+
+  (*get_data)(internal_context, &data_buf, &offset, &eof_received);
+
+  /* Make room for the message to be inserted */
+  orig_data_len = ssh_buffer_len(data_buf);
+  ssh_xbuffer_append_space(data_buf, &append_ptr, ssh_buffer_len(msg_buffer));
+  memmove(ssh_buffer_ptr(data_buf) + ssh_buffer_len(msg_buffer),
+          ssh_buffer_ptr(data_buf), orig_data_len);
+
+  /* Insert the message */
+  memcpy(ssh_buffer_ptr(data_buf), ssh_buffer_ptr(msg_buffer),
+         ssh_buffer_len(msg_buffer));
+
+  /* Empty the msg_buffer so that we can shortcircuit the filter on the
+     next call. */
+  ssh_buffer_consume(msg_buffer, ssh_buffer_len(msg_buffer));
+
+  /* Pass the whole thing */
+  (*completed)(internal_context, SSH_FILTER_ACCEPT(ssh_buffer_len(data_buf)));
+}
+
+void
+ssh_agent_ssh1_forwarding_filter_destroy(void *context)
+{
+  ssh_buffer_free((SshBuffer)context);
+}
+
+
 /* This function is called when a connecting to a real agent has been
    completed (possibly with error). */
 
@@ -197,7 +257,8 @@ void ssh_channel_open_ssh1_agent_connected(SshStream stream,
                                           void *context)
 {
   SshAgentConnection a = (SshAgentConnection)context;
-  SshBufferStruct buffer;
+  SshStream filter_stream;
+  SshBuffer buffer;
   unsigned char *cp;
 
   if (stream == NULL)
@@ -217,31 +278,29 @@ void ssh_channel_open_ssh1_agent_connected(SshStream stream,
 
   if (a->common->config->ssh_agent_compat == SSH_AGENT_COMPAT_SSH2)
     {
-      /* We are required to send a FORWARDING_NOTIFY to the agent to inform it
+      /* We are required to send a FORWARDING_NOTICE to the agent to inform it
          that the connection is actually forwarded.  Format that packet now. */
-      ssh_buffer_init(&buffer);
-      ssh_encode_buffer(&buffer,
+      buffer = ssh_xbuffer_allocate();
+      ssh_encode_buffer(buffer,
                         SSH_FORMAT_DATA, "1234", (size_t)4,
                         SSH_FORMAT_CHAR,
                         (unsigned int) SSH_AGENT_FORWARDING_NOTICE,
                         SSH_FORMAT_UINT32_STR,
-                        a->common->server_host_name,
-                        strlen(a->common->server_host_name),
+                          a->common->server_host_name,
+                          strlen(a->common->server_host_name),
                         SSH_FORMAT_UINT32_STR,
-                        a->common->remote_ip, strlen(a->common->remote_ip),
+                          a->common->remote_ip, strlen(a->common->remote_ip),
                         SSH_FORMAT_UINT32,
                         (SshUInt32) atol(a->common->remote_port),
                         SSH_FORMAT_END);
-      cp = ssh_buffer_ptr(&buffer);
-      SSH_PUT_32BIT(cp, ssh_buffer_len(&buffer) - 4);
+      cp = ssh_buffer_ptr(buffer);
+      SSH_PUT_32BIT(cp, ssh_buffer_len(buffer) - 4);
 
-      /* Write the buffer to the channel.  This is a kludge; this assumes that
-         we can always write this much to the internal buffers. */
-      if (ssh_stream_write(stream, ssh_buffer_ptr(&buffer),
-                           ssh_buffer_len(&buffer)) !=
-          ssh_buffer_len(&buffer))
-        ssh_fatal("ssh_channel_open_agent_connected: kludge failed");
-      ssh_buffer_uninit(&buffer);
+      /* Create a filter stream for inserting the message into the channel. */
+      filter_stream =
+        ssh_stream_filter_create(stream, 10240,
+                                 ssh_agent_ssh1_forwarding_notice_insert_filter,
+                                 NULL, ssh_agent_ssh1_forwarding_filter_destroy, buffer);
     }
 
   /* Create the channel. */
@@ -260,9 +319,13 @@ void ssh_channel_open_ssh1_agent_connected(SshStream stream,
 /* Processes an open request for an agent channel. */
 
 void ssh_channel_ssh1_agent_open(const char *type, int channel_id,
-                                const unsigned char *data, size_t len,
-                                SshConnOpenCompletionProc completion,
-                                void *completion_context, void *context)
+                                 const unsigned char *data, size_t len,
+
+
+
+                                 SshConnOpenCompletionProc completion,
+
+                                 void *completion_context, void *context)
 {
   SshCommon common = (SshCommon)context;
   SshChannelTypeSsh1Agent ct;
@@ -315,6 +378,10 @@ void ssh_channel_ssh1_agent_open(const char *type, int channel_id,
 
 
 
+
+
+
+
 }
 
 /***********************************************************************
@@ -373,8 +440,14 @@ void ssh_channel_ssh1_agent_send_request(SshCommon common,
 
   ct = ssh_channel_ssh1_agent_ct(common);
   
+
+
+
+
+
   ssh_conn_send_channel_request(common->conn, session_channel_id,
                                 "auth-ssh1-agent-req", NULL, 0, NULL, NULL);
+
   ct->agent_requested = TRUE;
 }
 
