@@ -19,8 +19,6 @@ sshchx11.c
 #include "sshfilterstream.h"
 #include "sshtcp.h"
 #include "sshencode.h"
-#include "sshtrans.h"
-#include "sshconn.h"
 #include "sshmsgs.h"
 #include "sshlocalstream.h"
 #include "sshcommon.h"
@@ -33,7 +31,7 @@ sshchx11.c
 #include "sshchx11.h"
 
 /* These headers are needed for Unix domain sockets. */
-#if defined(XAUTH_PATH) || defined(HPSUX_NONSTANDARD_X11_KLUDGE)
+#if defined(XAUTH_PATH) || defined(HPUX_NONSTANDARD_X11_KLUDGE)
 #include <sys/socket.h>
 #include <netinet/in.h>
 #ifdef HAVE_NETINET_IN_SYSTM_H
@@ -53,10 +51,12 @@ sshchx11.c
 #ifdef HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
 #endif /* HAVE_SYS_UTSNAME_H */
-#endif /* XAUTH_PATH || HPSUX_NONSTANDARD_X11_KLUDGE */
+#endif /* XAUTH_PATH || HPUX_NONSTANDARD_X11_KLUDGE */
 
 
+#ifndef VXWORKS
 #include <syslog.h>
+#endif /* VXWORKS */
 
 
 #ifdef NEED_SYS_SYSLOG_H
@@ -246,15 +246,16 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
     {
 #ifdef XAUTH_PATH
       /* Try to get Xauthority information for the display. */
-      snprintf(line, sizeof(line), "%s list %s 2>/dev/null",
-               XAUTH_PATH, getenv("DISPLAY"));
+      ssh_snprintf(line, sizeof(line), "%s list %s 2>/dev/null",
+                   XAUTH_PATH, getenv("DISPLAY"));
 
       /* XXX async, please! */
       f = popen(line, "r");
       if (f)
         {     
           while(!(cp = fgets(line, sizeof(line), f)) && errno == EINTR)
-            ;
+            errno = 0; /* For broken Solaris fgets(). */
+          
           if (cp && sscanf(line, "%*s %s %s", proto, data) == 2)
             got_data = TRUE;
         }
@@ -264,7 +265,9 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
       if (f)
         pclose(f);
 #else /* XAUTH_PATH */
+
       ssh_debug("No xauth program was found at configure time.");
+
 #endif /* XAUTH_PATH */
       /* If we didn't get authentication data, just make up some data.  The
          forwarding code will check the validity of the response anyway, and
@@ -275,8 +278,8 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
         {
           strcpy(proto, "MIT-MAGIC-COOKIE-1");
           for (i = 0; i < 16; i++)
-            snprintf(data + 2 * i, 3, "%02x",
-                     ssh_random_get_byte());
+            ssh_snprintf(data + 2 * i, 3, "%02x",
+                         ssh_random_get_byte());
         }
 
       /* Save protocol name. */
@@ -313,8 +316,8 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
   /* Convert the fake data into hex for transmission. */
   new_data = ssh_xmalloc(2 * ct->x11_fake_cookie_len + 1);
   for (i = 0; i < ct->x11_fake_cookie_len; i++)
-    snprintf(new_data + 2 * i, 3, "%02x",
-             (unsigned char)ct->x11_fake_cookie[i]);
+    ssh_snprintf(new_data + 2 * i, 3, "%02x",
+                 (unsigned char)ct->x11_fake_cookie[i]);
 
   ssh_buffer_init(&buffer);
   ssh_encode_buffer(&buffer,
@@ -325,9 +328,15 @@ void ssh_channel_x11_send_request(SshCommon common, int session_channel_id)
                     SSH_FORMAT_UINT32, screen_number,
                     SSH_FORMAT_END);
   /* XXX We want reply. (set callback and context to non-NULL. */
+
+
+
+
+
   ssh_conn_send_channel_request(common->conn, session_channel_id,
                                 "x11-req", ssh_buffer_ptr(&buffer),
                                 ssh_buffer_len(&buffer), NULL, NULL);
+
   ssh_buffer_uninit(&buffer);
   ssh_xfree(new_data);
 
@@ -385,7 +394,7 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
   
   /* Format the initiator string for describing where the connection
      came from. */
-  snprintf(msg, sizeof(msg), "X11 connection from %s:%s", ip, port);
+  ssh_snprintf(msg, sizeof(msg), "X11 connection from %s:%s", ip, port);
 
   /* Increase the number of open channels.  This will be decremented in
      the destroy function. */
@@ -400,7 +409,8 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
 
     if (old_handler == SIG_ERR)
       {
-        ssh_warning("ssh_channel_x11_connection: Could not set SIGCHLD signal handler.");
+        ssh_warning("ssh_channel_x11_connection: Could not set SIGCHLD "
+                    "signal handler.");
         old_handler = SIG_IGN;
       }
 
@@ -425,7 +435,8 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
 
     if (signal(SIGCHLD, old_handler) == SIG_ERR && old_handler != SIG_IGN)
       {
-        ssh_warning("ssh_channel_x11_connection: Could not reset SIGCHLD signal handler.");
+        ssh_warning("ssh_channel_x11_connection: Could not reset SIGCHLD "
+                    "signal handler.");
       }
   }
 #endif /* HAVE_LIBWRAP */
@@ -437,10 +448,9 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
       remote_host_name = "UNKNOWN";
     
     ssh_log_event(session->common->config->log_facility, SSH_LOG_INFORMATIONAL,
-                  "%s (%s)", msg,
-                  remote_host_name);
+                  "%s (%s)", msg, remote_host_name);
     if (session->common->client)
-      SSH_TRACE(0, ("%s (%s)", msg, remote_host_name));
+      ssh_debug("%s (%s)", msg, remote_host_name);
         
   }
   
@@ -450,7 +460,13 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
   /* Send a channel open request to the other side. */
   ssh_buffer_init(&buffer);
 
-  if (!(*session->common->compat_flags->x11_channel_open_draft_incompatibility))
+  if (
+
+
+
+      !(*session->common->compat_flags->x11_channel_open_draft_incompatibility)
+
+      )
     ssh_encode_buffer(&buffer,
                       SSH_FORMAT_UINT32_STR, ip, strlen(ip),
                       SSH_FORMAT_UINT32, port_number,
@@ -460,12 +476,23 @@ void ssh_channel_x11_connection(SshIpError error, SshStream stream,
                       SSH_FORMAT_UINT32_STR, msg, strlen(msg),
                       SSH_FORMAT_END);
   
+
+
+
+
+
+
+
+
+
+
   ssh_conn_send_channel_open(session->common->conn, "x11", stream, TRUE, TRUE,
                              X11_WINDOW_SIZE, X11_PACKET_SIZE,
                              ssh_buffer_ptr(&buffer), ssh_buffer_len(&buffer),
                              NULL,
                              ssh_channel_x11_connection_destroy,
                              (void *)session->common, NULL, NULL);
+
   ssh_buffer_uninit(&buffer);
 }
 
@@ -520,7 +547,7 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
                        SSH_FORMAT_UINT32_STR,
                          &session->auth_cookie, NULL,
                        SSH_FORMAT_UINT32, &temp,
-                       SSH_FORMAT_END) != len)
+                       SSH_FORMAT_END) != len || len == 0)
     {
       SSH_DEBUG(0, ("bad data"));
       return FALSE;
@@ -545,11 +572,11 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
        display_number++)
     {
       /* Format a port number for the display. */
-      snprintf(buf, sizeof(buf), "%d", 6000 + display_number);
+      ssh_snprintf(buf, sizeof(buf), "%d", 6000 + display_number);
 
       /* Open a TCP/IP socket for listening the port. */
       session->x11_listener =
-        ssh_tcp_make_listener("0.0.0.0", buf,
+        ssh_tcp_make_listener(SSH_IPADDR_ANY_IPV4, buf,
                               NULL, ssh_channel_x11_connection,
                               (void *)session);
 
@@ -566,8 +593,8 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
     }
 
   /* Set up a suitable value for the DISPLAY variable. */
-#ifdef HPSUX_NONSTANDARD_X11_KLUDGE
-  /* HPSUX has some special shared memory stuff in their X server, which
+#ifdef HPUX_NONSTANDARD_X11_KLUDGE
+  /* HPUX has some special shared memory stuff in their X server, which
      appears to be enabled if the host name matches that of the local machine.
      However, it can be circumvented by using the IP address of the local
      machine instead.  */
@@ -585,17 +612,17 @@ Boolean ssh_channel_x11_process_request(void *session_placeholder,
         return FALSE;
       }
     memcpy(&addr, hp->h_addr_list[0], sizeof(addr));
-    snprintf(buf, sizeof(buf), "%s:%d.%d",
-             inet_ntoa(addr), display_number, session->screen_number);
+    ssh_snprintf(buf, sizeof(buf), "%s:%d.%d",
+                 inet_ntoa(addr), display_number, session->screen_number);
   }
-#else /* HPSUX_NONSTANDARD_X11_KLUDGE */
+#else /* HPUX_NONSTANDARD_X11_KLUDGE */
   /* Get the name of the local host. */
   ssh_tcp_get_host_name(hostname, sizeof(hostname));
 
   /* Format the DISPLAY value. */
-  snprintf(buf, sizeof(buf), "%s:%d.%lu",
-           hostname, display_number, session->screen_number);
-#endif /* HPSUX_NONSTANDARD_X11_KLUDGE */
+  ssh_snprintf(buf, sizeof(buf), "%s:%d.%lu",
+               hostname, display_number, session->screen_number);
+#endif /* HPUX_NONSTANDARD_X11_KLUDGE */
 
   /* Save the display name. */
   session->display = ssh_xstrdup(buf);
@@ -617,7 +644,11 @@ typedef struct SshX11ConnectionRec
   SshCommon common;
   SshChannelTypeX11 ct;
   int channel_id;
+
+
+
   SshConnOpenCompletionProc completion;
+
   void *context;
   char *originator_ip;
   SshUInt32 originator_port;
@@ -641,15 +672,20 @@ void ssh_channel_x11_x_destroy(SshX11Connection x)
    been entirely received, at which point this will replace the authentication
    cookie in the packet by the real cookie, and shortcircuit the filter. */
 
-int ssh_channel_x11_filter(SshBuffer data,
-                           size_t offset,
-                           Boolean eof_received,
-                           void *context)
+void ssh_channel_x11_filter(void *context,
+                            SshFilterGetCB get_data,
+                            SshFilterCompletionCB completed,
+                            void *internal_context)
 {
   SshX11Connection x = (SshX11Connection)context;
   unsigned char *ucp;
   size_t received_len, data_len, proto_len, desired_len;
+  SshBuffer data;
+  size_t offset;
+  Boolean eof_received;
 
+  get_data(internal_context, &data, &offset, &eof_received);
+  
   SSH_DEBUG(6, ("filter: data len %d, offset %d",
                 (int)ssh_buffer_len(data), (int)offset));
 
@@ -663,9 +699,15 @@ int ssh_channel_x11_filter(SshBuffer data,
     {
       /* Not yet...  Keep receiving, unless we've got EOF. */
       if (eof_received)
-        return SSH_FILTER_DISCONNECT; /* Abort the connection. */
+        {
+          completed(internal_context, SSH_FILTER_DISCONNECT); /* Abort the connection. */
+          return;
+        }
       else
-        return SSH_FILTER_HOLD; /* Wait for more data. */
+        {
+          completed(internal_context, SSH_FILTER_HOLD); /* Wait for more data. */
+          return;
+        }
     }
 
   /* Parse the lengths of variable-length fields. */
@@ -684,7 +726,8 @@ int ssh_channel_x11_filter(SshBuffer data,
       {
         ssh_warning("Initial X11 packet contains bad byte order byte: 0x%x",
                     ucp[0]);
-        return SSH_FILTER_DISCONNECT;
+        completed(internal_context, SSH_FILTER_DISCONNECT);
+        return;
       }
 
   /* Compute the length of the packet we must receive. */
@@ -692,15 +735,24 @@ int ssh_channel_x11_filter(SshBuffer data,
 
   /* Sanity check: it must not be larger than we are willing to receive. */
   if (desired_len > X11_MAX_AUTH_PACKET_SIZE)
-    return SSH_FILTER_DISCONNECT;
+    {
+      completed(internal_context, SSH_FILTER_DISCONNECT);
+      return;
+    }
 
   /* Check if the whole packet is in buffer. */
   if (received_len < desired_len)
     {
       if (eof_received)
-        return SSH_FILTER_DISCONNECT; /* Abort the connection. */
+        {
+          completed(internal_context, SSH_FILTER_DISCONNECT); /* Abort the connection. */
+          return;
+        }
       else
-        return SSH_FILTER_HOLD;
+        {
+          completed(internal_context, SSH_FILTER_HOLD); /* Wait for more data. */
+          return;
+        }
     }
 
   /* Check if authentication protocol matches. */
@@ -712,7 +764,8 @@ int ssh_channel_x11_filter(SshBuffer data,
       ssh_warning("X11 connection requests different authentication protocol: '%.*s' vs. '%.*s'.",
                   x->ct->x11_fake_proto_len, x->ct->x11_fake_proto,
                   proto_len, (const char *)(ucp + 12));
-      return SSH_FILTER_DISCONNECT;
+      completed(internal_context, SSH_FILTER_DISCONNECT);
+      return;
     }
 
   /* Check if authentication data matches our fake data. */
@@ -721,7 +774,8 @@ int ssh_channel_x11_filter(SshBuffer data,
              x->ct->x11_fake_cookie, x->ct->x11_fake_cookie_len) != 0)
     {
       ssh_warning("X11 auth data does not match fake data.");
-      return SSH_FILTER_DISCONNECT;
+      completed(internal_context, SSH_FILTER_DISCONNECT);
+      return;
     }
 
   /* Received authentication protocol and data match our fake data.
@@ -731,7 +785,7 @@ int ssh_channel_x11_filter(SshBuffer data,
          x->ct->x11_real_cookie, x->ct->x11_real_cookie_len);
 
   /* Otherwise, we accept and shortcircuit any further communications. */
-  return SSH_FILTER_SHORTCIRCUIT;
+  completed(internal_context, SSH_FILTER_SHORTCIRCUIT);
 }
 
 /* This is called when the filter stream is being destroyed.  We free the
@@ -829,7 +883,11 @@ void ssh_channel_x11_open_connected_tcp(SshIpError error,
 
 void ssh_channel_x11_open(const char *type, int channel_id,
                           const unsigned char *data, size_t len,
+
+
+
                           SshConnOpenCompletionProc completion,
+
                           void *completion_context, void *context)
 {
   SshCommon common = (SshCommon)context;
@@ -845,12 +903,18 @@ void ssh_channel_x11_open(const char *type, int channel_id,
 
   ct = (SshChannelTypeX11)ssh_common_get_channel_type_context(common, "x11");
 
-  if (!(*common->compat_flags->x11_channel_open_draft_incompatibility))
+  if (
+
+
+
+      !(*common->compat_flags->x11_channel_open_draft_incompatibility)
+
+      )
     {
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32_STR, &originator_ip, NULL,
                            SSH_FORMAT_UINT32, &originator_port,
-                           SSH_FORMAT_END) != len)
+                           SSH_FORMAT_END) != len || len == 0)
         {
           SSH_DEBUG(0, ("x11_open: bad data"));
           goto fail;
@@ -861,7 +925,7 @@ void ssh_channel_x11_open(const char *type, int channel_id,
       /* Remote end has x11 channel open draft inconsistence bug. */
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32_STR, &originator_ip, NULL,
-                           SSH_FORMAT_END) != len)
+                           SSH_FORMAT_END) != len || len == 0)
         {
           SSH_DEBUG(0, ("x11_open: bad data"));
           goto fail;
@@ -871,8 +935,15 @@ void ssh_channel_x11_open(const char *type, int channel_id,
   if (!ct->x11_requested)
     {
       ssh_warning("Received X11 connection without requesting forwarding!");
-      if (!(*common->compat_flags->x11_channel_open_draft_incompatibility))
-        ssh_warning("Originator IP: '%s', port: '%s'", originator_ip, originator_port);
+      if (
+
+
+
+          !(*common->compat_flags->x11_channel_open_draft_incompatibility)
+
+          )
+        ssh_warning("Originator IP: '%s', port: '%s'", originator_ip,
+                    originator_port);
       else
         ssh_warning("Originator message: '%s'", originator_ip);
       ssh_xfree(originator_ip);
@@ -887,7 +958,13 @@ void ssh_channel_x11_open(const char *type, int channel_id,
   x->context = completion_context;
   x->originator_ip = originator_ip;
   x->originator_port = originator_port;
-  if (!(*common->compat_flags->x11_channel_open_draft_incompatibility))
+  if (
+
+
+
+      !(*common->compat_flags->x11_channel_open_draft_incompatibility)
+
+      )
     ssh_dsprintf(&x->originator_string, "%s:%ld", originator_ip, originator_port);
   else
     x->originator_string = ssh_xstrdup(originator_ip);
@@ -915,9 +992,9 @@ void ssh_channel_x11_open(const char *type, int channel_id,
         }
 
       /* Determine the name to use for the socket. */
-#ifdef HPSUX_NONSTANDARD_X11_KLUDGE
+#ifdef HPUX_NONSTANDARD_X11_KLUDGE
       {
-        /* HPSUX release 10.X uses /var/spool/sockets/X11/0 for the
+        /* HPUX release 10.X uses /var/spool/sockets/X11/0 for the
            unix-domain sockets, while earlier releases stores the
            socket in /usr/spool/sockets/X11/0 with soft-link from
            /tmp/.X11-unix/`uname -n`0 */
@@ -926,32 +1003,32 @@ void ssh_channel_x11_open(const char *type, int channel_id,
 
         if (stat("/var/spool/sockets/X11", &st) == 0)
           {
-            snprintf(buf, sizeof(buf), "%s/%d",
-                     "/var/spool/sockets/X11", display_number);
+            ssh_snprintf(buf, sizeof(buf), "%s/%d",
+                         "/var/spool/sockets/X11", display_number);
           }
         else
           {
             if (stat("/usr/spool/sockets/X11", &st) == 0)
               {
-                snprintf(buf, sizeof(buf), "%s/%d",
-                         "/usr/spool/sockets/X11", display_number);
+                ssh_snprintf(buf, sizeof(buf), "%s/%d",
+                             "/usr/spool/sockets/X11", display_number);
               }
             else
               {
                 struct utsname utsbuf;
-                /* HPSUX stores unix-domain sockets in
+                /* HPUX stores unix-domain sockets in
                    /tmp/.X11-unix/`hostname`0
                    instead of the normal /tmp/.X11-unix/X0. */
                 if (uname(&utsbuf) < 0)
                   ssh_fatal("uname: %.100s", strerror(errno));
-                snprintf(buf, sizeof(buf), "%.20s/%.64s%d",
-                         X11_DIR, utsbuf.nodename, display_number);
+                ssh_snprintf(buf, sizeof(buf), "%.20s/%.64s%d",
+                             X11_DIR, utsbuf.nodename, display_number);
               }
           }
       }
-#else /* HPSUX_NONSTANDARD_X11_KLUDGE */
-      snprintf(buf, sizeof(buf), "%.80s/X%d", X11_DIR, display_number);
-#endif /* HPSUX_NONSTANDARD_X11_KLUDGE */
+#else /* HPUX_NONSTANDARD_X11_KLUDGE */
+      ssh_snprintf(buf, sizeof(buf), "%.80s/X%d", X11_DIR, display_number);
+#endif /* HPUX_NONSTANDARD_X11_KLUDGE */
 
       /* Connect to the X11 socket. */
       ssh_local_connect(buf, ssh_channel_x11_open_connected, (void *)x);
@@ -987,7 +1064,7 @@ void ssh_channel_x11_open(const char *type, int channel_id,
     }
 
   /* Host name is now in ``buf''.  Format port number in ``port''. */
-  snprintf(port, sizeof(port), "%d", 6000 + display_number);
+  ssh_snprintf(port, sizeof(port), "%d", 6000 + display_number);
 
   ssh_tcp_connect(buf, port, NULL,
                   ssh_channel_x11_open_connected_tcp,
