@@ -58,10 +58,18 @@
 
 struct {
   const char *name;
+
+
+
   SshConnChannelOpenProc open_proc;
+
   SshChannelTypeCreateProc type_create_proc;
   SshChannelTypeDestroyProc type_destroy_proc;
+
+
+
   SshConnGlobalRequestStruct global_requests[10];
+
 } ssh_channel_types[] =
 {
 
@@ -108,14 +116,24 @@ struct {
   { "forwarded-tcpip", ssh_channel_ftcp_open_request, ssh_channel_ftcp_create,
     ssh_channel_ftcp_destroy },
     /* Client doesn't accept forwards from the server. Hence the NULL below.*/
-    { "direct-tcpip", NULL, ssh_channel_dtcp_create,
-      ssh_channel_dtcp_destroy },
+  { "direct-tcpip", NULL, ssh_channel_dtcp_create,
+    ssh_channel_dtcp_destroy },
 #endif /* !SSH_WIN_CLIENT */
 #endif /* SSH_CHANNEL_TCPFWD */
 
   { NULL },
 };
 
+static void destroy_forward_rec(void *obj, void *context)
+{
+  SshTcpForwardActive fwd = (SshTcpForwardActive)obj;
+  ssh_xfree(fwd->connect_from_host);
+  ssh_xfree(fwd->port);
+  ssh_xfree(fwd->connect_to_host);
+  ssh_xfree(fwd->connect_to_port);
+  ssh_xfree(fwd);
+}
+
 /* Timeout to destroy the common structure from the bottom of the event
    loop. */
 
@@ -136,21 +154,6 @@ void ssh_common_disconnect(int reason, Boolean locally_generated,
 
   SSH_DEBUG(2, ("DISCONNECT received: %s", msg));
 
-#if 0
-  /* When used with e.g. SCP, when the local scp closes the channel,
-     the connection protocol sends a CLOSE message to the server, which
-     responds immediately.  The server will then exit immediately, because all
-     channels have been closed.  Thus, the client will receive the server's CLOSE
-     response immediately followed by EOF from the connection.  The client will
-     get to process the EOF condition before it has destroyed its own
-     protocol modules, which may generate spurious Connection Lost messages
-     to the user.  This eliminates such messages when all channels
-     have been closed. */
-  if (common->client == TRUE && reason == SSH_DISCONNECT_CONNECTION_LOST &&
-      common->num_channels == 0)
-    return;
-#endif  
-  
   /* Log the disconnect in the system log. */
   ssh_log_event(common->config->log_facility, SSH_LOG_INFORMATIONAL,
                 "%s disconnected: %s",
@@ -170,6 +173,7 @@ void ssh_common_disconnect(int reason, Boolean locally_generated,
   (*common->disconnect)(reason, locally_generated, msg, common->context);
 }
 
+
 /* Processes a debug message received from the connection protocol. */
 
 void ssh_common_debug(int type, const char *msg, void *context)
@@ -183,8 +187,13 @@ void ssh_common_debug(int type, const char *msg, void *context)
     (*common->debug)(type, msg, common->context);
 }
 
+
 void ssh_common_request_rekey(SshCommon common)
 {
+
+
+
+
   SSH_PRECOND(common);
   SSH_PRECOND(common->chosen_host_key_alg);
   SSH_PRECOND(common->chosen_cipher_c_to_s);
@@ -233,6 +242,7 @@ void ssh_common_request_rekey(SshCommon common)
                        common->chosen_host_key_alg,
                        strlen(common->chosen_host_key_alg),
                        SSH_FORMAT_END);
+
 }
 
 void ssh_common_rekey_timeout_callback(void *context)
@@ -260,6 +270,8 @@ void ssh_common_rekey_notify_cb(void *context)
   common->doing_rekey = TRUE;
 }
                                 
+
+
 /* Processes a special packet received from the connection protocol. */
 
 void ssh_common_special(SshCrossPacketType type, const unsigned char *data,
@@ -359,8 +371,8 @@ void ssh_common_special(SshCrossPacketType type, const unsigned char *data,
         {
           /* Register rekey timeout. */
           if (common->config->rekey_interval_seconds > 0L)
-            ssh_register_timeout(common->config->rekey_interval_seconds, 0L,
-                                 ssh_common_rekey_timeout_callback,
+            ssh_register_timeout(common->config->rekey_interval_seconds,
+                                 0L, ssh_common_rekey_timeout_callback,
                                  common);
         }
       
@@ -373,6 +385,7 @@ void ssh_common_special(SshCrossPacketType type, const unsigned char *data,
     }
 }
 
+
 void ssh_common_finalize(SshIpError error,
                          const char *result,
                          void *context)
@@ -386,17 +399,29 @@ void ssh_common_finalize(SshIpError error,
     }
   else
     {
-      SSH_DEBUG(2, ("DNS lookup failed. Using host-IP-number (\"%s\") " \
-                    "as hostname.", common->remote_ip));
-      ssh_log_event(common->config->log_facility,
-                    SSH_LOG_WARNING,
-                    "DNS lookup failed for \"%s\".",
-                    common->remote_ip);
-      
+      if (common->config->try_reverse_mapping)
+        {
+          SSH_DEBUG(2, ("DNS lookup failed. Using host-IP-number (\"%s\") " \
+                        "as hostname.", common->remote_ip));
+          ssh_log_event(common->config->log_facility,
+                        SSH_LOG_WARNING,
+                        "DNS lookup failed for \"%s\".",
+                        common->remote_ip);
+        }
+      else
+        {
+          SSH_DEBUG(2, ("DNS lookup disabled in the configuration"));
+        }
       common->remote_host = ssh_xstrdup(common->remote_ip);
     }
   
   /* Create connection protocol object. */
+
+
+
+
+
+
   SSH_DEBUG(5, ("creating connection protocol"));
   common->conn = ssh_conn_wrap(common->auth, SSH_CONNECTION_SERVICE,
                                common->global_requests,
@@ -407,6 +432,7 @@ void ssh_common_finalize(SshIpError error,
                                common->compat_flags,
                                (void *)common);
   SSH_DEBUG(5, ("connection protocol created"));
+
 }
 
 /* Creates the common processing object for the SSH server/client connection.
@@ -433,15 +459,25 @@ void ssh_common_finalize(SshIpError error,
    a ``close_notify'' callback (see below).  */
 
 SshCommon ssh_common_wrap(SshStream connection,
+
+
+
                           SshStream auth,
+
                           Boolean client,
                           SshConfig config,
                           const char *server_host_name,
+
+
+
                           SshConnDisconnectProc disconnect,
                           SshConnDebugProc debug,
+
                           SshCommonAuthenticationNotify authentication_notify,
                           SshCommonDestroyNotify destroy_notify,
+
                           SshTransportCompat compat_flags,
+
                           void *context)
 {
   SshCommon common;
@@ -458,26 +494,46 @@ SshCommon ssh_common_wrap(SshStream connection,
   common->client = client;
   common->config = config;
   common->num_channels = 0;
-  common->active_local_forwards = ssh_dllist_allocate();
-  common->active_remote_forwards = ssh_dllist_allocate();
-  common->active_x11_forwards = ssh_dllist_allocate();
+  common->active_local_forwards = ssh_adt_create_generic(SSH_ADT_LIST,
+                                                         SSH_ADT_DESTROY,
+                                                         destroy_forward_rec,
+                                                         SSH_ADT_ARGS_END);
+  common->active_remote_forwards = ssh_adt_create_generic(SSH_ADT_LIST,
+                                                          SSH_ADT_DESTROY,
+                                                          destroy_forward_rec,
+                                                          SSH_ADT_ARGS_END);
+  common->active_x11_forwards = ssh_adt_create_generic(SSH_ADT_LIST,
+                                                       SSH_ADT_DESTROY,
+                                                       destroy_forward_rec,
+                                                       SSH_ADT_ARGS_END);
 
 
 
   common->disconnect = disconnect;
+
   common->debug = debug;
+  common->auth = auth;
+
   common->authentication_notify = authentication_notify;
   common->destroy_notify = destroy_notify;
   common->context = context;
   common->server_host_name =
     server_host_name ? ssh_xstrdup(server_host_name) : NULL;
   common->being_destroyed = FALSE;
-  common->auth = auth;
+
+
+
+
+
+
+  common->publickey_options = NULL;  
+
 
   if (compat_flags)
     common->compat_flags = compat_flags;
   else
     common->compat_flags = ssh_xcalloc(1, sizeof(*compat_flags));
+
   
   common->last_login_time = 0;
   common->sizeof_last_login_from_host = 100;
@@ -491,6 +547,8 @@ SshCommon ssh_common_wrap(SshStream connection,
   common->remote_port = ssh_xmalloc(100);
   if (!ssh_tcp_get_remote_port(connection, common->remote_port, 100))
     strcpy(common->remote_port, "UNKNOWN");
+  common->local_hostname = ssh_xmalloc(256);
+  ssh_tcp_get_host_name(common->local_hostname, 256);
   common->local_ip = ssh_xmalloc(100);
   if (!ssh_tcp_get_local_address(connection, common->local_ip, 100))
     strcpy(common->local_ip, "UNKNOWN");
@@ -516,6 +574,37 @@ SshCommon ssh_common_wrap(SshStream connection,
     }
   SSH_ASSERT(num_types == i);
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   /* Allocate memory for channel types and global requests. */
   common->global_requests = ssh_xcalloc(num_requests + 1,
                                         sizeof(common->global_requests[0]));
@@ -544,20 +633,24 @@ SshCommon ssh_common_wrap(SshStream connection,
       else
         common->type_contexts[i] = NULL;
     }
+
   
   /* Set remote host name to ip address. There is no need to do
      reverse mapping in the client (yet) */
   if (!client)
     {
-      ssh_tcp_get_host_by_addr(common->remote_ip, ssh_common_finalize,
-                               (void *)common);
+      if (config->try_reverse_mapping)
+        ssh_tcp_get_host_by_addr(common->remote_ip, ssh_common_finalize,
+                                 (void *)common);
+      else
+        ssh_common_finalize(SSH_IP_FAILURE, common->remote_ip, common);
     }
   else
     {
       /* common->remote_host is allocated in ssh_common_finalize() */
     }
 
-  if(client)
+  if (client)
     ssh_common_finalize(SSH_IP_OK, common->remote_ip, common);
   
   return common;
@@ -601,30 +694,44 @@ void ssh_common_destroy(SshCommon common)
   ssh_xfree(common->type_contexts);
   common->type_contexts = NULL;
   
+
   /* Destroy global requests and open procs. */
   ssh_xfree(common->global_requests);
   ssh_xfree(common->channel_opens);
+
   
   /* Destroy the connection protocol context. */
+
+
+
+
+
   if (common->conn)
     ssh_conn_destroy(common->conn);
   common->conn = NULL;
 
+
   /* Destroy the active channel lists */
-  ssh_dllist_free(common->active_local_forwards);
-  ssh_dllist_free(common->active_remote_forwards);
-  ssh_dllist_free(common->active_x11_forwards);
+  ssh_adt_destroy(common->active_local_forwards);
+  ssh_adt_destroy(common->active_remote_forwards);
+  ssh_adt_destroy(common->active_x11_forwards);
  
   /* Free allocated fields. */
   ssh_xfree(common->server_host_name);
   ssh_xfree(common->user);
+
   if (common->user_data)
     ssh_user_free(common->user_data, FALSE);
+
   ssh_xfree(common->remote_ip);
   ssh_xfree(common->remote_port);
+  ssh_xfree(common->local_hostname);
   ssh_xfree(common->local_ip);
   ssh_xfree(common->local_port);
   ssh_xfree(common->compat_flags);
+
+
+
   ssh_xfree(common->chosen_host_key_alg);
   ssh_xfree(common->chosen_cipher_c_to_s);
   ssh_xfree(common->chosen_cipher_s_to_c);
@@ -632,6 +739,7 @@ void ssh_common_destroy(SshCommon common)
   ssh_xfree(common->chosen_mac_s_to_c);
   ssh_xfree(common->chosen_compression_c_to_s);
   ssh_xfree(common->chosen_compression_s_to_c);
+
   
   if (common->remote_host)
     ssh_xfree(common->remote_host);
@@ -639,6 +747,23 @@ void ssh_common_destroy(SshCommon common)
   if (common->last_login_from_host)
     ssh_xfree(common->last_login_from_host);
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+  ssh_publickey_options_free(common->publickey_options);
+  common->publickey_options = NULL;
+  
   memset(common, 'F', sizeof(*common));
   ssh_xfree(common);
 }
@@ -708,7 +833,7 @@ void ssh_common_new_channel(SshCommon common)
 
 void ssh_common_add_active_forward(SshTcpForwardActive fwd_active)
 {
-  SshDlList list = NULL;
+  SshADTContainer list = NULL;
   
   switch(fwd_active->channel_type)
     {
@@ -723,9 +848,8 @@ void ssh_common_add_active_forward(SshTcpForwardActive fwd_active)
       break;
     }
   
-  ssh_dllist_add_item(list,
-                      (void*)fwd_active,
-                      SSH_DLLIST_BEGIN);
+  SSH_VERIFY(ssh_adt_insert_to(list, SSH_ADT_END, (void*)fwd_active) !=
+             SSH_ADT_INVALID);
 }
 
 
@@ -734,7 +858,7 @@ void ssh_common_delete_active_forward(void* context)
 {
   SshTcpForwardActive delete_node = (SshTcpForwardActive)context;
   SshCommon common;
-  SshDlList list = NULL;
+  SshADTContainer list = NULL;
   
   if (context == NULL) return;
   
@@ -755,22 +879,31 @@ void ssh_common_delete_active_forward(void* context)
   
   /* find the node in the list and delete it */
   
-  if (ssh_dllist_find(list, context) == SSH_DLLIST_OK)
-    {
-      /* item found, delete it */
-      ssh_dllist_delete_current(list);
-    }
-  else
+  if (ssh_adt_detach_object(list, context) == SSH_ADT_INVALID)
     {
       SSH_TRACE(2, ("Forward not found in list (bug?)."));
     }
-  
-  ssh_xfree(delete_node->connect_from_host);
-  ssh_xfree(delete_node->port);
-  ssh_xfree(delete_node->connect_to_host);
-  ssh_xfree(delete_node->connect_to_port);
-  ssh_xfree(delete_node);
+
+  destroy_forward_rec(delete_node, NULL);
 
   /* tell the common object that the channel is destroyed */
   ssh_common_destroy_channel(common);
 }
+
+void ssh_publickey_options_free(SshAuthPublicKeyOptions pubkey_options)
+{
+  if (pubkey_options == NULL)
+    return;
+
+  if (pubkey_options->use_allowed_from)
+    ssh_adt_destroy(pubkey_options->use_allowed_from);
+  if (pubkey_options->use_denied_from)
+    ssh_adt_destroy(pubkey_options->use_denied_from);
+  
+  ssh_xfree(pubkey_options->command);
+
+  if (pubkey_options->environment_vars)
+    ssh_adt_destroy(pubkey_options->environment_vars);
+  
+  ssh_xfree(pubkey_options);  
+}
