@@ -22,9 +22,17 @@
 #include "sshstream.h"
 #include "sshconfig.h"
 #include "sshcrypt.h"
+
+
+
 #include "sshconn.h"
 #include "sshtrans.h"
-#include "sshdllist.h"
+
+
+
+
+
+
 
 
 
@@ -78,28 +86,41 @@ typedef struct SshTcpForwardActiveRec
   SshCommon common;
 } *SshTcpForwardActive;
 
-
+/* Forward declaration. */
+typedef struct SshAuthPublicKeyOptionsRec SshAuthPublicKeyOptionsStruct,
+  *SshAuthPublicKeyOptions;
 struct SshCommonRec
 {
   /* TRUE if we are a client, FALSE if we are a server. */
   Boolean client;
-  
+
+
+
+
+
+
+
+
   /* The connection protocol object. */
   SshConn conn;
 
+  
   /* Configuration data. */
   SshConfig config;
-  
+
   /* Number of active channels on the common. */
   unsigned int num_channels;
 
   /* List of active local port forwardings. */
-  SshDlList active_local_forwards;
+  SshADTContainer active_local_forwards;
   /* List of active remote port forwardings. */
-  SshDlList active_remote_forwards;
+  SshADTContainer active_remote_forwards;
   /* List of active X11 port forwardings. */
-  SshDlList active_x11_forwards;
-  
+  SshADTContainer active_x11_forwards;
+
+
+
+
 
 
 
@@ -108,7 +129,7 @@ struct SshCommonRec
   /* Function to call when a disconnect message is received.  This function
      is supposed to destroy the object. */
   SshConnDisconnectProc disconnect;
-  
+
   /* Function to call when a debug message is received, or NULL. */
   SshConnDebugProc debug;
 
@@ -119,7 +140,7 @@ struct SshCommonRec
   /* Function to call when common object is being destroyed (may be
      NULL) */
   SshCommonDestroyNotify destroy_notify;
-  
+
   /* Context argument to ``disconnect'' and ``debug'' callbacks. */
   void *context;
 
@@ -132,7 +153,7 @@ struct SshCommonRec
 
   /* Authenticated user name. */
   char *user;
-  
+
   /* Data for the user. */
   SshUser user_data;
 
@@ -141,6 +162,9 @@ struct SshCommonRec
   char *remote_port;
   char *remote_host; /* Hostname or ip number */
 
+
+
+
   /* Authentication protocol object, needed here because of
      ssh_common_finalize */
   SshStream auth;
@@ -150,6 +174,7 @@ struct SshCommonRec
   SshTransportCompat compat_flags;
 
   /* Local ip address and port. */
+  char *local_hostname;
   char *local_ip;
   char *local_port;
 
@@ -157,13 +182,16 @@ struct SshCommonRec
   SshTime last_login_time;
   char *last_login_from_host;
   unsigned int sizeof_last_login_from_host;
-  
+
   /* TRUE if authentication has been successful, FALSE if not. */
   Boolean authenticated;
 
   /* TRUE, if transport layer is in the process of rekeying.*/
   Boolean doing_rekey;
-  
+
+
+
+
   /* Relevant parts of a SSH_CROSS_ALGORITHMS packet. Used in rekey
      etc.*/
   char *chosen_host_key_alg;
@@ -174,10 +202,15 @@ struct SshCommonRec
   char *chosen_compression_c_to_s;
   char *chosen_compression_s_to_c;
 
+
+
+
+
   /* An array of contexts returned by the type create function for each
      channel type. */
   void **type_contexts;
-  
+
+
   /* Dynamically built array of global requests.  This array is allocated
      using ssh_xmalloc. */
   unsigned int num_global_requests;
@@ -188,6 +221,7 @@ struct SshCommonRec
   unsigned int num_channel_opens;
   SshConnChannelOpen channel_opens;
 
+
   /* TRUE if context is being destroyed. */
   Boolean being_destroyed;
 
@@ -195,6 +229,47 @@ struct SshCommonRec
      authenticated user before SSH_CROSS_AUTHENTICATED is received.
      This can also be NULL. */
   SshUser *ucp;
+
+  /* Only for the server. */
+
+
+
+
+
+
+
+
+
+  /* Options for the public key the user used in authentication. */
+  SshAuthPublicKeyOptions publickey_options;
+};
+
+struct SshAuthPublicKeyOptionsRec
+{
+  /* From which host (regexs) the publickey can or can't be used
+     from. Follows {Allow,Deny}Hosts logic. */
+  SshADTContainer use_allowed_from;
+  SshADTContainer use_denied_from;
+
+  /* Forced command.*/
+  char *command;
+
+  /* Environment variables to be put to user's environment. (used in
+     sshchsession.c)*/
+  SshADTContainer environment_vars;
+
+  /* These are pretty straightforward. */
+
+  int idle_timeout;
+
+  Boolean no_port_forwarding;
+
+  Boolean no_x11_forwarding;
+
+  Boolean no_agent_forwarding;
+
+  /* Whether allocating a pty should be denied. (used in sshchsession.c)*/
+  Boolean no_pty;
 };
 
 /* Creates the common processing object for the SSH server/client connection.
@@ -221,15 +296,25 @@ struct SshCommonRec
    The object should be destroyed from the ``disconnect'' callback or from
    a ``close_notify'' callback (see below).  */
 SshCommon ssh_common_wrap(SshStream connection,
+
+
+
                           SshStream auth,
+
                           Boolean client,
                           SshConfig config,
                           const char *server_host_name,
+
+
+
                           SshConnDisconnectProc disconnect,
                           SshConnDebugProc debug,
+
                           SshCommonAuthenticationNotify authentication_notify,
                           SshCommonDestroyNotify destroy_notify,
+
                           SshTransportCompat compat_flags,
+
                           void *context);
 
 /* Destroys the common protocol object.  This will not call the disconnect
@@ -288,4 +373,7 @@ void ssh_common_add_active_forward(SshTcpForwardActive fwd_active);
    ssh_common_destroy_channel to delete the channel. */
 void ssh_common_delete_active_forward(void* context);
 
+/* Destroy struct, and the items it contains. */
+void ssh_publickey_options_free(SshAuthPublicKeyOptions pubkey_options);
+
 #endif /* SSHCOMMON_H */
