@@ -18,19 +18,24 @@
 #ifndef SSHCONFIG_H
 #define SSHCONFIG_H
 
+typedef struct SshConfigRec *SshConfig;
+
 #include "sshuser.h"
 #include "sshcrypt.h"
-#include "sshdllist.h"
+#include "sshhostkey.h"
+#include "sshadt.h"
+#include "sshregex.h"
+
+
+
+
+
+
+
 
 #define SUBSYSTEM_PREFIX "subsystem-"
 #define SUBSYSTEM_PREFIX_LEN 10
 
-typedef struct SshSubsystemRec
-{
-  char *name;                          /* name of the subsystem */
-  char *path;                          /* command and arguments to execute */
-} *SshSubsystem;
-
 /* Definition for SshForward */
 
 typedef struct SshForwardRec {
@@ -68,6 +73,19 @@ typedef enum {
   SSH_STRICT_HOSTKEY_CHECKING_ASK = 2
 } SshStrictHostKeyChecking;
 
+typedef struct SshPatternHolderRec
+{
+  char *pattern;
+  SshRegexSyntax regex_syntax;
+} SshPatternHolderStruct, *SshPatternHolder;
+
+typedef struct SshMetaConfigRec
+{
+  int version_major;
+  int version_minor;
+  SshRegexSyntax regex_syntax;
+} SshMetaConfigStruct, *SshMetaConfig;
+
 /* Data type for SSH server configuration data. */
 
 struct SshConfigRec
@@ -75,20 +93,9 @@ struct SshConfigRec
   /* TRUE if the config data is for a client. */
   Boolean client;
 
-  /* The private host key */
-  /* XXX multiple host keys. These should probably be put into a
-     separate struct, and one for each keytype should be help here. */
-  SshPrivateKey private_host_key;
-  unsigned char *public_host_key_blob;
-  size_t public_host_key_blob_len;
-  char *public_key_algorithm;
+  /* The list of host keys */
+  SshHostKeysContext host_keys_ctx;
   
-  /* A context structure to be used with various callbacks. 
-     No use at *nix at this time, but Windows-code uses it
-     to pass registry-arguments. */
-
-  void * callback_context;
-
   /* common for both client and server */
 
   /* For client ...*/
@@ -107,8 +114,8 @@ struct SshConfigRec
   Boolean verbose_mode;
   Boolean compression;
   int compression_level;
-  SshDlList allowed_authentications;
-  SshDlList required_authentications;
+  SshADTContainer allowed_authentications;
+  SshADTContainer required_authentications;
   Boolean user_known_hosts;
   
   char *port;      
@@ -119,6 +126,10 @@ struct SshConfigRec
   char *authorization_file;
   char *random_seed_file;
 
+
+
+
+
   char *pgp_public_key_file;
   char *pgp_secret_key_file;
 
@@ -126,6 +137,11 @@ struct SshConfigRec
   int password_guesses;
   int number_of_password_prompts;
 
+  /* Environment variables to be set in the remote server. */
+  SshADTContainer remote_env;
+  /* Settable environment vars (in server). */
+  SshADTContainer settable_env_vars;
+
   SshUInt32 rekey_interval_bytes;
   SshUInt32 rekey_interval_seconds;
 
@@ -138,19 +154,23 @@ struct SshConfigRec
   SshForward local_forwards;
   SshForward remote_forwards;
 
-  SshDlList allowed_hosts;
-  SshDlList denied_hosts;
-  SshDlList allowed_shosts;
-  SshDlList denied_shosts;
+  SshADTContainer allowed_hosts;
+  SshADTContainer denied_hosts;
+  SshADTContainer allowed_shosts;
+  SshADTContainer denied_shosts;
 
-  SshDlList allowed_users;
-  SshDlList denied_users;
-  SshDlList allowed_groups;
-  SshDlList denied_groups;
+  SshADTContainer allowed_users;
+  SshADTContainer denied_users;
+  SshADTContainer allowed_groups;
+  SshADTContainer denied_groups;
   
-  Boolean require_reverse_mapping;
+  Boolean require_reverse_mapping;  /* is reverse DNS mapping required */
+  Boolean try_reverse_mapping; /* if DNS mapping is not required, this
+                                  specifies whether it is done anyway */
 
   SshLogFacility log_facility;
+  SshLogFacility sftp_server_log_facility;
+  char *debug_log_file_name;
   
   Boolean fall_back_to_rsh;
   Boolean use_rsh;
@@ -173,35 +193,35 @@ struct SshConfigRec
   Boolean quiet_mode;
   Boolean fascist_logging;
   Boolean print_motd;
-  Boolean allow_cshrc_sourcing_with_subsystems;
   Boolean check_mail;
   Boolean auth_success_msg;
   Boolean keep_alive;
   Boolean no_delay;
   Boolean inetd_mode;
+
+  Boolean hostbased_force_client_hostname_dns_match;
+  
   char *listen_address;
   int login_grace_time;
-  char *host_key_file;
-  char *public_host_key_file;
   
-  char *forced_command;
+  SshADTContainer chroot_users;
+  SshADTContainer chroot_groups;
 
-  SshDlList chroot_users;
-  SshDlList chroot_groups;
-
-  SshDlList allowed_tcp_forwarding_users;
-  SshDlList denied_tcp_forwarding_users;
-  SshDlList denied_tcp_forwarding_groups;
-  SshDlList allowed_tcp_forwarding_groups;
-
-  size_t no_subsystems;
-  size_t subsystems_allocated;
-  SshSubsystem *subsystems;
+  SshADTContainer allowed_tcp_forwarding_users;
+  SshADTContainer denied_tcp_forwarding_users;
+  SshADTContainer allowed_tcp_forwarding_groups;
+  SshADTContainer denied_tcp_forwarding_groups;
 
+  SshADTContainer subsystems;
+  
 #ifdef SSH_SERVER_WITH_SECURID
   int securid_guesses; /* number of guesses at passphrase allowed */
 #endif /* SSH_SERVER_WITH_SECURID */
 
+
+
+
+
   /* How many UDP broadcast commands are handled by server each second.
      If this is zero, no broadcasts are received at all. */
   int broadcasts_allowed_per_second;
@@ -215,12 +235,18 @@ struct SshConfigRec
   /* Path to ssh1/sshd1.*/
   char *ssh1_path;
 
+  /* Path to alternate sshd1 configuration file. */
+  char *sshd1_config_file_path;
+  
+#ifdef SSHDIST_SSH2_INTERNAL_SSH1_EMULATION
+#ifdef WITH_INTERNAL_SSH1_EMULATION
+  /* Use internal ssh1 protocol emulation (client only). */
+  Boolean ssh1_emulation_internal; 
 
-
-
-
-
-
+  /* If TRUE, don't try to mask password length. */
+  Boolean ssh1_no_ignore_packets_in_password_auth;
+#endif /* WITH_INTERNAL_SSH1_EMULATION */
+#endif /* SSHDIST_SSH2_INTERNAL_SSH1_EMULATION */
 
   /* Ssh1 arguments for compatibility. If this is NULL, there will be no
      ssh1 compatibility.  The first argument should be the program name. */
@@ -246,9 +272,42 @@ struct SshConfigRec
   /* The banner message. */
   char *banner_msg;
 
-  /* Server idle timeout in minutes for the connection. 0 = disabled */
+  /* Server idle timeout in seconds for the connection. 0 = disabled */
   int idle_timeout;
 
+  /* Protocols that client uses to make connection.  See protocol_mask
+     in SshTcpConnectParams in sshtcp.h */
+  SshUInt32 protocol_mask;
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -286,20 +345,22 @@ struct SshConfigRec
 
 };
 
-typedef struct SshConfigRec *SshConfig;
-
 /* This a template for a function which is used to check whether a
    given parameter (represented as a standard C-string) is valid by
    certain qualifiers. Return FALSE if so, TRUE if not.*/
 typedef Boolean (*SshParameterValidityProc)(const char *param,
                                             void *data);
 
-/* Parse a configuration/authorization file into an array of 
-   variable name <-> value pairs. Return the number of variables or -1 on 
-   error. Pointers to tables of pointers to null-terminated strings are
-   placed at *variables and *values */
+/* Parse a configuration/authorization file into an array of variable
+   name <-> value pairs. Return the number of variables or -1 on
+   error. Pointers to tables of pointers to null-terminated strings
+   are placed at *variables and *values. ``metaconfig'' will store
+   "configuration for the configuration" and can be NULL, if caller
+   doesn't care. */
 int ssh2_parse_config(SshUser user, const char *instance, const char *path, 
-                      char ***variables, char ***values, void *context);
+                      char ***variables, char ***values,
+                      SshMetaConfig metaconfig,
+                      Boolean remove_quotes);
 
 /* Free the "vars" and "vals" arrays */
 void ssh_free_varsvals(int n, char **vars, char **vals);
@@ -318,41 +379,47 @@ void ssh_config_init_finalize(SshConfig config);
 /* Frees client configuration data. */
 void ssh_config_free(SshConfig config);
 
+
+
+
+
+
 /* Reads config data from the given file.  Returns FALSE if an error
    occurs (displays error messages with ssh_warning) */
 Boolean ssh_config_read_file(SshUser user, SshConfig config, char *instance, 
-                             const char *filename, void *context);
+                             const char *filename);
 
 /* Reads the host key that is defined in the config data. Returns
    TRUE if succesful. */
 Boolean ssh_server_load_host_key(SshConfig config,
                                  SshPrivateKey *private_host_key,
                                  unsigned char **public_host_key_blob,
-                                 size_t *public_host_key_blob_len, 
-                                 void *context);
+                                 size_t *public_host_key_blob_len);
 
 /* Set the variable corresponding to `var' to `val' in config. Return
    TRUE if an error occurs. */
-Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val);
+Boolean ssh_config_set_parameter(SshConfig config, SshMetaConfig metaconfig,
+                                 char *var, char *val);
 
 /* Parse a line of input. Return TRUE if an error occurs. */
-Boolean ssh_config_parse_line(SshConfig config, char *line);
+Boolean ssh_config_parse_line(char *line, char **var, char **val);
 
-/* Parses a given comma-separated list to tokens, which are stored in
-   a SshDlList. The list is allocated and returned by this
-   function. On error, returns NULL. function and context are used to
-   check whether a given list-item is valid for a given config
-   parameter. function and context can be NULL, if the parameter is
-   not needed to be checked or the context isn't needed,
-   respectively. */
-SshDlList ssh_config_parse_list(char *string,
-                                SshParameterValidityProc function,
-                                void *context);
+/* Parse a line of input, and set variables in the config
+   struct. Return TRUE if an error occurs. */
+Boolean ssh_config_parse_line_and_set_params(SshConfig config, char *line);
 
-/* separate (commandline)options from their parameters */
-void ssh_split_arguments(int argc, char **argv, int *dest_ac, char ***dest_av);
+/* Allocatar function for SshPatternHolder */
+SshPatternHolder ssh_config_pattern_alloc(char *token,
+                                          SshMetaConfig metaconfig);
+
+/* Destroys an SshPatternHolder object (this is a valid callback to give
+   to ssh_adt_create_generic()) */
+void ssh_config_pattern_destructor(SshPatternHolder holder, void *context);
 
 /* Parse forwarding definitions. Format is port:remotehost:remoteport */
 Boolean ssh_parse_forward(SshForward *forwards, char *spec);
 
+/* Return idle-timeout value. Return -1 if parsing fails. */
+int ssh_config_parse_idle_timeout(const char *value);
+
 #endif /* SSHCONFIG_H */
