@@ -3,7 +3,7 @@
 
   Author: Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1999-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1999-2001 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Connection handling for SshFileCopy.
@@ -13,7 +13,7 @@
 #define SSHFC_CONN_H
 
 #include "sshincludes.h"
-#include "sshfsm.h"
+#include "ssholdfsm.h"
 #include "sshfilecopy.h"
 
 /* Flag values. */
@@ -28,14 +28,14 @@
 #define SSH_FCC_CONN_ABORT (SshUInt32)1
 #define SSH_FCC_CONN_CLOSE (SshUInt32)2
 
-SshFSMThread
-ssh_file_copy_connection_thread_spawn(SshFSM fsm,
-                                      SshFSMCondition waiting,
-                                      SshFSMCondition finished,
+SshOldFSMThread
+ssh_file_copy_connection_thread_spawn(SshOldFSM fsm,
+                                      SshOldFSMCondition waiting,
+                                      SshOldFSMCondition finished,
                                       SshFileCopyConnection conn,
                                       SshUInt8 *conn_flags);
 
-#define SSH_FSM_PROTOTYPES
+#define SSH_OLDFSM_PROTOTYPES
 #include "sshfc_conn_states.h"
 
 #endif /* SSHFC_CONN_H */
