@@ -3,20 +3,21 @@
 
   Author: Sami Lehtinen <sjl@ssh.com>
 
-  Copyright (C) 1999-2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 1999-2001 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Connection handling for SshFileCopy. States used by SshFileCopyConn.
  */
 /* Include this into a state array. */
 
-#ifdef SSH_FSM_PROTOTYPES
-#define SSH_FSM_DECL(x,y,z) SSH_FSM_STEP(z);
+#ifdef SSH_OLDFSM_PROTOTYPES
+#define SSH_FSM_DECL(x,y,z) SSH_OLDFSM_STEP(z);
 #else
-#define SSH_FSM_DECL SSH_FSM_STATE
-#endif /* SSH_FSM_PROTOTYPES */
+#define SSH_FSM_DECL SSH_OLDFSM_STATE
+#endif /* SSH_OLDFSM_PROTOTYPES */
 
-SSH_FSM_DECL("fcc_establish_connection", "Establish connection",        \
+SSH_FSM_DECL("fcc_establish_connection",        \
+             "Establish connection",            \
              ssh_fc_conn_establish)
 
 SSH_FSM_DECL("fcc_close_connection",            \
@@ -39,5 +40,5 @@ SSH_FSM_DECL("fcc_connection_ready",            \
              "Connection succeeded",            \
              ssh_fc_conn_ready)
 
-#undef SSH_FSM_PROTOTYPES
+#undef SSH_OLDFSM_PROTOTYPES
 #undef SSH_FSM_DECL
