@@ -4,7 +4,7 @@
   Author: Sami Lehtinen <sjl@ssh.com>
           Antti Huima <huima@ssh.com>
 
-  Copyright (C) 2000 SSH Communications Security Corp, Helsinki, Finland
+  Copyright (C) 2000, 2001 SSH Communications Security Corp, Helsinki, Finland
   All rights reserved.
 
   Functions that perform the globbing in file transfer.
@@ -17,6 +17,9 @@
 #include "sshregex.h"
 #include "sshglob.h"
 
+
+
+
 #define SSH_DEBUG_MODULE "SshFCGlob"
 
 /**********
@@ -25,7 +28,7 @@
 
 typedef struct SshFileCopyGlobContextRec
 {
-  SshFSM fsm;
+  SshOldFSM fsm;
 
   /* The attrs, that can be used to affect the globbing behaviour. */
   SshFileCopyGlobAttrs attrs;
@@ -38,7 +41,7 @@ typedef struct SshFileCopyGlobContextRec
   SshFileCopyGlobErrorCallback error_cb;
   /* Context, which is passed to the above callabacks.*/
   void *callback_context;
-  
+
   /* Connection used. */
   SshFileCopyConnection connection;
   /* 'Raw' filenames are stored here. */
@@ -46,14 +49,14 @@ typedef struct SshFileCopyGlobContextRec
   /* This contains the final file_list. */
   SshDlList new_file_list;
 
-  SshFSMThread main_thread;
+  SshOldFSMThread main_thread;
 
 } *SshFileCopyGlobContext;
 
 typedef struct GlobThreadContextRec
 {
   char *current_basedir;
-  
+
   /* Used in calls to ssh_file_copy_recurse_dirs(), contains
      SshFileCopyFileListItems. */
   SshDlList temp_list;
@@ -74,7 +77,7 @@ void glob_fsm_destructor(void *gdata)
   SSH_PRECOND(global_data);
 
   SSH_DEBUG(3, ("Destroying global data..."));
-  ssh_xfree(global_data->attrs);  
+  ssh_xfree(global_data->attrs);
 }
 
 /* Helper to wrap the ssh_file_copy_file_list_item_destroy()s
@@ -83,7 +86,7 @@ void file_item_destroyer(void *item)
 {
   SshFileCopyFileListItem to_be_destroyed = (SshFileCopyFileListItem) item;
   SSH_PRECOND(to_be_destroyed);
-  
+
   SSH_DEBUG(3, ("Destroying thread data..."));
   ssh_file_copy_file_list_item_destroy(to_be_destroyed);
 }
@@ -93,7 +96,7 @@ void glob_thread_destructor(void *tdata)
   GlobThreadContext thread_data = (GlobThreadContext)tdata;
 
   SSH_PRECOND(thread_data);
-  
+
   ssh_xfree(thread_data->current_basedir);
   ssh_app_free_list(thread_data->temp_list, file_item_destroyer);
   if (thread_data->pattern_matcher)
@@ -108,16 +111,16 @@ Boolean glob_recurse_file_func(const char *filename,
                                SshFileCopyRecurseAttrs attrs,
                                void *context)
 {
-  SshFSMThread thread = (SshFSMThread) context;
+  SshOldFSMThread thread = (SshOldFSMThread) context;
   SshFileCopyFileListItem temp_item;
   char *trunc_filename = NULL, *temp_filename;
-  SSH_FSM_DATA(SshFileCopyGlobContext, GlobThreadContext);
-  
+  SSH_OLDFSM_DATA(SshFileCopyGlobContext, GlobThreadContext);
+
   SSH_DEBUG(5, ("Got file %s.", filename));
 
   if (strcmp(tdata->current_basedir, filename) == 0)
     return TRUE;
-  
+
   if (strcmp(tdata->current_basedir, "/") == 0)
     trunc_filename = (char *)&filename[1];
   else
@@ -126,7 +129,7 @@ Boolean glob_recurse_file_func(const char *filename,
   SSH_DEBUG(5, ("Truncated filename is %s.", trunc_filename));
 
   if ((file_attributes->permissions & S_IFMT) == S_IFDIR)
-    {      
+    {
       if (!(ssh_regex_match_cstr_prefix(tdata->pattern_matcher,
                                         trunc_filename)))
         return FALSE;
@@ -143,7 +146,7 @@ Boolean glob_recurse_file_func(const char *filename,
       /* Accept this file. */
     full_match:
       temp_item = ssh_file_copy_file_list_item_allocate();
-      
+
       temp_item->original_filename =
         ssh_xstrdup(ssh_file_copy_file_get_name(tdata->current_file));
 
@@ -161,21 +164,21 @@ Boolean glob_recurse_file_func(const char *filename,
         {
           temp_filename = ssh_xstrdup(trunc_filename);
         }
-      
+
       SSH_TRACE(3, ("Adding file %s to file list...", temp_filename));
-      
+
       ssh_file_copy_location_add_file(temp_item->files,
                                       file_attributes,
                                       temp_filename);
 
       ssh_xfree(temp_filename);
-      
+
       SSH_VERIFY(ssh_dllist_add_item(gdata->new_file_list,  \
                                      temp_item,             \
                                      SSH_DLLIST_END)        \
                  == SSH_DLLIST_OK);
 
-      return FALSE;      
+      return FALSE;
     }
   SSH_NOTREACHED;
 }
@@ -188,14 +191,14 @@ void glob_recurse_ready_cb(SshFileCopyError error,
                            SshDlList file_list,
                            void *context)
 {
-  SshFSMThread thread = (SshFSMThread) context;
-  SSH_FSM_GDATA(SshFileCopyGlobContext);
+  SshOldFSMThread thread = (SshOldFSMThread) context;
+  SSH_OLDFSM_GDATA(SshFileCopyGlobContext);
 
   SSH_PRECOND(connection == gdata->connection);
   SSH_PRECOND(file_list == NULL);
-  
-  SSH_FSM_SET_NEXT("fcg_next_file");
-  
+
+  SSH_OLDFSM_SET_NEXT("fcg_next_file");
+
   switch(error)
     {
     case SSH_FC_OK:
@@ -213,54 +216,86 @@ void glob_recurse_ready_cb(SshFileCopyError error,
     case SSH_FC_ERROR_PROTOCOL_MISMATCH:
       (*gdata->ready_cb)(error, error_message, gdata->connection,
                          NULL, gdata->callback_context);
-      SSH_FSM_SET_NEXT("fcg_finish");
+      SSH_OLDFSM_SET_NEXT("fcg_finish");
       break;
     }
 
-  SSH_FSM_CONTINUE_AFTER_CALLBACK(thread);
+  SSH_OLDFSM_CONTINUE_AFTER_CALLBACK(thread);
 }
 
 void glob_recurse_error_cb(SshFileCopyError error,
                            const char *error_message,
                            void *context)
 {
-  SshFSMThread thread = (SshFSMThread) context;
-  SSH_FSM_GDATA(SshFileCopyGlobContext);
-  
+  SshOldFSMThread thread = (SshOldFSMThread) context;
+  SSH_OLDFSM_GDATA(SshFileCopyGlobContext);
+
   (*gdata->error_cb)(error, error_message, gdata->callback_context);
 }
 
-SSH_FSM_STEP(glob_next_file)
+SSH_OLDFSM_STEP(glob_next_file)
 {
   SshFileCopyFile temp_file;
   SshFileCopyFileListItem temp_item;
   char *temp_filename, *p, *to_be_deleted;
-  SSH_FSM_DATA(SshFileCopyGlobContext, GlobThreadContext);
+
+
+
+
+
+  SSH_OLDFSM_DATA(SshFileCopyGlobContext, GlobThreadContext);
 
   if (!ssh_dllist_is_current_valid(gdata->orig_location->file_list))
     {
       SSH_DEBUG(3, ("File list is at it's end."));
-      SSH_FSM_SET_NEXT("fcg_ready");
-      return SSH_FSM_CONTINUE;
+      SSH_OLDFSM_SET_NEXT("fcg_ready");
+      return SSH_OLDFSM_CONTINUE;
     }
-      
+
   temp_file =
     ssh_dllist_current(gdata->orig_location->file_list);
 
   SSH_ASSERT(temp_file);
 
   tdata->current_file = temp_file;
-  
+
   ssh_dllist_fw(gdata->orig_location->file_list, 1);
 
   temp_item = ssh_file_copy_file_list_item_allocate();
-      
+
   temp_item->original_filename =
     ssh_xstrdup(ssh_file_copy_file_get_name(temp_file));
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   temp_filename = ssh_xstrdup(temp_item->original_filename);
-                                                     
+
+
+
+
+
+
+
+
   if ((p = ssh_glob_next_unescaped_wildchar(temp_filename)) != NULL)
+
     {
       *p = '\0';
 
@@ -299,50 +334,52 @@ SSH_FSM_STEP(glob_next_file)
       SSH_TRACE(3, ("Adding file %s to file list without globbing...",  \
                     temp_filename));
       to_be_deleted = temp_filename;
-      temp_filename = ssh_glob_strip_escapes(temp_filename);
+
+          temp_filename = ssh_glob_strip_escapes(temp_filename);
       ssh_xfree(to_be_deleted);
 
+
       /* XXX should be fixed in recurse. */
       if (strlen(temp_filename) < 1)
         {
           ssh_xfree(temp_filename);
           temp_filename = ssh_xstrdup(".");
         }
-      
+
       ssh_file_copy_location_add_file(temp_item->files,
                                       NULL,
                                       temp_filename);
 
       ssh_xfree(temp_filename);
-      
+
       SSH_VERIFY(ssh_dllist_add_item(gdata->new_file_list,  \
                                      temp_item,             \
                                      SSH_DLLIST_END)        \
                  == SSH_DLLIST_OK);
-      return SSH_FSM_CONTINUE;
-      
+      return SSH_OLDFSM_CONTINUE;
+
     }
-  
+
   if (tdata->pattern_matcher)
     {
       ssh_regex_free(tdata->pattern_matcher);
     }
-  
+
   tdata->pattern_matcher = ssh_regex_create(tdata->regex_context,
                                             temp_filename,
                                             SSH_REGEX_SYNTAX_ZSH_FILEGLOB);
-  
+
   if (!tdata->pattern_matcher)
     {
       /* XXX FIXME: error message to application. */
       SSH_TRACE(2, ("Pattern ``%s'' was deemed invalid by the " \
                     "sshregex-library.",                        \
                     temp_filename));
-      return SSH_FSM_CONTINUE;
+      return SSH_OLDFSM_CONTINUE;
     }
-  
+
   ssh_xfree(temp_filename);
-  
+
   /* Strip escaped from basedir before using it. */
   to_be_deleted = tdata->current_basedir;
   tdata->current_basedir = ssh_glob_strip_escapes(tdata->current_basedir);
@@ -354,13 +391,13 @@ SSH_FSM_STEP(glob_next_file)
       ssh_xfree(tdata->current_basedir);
       tdata->current_basedir = ssh_xstrdup(".");
     }
-  
+
   /* Make a list (that contains only the directory, where we start
      globbing), that can be fed to ssh_file_copy_recurse_dirs(). */
   ssh_file_copy_location_add_file(temp_item->files,
                                   NULL,
                                   tdata->current_basedir);
-      
+
   ssh_file_copy_file_register_filename(temp_item->files->file,
                                        ssh_xstrdup(""));
 
@@ -370,49 +407,49 @@ SSH_FSM_STEP(glob_next_file)
     }
 
   tdata->temp_list = ssh_dllist_allocate();
-      
+
   SSH_VERIFY(ssh_dllist_add_item(tdata->temp_list,  \
                                  temp_item,         \
                                  SSH_DLLIST_END)    \
              == SSH_DLLIST_OK);
-      
+
   SSH_TRACE(2, ("Starting glob of %s...",                   \
                 ssh_file_copy_file_get_name(temp_file)));
   SSH_DEBUG(3, ("basedir: %s", tdata->current_basedir));
 
-  
-  SSH_FSM_ASYNC_CALL(ssh_file_copy_recurse_dirs \
+
+  SSH_OLDFSM_ASYNC_CALL(ssh_file_copy_recurse_dirs \
                      (gdata->connection,        \
                       tdata->temp_list,         \
                       NULL,                     \
                       glob_recurse_file_func,   \
                       glob_recurse_ready_cb,    \
                       glob_recurse_error_cb,    \
-                      thread));  
+                      thread));
 }
 
-SSH_FSM_STEP(glob_ready)
+SSH_OLDFSM_STEP(glob_ready)
 {
-  SSH_FSM_GDATA(SshFileCopyGlobContext);
+  SSH_OLDFSM_GDATA(SshFileCopyGlobContext);
+
+  SSH_OLDFSM_SET_NEXT("fcg_finish");
 
-  SSH_FSM_SET_NEXT("fcg_finish");
-  
   (*gdata->ready_cb)(SSH_FC_OK, "Globbing successful.", gdata->connection,
                      gdata->new_file_list, gdata->callback_context);
-  
-  return SSH_FSM_CONTINUE;
+
+  return SSH_OLDFSM_CONTINUE;
 }
 
-SSH_FSM_STEP(glob_finish)
+SSH_OLDFSM_STEP(glob_finish)
 {
-  SSH_FSM_GDATA(SshFileCopyGlobContext);
+  SSH_OLDFSM_GDATA(SshFileCopyGlobContext);
 
-  ssh_fsm_destroy(gdata->fsm);
-  
-  return SSH_FSM_FINISH;
+  ssh_oldfsm_destroy(gdata->fsm);
+
+  return SSH_OLDFSM_FINISH;
 }
 
-SshFSMStateMapItemStruct glob_states_array[] =
+SshOldFSMStateMapItemStruct glob_states_array[] =
 { { "fcg_next_file", "Set next file to glob", glob_next_file },
   { "fcg_ready", "Globbing ready", glob_ready },
   { "fcg_finish", "Finish", glob_finish }
@@ -425,7 +462,7 @@ void ssh_file_copy_glob(SshFileCopyConnection connection,
                         SshFileCopyGlobErrorCallback error_cb,
                         void *context)
 {
-  SshFSM fsm;
+  SshOldFSM fsm;
   SshFileCopyGlobContext glob_context;
   GlobThreadContext tdata;
 
@@ -433,15 +470,15 @@ void ssh_file_copy_glob(SshFileCopyConnection connection,
   SSH_PRECOND(orig_location);
   SSH_PRECOND(ready_cb);
   SSH_PRECOND(error_cb);
-  
-  fsm = ssh_fsm_allocate(sizeof(*glob_context),
+
+  fsm = ssh_oldfsm_allocate(sizeof(*glob_context),
                          glob_states_array,
-                         SSH_FSM_NUM_STATES(glob_states_array),
+                         SSH_OLDFSM_NUM_STATES(glob_states_array),
                          glob_fsm_destructor);
 
-  glob_context = ssh_fsm_get_gdata_fsm(fsm);
+  glob_context = ssh_oldfsm_get_gdata_fsm(fsm);
   memset(glob_context, 0, sizeof(*glob_context));
-  
+
   glob_context->fsm = fsm;
 
   if (attrs == NULL)
@@ -456,19 +493,28 @@ void ssh_file_copy_glob(SshFileCopyConnection connection,
   glob_context->error_cb = error_cb;
   glob_context->callback_context = context;
   glob_context->new_file_list = ssh_dllist_allocate();
-  
+
   ssh_dllist_rewind(orig_location->file_list);
 
   glob_context->main_thread =
-    ssh_fsm_spawn(fsm, sizeof(*tdata),
+    ssh_oldfsm_spawn(fsm, sizeof(*tdata),
                   "fcg_next_file",
                   NULL /* XXX */, glob_thread_destructor);
 
-  tdata = ssh_fsm_get_tdata(glob_context->main_thread);
-  memset(tdata, 0, sizeof(*tdata));  
+  tdata = ssh_oldfsm_get_tdata(glob_context->main_thread);
+  memset(tdata, 0, sizeof(*tdata));
 
   tdata->regex_context = ssh_regex_create_context();
 
-  ssh_fsm_set_thread_name(glob_context->main_thread,
+  ssh_oldfsm_set_thread_name(glob_context->main_thread,
                           "main_thread");
 }
+
+
+
+
+
+
+
+
+
