@@ -18,12 +18,6 @@
 
   - Improve error-handling (those SSH_NOTREACHEDs have to go)
 
-  - add 'multiple buffer'-support to
-    ssh_file_copy_transfer_files(). The speed of the transfer can be
-    greatly increased with several concurrent transfer to/from a file,
-    so that we don't idle waiting for ACK from the server end (either
-    end)
-
   - Find out why killing of the writer (usually ssh2) in transfer
     leads to corrupted files, but the killing of reader is handled
     correctly
@@ -88,6 +82,13 @@ void ssh_file_copy_file_destroy(void *item)
   if (file->handle)
     {
       SSH_DEBUG(7, ("Closing file-handle..."));
+      /* XXX This will nevertheless cause an SSH_FXP_STATUS message to
+         be sent by the file server. With remote connections, this
+         message will be written to the stream by ssh2. Unfortunately,
+         if scp2 (for example) has already exited, ssh2 will fail with
+         SIGPIPE. Kludging this in scp2.c for the moment, but this
+         probably should be fixed properly by catching the status
+         packets. */
       ssh_file_client_close(file->handle, NULL, NULL);
       file->handle = NULL;
     }
@@ -182,6 +183,14 @@ void ssh_file_copy_file_register_long_name(SshFileCopyFile file,
   file->long_name = long_name;
 }
 
+/* Get handle from a SshFileCopyFile structure. */
+SshFileHandle ssh_file_copy_file_get_handle(SshFileCopyFile file)
+{
+  SSH_PRECOND(file);
+
+  return file->handle;
+}
+
 /* Duplicate a SshFileCopyFile structure. Notice, however, that if the
    file had bee opened in the original struct, it will be closed now
    (ie. the SshFileHandle is NULL in the copy). */
@@ -643,8 +652,10 @@ char *ssh_file_copy_get_basedir(const char *filename,
   filename_copy = ssh_xstrdup(filename);
 
   /* Find next wildchar. */
+
   file = ssh_glob_next_unescaped_wildchar(filename_copy);
 
+
   if (!file)
     {
       /* If no wildchars, find last '/' */
@@ -730,10 +741,12 @@ char *ssh_file_copy_strip_escapes_before_slashes(const char *filename)
 
   for(i = 0, str_ind = 0; i < len - 1; i++, str_ind++)
     {
+
       if (filename[i] == echar && filename[i + 1] == '/' &&
           _ssh_glob_isescaped(&filename[i + 1], filename, echar))
         i++;
 
+
       stripped[str_ind] = filename[i];
     }
 
@@ -911,7 +924,7 @@ ssh_file_copy_connect(SshFileCopyConnection connection,
       /* Make local connection. */
       ssh_stream_pair_create(&stream1, &stream2);
       /* XXX local_server should be freed. */
-      local_server = ssh_file_server_wrap(stream1);
+      local_server = ssh_file_server_wrap(stream1, -1);
       local_client = ssh_file_client_wrap(stream2);
 
       /* The server doesn't have to freed implicitly, as it is freed
@@ -924,9 +937,11 @@ ssh_file_copy_connect(SshFileCopyConnection connection,
     {
       ConnectionCompletionCtx new_ctx = ssh_xcalloc(1, sizeof(*new_ctx));
       
-      SSH_TRACE(2, ("Connecting to remote host. (host = %s, "       \
-                    "user = %s, port = %s)", connection->host,      \
-                    connection->user, connection->port));
+      SSH_TRACE(2, ("Connecting to remote host. (host = %s, "
+                    "user = %s, port = %s)",
+                    connection->host ? connection->host : "NULL",
+                    connection->user ? connection->user : "NULL",
+                    connection->port ? connection->port : "NULL"));
       /* use SshFileCopyConnectCallback to connect. */
       SSH_ASSERT(connect_callback);
       new_ctx->connection = connection;
