@@ -16,6 +16,13 @@
 #include "sshdllist.h"
 #include "sshoperation.h"
 
+#define SSH_FC_DEFAULT_BUF_SIZE 0x8000
+#define SSH_FC_DEFAULT_BUF_NUM 10
+
+
+
+
+
 typedef struct SshFileCopyConnectionRec *SshFileCopyConnection;
 typedef struct SshFileCopyFileRec *SshFileCopyFile;
 typedef struct SshFileCopyLocationRec *SshFileCopyLocation;
@@ -34,7 +41,7 @@ struct SshFileCopyConnectionRec
 typedef enum
 {
   /* operation was successful.*/
-  SSH_FC_OK,
+  SSH_FC_OK = 0,
   /* operation resulted in an undetermined error within sshfilecopy. */
   SSH_FC_ERROR,
   /* Destination is not directory, but it should be. */
@@ -154,6 +161,9 @@ void ssh_file_copy_file_register_long_name(SshFileCopyFile file,
 void ssh_file_copy_file_register_attributes(SshFileCopyFile file,
                                             SshFileAttributes attrs);
 
+/* Get handle from a SshFileCopyFile structure. */
+SshFileHandle ssh_file_copy_file_get_handle(SshFileCopyFile file);
+
 /*****************************************************************************
  * SshFileCopyConnection structure manipulating functions.
  *****************************************************************************/
@@ -357,21 +367,44 @@ typedef struct SshFileCopyTransferAttrsRec
   Boolean preserve_attributes;
   Boolean unlink_source;
   Boolean recurse_dirs;
+  Boolean preserve_destination_permissions;
+
+
+
+
+  size_t max_buffers;
+  size_t max_buf_size;
 } *SshFileCopyTransferAttrs;
 
+typedef enum
+{
+  /* Binary transfer mode */
+  SSH_FC_TRANSFER_MODE_BINARY,
+
+  /* Ascii transfer mode */
+  SSH_FC_TRANSFER_MODE_ASCII,
+
+  /* Tranfer mode is selected automatically from the file extensions */
+  SSH_FC_TRANSFER_MODE_AUTO
+} SshFileCopyTransferMode;
+
 SshOperationHandle
-ssh_file_copy_transfer_files (SshFileCopyConnection source,
-                              SshDlList file_list,
-                              SshFileCopyConnection destination,
-                              SshFileCopyLocation dest_location,
-                              SshFileCopyTransferAttrs attrs,
-                              SshFileCopyTransferReadyCallback ready_cb,
-                              SshFileCopyTransferErrorCallback error_cb,
-                              SshFileCopyTransferProgressCallback
-                              progress_cb,
-                              SshFileCopyTransferAbortNotifyCallback
-                              abort_notify_cb,
-                              void *context);
+ssh_file_copy_transfer_files(SshFileCopyConnection source,
+                             SshDlList file_list,
+                             SshFileCopyConnection destination,
+                             SshFileCopyLocation dest_location,
+                             SshFileCopyTransferAttrs attrs,
+                             SshFileCopyTransferMode transfer_mode,
+                             const char *source_newline,
+                             const char *dest_newline,
+                             const char *ascii_extensions,
+                             SshFileCopyTransferReadyCallback ready_cb,
+                             SshFileCopyTransferErrorCallback error_cb,
+                             SshFileCopyTransferProgressCallback
+                             progress_cb,
+                             SshFileCopyTransferAbortNotifyCallback
+                             abort_notify_cb,
+                             void *context);
 
 /* Internal definitions. Don't call these directly. */
 
@@ -386,4 +419,9 @@ ssh_file_copy_connect(SshFileCopyConnection connection,
                       SshFileCopyConnectCompleteCB completion_cb,
                       void *completion_context);
 
+
+
+
+
+
 #endif /* SSHFILECOPY_H */
