@@ -17,11 +17,13 @@ sshserver.c
 */
 
 #include "ssh2includes.h"
+#include "sshcommon.h"
+
 #include "sshtrans.h"
 #include "sshauth.h"
 #include "sshconn.h"
+
 #include "sshauthmethods.h"
-#include "sshcommon.h"
 #include "sshserver.h"
 #include "sshuserfiles.h"
 #include "sshcipherlist.h"
@@ -29,11 +31,12 @@ sshserver.c
 
 #define SSH_DEBUG_MODULE "SshServer"
 
+
 /* Fetches values for the transport parameters (e.g., encryption algorithms)
    from the config data. */
 
-Boolean ssh_server_update_transport_params(SshConfig config,
-                                           SshTransportParams params)
+void ssh_server_update_transport_params(SshConfig config,
+                                        SshTransportParams params)
 {
   char *hlp;
 
@@ -65,18 +68,12 @@ Boolean ssh_server_update_transport_params(SshConfig config,
           ssh_xfree(hlp);
         }
     }
-
-  hlp = ssh_public_key_list_canonicalize(params->host_key_algorithms);
-  ssh_xfree(params->host_key_algorithms);
-  params->host_key_algorithms = hlp;
-
-  hlp = ssh_hash_list_canonicalize(params->hash_algorithms);
-  ssh_xfree(params->hash_algorithms);
-  params->hash_algorithms = hlp;
-
-  return TRUE;
 }
 
+
+/* Forward declaration. */
+void ssh_server_destroy_finalize(void *context);
+
 /* Takes a stream, and creates an SSH server for processing that
    connection.  This closes the stream and returns NULL (without
    calling the destroy function) if an error occurs.  This does not
@@ -84,54 +81,95 @@ Boolean ssh_server_update_transport_params(SshConfig config,
    valid until the server is destroyed; it is not automatically freed.
      `stream'        the connection stream
      `config'        configuration data (not freed, must remain valid)
-     `private_server_key'   private key that changes every hour or NULL
      `disconnect'    function to call on disconnect
      `debug'         function to call on debug message (may be NULL)
      `version_check' version check callback (may be NULL)
      `context'       context to pass to the callbacks
    The object should be destroyed from the ``disconnect'' callback. */
 
-SshServer ssh_server_wrap(SshStream stream, SshConfig config,
-                          SshPrivateKey private_server_key,
+SshServer ssh_server_wrap(SshStream stream,
+
+
+
+                          SshConfig config,
+
+
+
+
                           SshConnDisconnectProc disconnect,
                           SshConnDebugProc debug,
                           SshVersionCallback version_check,
+
                           SshAuthPolicyProc auth_policy_proc,
                           SshCommonAuthenticationNotify authenticated_notify,
+                          const char *protocol_version,
                           void *context)
 {
   SshServer server;
+
+
+
   SshStream trans, auth;
   SshTransportParams params;
   SshTransportCompat compat_flags;
-  
-  /* Create parameters. */
-  params = ssh_transport_create_params();
-  if (!ssh_server_update_transport_params(config, params))
-    {
-      ssh_stream_destroy(stream);
-      ssh_transport_destroy_params(params);
-      return NULL;
-    }
 
-  /* Check the host key. */
-
-  if (config->private_host_key == NULL || 
-      config->public_host_key_blob == NULL)
-    ssh_fatal("ssh_server_wrap: no host key !");
+  SshAuthUninitCtx notify_ctx;
 
   /* Create the server object. */
   server = ssh_xcalloc(1, sizeof(*server));
   server->config = config;
-  
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+  /* Create parameters. */
+  params = ssh_transport_create_params();
+  ssh_server_update_transport_params(config, params);
+
   /* Create a transport layer protocol object. */
   ssh_debug("ssh_server_wrap: creating transport protocol");
   trans = ssh_transport_server_wrap(stream, 
-                                    SSH2_PROTOCOL_VERSION_STRING,
-                                    params, config->private_host_key,
-                                    private_server_key,
-                                    config->public_host_key_blob,
-                                    config->public_host_key_blob_len,
+                                    protocol_version,
+                                    params, config->host_keys_ctx,
                                     version_check,
                                     (void *)context,
                                     ssh_common_rekey_notify_cb,
@@ -148,8 +186,14 @@ SshServer ssh_server_wrap(SshStream stream, SshConfig config,
   /* Create an authentication protocol object. */
   ssh_debug("ssh_server_wrap: creating userauth protocol");
 
+  notify_ctx = ssh_xcalloc(1, sizeof(*notify_ctx));
+  notify_ctx->notify_callback = ssh_server_destroy_finalize;
+  notify_ctx->notify_context = server;
+
   auth = ssh_auth_server_wrap(trans, auth_policy_proc, (void *)server,
                               server->methods, (void *)server,
+                              ssh_server_authentication_uninitialize,
+                              (void *)notify_ctx,
                               config->banner_msg,
                               &server->uc);
 
@@ -160,9 +204,9 @@ SshServer ssh_server_wrap(SshStream stream, SshConfig config,
                                    NULL, compat_flags,
                                    context);
 
+
   if (server->common == NULL)
     {
-      ssh_server_authentication_uninitialize(server->methods);
       ssh_xfree(server);
       return NULL;
     }
@@ -182,16 +226,37 @@ extern void ssh_login_grace_time_exceeded(void *context);
   
 void ssh_server_destroy(SshServer server)
 {
+  SSH_DEBUG(3, ("Destroying server."));
+  
   /* Cancel grace timeout. */
   ssh_cancel_timeouts(ssh_login_grace_time_exceeded, SSH_ALL_CONTEXTS);
-  ssh_common_destroy(server->common);
-#if 0
-  /* XXX causes SIGSEGV, as the methods are called from
-     lib/sshproto/sshauths.c after this call. */
-  ssh_server_authentication_uninitialize(server->methods);
-#endif
+
+
+
+
+
   ssh_xfree(server->authenticated_client_user_name);
   ssh_xfree(server->authenticated_client_host);
+
+  /* ssh_common_destroy() might trigger a destruction of the
+     authentication layer _immediately_ (without going through the
+     event loop), which will lead to the `server' struct being
+     destroyed after this call (by
+     ssh_server_destroy_finalize()). Because of this,
+     ssh_common_destroy() must be called last here. //sjl */
+  ssh_common_destroy(server->common);
+}
+
+/* This is called by ssh_server_authentication_uninitialize(), which
+   is in turn called by the authentication layer, when the methods
+   have been cleared. */
+void ssh_server_destroy_finalize(void *context)
+{
+  SshServer server = (SshServer)context;
+  SSH_PRECOND(server);
+  
+  SSH_DEBUG(3, ("Destroying server completed."));
+
   memset(server, 'F', sizeof(*server));
   ssh_xfree(server);
 }
