@@ -14,9 +14,9 @@
 */
 
 #include "ssh2includes.h"
-
-
-
+#ifdef SSHDIST_SSH2_INTERNAL_SSH1_EMULATION
+#include "ssh1proto.h"
+#endif /* SSHDIST_SSH2_INTERNAL_SSH1_EMULATION */
 #include "sshfilterstream.h"
 #include "sshtcp.h"
 #include "sshcommon.h"
@@ -25,7 +25,6 @@
 #include "sshappcommon.h"
 #include "sshclient.h"
 #include "sshencode.h"
-#include "sshcross.h"
 #include "sshdsprintf.h"
 
 #define SSH_DEBUG_MODULE "SshStdIOFilter"
@@ -33,6 +32,9 @@
 #define STATE_BASIC     0
 #define STATE_HAVE_CR   1
 #define STATE_HAVE_ESC  2
+#define STATE_LINEMODE  3
+
+#define SSH_STDIO_FILTER_MAX_BUFFER_SIZE 2048
 
 static int stdio_filter_state = STATE_BASIC;
 
@@ -89,6 +91,10 @@ void ssh_escape_char_help(int esc_char)
           "  %cr   - initiate rekey immediately with old algorithms (ie. "
           "changes\n"
           "         encryption and integrity keys)\n", esc_char);
+  fprintf(stderr,
+          "  %cl   - go to line mode (keystrokes are gathered to an internal\n"
+          "         buffer, and will be output all at once when you enter\n"
+          "         a newline)\n", esc_char);
   fprintf(stderr,
           "  (Note that escapes are only recognized immediately "
           "after newline.)\n");
@@ -179,70 +185,74 @@ void ssh_escape_char_list_connections(int esc_char, SshClient client)
   ssh_cancel_nonblocking();
 
   common = client->common;
-
-  if (!ssh_dllist_is_empty(common->active_local_forwards))
+  
+  fprintf(stderr, "\r\n");
+  if (ssh_adt_num_objects(common->active_local_forwards) > 0)
     {      
-      fprintf(stderr, "\nActive local forwards:\n");
-      ssh_dllist_rewind(common->active_local_forwards);
-  
-      while ((tmp_active_ptr =
-              ssh_dllist_current(common->active_local_forwards)) != NULL)
-        {
-          fprintf(stderr, "%s : %s : %s from %s\n",
-                  tmp_active_ptr->port,
-                  tmp_active_ptr->connect_to_host,
-                  tmp_active_ptr->connect_to_port,
-                  tmp_active_ptr->connect_from_host);
-
-          ssh_dllist_fw(common->active_local_forwards, 1);
-        }
-    }
-  else
-    {
-      fprintf(stderr, "\nNo active local forwards.\n");
-    }
-  
-  if (!ssh_dllist_is_empty(common->active_remote_forwards))
-    {
-      fprintf(stderr, "\nActive remote forwards:\n");
-      ssh_dllist_rewind(common->active_remote_forwards);
-
-      while ((tmp_active_ptr =
-              ssh_dllist_current(common->active_remote_forwards)) != NULL)
-        {
-          fprintf(stderr, "%s : %s : %s from %s\r\n",
-                  tmp_active_ptr->port,
-                  tmp_active_ptr->connect_to_host,
-                  tmp_active_ptr->connect_to_port,
-                  tmp_active_ptr->connect_from_host);
-
-          ssh_dllist_fw(common->active_remote_forwards, 1);
-        }
-    }
-  else
-    {
-      fprintf(stderr, "No active remote forwards.\n");
-    }
-  
-  if (!ssh_dllist_is_empty(common->active_x11_forwards))
-    {
-      fprintf(stderr, "\nActive X11 forwards:\n");
-      ssh_dllist_rewind(common->active_x11_forwards);
-
-      while ((tmp_active_ptr = ssh_dllist_current(common->active_x11_forwards))
-             != NULL)
-        {
-          fprintf(stderr, "X11 forward from %s\r\n",
-                  tmp_active_ptr->connect_from_host);
+      SshADTHandle h;
       
-          ssh_dllist_fw(common->active_x11_forwards, 1);
+      fprintf(stderr, "Active local forwards:\r\n");
+      for (h = ssh_adt_enumerate_start(common->active_local_forwards);
+           h != SSH_ADT_INVALID;
+           h = ssh_adt_enumerate_next(common->active_local_forwards, h))
+        {
+          tmp_active_ptr = ssh_adt_get(common->active_local_forwards, h);
+
+          fprintf(stderr, "  %s : %s : %s from %s\r\n",
+                  tmp_active_ptr->port,
+                  tmp_active_ptr->connect_to_host,
+                  tmp_active_ptr->connect_to_port,
+                  tmp_active_ptr->connect_from_host);
         }
     }
   else
     {
-      fprintf(stderr, "No active X11 forwards.\n");
+      fprintf(stderr, "No active local forwards.\r\n");
     }
   
+  if (ssh_adt_num_objects(common->active_remote_forwards) > 0)
+    {      
+      SshADTHandle h;
+      
+      fprintf(stderr, "Active remote forwards:\r\n");
+      for (h = ssh_adt_enumerate_start(common->active_remote_forwards);
+           h != SSH_ADT_INVALID;
+           h = ssh_adt_enumerate_next(common->active_remote_forwards, h))
+        {
+          tmp_active_ptr = ssh_adt_get(common->active_remote_forwards, h);
+
+          fprintf(stderr, "  %s : %s : %s from %s\r\n",
+                  tmp_active_ptr->port,
+                  tmp_active_ptr->connect_to_host,
+                  tmp_active_ptr->connect_to_port,
+                  tmp_active_ptr->connect_from_host);
+        }
+    }
+  else
+    {
+      fprintf(stderr, "No active remote forwards.\r\n");
+    }
+  
+  if (ssh_adt_num_objects(common->active_x11_forwards) > 0)
+    {      
+      SshADTHandle h;
+      
+      fprintf(stderr, "Active X11 forwards:\r\n");
+      for (h = ssh_adt_enumerate_start(common->active_x11_forwards);
+           h != SSH_ADT_INVALID;
+           h = ssh_adt_enumerate_next(common->active_x11_forwards, h))
+        {
+          tmp_active_ptr = ssh_adt_get(common->active_x11_forwards, h);
+
+          fprintf(stderr, "  X11 forward from %s\r\n",
+                  tmp_active_ptr->connect_from_host);
+        }
+    }
+  else
+    {
+      fprintf(stderr, "No active X11 forwards.\r\n");
+    }
+
   ssh_set_nonblocking();
   ssh_enter_raw_mode(-1, FALSE);
 }
@@ -258,6 +268,8 @@ void ssh_escape_char_dump_version(int esc_char)
 
 void ssh_escape_char_dump_statistics(int esc_char, SshClient client)
 {
+
+
   SshTransportStatisticsStruct stats;
   char *local_version_string = NULL;
   /* XXX this should be somewhere else. */
@@ -271,23 +283,23 @@ void ssh_escape_char_dump_statistics(int esc_char, SshClient client)
   ssh_leave_raw_mode(-1);
   fprintf(stderr, "\n");
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+#ifdef SSHDIST_SSH2_INTERNAL_SSH1_EMULATION
+#ifdef WITH_INTERNAL_SSH1_EMULATION
+  if (client->ssh1_protocol_handle == NULL)
+    {
+      ssh_transport_get_statistics_tr(client->tr_context, &stats);
+    }
+  else
+    {
+      ssh1_get_statistics(client->ssh1_protocol_handle, &stats);
+    }
+#else /* WITH_INTERNAL_SSH1_EMULATION */
   ssh_transport_get_statistics_tr(client->tr_context, &stats);
+#endif /* WITH_INTERNAL_SSH1_EMULATION */
 
 
+#endif /* SSHDIST_SSH2_INTERNAL_SSH1_EMULATION */
+
   ssh_tcp_get_host_name(local_host_name, MAXHOSTNAMELEN);
   
   if (client->config->host_to_connect)
@@ -319,8 +331,8 @@ void ssh_escape_char_dump_statistics(int esc_char, SshClient client)
   fprintf(stderr, "\nAlgorithms:\n");
 
   fprintf(stderr, "Chosen key exchange algorithm: %s\n", stats.kex_name);
-  fprintf(stderr, "Host key algorithm: %s\n", stats.host_key_name);
-  fprintf(stderr, "Common host keys algorithms: %s\n", stats.host_key_names);
+  fprintf(stderr, "Chosen host key algorithm: %s\n", stats.host_key_name);
+  fprintf(stderr, "Common host key algorithms: %s\n", stats.host_key_names);
   fprintf(stderr, "\nAlgorithms client to server:\n");
   fprintf(stderr, "Cipher: %s\n", stats.cipher_c_to_s);
   fprintf(stderr, "MAC: %s\n", stats.mac_c_to_s);
@@ -331,6 +343,7 @@ void ssh_escape_char_dump_statistics(int esc_char, SshClient client)
   fprintf(stderr, "Compression: %s\n", stats.compression_s_to_c);
   
   ssh_enter_raw_mode(-1, FALSE);
+
 }
 
 void ssh_escape_char_request_rekey(int esc_char, SshClient client)
@@ -339,43 +352,79 @@ void ssh_escape_char_request_rekey(int esc_char, SshClient client)
   clearerr(stderr);        /* XXX */
   ssh_leave_raw_mode(-1);
   fprintf(stderr, "\n");
-  ssh_common_request_rekey(client->common);
-  fprintf(stderr, "Rekeying.\n");
+#ifdef WITH_INTERNAL_SSH1_EMULATION
+  if (client->ssh1_protocol_handle != NULL)
+    {
+      fprintf(stderr, "No rekeying in ssh1 emulation.\n");
+    }
+  else
+#endif /* WITH_INTERNAL_SSH1_EMULATION */
+    {
+      ssh_common_request_rekey(client->common);
+      fprintf(stderr, "Rekeying.\n");
+    }
   ssh_enter_raw_mode(-1, FALSE);
 }
 
-int ssh_stdio_output_filter(SshBuffer data,
-                            size_t offset,
-                            Boolean eof_received,
-                            void *context)
+void ssh_stdio_output_filter(void *context,
+                             SshFilterGetCB get_data,
+                             SshFilterCompletionCB completed,
+                             void *internal_context)
 {
-  size_t received_len;
+  size_t offset, received_len;
+  SshBuffer data;
+  Boolean eof_received;
+
+  (*get_data)(internal_context, &data, &offset, &eof_received);
 
   received_len = ssh_buffer_len(data) - offset;
 
-  return SSH_FILTER_ACCEPT(received_len);
+  if (received_len == 0)
+    {
+      (*completed)(internal_context, SSH_FILTER_HOLD);
+      return;
+    }
+  
+  (*completed)(internal_context, SSH_FILTER_ACCEPT(received_len));
 }
 
-int ssh_stdio_input_filter(SshBuffer data,
-                           size_t offset, 
-                           Boolean eof_received,
-                           void *context)
+void ssh_stdio_input_filter(void *context,
+                            SshFilterGetCB get_data,
+                            SshFilterCompletionCB completed,
+                            void *internal_context)
 {
-  size_t received_len;
+  size_t offset, received_len;
+  SshBuffer data;
+  Boolean eof_received;
+
   unsigned char *ucp;
   int i, c, e;
   SshClient client = (SshClient) context;
   
+  (*get_data)(internal_context, &data, &offset, &eof_received);
+
   if (!context)
-    return SSH_FILTER_SHORTCIRCUIT;
+    {
+      (*completed)(internal_context, SSH_FILTER_SHORTCIRCUIT);
+      return;
+    }
   e = *((unsigned char *)client->config->escape_char);
   if (!e)
-    return SSH_FILTER_SHORTCIRCUIT;
+    {
+      (*completed)(internal_context, SSH_FILTER_SHORTCIRCUIT);
+      return;
+    }
   /* Compute the start and length of data that we have in the filter. */
   ucp = ssh_buffer_ptr(data);
   ucp += offset;
   received_len = ssh_buffer_len(data) - offset;
 
+  if (received_len == 0)
+    {
+      (*completed)(internal_context, SSH_FILTER_HOLD);
+      return;
+    }
+
   for (i = 0; i < received_len; i++) {
     c = ucp[i];
     switch (stdio_filter_state) {
@@ -394,6 +443,48 @@ int ssh_stdio_input_filter(SshBuffer data,
           stdio_filter_state = STATE_BASIC;
       }
       break;
+    case STATE_LINEMODE:
+      if ((c == '\n') || (c == '\r') ||
+          (ssh_buffer_len(client->stdio_filter_buffer) >=
+           SSH_STDIO_FILTER_MAX_BUFFER_SIZE))
+        {
+          size_t stored_len;
+          
+          SSH_TRACE(3, ("Going to normal mode."));
+          /* Go back to normal state. */
+          /* Copy stuff from internal buffer to filter buffer to
+             be sent downstream. */
+          stdio_filter_state = STATE_BASIC;
+          stored_len = ssh_buffer_len(client->stdio_filter_buffer);
+          ssh_xbuffer_append(client->stdio_filter_buffer,
+                             ssh_buffer_ptr(data) + offset + i,
+                             ssh_buffer_len(data) - offset - i);
+          ssh_buffer_consume_end(data, ssh_buffer_len(data) - offset -i);
+          ssh_xbuffer_append(data, ssh_buffer_ptr(client->stdio_filter_buffer),
+                             ssh_buffer_len(client->stdio_filter_buffer));
+          /* Clear sensitive data from the buffer (in linemode the
+             stuff entered is usually password and such). */
+          memset(ssh_buffer_ptr(client->stdio_filter_buffer), 0,
+                 ssh_buffer_len(client->stdio_filter_buffer));
+          received_len += stored_len;
+          /* Re-examine current position. */
+          i--;
+          /* XXX Empty internal buffer. */
+          ssh_buffer_clear(client->stdio_filter_buffer);
+        }
+      else
+        {
+          SSH_TRACE(3, ("Processed."));
+          
+          /* Remove character from data buffer, and append to internal
+             buffer. */
+          ssh_buffer_consume_middle(data, offset + i, 1);
+          received_len--; /* Buffer is now shorter    */
+          i--;            /* Re-exam current position */
+          ssh_xbuffer_append(client->stdio_filter_buffer,
+                             (unsigned char *)&c, 1);
+        }
+      break;
     case STATE_HAVE_ESC:
       switch (c)
         {
@@ -425,7 +516,8 @@ int ssh_stdio_input_filter(SshBuffer data,
           ssh_buffer_consume_middle(data, offset + i, 1);
           received_len--; /* Buffer is now shorter    */
           i--;            /* Re-exam current position */
-          return SSH_FILTER_SHORTCIRCUIT;
+          (*completed)(internal_context, SSH_FILTER_SHORTCIRCUIT);
+          return;
         case 'V':
           ssh_buffer_consume_middle(data, offset + i, 1);
           received_len--; /* Buffer is now shorter    */
@@ -444,6 +536,15 @@ int ssh_stdio_input_filter(SshBuffer data,
           i--;            /* Re-exam current position */
           ssh_escape_char_request_rekey(e, client);
           break;
+        case 'l':
+          /* Line mode. We will exit line mode on CR, and then all
+             data typed by the user is sent in one blob. */
+          ssh_buffer_consume_middle(data, offset + i, 1);
+          received_len--; /* Buffer is now shorter    */
+          i--;            /* Re-exam current position */
+          stdio_filter_state = STATE_LINEMODE;
+          fprintf(stderr, "\r\n<in linemode>\r\n");
+          continue;
         default:
           if (c != e)
             {
@@ -460,7 +561,7 @@ int ssh_stdio_input_filter(SshBuffer data,
     }
   }
 
-  return SSH_FILTER_ACCEPT(received_len);
+  (*completed)(internal_context, SSH_FILTER_ACCEPT(received_len));
 }
 
 void ssh_stdio_filter_destroy(void *context)
