@@ -15,18 +15,276 @@
 
 */
 
-/* 
- * $Log: sshuserfiles.h,v $
+/*
+ * $Log: sshuserfiles.h,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
 #ifndef SSHUSERFILES_H
 #define SSHUSERFILES_H
 
+#include "sshincludes.h"
 #include "sshcrypt.h"
 #include "sshuser.h"
 #include "sshconfig.h"
-
+#include "sshkeyblob2.h"
 
 
 
@@ -34,6 +292,10 @@
 #define SSH_USER_DIR ".ssh2"
 #endif /* SSH_USER_DIR */
 
+#ifdef VXWORKS
+#define SSH_RANDSEED_FILE "rndseed"
+#endif /* VXWORKS */
+
 /* the random seed file */
 #ifndef SSH_RANDSEED_FILE
 #define SSH_RANDSEED_FILE "random_seed"
@@ -60,9 +322,6 @@
 
 
 
-
-
-
 
 
 
@@ -75,24 +334,16 @@
 #define SSH_PGP_SECRET_KEY_FILE "secring.pgp"
 #endif /* SSH_PGP_SECRET_KEY_FILE */
 
-/* the standard cipher used for passphrase encryption */
-
-#ifndef SSH_PASSPHRASE_CIPHER
-#define SSH_PASSPHRASE_CIPHER "3des-cbc"
-#endif /* SSH_PASSPHRASE_CIPHER */
-
 /* the standard "authorization" file */
 
+#ifdef VXWORKS
+#define SSH_AUTHORIZATION_FILE "authfile"
+#endif /* VXWORKS */
+
 #ifndef SSH_AUTHORIZATION_FILE
 #define SSH_AUTHORIZATION_FILE "authorization"
 #endif /* SSH_AUTHORIZATION_FILE */
 
-/* the separator for forced commands in the users
-   "authorization" file.*/
-#ifndef FORCED_COMMAND_ID
-#define FORCED_COMMAND_ID "command"
-#endif /* FORCED_COMMAND_ID */
-
 #ifndef SSH_USER_CONFIG_DIRECTORY
 #define SSH_USER_CONFIG_DIRECTORY "%D/" SSH_USER_DIR
 #endif /* SSH_USER_CONFIG_DIRECTORY */
@@ -121,68 +372,107 @@
 
 /* server directory */
 
+#ifndef PASSWD_PATH
+#define PASSWD_PATH "/bin/passwd"
+#endif /* !PASSWD_PATH */
+
+#ifdef VXWORKS
+#define ETCDIR "/fd0"
+#endif /* VXWORKS */
+
 #ifndef ETCDIR
-#define ETCDIR "/etc"
-#endif /* ETCDIR */
+#define ETCDIR "/etc/ssh2"
+#endif /* !ETCDIR */
 
 #ifndef SSH_SERVER_DIR
-#define SSH_SERVER_DIR ETCDIR "/ssh2"
-#endif /* SSH_SERVER_DIR */
+#define SSH_SERVER_DIR ETCDIR
+#endif /* !SSH_SERVER_DIR */
+
+#ifndef SSHD_PIDDIR
+#ifdef HAVE_VAR_RUN
+#define SSHD_PIDDIR "/var/run"
+#else /* HAVE_VAR_RUN */
+#define SSHD_PIDDIR SSH_SERVER_DIR
+#endif /* HAVE_VAR_RUN */
+#endif /* !SSHD_PIDDIR */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 #ifndef SSH_HOSTKEYS_DIR
 #define SSH_HOSTKEYS_DIR "/hostkeys"
-#endif /* SSH_HOSTKEYS_DIR */
+#endif /* !SSH_HOSTKEYS_DIR */
 
 #ifndef SSH_KNOWNHOSTS_DIR
 #define SSH_KNOWNHOSTS_DIR "/knownhosts"
-#endif /* SSH_KNOWNHOSTS_DIR */
+#endif /* !SSH_KNOWNHOSTS_DIR */
 
 #ifndef SSH_GLOBAL_HOSTKEYS_DIR
 #define SSH_GLOBAL_HOSTKEYS_DIR SSH_SERVER_DIR SSH_HOSTKEYS_DIR
-#endif /* SSH_GLOBAL_KNOWNHOSTS_DIR */
+#endif /* !SSH_GLOBAL_KNOWNHOSTS_DIR */
 
 #ifndef SSH_GLOBAL_KNOWNHOSTS_DIR
 #define SSH_GLOBAL_KNOWNHOSTS_DIR SSH_SERVER_DIR SSH_KNOWNHOSTS_DIR
-#endif /* SSH_GLOBAL_KNOWNHOSTS_DIR */
+#endif /* !SSH_GLOBAL_KNOWNHOSTS_DIR */
 
 /* File, from which a possible banner message is sent to the client
    before authentication. */
 #ifndef SSH_BANNER_MSG_FILE
 #define SSH_BANNER_MSG_FILE SSH_SERVER_DIR "/ssh_banner_message"
-#endif /* SSH_BANNER_MSG_FILE */
+#endif /* !SSH_BANNER_MSG_FILE */
 
 /* global configuration file for the client */
 
 #ifndef SSH_CLIENT_GLOBAL_CONFIG_FILE
 #define SSH_CLIENT_GLOBAL_CONFIG_FILE SSH_SERVER_DIR "/ssh2_config"
-#endif /* SSH_CLIENT_GLOBAL_CONFIG_FILE */
+#endif /* !SSH_CLIENT_GLOBAL_CONFIG_FILE */
 
 /* configuration file for the client */
 
 #ifndef SSH_CLIENT_CONFIG_FILE
 #define SSH_CLIENT_CONFIG_FILE "ssh2_config"
-#endif /* SSH_CLIENT_CONFIG_FILE */
+#endif /* !SSH_CLIENT_CONFIG_FILE */
+
+#ifdef VXWORKS
+/* VxWorks allows only 8 character long file names */
+#define SSH_SERVER_CONFIG_FILE "sshd2.cnf"
+#endif /* VXWORKS */
 
 /* configuration file for the server */
-
 #ifndef SSH_SERVER_CONFIG_FILE
 #define SSH_SERVER_CONFIG_FILE "sshd2_config"
-#endif /* SSH_SERVER_CONFIG_FILE */
+#endif /* !SSH_SERVER_CONFIG_FILE */
+
+/* sshd1-specific config file */
+#ifndef SSHD1_CONFIG_FILE
+#define SSHD1_CONFIG_FILE FOREIGN_ETCDIR "/sshd_config"
+#endif /* !SSHD1_CONFIG_FILE */
 
 /* Ssh-only version of /etc/hosts.equiv. */
 #ifndef SSH_HOSTS_EQUIV
-#define SSH_HOSTS_EQUIV ETCDIR "/shosts.equiv"
-#endif /* SSH_HOSTS_EQUIV */
+#define SSH_HOSTS_EQUIV FOREIGN_ETCDIR "/shosts.equiv"
+#endif /* !SSH_HOSTS_EQUIV */
+
+
+
+
+
+
+
+
 
-/* Magic identifying codes for private and public key files. */
 
-#define SSH_KEY_MAGIC_FAIL                      0
-#define SSH_KEY_MAGIC_PUBLIC                    0x73736801
-#define SSH_KEY_MAGIC_PRIVATE                   0x73736802
-#define SSH_KEY_MAGIC_PRIVATE_ENCRYPTED         0x73736803
-#define SSH_KEY_MAGIC_SSH1_PUBLIC               0x73733101
-#define SSH_KEY_MAGIC_SSH1_PRIVATE              0x73733102
-#define SSH_KEY_MAGIC_SSH1_PRIVATE_ENCRYPTED    0x73733103
 
 
 /* Return a pointer to user's ssh2 directory.  The directory is
@@ -192,14 +482,23 @@
    needed. */
 char *ssh_userdir(SshUser user, SshConfig config, Boolean create_if_needed);
 
+/* Forms an absolute path from `file', returning an xmallocated
+   string. If `file' is already absolute, it is used as is. If it is
+   relative, an absolute path is formed, using user's ssh2 directory
+   as the prefix. If user's ssh2 directory is invalid, the
+   `default_prefix' is used (this string should start with a '/'). */
+char *ssh_userdir_prefix(SshUser user, SshConfig config,
+                         const char *file, const char *default_prefix,
+                         Boolean create_if_needed);
+
 /* Return a pointer to the user's random seed file name.
-   Return NULL on failure. The file name is found from `config'. 
+   Return NULL on failure. The file name is found from `config'.
    If `config' is NULL, use the standard SSH_RANDSEED_FILE.
 
    'create' specifies whether an empty seed file is created if the file
    does not exist.
 
-   The caller is responsible for freeing the returned value with ssh_xfree 
+   The caller is responsible for freeing the returned value with ssh_xfree
    when no longer needed. */
 char *ssh_randseed_file(SshUser user, SshConfig config, Boolean create);
 
@@ -208,32 +507,6 @@ char *ssh_randseed_file(SshUser user, SshConfig config, Boolean create);
 
 void ssh_randseed_load(SshUser user,SshConfig config);
 
-
-/* Reads a blob into a buffer. Return TRUE on failure.  The caller must free
-   `*blob' with ssh_xfree when no longer needed. */
-Boolean ssh_blob_read(SshUser user, const char *fname, unsigned char **blob, 
-                      size_t *bloblen, void *context);
-
-/* Write a blob. Return TRUE on failure. */
-Boolean ssh_blob_write(SshUser user, const char *fname, mode_t mode,
-                       const unsigned char *blob, size_t bloblen, 
-                       void *context);
-
-/* Read a public/private key blob from a file. Return the magic code
-   or SSH_KEY_MAGIC_FAIL on failure.  The caller should free comment
-   with ssh_xfree when no longer needed. */
-unsigned long ssh2_key_blob_read(SshUser user, const char *fname, 
-                                 Boolean try_convert_ssh1_cert,
-                                 char **comment,
-                                 unsigned char **blob,
-                                 size_t *bloblen, void *context);
-
-/* Write a key blob. Return TRUE on failure. */
-Boolean ssh2_key_blob_write(SshUser user, const char *fname, mode_t mode,
-                           unsigned long magic,
-                           const char *comment, const unsigned char *key,
-                           size_t keylen, void * context);
-
 /* Get the random state from the file.  The caller is responsible for
    freeing the random number generator with ssh_random_free, or closing and
    updating to the seed file with ssh_randseed_close (recommended) when no
@@ -247,31 +520,15 @@ void ssh_randseed_open(SshUser user, SshConfig config);
    file. */
 void ssh_randseed_update(SshUser user, SshConfig config);
 
-/* Read a public key from a file. Return NULL on failure.  The caller is
-   responsible for freeing `comment' with ssh_xfree when no longer needed.
-   `comment' can be NULL. */
-SshPublicKey ssh_pubkey_read(SshUser user, const char *fname, char **comment, 
-                             void * context);
-
-/* Write a public key to a file. Returns TRUE on error. */
-Boolean ssh_pubkey_write(SshUser user, const char *fname, const char *comment,
-                         SshPublicKey key, void *context);
-
-/* Read a private key from a file. Return NULL on failure.  The caller should
-   free `comment' with ssh_xfree when no longer needed.  `comment' can be NULL. */
-SshPrivateKey ssh_privkey_read(SshUser user, const char *fname,
-                               const char *passphrase, 
-                               char **comment, void *context);
-
-/* Write a private key to a file with a passphrase. Return TRUE on error. */
-Boolean ssh_privkey_write(SshUser user,
-                          const char *fname, const char *passphrase,
-                          const char *comment,
-                          SshPrivateKey key, 
-                          void *context);
+typedef enum {
+  SSH_PRIVKEY_PLAIN,
+  SSH_PRIVKEY_CERT,
+  SSH_PRIVKEY_PGP
+} SshConfigPrivateKeyType;
 
 struct SshConfigPrivateKey {
   char *keyfile;
+  SshConfigPrivateKeyType type;
 #ifdef WITH_PGP
   char *pgp_keyring;
   char *pgp_name;
@@ -282,27 +539,20 @@ struct SshConfigPrivateKey {
 
 /* Build a list of private key files that should be tried when
    logging into `host'.  The list's last entry is NULL.
-   The caller should free the array and all strings in it with ssh_xfree when 
+   The caller should free the array and all strings in it with ssh_xfree when
    no longer needed. */
-struct SshConfigPrivateKey **ssh_privkey_list(SshUser user, 
-                                              char *host, 
+struct SshConfigPrivateKey **ssh_privkey_list(SshUser user,
+                                              char *host,
                                               SshConfig config);
 
-/* Generate a name string from any blob.  String consists of
-   caller given string and space and sha1 hash of the blob in hex. 
-   String is allocated with ssh_xmalloc. */
-char *ssh_generate_name_from_blob(char *name,
-                                  unsigned char *blob,
-                                  size_t bloblen);
-
 /* Generate a directory, where the config files of given user
    are.  This is usually %D/.ssh2 where %D is user's home
    directory. */
-char *ssh_user_conf_dir(SshConfig config, SshUser user);
+char *ssh_user_conf_dir(SshUser user, SshConfig config);
 
 /* Builds a directory name according to the dir template and user info.
    Extracts %D, %U, %IU and %IG in the template to the values
    specified by user. */
-char *ssh_user_build_dir(const char *path_template, SshUser user);
+char *ssh_user_build_dir(SshUser user, const char *path_template);
 
 #endif /* SSHUSERFILES_H */
