@@ -19,6 +19,10 @@ sharing, or other data copying over any stream.
 #include "sshstream.h"
 #include "sshoperation.h"
 
+#ifdef WIN32
+#define off_t SshUInt64
+#endif /* WIN32 */
+
 /* Data type for the file transfer client. */
 typedef struct SshFileClientRec *SshFileClient;
 
@@ -62,7 +66,7 @@ typedef struct SshFileHandleRec *SshFileHandle;
    completes. */
 typedef enum {
   /* The operation completed successfully. */
-  SSH_FX_OK,
+  SSH_FX_OK = 0,
 
   /* The operation failed because of trying to read at end of file. */
   SSH_FX_EOF,
@@ -87,6 +91,10 @@ typedef enum {
      performed. */
   SSH_FX_CONNECTION_LOST,
 
+  /* Requested operation failed because memory allocation failed in
+     the client library. */
+  SSH_FX_OUT_OF_MEMORY,
+
   /* Operation is unsupported by the fileserver. This is actually
      checked by the client, and the offending call is never sent to
      the server. */
@@ -300,7 +308,8 @@ typedef struct SshFileServerRec *SshFileServer;
 
 /* Wraps the given communications channel into a file transfer server.
    The server is automatically destroyed when the connection is closed. */
-SshFileServer ssh_file_server_wrap(SshStream stream);
+SshFileServer ssh_file_server_wrap(SshStream stream,
+                                   int log_facility);
 
 
 
