@@ -259,7 +259,7 @@ void ssh_file_client_return_status(SshFileClient client,
   /* Check if a request was found. */
   if (request == NULL)
     {
-      SSH_TRACE(0, ("ssh_file_client_return_status: id %d not found, " \
+      SSH_TRACE(0, ("ssh_file_client_return_status: id %d not found, "
                     "error %d", id, (int)error));
       return;
     }
@@ -976,7 +976,7 @@ void ssh_file_client_receive_proc(SshPacketType type,
     case SSH_FXP_VERSION:
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32, &u,
-                           SSH_FORMAT_END) != len)
+                           SSH_FORMAT_END) != len || len == 0)
         {
           SSH_TRACE(0, ("ssh_file_client_receive_proc: bad VERSION"));
           return;
@@ -990,7 +990,7 @@ void ssh_file_client_receive_proc(SshPacketType type,
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32, &id,
                            SSH_FORMAT_UINT32, &u,
-                           SSH_FORMAT_END) != len)
+                           SSH_FORMAT_END) != len || len == 0)
         {
           SSH_TRACE(0, ("ssh_file_client_receive_proc: bad STATUS"));
           return;
@@ -1002,7 +1002,7 @@ void ssh_file_client_receive_proc(SshPacketType type,
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32, &id,
                            SSH_FORMAT_UINT32_STR, &s, &slen,
-                           SSH_FORMAT_END) != len)
+                           SSH_FORMAT_END) != len || len == 0)
         {
           SSH_TRACE(0, ("ssh_file_client_receive_proc: bad HANDLE"));
           return;
@@ -1038,7 +1038,7 @@ void ssh_file_client_receive_proc(SshPacketType type,
       if (ssh_decode_array(data, len,
                            SSH_FORMAT_UINT32, &id,
                            SSH_FORMAT_UINT32_STR_NOCOPY, &s, &slen,
-                           SSH_FORMAT_END) != len)
+                           SSH_FORMAT_END) != len || len == 0)
         {
           SSH_TRACE(0, ("ssh_file_client_receive_proc: bad DATA"));
           return;
@@ -1099,10 +1099,12 @@ void ssh_file_client_receive_proc(SshPacketType type,
        * of a single name */
       
       if (handle == NULL)
-        {         
+        {
+          size_t ret = 0L;
+          
           if (u != 1)
             {
-              SSH_TRACE(0, ("ssh_file_client_receive_proc: null handle and " \
+              SSH_TRACE(0, ("ssh_file_client_receive_proc: null handle and "
                             "%lu names.", u));
               return;
             }
@@ -1115,15 +1117,22 @@ void ssh_file_client_receive_proc(SshPacketType type,
                                      &long_name, NULL,
                                    SSH_FORMAT_END);
 
-          bytes += ssh_file_attrs_decoder(data + offset + bytes,
-                                          len - offset - bytes, &attrs);
-
-          
           if (bytes == 0)
             {
               SSH_TRACE(0 ,("ssh_file_client_receive_proc: bad NAME"));
               return;
             }
+
+          ret = ssh_file_attrs_decoder(data + offset + bytes,
+                                       len - offset - bytes, &attrs);
+          bytes += ret;
+          
+          if (ret == 0)
+            {
+              SSH_TRACE(0 ,("ssh_file_client_receive_proc: bad attrs "
+                            "in NAME"));
+              return;
+            }
          
           /* Call the context */
 
@@ -1161,6 +1170,8 @@ void ssh_file_client_receive_proc(SshPacketType type,
 
       for (i = 0; i < u; i++)
         {
+          size_t ret = 0L;
+              
           bytes = ssh_decode_array(data + offset, len - offset,
                                    SSH_FORMAT_UINT32_STR,
                                      &handle->names[i], NULL,
@@ -1169,10 +1180,14 @@ void ssh_file_client_receive_proc(SshPacketType type,
                                    SSH_FORMAT_END);
 
           if (bytes != 0)
-            bytes += ssh_file_attrs_decoder(data + offset + bytes,
+            {
+              ret += ssh_file_attrs_decoder(data + offset + bytes,
                                             len - offset - bytes, 
                                             &handle->attrs[i]);
-          if (bytes == 0)
+              bytes += ret;
+            }
+          
+          if (bytes == 0 || ret == 0)
             {
               SSH_TRACE(0, ("ssh_file_client_receive_proc: bad NAME %d", i));
               for (; i > 0; i--)
@@ -1230,7 +1245,7 @@ void ssh_file_client_receive_proc(SshPacketType type,
       if (bytes)
         bytes += ssh_file_attrs_decoder(data + bytes, len - bytes, &attrs);
 
-      if (bytes != len)
+      if (bytes != len || len == 0)
         {
           SSH_TRACE(0, ("ssh_file_client_receive_proc: bad ATTRS"));
           return;
@@ -1259,7 +1274,7 @@ void ssh_file_client_receive_proc(SshPacketType type,
       break;
 
     default:
-      SSH_TRACE(0, ("ssh_file_client_receive_proc: unexpected packet %d", \
+      SSH_TRACE(0, ("ssh_file_client_receive_proc: unexpected packet %d",
                     (int)type));
     }
 
@@ -1334,7 +1349,8 @@ SshFileClient ssh_file_client_wrap(SshStream stream)
 
   /* Send initialization packet. */
   ssh_packet_wrapper_send_encode(client->conn, SSH_FXP_INIT,
-                                 SSH_FORMAT_UINT32, (SshUInt32) SSH_FILEXFER_VERSION,
+                                 SSH_FORMAT_UINT32,
+                                 (SshUInt32) SSH_FILEXFER_VERSION,
                                  SSH_FORMAT_END);
 
   return client;
