@@ -164,22 +164,22 @@ char *ssh_realpath(const char *path, char *resolved);
 
 /* Decodes the user given file name path to a locally suitable format.
    This does nothing on Unix. */
-char *decode_file_path(char *file);
-
-
-
-
-
-
-
-
-
-
-
+char *ssh_decode_file_path(char *file);
 
 
+#ifdef WINDOWS
 
+/* Returns the list of logical drives for the user.
+   Individual drive names are to be extracted from the returned list
+   with get_root_drive(). The list should be freed with ssh_xfree()
+   when no longer used.
+   Returns TRUE on success, FALSE on failure. */
+Boolean get_root_drive_list(char **list);
+/* Returns one drive name from a list returned by get_root_drive_list().
+   The format is "A:/". */
+const char *get_root_drive(const char *list, int index);
 
+#endif /* WINDOWS */
 
 
 #endif /* SSHFILEXFERI_H */
