@@ -24,14 +24,47 @@
   */
 
 /*
- * $Id: factor.c,v 1.6 2000/11/13 04:17:56 mkojo Exp $
- * $Log: factor.c,v $
+ * $Id: factor.c,v 1.9 2001/10/25 16:59:44 ksr Exp $
+ * $Log: factor.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
 #include "sshincludes.h"
 #include "sshmp.h"
-#include "sieve.h"
 #include "pollard.h"
 #include "sshbuffer.h"
 
@@ -61,6 +94,19 @@ typedef struct SshECMCurveRec
   SshECMPoint q1, q2;
 } SshECMCurve;
 
+void ssh_mprz_out_str(FILE *fp, unsigned int base,
+                      SshMPIntegerConst op)
+{
+  char *str;
+
+  str = ssh_mprz_get_str(op, base);
+  if (fp == NULL)
+    fputs(str, stdout);
+  else
+    fputs(str, fp);
+  ssh_free(str);
+}
+
 void ssh_ecm_init_point(SshECMPoint *n, const SshECMCurve *E)
 {
   ssh_mpm_init(&n->x, &E->mont_q);
@@ -1245,7 +1291,6 @@ int main(int ac, char *av[])
   printf("Pollard rho to find out medium size factors.\n");
   while (1)
     {
-      int e;
       if (ssh_pollard_rho(&factor, &c,
                           1600000, 1) == FALSE)
         {
