@@ -4,7 +4,7 @@
 
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (C) 1996,1997 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996,1997, 2001 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Created: Wed Apr 29 02:10:22 1998 [mkojo]
@@ -21,8 +21,169 @@
   */
 
 /*
- * $Id: t-mathtest.c,v 1.32 2000/11/10 15:43:01 mkojo Exp $
- * $Log: t-mathtest.c,v $
+ * $Id: t-mathtest.c,v 1.42 2001/10/29 10:53:05 irwin Exp $
+ * $Log: t-mathtest.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -31,11 +192,22 @@
 
 
 
-
-
-
 #include "sshtimemeasure.h"
-#include "sieve.h"
+
+/* Out integer to a stream. */
+void ssh_mprz_out_str(FILE *fp, unsigned int base,
+                      SshMPIntegerConst op)
+{
+  char *str;
+
+  str = ssh_mprz_get_str(op, base);
+  if (fp == NULL)
+    fputs(str, stdout);
+  else
+    fputs(str, fp);
+  ssh_free(str);
+}
+
 
 /* Printing of different types to the screen, these are helpful when
    trying to figure out what was wrong. And also, sometimes to compare
@@ -45,7 +217,7 @@ void print_int(char *str, SshMPInt op)
 {
   char *mstr;
 
-  mstr = ssh_mp_get_str(NULL, 10, op);
+  mstr = ssh_mp_get_str(NULL, 16, op);
   printf("%s %s\n", str, mstr);
   ssh_xfree(mstr);
 }
@@ -58,7 +230,7 @@ void print_mont(char *str, SshMPIntModQ op)
   ssh_mp_init(&a);
   ssh_mp_set_mpm(&a, op);
 
-  mstr = ssh_mp_get_str(NULL, 10, &a);
+  mstr = ssh_mp_get_str(NULL, 16, &a);
   printf("%s %s\n", str, mstr);
   ssh_xfree(mstr);
 
@@ -107,7 +279,7 @@ void test_int(int flag, int bits)
   ssh_mp_init(&g);
 
   printf("Running integer tests (%u bits):\n", bits);
-  
+
   printf(" * addition/subtraction test\n");
   for (j = 0; j < 1000; j++)
     {
@@ -223,14 +395,12 @@ void test_int(int flag, int bits)
       if (ssh_mp_cmp(&c, &b) != 0)
         {
           printf("error: multiplication/squaring failed.\n");
-          ssh_mp_dump(&a);
-          ssh_mp_dump(&b);
-          ssh_mp_dump(&c);
+          print_int("a = ", &a);
+          print_int("b = ", &b);
+          print_int("c = ", &c);
 
           print_int("a*a = ", &b);
-          ssh_mp_dump(&b);
           print_int("a^2 = ", &c);
-          ssh_mp_dump(&c);
           exit(1);
         }
     }
@@ -292,6 +462,11 @@ void test_int(int flag, int bits)
       if (ssh_mp_cmp_ui(&a, 3) < 0)
         continue;
 
+#ifdef SSHMATH_ODD_MODULUS
+      /* Make the modulus odd. */ 
+      ssh_mprz_set_bit(&a, 0); 
+#endif /* SSHMATH_ODD_MODULUS */
+
       k = random();
       ssh_mp_set_ui(&b, k);
       ssh_mp_mod(&b, &b, &a);
@@ -326,6 +501,11 @@ void test_int(int flag, int bits)
           if (ssh_mp_cmp_ui(&a, 3) < 0)
             continue;
 
+#ifdef SSHMATH_ODD_MODULUS
+      /* Make the modulus odd. */ 
+      ssh_mprz_set_bit(&a, 0); 
+#endif /* SSHMATH_ODD_MODULUS */ 
+
           true_rand(&b, bits);
           ssh_mp_abs(&b, &b);
 
@@ -353,7 +533,13 @@ void test_int(int flag, int bits)
       if (ssh_mp_cmp_ui(&a, 3) < 0)
         continue;
 
+#ifdef SSHMATH_ODD_MODULUS
+      /* Make the modulus odd. */ 
+      ssh_mprz_set_bit(&a, 0); 
+#endif /* SSHMATH_ODD_MODULUS */
+
       k = random();
+
       ssh_mp_set_ui(&b, k);
       ssh_mp_mod(&b, &b, &a);
       ssh_mp_set(&c, &b);
@@ -389,12 +575,20 @@ void test_int(int flag, int bits)
           if (ssh_mp_cmp_ui(&a, 3) < 0)
             continue;
 
+#ifdef SSHMATH_ODD_MODULUS
+      /* Make the modulus odd. */ 
+      ssh_mprz_set_bit(&a, 0); 
+#endif /* SSHMATH_ODD_MODULUS */
+
           true_rand(&b, bits);
           ssh_mp_mod(&b, &b, &a);
           true_rand(&c, bits);
+          ssh_mp_abs(&c, &c);
+          
           true_rand(&d, bits);
           ssh_mp_mod(&d, &d, &a);
           true_rand(&e, bits);
+          ssh_mp_abs(&e, &e);
 
           ssh_mp_powm(&f, &b, &c, &a);
           ssh_mp_powm(&g, &d, &e, &a);
@@ -431,8 +625,10 @@ void test_int(int flag, int bits)
         ssh_mp_add_ui(&a, &a, 1);
 
       k = random();
+   
       ssh_mp_set_ui(&b, k);
       true_rand(&e, bits);
+      ssh_mp_abs(&e, &e);
 
       ssh_mp_powm(&c, &b, &e, &a);
       ssh_mp_powm_ui(&d, k, &e, &a);
@@ -514,6 +710,7 @@ void test_int(int flag, int bits)
         }
     }
 
+#ifndef SSHMATH_ODD_MODULUS 
   if (flag)
     {
       printf(" * full exponentiation test with base\n");
@@ -553,7 +750,7 @@ void test_int(int flag, int bits)
             }
         }
     }
-
+#endif /* SSHMATH_ODD_MODULUS */
   if (flag)
     {
       printf(" * square tests\n");
@@ -644,7 +841,7 @@ void test_int(int flag, int bits)
           exit(1);
         }
     }
-  
+
   printf(" * buffer testing.\n");
   for (i = 0; i < 1000; i++)
     {
@@ -653,7 +850,8 @@ void test_int(int flag, int bits)
 
       true_rand(&a, bits);
       ssh_mp_abs(&a, &a);
-      
+    
+
       buffer_len = (bits+7)/8;
       buffer = ssh_xmalloc(buffer_len);
 
@@ -664,15 +862,12 @@ void test_int(int flag, int bits)
         {
           printf("error: buffer <-> integer failed (%u bytes).\n", buffer_len);
           print_int("a = ", &a);
-          ssh_mp_dump(&a);
           print_int("b = ", &b);
-          ssh_mp_dump(&b);
           ssh_xfree(buffer);
           exit(1);
         }
 
       ssh_xfree(buffer);
-
       /* Small buffer. */
       buffer_len = ssh_mp_get_size(&a, 256) + 4; /* (random() % buffer_len) + 1; */
       buffer     = ssh_xmalloc(buffer_len);
@@ -688,17 +883,14 @@ void test_int(int flag, int bits)
           printf("error: buffer <-> integer failed (%u bytes / %u).\n", buffer_len,
                  ssh_mp_get_size(&a, 256));
           print_int("a = ", &a);
-          ssh_mp_dump(&a);
           print_int("c = ", &c);
-          ssh_mp_dump(&c);
           print_int("b = ", &b);
-          ssh_mp_dump(&b);
           ssh_xfree(buffer);
           exit(1);
         }
       ssh_xfree(buffer);
     }
-  
+
   printf(" * conversion testing.\n");
   for (i = 0; i < 1000; i++)
     {
@@ -720,9 +912,7 @@ void test_int(int flag, int bits)
         {
           printf("error: conversion to integer failed in base %d.\n", base);
           print_int("a = ", &a);
-          ssh_mp_dump(&a);
           print_int("b = ", &b);
-          ssh_mp_dump(&b);
           printf("Output: %s\n", str);
           ssh_xfree(str);
           exit(1);
@@ -752,9 +942,7 @@ void test_int(int flag, int bits)
         {
           printf("error: automatic recognition of base %d.\n", base);
           print_int("a = ", &a);
-          ssh_mp_dump(&a);
           print_int("b = ", &b);
-          ssh_mp_dump(&b);
           printf("Output: %s\n", str);
           ssh_xfree(str);
           exit(1);
@@ -762,6 +950,32 @@ void test_int(int flag, int bits)
       ssh_xfree(str);
 
     }
+  for (i = 0; i < 10000; i += 19)
+    {
+      char *tt;
+      size_t tt_len;
+      ssh_mprz_rand(&a, i+1);
+      tt = ssh_mprz_get_binary(&tt_len, &a);
+      if (ssh_mprz_set_binary(&b, tt, tt_len) == tt_len)
+        {
+          if (ssh_mprz_cmp(&a, &b) != 0)
+            {
+              printf("error: binary encoding failed with i = %u.\n", i);
+              print_int("a = ", &a);
+              print_int("b = ", &b);
+              ssh_xfree(tt);
+              exit(1);
+            }
+        }
+      else
+        {
+          printf("error: could not binary decode i = %u.\n", i);
+          print_int("a = ", &a);
+          ssh_xfree(tt);
+          exit(1);
+        }
+      ssh_xfree(tt);
+    }
 
   ssh_mp_clear(&a);
   ssh_mp_clear(&b);
@@ -789,13 +1003,12 @@ void test_mod(int flag, int bits)
 
   printf(" * random moduli search\n");
 
-  do
-    {
-      ssh_mp_rand(&m1, bits);
-      while (ssh_mp_next_prime(&m1, &m1) == FALSE)
-        ssh_mp_rand(&m1, bits);
-    }
-  while (ssh_mpm_init_m(&m0, &m1) == FALSE);
+  ssh_mp_rand(&m1, bits);
+#if 0
+  while (ssh_mp_next_prime(&m1, &m1) == FALSE)
+    ssh_mp_rand(&m1, bits);
+#endif
+  ssh_mpm_init_m(&m0, &m1);
 
   ssh_mpm_init(&a0, &m0);
   ssh_mpm_init(&b0, &m0);
@@ -874,7 +1087,6 @@ void test_mod(int flag, int bits)
           print_mont("  a0      = ", &a0);
           print_mont("  b0      = ", &b0);
           print_mont("  a0 * b0 = ", &c0);
-          ssh_mpm_dump(&c0);
           exit(1);
         }
     }
@@ -965,6 +1177,7 @@ void test_mod(int flag, int bits)
         }
     }
 
+#if 0
   printf(" * div 2exp test\n");
   for (i = 0; i < 1000; i++)
     {
@@ -987,6 +1200,7 @@ void test_mod(int flag, int bits)
           exit(1);
         }
     }
+#endif
 
 
 
@@ -2253,6 +2467,398 @@ void test_mod(int flag, int bits)
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -2281,7 +2887,9 @@ void timing_int(int bits)
   SshMPIntStruct a, b, c, d, e, f[100];
   struct SshTimeMeasureRec tmit = SSH_TIME_MEASURE_INITIALIZER;
   unsigned int i, j;
-  SshMPPowmBaseStruct base;
+#ifndef SSHMATH_ODD_MODULUS
+  SshMPPowmBaseStruct base; 
+#endif 
 
   ssh_mp_init(&a);
   ssh_mp_init(&b);
@@ -2398,13 +3006,12 @@ void timing_int(int bits)
     }
   while (ssh_mp_get_size(&b, 2) < bits-1);
 
+#ifndef SSHMATH_ODD_MODULUS
+
   /* Create the base. */
   ssh_mp_sub_ui(&a, &f[2], 1);
   ssh_mp_powm_with_base_init(&b, &f[2], &a, &base);
 
-  if (base.defined == FALSE)
-    ssh_fatal("error: could not define base.");
-
   printf("Timing modexp with fixed base [%u^%u %% %u = %u] \n",
          bits, bits, bits, bits);
 
@@ -2414,6 +3021,7 @@ void timing_int(int bits)
     ssh_mp_powm_with_base(&a, &f[j + 1], &base);
   ssh_time_measure_stop(&tmit);
 
+
   printf("  * %g modexps per sec\n",
          ((double)10) /
          ((double) ssh_time_measure_get(&tmit,
@@ -2421,7 +3029,7 @@ void timing_int(int bits)
           1000000.0));
 
   ssh_mp_powm_with_base_clear(&base);
-
+#endif /* SSHMATH_ODD_MODULUS */
 
 #define ENTROPY_BITS 256
 
@@ -2476,7 +3084,7 @@ void timing_int(int bits)
 
     buffer_len = (bits + 7)/8;
     buffer     = ssh_xmalloc(buffer_len);
-    
+
     for (j = 0, i = 0; i < 10000; i++, j += 2)
       {
         ssh_mp_get_buf(buffer, buffer_len, &f[j % 20]);
@@ -2492,8 +3100,8 @@ void timing_int(int bits)
                                         SSH_TIME_GRANULARITY_MICROSECOND) /
           1000000.0));
 
-  
-  
+
+
   ssh_mp_clear(&a);
   ssh_mp_clear(&b);
   ssh_mp_clear(&c);
@@ -2523,8 +3131,8 @@ void timing_modular(int bits)
   while (ssh_mp_get_size(&a, 2) < bits - 1);
 
   printf("Timing modular arithmetic.\n");
-  if (ssh_mpm_init_m(&m, &a) == FALSE)
-    ssh_fatal("timing_modular: could not initialize modular arithmetic.");
+
+  ssh_mpm_init_m(&m, &a);
 
   printf("Bits = %u\n", bits);
 
@@ -2814,6 +3422,324 @@ void timing_modular(int bits)
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -2856,6 +3782,8 @@ typedef struct CommandRec
 #define C_ECP     8
 #define C_EC2N    9
 #define C_FEC2N   10
+#define C_OEF     11
+#define C_ECOEF   12
 
 #define C_BITS     20
 #define C_BITS_ADV 21
@@ -2900,6 +3828,12 @@ const Command commands[] =
 
 
 
+
+
+
+
+
+
   { NULL }
 };
 
@@ -2938,6 +3872,10 @@ void usage(void)
 
 
 
+
+
+
+
          );
   exit(1);
 }
@@ -2965,12 +3903,12 @@ void my_fatal_cb(const char *message, void *context)
 int main(int ac, char *av[])
 {
   int i, all, itr, type, args;
-  int gf2n, mod, integer, ecp, ec2n, fec2n, poly2n, bpoly,
+  int gf2n, mod, oef, integer, ecp, ecoef, ec2n, fec2n, poly2n, bpoly,
     bits, bits_advance, timing;
   int fatal_cb;
 
   printf("Arithmetic library test suite\n"
-         "Copyright (C) 1998 SSH Communications Security, Ltd.\n"
+         "Copyright (C) 1998-2001 SSH Communications Security, Ltd.\n"
          "              All rights reserved.\n"
          "\n"
          "Features: \n"
@@ -2983,10 +3921,15 @@ int main(int ac, char *av[])
 
 
 
+
+
+
+
+
          "\n");
 
   /* Randomize the random number generator. */
-  srandom(ssh_time());
+  ssh_rand_seed((unsigned int)(ssh_time()));
 
   /* Don't use this if you want to test the mathlibrary :) */
   /*extra_test(); */
@@ -3004,9 +3947,11 @@ int main(int ac, char *av[])
 
   gf2n     = 0;
   integer  = 1;
+  oef      = 0;
   mod      = 0;
   bpoly    = 0;
   ecp      = 0;
+  ecoef    = 0;
   ec2n     = 0;
   fec2n    = 0;
   poly2n   = 0;
@@ -3052,6 +3997,18 @@ int main(int ac, char *av[])
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -3141,6 +4098,22 @@ int main(int ac, char *av[])
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
