@@ -1,10 +1,10 @@
 /*
 
   t-sophie-germain.c
-  
+
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (C) 1996,1997 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996,1997, 2001 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Created: Wed Jul 23 22:36:43 1997 [mkojo]
@@ -19,16 +19,103 @@
   */
 
 /*
- * $Id: t-sophie-germain.c,v 1.15 2000/11/13 04:17:56 mkojo Exp $
- * $Log: t-sophie-germain.c,v $
+ * $Id: t-sophie-germain.c,v 1.18 2001/09/10 10:26:02 tmo Exp $
+ * $Log: t-sophie-germain.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
 #include "sshincludes.h"
-#include "sshmp.h" /* was "gmp.h" */
-#include "sieve.h"
+#include "sshmp.h"
 #include "sshtimemeasure.h"
 
+/* Out integer to a stream. */
+void ssh_mprz_out_str(FILE *fp, unsigned int base,
+                      SshMPIntegerConst op)
+{
+  char *str;
+
+  str = ssh_mprz_get_str(op, base);
+  if (fp == NULL)
+    fputs(str, stdout);
+  else
+    fputs(str, fp);
+  ssh_free(str);
+}
+
+
 /*
   This implementation uses a slightly different sieving idea, which might
   be less good.
@@ -59,7 +146,7 @@
 
   We need to compute these sequences fast. Thus having tables for all
   y_n (mod p_i), b(2*a + b - 1) (mod p_i) and nb^2 (mod p_i) seems
-  necessary. 
+  necessary.
  */
 
 int find_safe_prime(unsigned int sieve_size, SshMPInt a, SshMPInt b,
@@ -78,7 +165,7 @@ int find_safe_prime(unsigned int sieve_size, SshMPInt a, SshMPInt b,
   for (len = 0, p = 2; p; p = ssh_sieve_next_prime(p, &sieve), len++)
     ;
   len--;
-  
+
   ssh_mp_init(&v);
   ssh_mp_init(&s);
   ssh_mp_init(&ret);
@@ -86,7 +173,7 @@ int find_safe_prime(unsigned int sieve_size, SshMPInt a, SshMPInt b,
 
   ssh_mp_init(&b2);
   ssh_mp_init(&ba);
-  
+
   /* Compute b2 = b^2. */
   ssh_mp_square(&b2, b);
 
@@ -103,7 +190,7 @@ int find_safe_prime(unsigned int sieve_size, SshMPInt a, SshMPInt b,
   ssh_mp_div_ui(&aux, &aux, 2);
 
   printf("Initializing tables.\n");
-  
+
   y_table   = ssh_xmalloc(len*sizeof(*y_table));
   b_table   = ssh_xmalloc(len*sizeof(*b_table));
   b2_table  = ssh_xmalloc(len*sizeof(*b2_table));
@@ -123,7 +210,7 @@ int find_safe_prime(unsigned int sieve_size, SshMPInt a, SshMPInt b,
 
   /* Initialize the return table. */
   ssh_mp_set(&ret, a);
-  
+
   /* We assume that only 2^30 choices are needed. */
   for (i = 0, mult_k = 0; i < steps; i++)
     {
@@ -163,7 +250,7 @@ int find_safe_prime(unsigned int sieve_size, SshMPInt a, SshMPInt b,
             b_table[j] -= p;
         }
       if (rv == FALSE)
-        continue; 
+        continue;
 
       /* Inform the user. */
       printf("x");
@@ -173,7 +260,7 @@ int find_safe_prime(unsigned int sieve_size, SshMPInt a, SshMPInt b,
       ssh_mp_mul_ui(&s, b, i - mult_k);
       ssh_mp_add(&ret, &ret, &s);
       ssh_mp_set(&v, &ret);
-      
+
       /* Remember the last try. */
       mult_k = i;
 
@@ -230,7 +317,7 @@ int find_safe_prime(unsigned int sieve_size, SshMPInt a, SshMPInt b,
 
   ssh_mp_clear(&ba);
   ssh_mp_clear(&b2);
-  
+
   ssh_mp_clear(&v);
   ssh_mp_clear(&s);
   ssh_mp_clear(&ret);
@@ -249,8 +336,8 @@ int find_safe_prime(unsigned int sieve_size, SshMPInt a, SshMPInt b,
    Useful in a way such that one needs to do ECPP for only (p-1)/2, although
    in practice ECPP seems to do this itself. (Which is nice.)
 
-   The table should contain the factors that are provably prime. 
-   
+   The table should contain the factors that are provably prime.
+
    */
 
 int primality_proof(SshMPInt n, SshMPInt table, unsigned int table_n)
@@ -269,14 +356,14 @@ int primality_proof(SshMPInt n, SshMPInt table, unsigned int table_n)
       printf(" -- given input was definitely composite.\n");
       return 0;
     }
-    
+
   ssh_mp_init(&f);
   ssh_mp_init(&u);
   ssh_mp_init(&g);
 
   ssh_mp_set_ui(&f, 1);
   ssh_mp_sub_ui(&u, n, 1);
-  
+
   for (i = 0; i < table_n; i++)
     {
       printf(" -- prime factor no. %u is = ", i+1);
@@ -288,7 +375,7 @@ int primality_proof(SshMPInt n, SshMPInt table, unsigned int table_n)
           printf(" -- given factor was definitely composite.\n");
           continue;
         }
-      
+
       ssh_mp_mod(&g, &u, &table[i]);
       if (ssh_mp_cmp_ui(&g, 0) == 0)
         {
@@ -325,7 +412,7 @@ int primality_proof(SshMPInt n, SshMPInt table, unsigned int table_n)
       ssh_mp_sub_ui(&d, n, 1);
 
       printf(" -- proceeding to find the suitable proving elements.\n");
-      
+
       for (i = 0; i < table_n; i++)
         {
           int j;
@@ -385,7 +472,7 @@ int primality_proof(SshMPInt n, SshMPInt table, unsigned int table_n)
 
   if (rv == 1)
     printf(" -- Pocklington's primality proof was a success.\n");
-  
+
   ssh_mp_clear(&f);
   ssh_mp_clear(&u);
   ssh_mp_clear(&g);
@@ -431,7 +518,7 @@ double mypow_base16(long e)
       sign = 1;
       e = -e;
     }
-  
+
   while (e)
     {
       if (e & 1)
@@ -452,7 +539,7 @@ double f_series(long m, long c)
 
   /* Take a small enough epsilon. */
 #define EPSILON 1e-16
-  
+
   s = 0.0;
 
   /* Sum the series up to c. */
@@ -486,7 +573,7 @@ double f_series(long m, long c)
 long pi_digit(long c)
 {
   double s1, s2, s3, s4, d;
-  
+
   s1 = f_series(1, c);
   s2 = f_series(4, c);
   s3 = f_series(5, c);
@@ -513,13 +600,13 @@ void pi_comp_bbp(SshMPInt ret, int bits)
 
   /* Set the first two bits. */
   ssh_mp_set_ui(ret, 3);
-  
+
   for (i = 2; i < bits; i += 4)
     {
       long d;
 
       d = pi_digit((i-2)/4);
-      
+
       ssh_mp_mul_ui(ret, ret, 16);
       ssh_mp_add_ui(ret, ret, d);
     }
@@ -533,7 +620,7 @@ void pi_comp_bbp(SshMPInt ret, int bits)
 /* A quadratically convergent algorithm for computing the Pi applying
    the algorithm by Borwein and Borwein (it is related to the AGM). This
    is slightly easier to write than the Gauss-Legendre method by
-   Brent-Salamin. 
+   Brent-Salamin.
 
    The function returns the asked number of bits of the Pi in the integer
    such that the decimal point is at the position bits
@@ -550,19 +637,19 @@ void pi_comp_agm(SshMPInt ret, int bits)
   guard = bits / 50;
   if (guard < 32)
     guard = 32;
-  
+
   dp    = bits - 2 + guard;
-  
+
   if (dp <= 0)
     {
       printf("error: pi was not computed due to too few bits asked.\n");
       exit(1);
     }
-  
+
   ssh_mp_init(&a);
   ssh_mp_init(&b);
   ssh_mp_init(&p);
-  
+
   ssh_mp_init(&an);
   ssh_mp_init(&bn);
   ssh_mp_init(&pn);
@@ -571,7 +658,7 @@ void pi_comp_agm(SshMPInt ret, int bits)
   ssh_mp_init(&t1);
   ssh_mp_init(&t2);
   ssh_mp_init(&t3);
-  
+
   /* Compute the initial values. */
 
   /* The fixed precision operations:
@@ -589,7 +676,7 @@ void pi_comp_agm(SshMPInt ret, int bits)
      sqrt(a) -> sqrt(a*R^2) = sqrt(a)*R
 
      */
-       
+
   /* a_0 = sqrt(2) */
   ssh_mp_set_bit(&a, dp + 1 + dp);
   ssh_mp_sqrt(&a, &a);
@@ -618,7 +705,7 @@ void pi_comp_agm(SshMPInt ret, int bits)
       /* Compute a_k+1 */
       ssh_mp_add(&an, &t1, &t2);
       ssh_mp_div_2exp(&an, &an, 1);
-      
+
       /* Compute sqrt(a)*(1 + b) */
       ssh_mp_set_ui(&t2, 0);
       ssh_mp_set_bit(&t2, dp);
@@ -658,7 +745,7 @@ void pi_comp_agm(SshMPInt ret, int bits)
   /* Remove the guard bits and copy. */
   ssh_mp_div_2exp(&p, &p, guard);
   ssh_mp_set(ret, &p);
-  
+
   /* Clean up. */
   ssh_mp_clear(&a);
   ssh_mp_clear(&b);
@@ -675,8 +762,8 @@ void pi_comp_agm(SshMPInt ret, int bits)
 
 /*
   Here is a Pari/GP program for generating an IKE prime. This is useful
-  for checking purposes. 
-  
+  for checking purposes.
+
   { ikeprime(b, k) =
     \\ Enough of Pi.
     epi = floor(Pi*2^(b-2*fixedbits-2)) + k;
@@ -694,7 +781,7 @@ void ike_start(SshMPInt ret, int bits, long my_index, int fixed_bits)
              2*fixed_bits + 2);
       exit(1);
     }
-  
+
   ssh_mp_init(&pi);
 
 #ifdef PI_VERIFY
@@ -718,7 +805,7 @@ void ike_start(SshMPInt ret, int bits, long my_index, int fixed_bits)
 #else
   pi_comp_agm(&pi, bits - 2*fixed_bits);
 #endif
-  
+
   ssh_mp_add_ui(&pi, &pi, my_index);
 
   ssh_mp_set_ui(ret, 0);
@@ -729,7 +816,7 @@ void ike_start(SshMPInt ret, int bits, long my_index, int fixed_bits)
 
   ssh_mp_mul_2exp(&pi, &pi, fixed_bits);
   ssh_mp_add(ret, ret, &pi);
-  
+
   ssh_mp_clear(&pi);
 }
 
@@ -740,7 +827,7 @@ int ike_find(SshMPInt ret, int bits, long my_index, int max_steps,
 {
   SshMPIntStruct start, add;
   int rv;
-  
+
   ssh_mp_init(&start);
   ssh_mp_init(&add);
   ike_start(&start, bits, my_index + id, fixed_bits);
@@ -750,7 +837,7 @@ int ike_find(SshMPInt ret, int bits, long my_index, int max_steps,
   printf("IKE start value (%u bits): \n", bits);
   ssh_mp_out_str(NULL, 10, &start);
   printf("\n");
-  
+
   rv =  find_safe_prime(sieve_size, &start, &add, ret, max_steps,
                         id + my_index, num);
 
@@ -769,7 +856,7 @@ void ike_batch(char *filename, int from_bits, int to_bits, int step_bits,
   int *status;
   unsigned long status_len, my_index;
   unsigned long i, done;
-  
+
   if (to_bits <= from_bits)
     {
       printf("error: batch mode demands larger range.\n");
@@ -784,7 +871,7 @@ void ike_batch(char *filename, int from_bits, int to_bits, int step_bits,
       exit(1);
     }
   printf("Output to '%s'.\n", filename);
-  
+
   status_len = (to_bits - from_bits)/step_bits;
 
   status = ssh_xmalloc(sizeof(*status) * status_len);
@@ -796,7 +883,7 @@ void ike_batch(char *filename, int from_bits, int to_bits, int step_bits,
   fprintf(fp, "SOPHIE GERMAIN PRIME SEARCH\n");
   fprintf(fp, "FIXED %u bits.\n", fixed_bits);
   fflush(fp);
-  
+
   for (my_index = start_index; my_index < ((unsigned long)1 << 30);
        my_index += max_steps)
     {
@@ -816,7 +903,7 @@ void ike_batch(char *filename, int from_bits, int to_bits, int step_bits,
 
           fprintf(fp, ".");
           fflush(fp);
-          
+
           /* Compute the bits for this status. */
           bits = from_bits + step_bits*i;
 
@@ -846,7 +933,7 @@ void ike_batch(char *filename, int from_bits, int to_bits, int step_bits,
 
   printf("Finished the range searching.\n");
 }
-               
+
 void gather(char *av[], int len, int pock)
 {
   FILE *fp;
@@ -856,16 +943,16 @@ void gather(char *av[], int len, int pock)
   char *buffer, *buf;
   size_t buf_len;
   int i, j, fixed_bits;
-  
+
   table_n    = 10;
   table      = ssh_xmalloc(sizeof(*table)*table_n);
   table_used = 0;
 
   fixed_bits = 64;
-  
+
   buf_len = 100000;
   buffer = ssh_xmalloc(sizeof(char) * buf_len);
-  
+
   for (i = 0; i < len; i++)
     {
       int bits;
@@ -881,7 +968,7 @@ void gather(char *av[], int len, int pock)
               SshMPInt mp;
 
               printf("Prime with bits %u found.\n", bits);
-              
+
               buf = fgets(buffer, buf_len, fp);
               if (buf == NULL)
                 break;
@@ -961,14 +1048,14 @@ void gather(char *av[], int len, int pock)
 
       if (pock)
         ptable[i] = primality_proof(table[i], &mtab[0], 1);
-      
+
       ssh_mp_clear(&p);
       ssh_mp_clear(&p1);
       for (j = 0; j < 10; j++)
         ssh_mp_clear(&mtab[j]);
     }
 
-  
+
   /* Now printout the stuff. */
 
   printf("Table for IKE primes in <bits, index> convention:\n");
@@ -982,7 +1069,7 @@ void gather(char *av[], int len, int pock)
       ssh_mp_sub(&tmp, table[i], &tmp);
       ssh_mp_div_2exp(&tmp, &tmp, fixed_bits);
 
-      
+
       bits = ssh_mp_get_size(table[i], 2);
       ike_index = ssh_mp_get_ui(&tmp);
 
@@ -1026,9 +1113,9 @@ void expand(char *av[], int len, int pock)
   failures = 0;
 
   fixed_bits = 64;
-  
+
   printf("Trying to find suitable expandable IKE primes (based on pi).\n");
-  
+
   for (i = 0; i < len; i++)
     {
       printf("Handling file '%s':\n", av[i]);
@@ -1040,7 +1127,7 @@ void expand(char *av[], int len, int pock)
             {
               continue;
             }
-          
+
           if (sscanf(buf, " < %lu , %lu > ", &bits, &pindex) == 2)
             {
               SshMPInt mp;
@@ -1062,16 +1149,16 @@ void expand(char *av[], int len, int pock)
 
               table[table_used] = mp;
               table_used++;
-              
+
               printf(" ---- added to the list.\n");
             }
         }
     }
 
   /* XXX Sort. */
-  
+
   /* Same as with gathering. */
-  
+
   /* Do other processing. */
   printf("Running checks:\n");
 
@@ -1096,20 +1183,20 @@ void expand(char *av[], int len, int pock)
 
       if (pock)
         ptable[i] = primality_proof(table[i], &mtab[0], 1);
-      
+
       ssh_mp_clear(&p);
       ssh_mp_clear(&p1);
       for (j = 0; j < 10; j++)
         ssh_mp_clear(&mtab[j]);
     }
 
-  
+
   /* Now printout the stuff. */
 
   printf("Table for IKE primes in <bits, index> convention:\n");
 
   printf("(Fixed %u bits.)\n", fixed_bits);
-  
+
   for (i = 0; i < table_used; i++)
     {
       SshMPIntStruct tmp;
@@ -1120,7 +1207,7 @@ void expand(char *av[], int len, int pock)
       ssh_mp_sub(&tmp, table[i], &tmp);
       ssh_mp_div_2exp(&tmp, &tmp, fixed_bits);
 
-      
+
       bits = ssh_mp_get_size(table[i], 2);
       ike_index = ssh_mp_get_ui(&tmp);
 
@@ -1153,7 +1240,7 @@ int main(int ac, char *av[])
     ike_id, ike_num, ike_fixed_bits;
   unsigned long max_steps, my_index;
   char *filename;
-  
+
   ssh_mp_init(&input);
   ssh_mp_init(&add);
   ssh_mp_init(&prime);
@@ -1170,17 +1257,17 @@ int main(int ac, char *av[])
   ike_num = 1;
 
   pi_bits = 0;
-  
+
   sieve_size = 20000;
 
   filename = NULL;
-  
+
   my_index = 0;
   max_steps = ((unsigned long)1 << 30);
-  
+
   ssh_mp_set_ui(&input, 1);
   ssh_mp_set_ui(&add, 2);
-  
+
   for (pos = 1; pos < ac; pos++)
     {
       if (strcmp(av[pos], "-fb") == 0)
@@ -1282,7 +1369,7 @@ int main(int ac, char *av[])
             printf("error: argument assumed.\n");
           continue;
         }
-      
+
       if (strcmp(av[pos], "-s") == 0)
         {
           if (pos + 1 < ac)
@@ -1350,7 +1437,7 @@ int main(int ac, char *av[])
             printf("error: argument assumed.\n");
           exit(0);
         }
-      
+
       if (strcmp(av[pos], "-h") == 0)
         {
           printf("Program for finding Sophie Germain primes.\n");
@@ -1388,17 +1475,17 @@ int main(int ac, char *av[])
     {
       SshMPIntStruct pi_agm, pi_bbp;
       struct SshTimeMeasureRec tmit = SSH_TIME_MEASURE_INITIALIZER;
-      
+
       ssh_mp_init(&pi_agm);
       ssh_mp_init(&pi_bbp);
-      
+
       printf("Computing %u bits of Pi for your pleasure...\n", pi_bits);
 
       printf("Using AGM class algorithm Borwein-Borwein:\n");
 
       ssh_time_measure_reset(&tmit);
       ssh_time_measure_start(&tmit);
-      
+
       pi_comp_agm(&pi_agm, pi_bits);
 
       ssh_time_measure_stop(&tmit);
@@ -1437,7 +1524,7 @@ int main(int ac, char *av[])
         {
           unsigned int digits;
           int pos, start;
-          
+
           printf("The Bailey-Borwein-Plouffe digit extraction algorithm is\n"
                  "too slow to be used for full Pi computation for your\n"
                  "requested bit amount.\n"
@@ -1471,7 +1558,7 @@ int main(int ac, char *av[])
         }
       exit(1);
     }
-  
+
   /* Compute GCD. */
   {
     SshMPIntStruct g;
