@@ -11,8 +11,74 @@ Parse pgp keyblobs.
 
 */
 /*
- * $Id: pgp_key.c,v 1.14 2000/06/14 00:42:43 kivinen Exp $
- * $Log: pgp_key.c,v $
+ * $Id: pgp_key.c,v 1.16.16.1 2001/11/12 06:27:05 sjl Exp $
+ * $Log: pgp_key.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -148,17 +214,17 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
           ssh_hash_final(hash, digest);
           for (i = 0; i < 8; i++)
             {
-              snprintf(&(print[i * 3]), 
-                       sizeof (print) - (i * 3), 
-                       "%02X     ",
-                       (((unsigned int)digest[i])));
+              ssh_snprintf(&(print[i * 3]), 
+                           sizeof (print) - (i * 3), 
+                           "%02X     ",
+                           (((unsigned int)digest[i])));
             }
           for (i = 0; i < 8; i++)
             {
-              snprintf(&(print[((i + 8) * 3) + 1]), 
-                       sizeof (print) - (((i + 8) * 3) + 1), 
-                       "%02X     ",
-                       (((unsigned int)digest[i + 8])));
+              ssh_snprintf(&(print[((i + 8) * 3) + 1]), 
+                           sizeof (print) - (((i + 8) * 3) + 1), 
+                           "%02X     ",
+                           (((unsigned int)digest[i + 8])));
             }
           key_fingerprint = ssh_xstrdup(print); 
           ssh_hash_free(hash);
@@ -292,18 +358,18 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
           ssh_hash_update(hash, pubkey_data, pubkey_data_len);
           ssh_hash_final(hash, digest);
           for (i = 0; i < 5; i++) {
-            snprintf(&(print[i * 5]), 
-                     sizeof (print) - (i * 5), 
-                     "%04X     ",
-                     ((((unsigned int)digest[i * 2]) << 8) |
-                      ((unsigned int)digest[(i * 2) + 1])));
+            ssh_snprintf(&(print[i * 5]), 
+                         sizeof (print) - (i * 5), 
+                         "%04X     ",
+                         ((((unsigned int)digest[i * 2]) << 8) |
+                          ((unsigned int)digest[(i * 2) + 1])));
           }
           for (i = 0; i < 5; i++) {
-            snprintf(&(print[((i + 5) * 5) + 1]), 
-                     sizeof (print) - (((i + 5) * 5) + 1), 
-                     "%04X     ",
-                     ((((unsigned int)digest[(i + 5) * 2]) << 8) |
-                      ((unsigned int)digest[((i + 5) * 2) + 1])));
+            ssh_snprintf(&(print[((i + 5) * 5) + 1]), 
+                         sizeof (print) - (((i + 5) * 5) + 1), 
+                         "%04X     ",
+                         ((((unsigned int)digest[(i + 5) * 2]) << 8) |
+                          ((unsigned int)digest[((i + 5) * 2) + 1])));
           }
           key_fingerprint = ssh_xstrdup(print); 
           key_id_high = ((((SshUInt32)digest[12]) << 24) |
@@ -936,15 +1002,15 @@ SshPrivateKey ssh_pgp_define_secret_rsa_key(SshPublicKey public_key,
                                SSH_PKF_END);
   if (cs != SSH_CRYPTO_OK)
     goto failed;
-  cs = ssh_private_key_generate(&key,
-                                SSH_PGP_CANONICAL_RSA_NAME,
-                                SSH_PKF_MODULO_N, &n,
-                                SSH_PKF_PUBLIC_E, &e,
-                                SSH_PKF_PRIME_P, p,
-                                SSH_PKF_PRIME_Q, q,
-                                SSH_PKF_SECRET_D, d,
-                                SSH_PKF_INVERSE_U, u,
-                                SSH_PKF_END);
+  cs = ssh_private_key_define(&key,
+                              SSH_PGP_CANONICAL_RSA_NAME,
+                              SSH_PKF_MODULO_N, &n,
+                              SSH_PKF_PUBLIC_E, &e,
+                              SSH_PKF_PRIME_P, p,
+                              SSH_PKF_PRIME_Q, q,
+                              SSH_PKF_SECRET_D, d,
+                              SSH_PKF_INVERSE_U, u,
+                              SSH_PKF_END);
   if (cs != SSH_CRYPTO_OK)
     {
       key = NULL;
@@ -980,13 +1046,13 @@ SshPrivateKey ssh_pgp_define_secret_elgamal_key(SshPublicKey public_key,
   if (cs != SSH_CRYPTO_OK)
     goto failed;
 
-  cs = ssh_private_key_generate(&key,
-                                SSH_PGP_CANONICAL_ELGAMAL_NAME,
-                                SSH_PKF_PRIME_P, &p,
-                                SSH_PKF_GENERATOR_G, &g,
-                                SSH_PKF_PUBLIC_Y, &y,
-                                SSH_PKF_SECRET_X, x,
-                                SSH_PKF_END);
+  cs = ssh_private_key_define(&key,
+                              SSH_PGP_CANONICAL_ELGAMAL_NAME,
+                              SSH_PKF_PRIME_P, &p,
+                              SSH_PKF_GENERATOR_G, &g,
+                              SSH_PKF_PUBLIC_Y, &y,
+                              SSH_PKF_SECRET_X, x,
+                              SSH_PKF_END);
   if (cs != SSH_CRYPTO_OK)
     {
       key = NULL;
@@ -1025,14 +1091,14 @@ SshPrivateKey ssh_pgp_define_secret_dsa_key(SshPublicKey public_key,
   if (cs != SSH_CRYPTO_OK)
     goto failed;
 
-  cs = ssh_private_key_generate(&key,
-                                SSH_PGP_CANONICAL_DSA_NAME,
-                                SSH_PKF_PRIME_P, &p,
-                                SSH_PKF_PRIME_Q, &q,
-                                SSH_PKF_GENERATOR_G, &g,
-                                SSH_PKF_PUBLIC_Y, &y,
-                                SSH_PKF_SECRET_X, x,
-                                SSH_PKF_END);
+  cs = ssh_private_key_define(&key,
+                              SSH_PGP_CANONICAL_DSA_NAME,
+                              SSH_PKF_PRIME_P, &p,
+                              SSH_PKF_PRIME_Q, &q,
+                              SSH_PKF_GENERATOR_G, &g,
+                              SSH_PKF_PUBLIC_Y, &y,
+                              SSH_PKF_SECRET_X, x,
+                              SSH_PKF_END);
   if (cs != SSH_CRYPTO_OK)
     {
       key = NULL;
