@@ -2,15 +2,15 @@
 
 pty-ptmx.c
 
-Author: Tatu Ylonen <ylo@ssh.fi>
-        Sami Lehtinen <sjl@ssh.fi>
+  Author: Tatu Ylonen <ylo@ssh.fi>
+          Sami Lehtinen <sjl@ssh.fi>
 
-Copyright (c) 1997 SSH Communications Security, Finland
-                   All rights reserved
+  Copyright (c) 1997-2001 SSH Communications Security, Finland
+                          All rights reserved
 
 Allocating a pty using /dev/ptmx.  This is used e.g. on Solaris 2.x
 and SysVr4.  (Note that Solaris 2.3 also has bsd-style ptys, but they
-simply do not work.)
+simply do not work.) Also used in e.g. Linux with Unix98 style ptys.
 
 */
 
@@ -51,9 +51,11 @@ Boolean ssh_pty_internal_allocate(int *ptyfd, int *ttyfd, char *namebuf)
     }
   {
 #ifdef HAVE_SIGNAL
+    /* Setting/restoring the SIGCHLD signal handler is needed, because
+       some systems implement granpt() by using a child process. */
     void (*old_handler)(int);
     old_handler = signal(SIGCHLD, SIG_DFL);
-#endif
+#endif /* HAVE_SIGNAL */
     if (grantpt(ptm) < 0)
       {
         ssh_warning("grantpt: %.100s", strerror(errno));
@@ -61,7 +63,7 @@ Boolean ssh_pty_internal_allocate(int *ptyfd, int *ttyfd, char *namebuf)
       }
 #ifdef HAVE_SIGNAL
     signal(SIGCHLD, old_handler);
-#endif
+#endif /* HAVE_SIGNAL */
   }
 
   if (unlockpt(ptm) < 0)
@@ -133,7 +135,7 @@ Boolean ssh_pty_internal_make_ctty(int *ttyfd, const char *ttyname)
   /* Make it our controlling tty. */
 #ifdef TIOCSCTTY
   ssh_debug("Setting controlling tty using TIOCSCTTY.");
-  /* We ignore errors from this, because HPSUX defines TIOCSCTTY, but returns
+  /* We ignore errors from this, because HP-UX defines TIOCSCTTY, but returns
      EINVAL with these arguments, and there is absolutely no documentation. */
   ioctl(*ttyfd, TIOCSCTTY, NULL);
 #endif /* TIOCSCTTY */
@@ -145,12 +147,12 @@ Boolean ssh_pty_internal_make_ctty(int *ttyfd, const char *ttyname)
 #ifdef CRAY
   ssh_debug("Setting controlling tty using TCSETCTTY.");
   ioctl(*ttyfd, TCSETCTTY, NULL);
-#endif
+#endif /* CRAY */
 
 #ifdef HAVE_SETPGID
   /* This appears to be necessary on some machines...  */
   setpgid(0, 0);
-#endif
+#endif /* HAVE_SETPGID */
 
   fd = open(ttyname, O_RDWR);
   if (fd < 0)
