@@ -10,8 +10,90 @@ Copyright (c) 1997 SSH Communications Security, Finland
 */
 
 /*
- * $Id: sshunixptystream.c,v 1.9 2000/06/14 00:43:04 kivinen Exp $
- * $Log: sshunixptystream.c,v $
+ * $Id: sshunixptystream.c,v 1.16 2001/08/07 16:52:20 nordlund Exp $
+ * $Log: sshunixptystream.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -45,7 +127,7 @@ typedef struct {
   int exit_status;
 } *PtyStream;
 
-extern const SshStreamMethodsStruct ssh_pty_methods;
+extern SshStreamMethodsStruct ssh_pty_methods;
 
 /* This callback is called after the parent has exited; this will
    call the application callback so that it gets notified of the EOF
@@ -223,6 +305,7 @@ SshPtyStatus ssh_pty_allocate_and_fork(uid_t owner_uid, gid_t owner_gid,
         }
     }
 
+#ifndef VXWORKS
   /* Initialize SIGCHLD handling.  This will ensure the SIGCHLD won't get
      delivered until we register the handler for the new process below. */
   ssh_sigchld_initialize();
@@ -274,6 +357,7 @@ SshPtyStatus ssh_pty_allocate_and_fork(uid_t owner_uid, gid_t owner_gid,
       *master_return = NULL;
       return SSH_PTY_CHILD_OK;
     }
+#endif /* VXWORKS */
 
   /* Parent */
   pty = ssh_xcalloc(sizeof(*pty), 1);
@@ -286,7 +370,9 @@ SshPtyStatus ssh_pty_allocate_and_fork(uid_t owner_uid, gid_t owner_gid,
   pty->callback_context = NULL;
   pty->status_returned = FALSE;
   pty->exit_status = -1;
+#ifndef VXWORKS
   close(ttyfd);
+#endif /* VXWORKS */
 
   /* Set the pty to the appropriate mode. */
   /* XXX */
@@ -306,6 +392,7 @@ SshPtyStatus ssh_pty_allocate_and_fork(uid_t owner_uid, gid_t owner_gid,
   return SSH_PTY_PARENT_OK;
 }
 
+
 /* Returns the process id of the child process. */
 
 pid_t ssh_pty_get_pid(SshStream stream)
@@ -447,6 +534,10 @@ int ssh_pty_stream_read(void *context, unsigned char *buf, size_t size)
                This is to ensure that a valid exit status is available after
                we return EOF. */
             len = -1;
+#ifdef VXWORKS
+            /* Call the sigchld handler */
+            ssh_pty_sigchld_handler(pty->pid, 0, pty);
+#endif /* VXWORKS */
           }
       return len;
 
@@ -541,7 +632,7 @@ void ssh_pty_stream_destroy(void *context)
   ssh_xfree(pty);
 }
 
-const SshStreamMethodsStruct ssh_pty_methods = {
+SshStreamMethodsStruct ssh_pty_methods = {
   ssh_pty_stream_read,
   ssh_pty_stream_write,
   ssh_pty_stream_output_eof,
