@@ -2,10 +2,11 @@
 
 sshunixuser.c
 
-Author: Tatu Ylonen <ylo@ssh.fi>
-
-Copyright (c) 1998 SSH Communications Security, Finland
-                   All rights reserved
+Author: Tatu Ylonen   <ylo@ssh.fi>
+        Sami Lehtinen <sjl@ssh.com>
+        
+Copyright (c) 1998,2001 SSH Communications Security, Finland
+                        All rights reserved
 
 Manipulating user information in SSH server (mostly password validation).
 This is a simple implementation for generic unix platforms.
@@ -28,20 +29,13 @@ This is a simple implementation for generic unix platforms.
 # include <prot.h>
 # ifdef HAVE_SHADOW_H
 #  include <shadow.h>
-# endif
+# endif /* HAVE_SHADOW_H */
 #else /* HAVE_SCO_ETC_SHADOW */
-# ifdef HAVE_HPUX_TCB_AUTH
-#  undef ctime
-#  include <sys/types.h>
-#  include <hpsecurity.h>
-#  include <prot.h>
-# else /* HAVE_HPUX_TCB_AUTH */
-#  ifdef HAVE_ETC_SHADOW
-#   ifdef HAVE_SHADOW_H
-#    include <shadow.h>
-#   endif
-#  endif /* HAVE_ETC_SHADOW */
-# endif /* HAVE_HPUX_TCB_AUTH */
+# ifdef HAVE_ETC_SHADOW
+#  ifdef HAVE_SHADOW_H
+#   include <shadow.h>
+#  endif /* HAVE_SHADOW_H */
+# endif /* HAVE_ETC_SHADOW */
 #endif /* HAVE_SCO_ETC_SHADOW */
 #ifdef HAVE_ETC_SECURITY_PASSWD_ADJUNCT
 # include <sys/label.h>
@@ -54,10 +48,34 @@ This is a simple implementation for generic unix platforms.
 #endif /* HAVE_ULTRIX_SHADOW_PASSWORDS */
 #include <netdb.h>
 
+#ifdef HAVE_LIBBSM
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif /* !HAVE_SYS_PARAM_H */
+#ifdef HAVE_BSM_AUDIT_H
+#include <bsm/audit.h>
+#endif /* !HAVE_BSM_AUDIT_H */
+#ifdef HAVE_BSM_LIBBSM_H
+#include <bsm/libbsm.h>
+#endif /* !HAVE_BSM_LIBBSM_H */
+
+#endif /* HAVE_LIBBSM */
+
+#ifdef _HPUX_SOURCE
+#undef ctime
+#include <sys/types.h>
+#include <hpsecurity.h>
+#include <prot.h>
+#endif /* _HPUX_SOURCE */
+
 #ifdef HAVE_USERSEC_H
 #include <usersec.h>
 #endif /* HAVE_USERSEC_H */
 
+#ifdef HAVE_LOGINRESTRICTIONS
+#include <login.h>
+#endif /* HAVE_LOGINRESTRICTIONS */
+
 #ifdef HAVE_GRP_H
 #include <grp.h>
 #endif /* HAVE_GRP_H */
@@ -135,6 +153,10 @@ struct SshUserRec
   /* Ticket granting ticket to save in the user's credentials after login,
      or NULL if none. */
   krb5_creds *krb_tgt;
+  
+  /* Local username obtained from krb5_aname_to_localname from
+     kerberos name */
+  char *krb_local_username;
 #endif /* KERBEROS */
 
 #ifdef DAEMON_WITH_PAM
@@ -147,13 +169,49 @@ struct SshUserRec
 /* Returns true if logging in as the specified user is permitted.  Returns
    false if login is not permitted (e.g., the account is expired). */
 
-Boolean ssh_login_permitted(const char *user, SshUser uc)
+Boolean ssh_login_permitted(SshUser uc)
 {
   char passwd[20];              /* Only for account lock check */
- 
+  const char *user = ssh_user_name(uc);
+  
   strncpy(passwd, uc->correct_encrypted_passwd, sizeof(passwd));
   passwd[sizeof(passwd) - 1] = '\0';
 
+#ifdef HAVE_LOGINRESTRICTIONS
+  {
+    char *msg;
+
+    /* XXX memory leak here, since we must free() the msg */
+    if (loginrestrictions(user, S_RLOGIN, NULL, &msg) < 0)
+      {
+        SSH_DEBUG(2, ("loginrestrictions S_RLOGIN failed: %.200s",
+                      msg));
+        return FALSE;
+      }
+  }
+#endif /* HAVE_LOGINRESTRICTIONS */
+
+#ifdef HAVE_PASSWDEXPIRED
+  {
+    char *message;
+    int ret;
+
+    if (ret = passwdexpired((char*)user, &message) != 0)
+      {
+        if (ret > 0)
+          uc->password_needs_change = TRUE;
+        else
+          {
+            SSH_DEBUG(2, ("passwdexpired failed: %.200s",
+                           message));
+            return FALSE;
+          } 
+      }
+      /* XXX mem leak, must free message */
+  }
+
+#endif /* HAVE_PASSWDEXPIRED */
+
 #ifdef HAVE_USERSEC_H
   {
     char *expiration, current_time[100], normalized[100];
@@ -227,9 +285,9 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
       }
     t = ssh_time();
     ssh_calendar_time(t, tm, TRUE);
-    snprintf(current_time, sizeof(current_time), "%04d%02d%02d%02d%02d",
-             tm->year, tm->month + 1, tm->monthday,
-             tm->hour, tm->minute);
+    ssh_snprintf(current_time, sizeof(current_time), "%04d%02d%02d%02d%02d",
+                 tm->year, tm->month + 1, tm->monthday,
+                 tm->hour, tm->minute);
     if (expiration[8] < '7') /* Assume year < 70 is 20YY. */
       strcpy(normalized, "20");
     else
@@ -247,54 +305,77 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
   }
 #endif /* HAVE_USERSEC_H */
 
-#if defined(HAVE_HPUX_TCB_AUTH)
-  /* Check whether the account has been locked. */
-  {
-    struct pr_passwd *pr = getprpwnam(ssh_user_name(uc));
-    pr = getprpwnam(ssh_user_name(uc));
-    if (pr)
-      {
-        /* Is administrative lock is set on the account? */
-        if (pr->uflg.fg_lock && pr->ufld.fd_lock)
+#ifdef _HPUX_SOURCE
+  if (iscomsec())
+    {
+      /* System is in trusted mode */
+      struct pr_passwd *pr = getprpwnam((char *)user);
+      
+      if (!pr || locked_out(pr))
+        {
+          SSH_DEBUG(2, ("Account %.100s not valid - access denied.", user));
+          endprpwent();
           return FALSE;
-        /* Are there too many unsuccessful login attempts? */
-        if (pr->uflg.fg_nlogins && pr->uflg.fg_max_tries &&
-            pr->ufld.fd_nlogins >= pr->ufld.fd_max_tries)
+        }
+      
+      /* Check whether the password has expired and needs to be changed. */
+      if (pr->uflg.fg_expire && pr->uflg.fg_schange)
+        {
+          if (pr->ufld.fd_expire > 0 &&
+              (pr->ufld.fd_expire + pr->ufld.fd_schange) <= ssh_time())
+            uc->password_needs_change = TRUE;
+        }
+      else
+        {
+          if (pr->sflg.fg_expire && pr->uflg.fg_schange &&
+              pr->sfld.fd_expire > 0 &&
+              ((pr->sfld.fd_expire + pr->ufld.fd_schange) <= ssh_time()))
+            uc->password_needs_change = TRUE;
+        }
+      endprpwent();
+    }
+  else
+    {
+      /* The system is not in trusted mode. Check whether the password
+         needs to be changed, using the traditional interface... */
+      struct passwd *pwd = getpwnam((char *)user);
+      SshTime current_time = ssh_time();
+      long pw_expiry,
+        pw_max, /* Max validity in days */
+        pw_secs; /* Last change */
+      
+      if (!pwd)
+        {
+          SSH_DEBUG(2, ("Account %.100s not valid - access denied.", user));
+          endpwent();
           return FALSE;
-        else
-          if (pr->uflg.fg_nlogins && pr->sflg.fg_max_tries &&
-              pr->ufld.fd_nlogins >= pr->sfld.fd_max_tries)
-            return FALSE;
-        /* Password lifetime exceeded? */
-        if (pr->uflg.fg_lifetime && pr->ufld.fd_lifetime <= 0)
-          return FALSE;
-        /* Account is expired? */
-        if (pr->uflg.fg_acct_expire && pr->ufld.fd_acct_expire <= 0)
-          return FALSE; 
-        /* Maximum time since last successful login exceeded? */
-        if ((pr->uflg.fg_max_llogin && pr->uflg.fg_slogin) ||
-            (pr->sflg.fg_max_llogin && pr->uflg.fg_slogin))
-          {
-            struct timeval t;
-            if (gettimeofday(&t, NULL) == 0)
-              {
-                if (pr->uflg.fg_max_llogin &&
-                    (t.tv_sec - pr->ufld.fd_slogin) > pr->ufld.fd_max_llogin)
-                  return FALSE;
-                else
-                  if (pr->sflg.fg_max_llogin &&
-                      (t.tv_sec - pr->ufld.fd_slogin) > pr->sfld.fd_max_llogin)
-                    return FALSE;
-              }
-          }
-      }
-  }
-#endif /* defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH) */
-        
+        }
+
+      if (pwd->pw_age && strlen(pwd->pw_age) > 0)
+        {
+          pw_expiry = a64l(pwd->pw_age);
+          pw_max = (pw_expiry & 077) * 7;
+          pw_secs = (pw_expiry >> 12) * 7 * 24 * 60 * 60;
+          
+          if (pw_max >= 0 &&
+              ((current_time - pw_secs) / 60 / 60 / 24) > pw_max)
+            {
+              SSH_DEBUG(2, ("Password expired, must be changed"));
+              uc->password_needs_change = TRUE;
+            }
+        }
+      
+      endpwent();
+    }
+  
+#endif /* _HPUX_SOURCE */
+  
 #ifdef HAVE_ETC_SHADOW
   {
     struct spwd *sp;
-    
+#ifdef HAVE_SETSPENT
+    setspent();
+#endif /* HAVE_SETSPENT */
     sp = (struct spwd *)getspnam(user);
 #if defined(SECURE_RPC) && defined(NIS_PLUS)
     if (geteuid() == UID_ROOT && ssh_user_uid(uc) != UID_ROOT
@@ -341,7 +422,7 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
             endspent();
             return FALSE;
           }
-#endif
+#endif /* HAVE_STRUCT_SPWD_EXPIRE */
         
 #ifdef HAVE_STRUCT_SPWD_INACT
         /* Check for last login */
@@ -353,20 +434,20 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
             llt = ssh_user_get_last_login_time(uc, buf, sizeof(buf));
             if (llt && (today - llt/24/60/60) > sp->sp_inact)
               {
-                SSH_DEBUG(2, ("Account %.100s was inactive for more than %d days.",
-                              user, sp->sp_inact));
+                SSH_DEBUG(2, ("Account %.100s was inactive for more than %d "
+                              "days.", user, sp->sp_inact));
                 endspent();
                 return FALSE;
               }
           }
-#endif
+#endif /* HAVE_STRUCT_SPWD_INACT */
         
         /* Check if password is valid */
         if (sp->sp_lstchg == 0 ||
             (sp->sp_max > 0 && today > sp->sp_lstchg + sp->sp_max))
           {
-            SSH_DEBUG(2, ("Account %.100s's password is too old - forced to change.",
-                          user));
+            SSH_DEBUG(2, ("Account %.100s's password is too old - forced "
+                          "to change.", user));
             uc->password_needs_change = TRUE;
           }
         strncpy(passwd, sp->sp_pwdp, sizeof(passwd));
@@ -379,13 +460,12 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
    * Check if account is locked. Check if encrypted password starts
    * with "*LK*".
    */
-  {
-    if (strncmp(passwd,"*LK*", 4) == 0)
-      {
-        SSH_DEBUG(2, ("Account %.100s is locked.", user));
-        return FALSE;
-      }
-  }
+
+  if (strncmp(passwd,"*LK*", 4) == 0)
+    {
+      SSH_DEBUG(2, ("Account %.100s is locked.", user));
+      return FALSE;
+    }
 #ifdef CHECK_ETC_SHELLS
   {
     int  invalid = 1;
@@ -409,6 +489,50 @@ Boolean ssh_login_permitted(const char *user, SshUser uc)
   return TRUE;
 }
 
+void ssh_user_record_login_failure(SshUser uc, const char *remote_host)
+{
+  SSH_PRECOND(uc);
+  
+#ifdef HAVE_LOGINFAILED
+  /* record to the system's database that we have an unsucessful login */
+  /* XXX this should be done on all systems, not just on those with
+     loginfailed */
+  /* Last parameter is `tty', but we haven't allocated one yet. Use
+     dummy "ssh" value. */
+  loginfailed(ssh_user_name(uc), remote_host, "ssh");
+#endif /* HAVE_LOGINFAILED */
+
+#ifdef _HPUX_SOURCE
+  if (iscomsec())
+    {
+      /* System is in trusted mode */
+      struct pr_passwd save_pr, *pr = getprpwnam((char *)ssh_user_name(uc));
+      if (pr)
+        {
+          save_pr = *pr;
+          pr = &save_pr;  /* Point to a copy, not the original */
+          
+          if (pr->uflg.fg_nlogins)
+            pr->ufld.fd_nlogins++;
+          else
+            {
+              pr->uflg.fg_nlogins = 1;
+              pr->ufld.fd_nlogins = 1;
+            }
+
+          pr->uflg.fg_ulogin = 1;
+          pr->ufld.fd_ulogin = ssh_time();
+
+          putprpwnam((char *)ssh_user_name(uc), pr);
+        }
+      endprpwent();
+    }
+#endif /* _HPUX_SOURCE */
+  
+  uc->login_allowed = ssh_login_permitted(uc);
+}
+
+
 /* Forward declaration. */
 SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged);
 
@@ -450,7 +574,7 @@ SshUser ssh_user_initialize(const char *user, Boolean privileged)
       /* The name is a kerberos name.  Map it to a local name, and parse
          the kerberos principal into krb_user.  First normalize the '%'
          syntax to the normal kerberos syntax. */
-      snprintf(buf, sizeof(buf), "%s", user);
+      ssh_snprintf(buf, sizeof(buf), "%s", user);
       if (strchr(buf, '%'))
         *strchr(buf, '%') = '\0';
 
@@ -482,7 +606,7 @@ SshUser ssh_user_initialize(const char *user, Boolean privileged)
             goto nokerberos;
           /* Copy the local name to our own buffer to make sure it remains
              valid during the parse call (in case it calls getpw*). */
-          snprintf(user_copy, sizeof(user_copy), "%s", pw->pw_name);
+          ssh_snprintf(user_copy, sizeof(user_copy), "%s", pw->pw_name);
           user = user_copy;
         }
       
@@ -496,7 +620,7 @@ SshUser ssh_user_initialize(const char *user, Boolean privileged)
         }
 
       /* Use the same name as the local user login name. */
-      snprintf(local_user, sizeof(local_user), "%s", user);
+      ssh_snprintf(local_user, sizeof(local_user), "%s", user);
     }
   /* local_user now contains the local user name, and krb_user the kerberos
      name. */
@@ -519,6 +643,7 @@ SshUser ssh_user_initialize(const char *user, Boolean privileged)
     }
 
   /* Store the kerberos data into the SshUser object. */
+  uc->krb_local_username = ssh_xstrdup(local_user);
   uc->kcontext = kcontext;
   uc->krb_user = krb_user;
   uc->krb_user_name = krb_user_name;
@@ -580,17 +705,18 @@ SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
       /* Save the encrypted password. */
       strncpy(correct_passwd, pw->pw_passwd, sizeof(correct_passwd));
 
-#ifdef SSHDIST_SESSION_SIA
 #ifdef HAVE_SIA
+#ifdef SSHDIST_SESSION_SIA
       /* pr->pw_passwd may not be the real encrypted password (it won't be
          under Enhanced Security), but we don't care because we never look at
          uc->correct_encrypted_passwd.  We let ssh_sia_validate_user() do all
          the work instead. */
+#endif /* SSHDIST_SESSION_SIA */
 #else /* HAVE_SIA */
       /* If we have shadow passwords, lookup the real encrypted password from
          the shadow file, and replace the saved encrypted password with the
          real encrypted password. */
-#if defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH)
+#if defined(HAVE_SCO_ETC_SHADOW)
       {
         struct pr_passwd *pr = getprpwnam(ssh_user_name(uc));
         pr = getprpwnam(ssh_user_name(uc));
@@ -598,10 +724,25 @@ SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
           strncpy(correct_passwd, pr->ufld.fd_encrypt, sizeof(correct_passwd));
         endprpwent();
       }
-#else /* defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH) */
+#else /* defined(HAVE_SCO_ETC_SHADOW) */
+#ifdef _HPUX_SOURCE
+      {
+        if (iscomsec())
+          {
+            struct pr_passwd *pr = getprpwnam((char *)ssh_user_name(uc));
+            if (pr && pr->uflg.fg_encrypt)
+              strncpy(correct_passwd, pr->ufld.fd_encrypt, sizeof(correct_passwd));
+            endprpwent();
+          }
+      }
+#else /* _HPUX_SOURCE */
 #ifdef HAVE_ETC_SHADOW
       {
-        struct spwd *sp = getspnam(ssh_user_name(uc));
+        struct spwd *sp;
+#ifdef HAVE_SETSPENT
+        setspent();
+#endif /* HAVE_SETSPENT */
+        sp = getspnam(ssh_user_name(uc));
 #if defined(SECURE_RPC) && defined(NIS_PLUS)
         if (geteuid() == UID_ROOT && ssh_user_uid(uc) != UID_ROOT &&
             (!sp || !sp->sp_pwdp || !strcmp(sp->sp_pwdp,"*NP*")))
@@ -632,8 +773,8 @@ SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
         f = fopen("/etc/security/passwd", "r");
         if (f)
           {
-            snprintf(looking_for_user, sizeof(looking_for_user), 
-                     "%.190s:", ssh_user_name(uc));
+            ssh_snprintf(looking_for_user, sizeof(looking_for_user), 
+                         "%.190s:", ssh_user_name(uc));
             while (fgets(line, sizeof(line), f))
               {
                 if (strchr(line, '\n'))
@@ -665,93 +806,13 @@ SshUser ssh_user_initialize_with_pw(struct passwd *pw, Boolean privileged)
 #endif /* HAVE_ETC_SECURITY_PASSWD */
 #endif /* HAVE_ETC_SECURITY_PASSWD_ADJUNCT */
 #endif /* HAVE_ETC_SHADOW */
+#endif /* _HPUX_SOURCE */
 #endif /* HAVE_SCO_ETC_SHADOW */
 #endif /* HAVE_SIA */
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-#endif /* SSHDIST_SESSION_SIA */
-
       uc->correct_encrypted_passwd = ssh_xstrdup(correct_passwd);
 
-      uc->login_allowed = ssh_login_permitted(uc->name, uc);
+      uc->login_allowed = ssh_login_permitted(uc);
     }
   else /* !privileged */
     {
@@ -838,19 +899,9 @@ Boolean ssh_user_login_is_allowed(SshUser uc)
 
 /* Returns TRUE if login is allowed with the given local password. */
 
-#ifdef SSHDIST_SESSION_SIA
-#ifdef HAVE_SIA
 Boolean ssh_user_validate_local_password(SshUser uc,
                                          const char *password,
                                          const char *remote_host)
-#else /* HAVE_SIA */
-Boolean ssh_user_validate_local_password(SshUser uc,
-                                         const char *password)
-#endif /* HAVE_SIA */
-
-
-
-#endif /* SSHDIST_SESSION_SIA */
 {
   char *encrypted_password;
   const char *correct_passwd = uc->correct_encrypted_passwd;
@@ -912,15 +963,30 @@ Boolean ssh_user_validate_local_password(SshUser uc,
 #endif /* SSHDIST_SESSION_SIA */
 
   /* Encrypt the candidate password using the proper salt. */
-#if defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH)
-  encrypted_password = bigcrypt(password, 
+#if defined(HAVE_SCO_ETC_SHADOW)
+  encrypted_password = bigcrypt((char *)password, 
                                 (correct_passwd[0] && correct_passwd[1]) ?
-                                correct_passwd : "xx");
-#else /* defined(HAVE_SCO_ETC_SHADOW) || defined(HAVE_HPUX_TCB_AUTH) */
+                                (char *)correct_passwd : "xx");
+#else /* HAVE_SCO_ETC_SHADOW */
+# if defined(_HPUX_SOURCE)
+  if (iscomsec())
+    {
+      encrypted_password = bigcrypt((char *)password, 
+                                    (correct_passwd[0] && correct_passwd[1]) ?
+                                    (char *)correct_passwd : "xx");
+    }
+  else
+    {
+      encrypted_password = crypt(password, 
+                                 (correct_passwd[0] && correct_passwd[1]) ?
+                                 correct_passwd : "xx");
+    }
+# else /* defined(_HPUX_SOURCE) */
   encrypted_password = crypt(password, 
                              (correct_passwd[0] && correct_passwd[1]) ?
                              correct_passwd : "xx");
-#endif /* HAVE_SCO_ETC_SHADOW */
+# endif /* defined(_HPUX_SOURCE) */
+#endif /* defined(HAVE_SCO_ETC_SHADOW) */
 #ifdef _AIX41
   /* AIX authenticate loop, KG,04.00 */
   {
@@ -938,17 +1004,21 @@ Boolean ssh_user_validate_local_password(SshUser uc,
   }
 #endif /* _AIX41 */
 
+
+
+
+
+
+  
+  if (strlen(correct_passwd) < 13)
+    return FALSE;
+
   /* Authentication is accepted if the encrypted passwords are identical. */
-#ifdef HAVE_HPUX_TCB_AUTH
   return strncmp(encrypted_password, correct_passwd,
                  strlen(correct_passwd)) == 0;
-#else /* HAVE_HPUX_TCB_AUTH */
-  return strcmp(encrypted_password, correct_passwd) == 0;
-#endif /* HAVE_HPUX_TCB_AUTH */
 }
 
 /* Returns TRUE if the user's password needs to be changed. */
-
 Boolean ssh_user_password_must_be_changed(SshUser uc,
                                           char **prompt_return)
 {
@@ -1121,7 +1191,8 @@ Boolean ssh_user_validate_kerberos_password(SshUser uc,
   memset(&creds, 0, sizeof(creds));
   
   /* Resolve the credentials cache path.  Make it unique to this process. */
-  snprintf(ccname, sizeof(ccname), "FILE:/tmp/krb5cc_l%ld", (long)getpid());
+  ssh_snprintf(ccname, sizeof(ccname), "FILE:/tmp/krb5cc_l%ld",
+               (long)getpid());
   kerr = krb5_cc_resolve(uc->kcontext, ccname, &ccache);
   if (kerr)
     {
@@ -1265,9 +1336,7 @@ Boolean ssh_user_validate_secure_rpc_password(SshUser uc,
 #include <sys/category.h>
 extern char *setlimits();
 
-int ssh_cray_setup(uid, username)
-     uid_t uid;
-     char *username;
+int ssh_cray_setup(uid_t uid, char *username)
 {
   register struct udb *p;
   extern struct udb *getudb();
@@ -1311,7 +1380,7 @@ int ssh_cray_setup(uid, username)
   if (acctid(0, accts[0]) < 0) 
     {
       SSH_DEBUG(2, ("ssh_cray_setup: System call acctid failed, accts[0]=%d",
-                accts[0]));
+                    accts[0]));
       return(-1);
     } 
  
@@ -1389,27 +1458,24 @@ Boolean ssh_user_become(SshUser uc)
      open. */
   endgrent();
 
-#ifdef SSHDIST_SESSION_SIA
 #ifdef HAVE_SIA
+#ifdef SSHDIST_SESSION_SIA
   /* Close later on, after calling sia_become_user().  Closing now could
      break that call. */
+#endif /* SSHDIST_SESSION_SIA */
 #else /* HAVE_SIA */
   for (i = 3; i < 1000; i++)
     close(i);
 #endif /* HAVE_SIA */
 
-
-
-#endif /* SSHDIST_SESSION_SIA */
-
 #ifdef CRAY   /* set up accounting account number, job, limits, permissions  */
-  if (cray_setup(ssh_user_uid(uc), ssh_user_name(uc)) < 0)
+  if (ssh_cray_setup(ssh_user_uid(uc), ssh_user_name(uc)) < 0)
     {
       SSH_DEBUG(2, ("ssh_user_become: Failure in Cray job setup for user %d.",
                     (int)ssh_user_uid(uc)));
       return FALSE;
     }
-#endif
+#endif /* CRAY */
 
   /* Set uid, gid, and groups. */
   if (getuid() == UID_ROOT || geteuid() == UID_ROOT)
@@ -1428,9 +1494,36 @@ Boolean ssh_user_become(SshUser uc)
         }
 #endif /* HAVE_INITGROUPS */
       endgrent();
-          
-#ifdef SSHDIST_SESSION_SIA
+
+#ifdef HAVE_LIBBSM
+      {
+        struct auditinfo au_info;
+        memset(&au_info, 0, sizeof(au_info));
+        if (getaudit(&au_info) < 0)
+          {
+            SSH_DEBUG(2, ("ssh_user_become: getauid %d: %s",
+                          (int)ssh_user_uid(uc), strerror(errno)));
+          }
+        else
+          {
+            au_info.ai_auid = (au_id_t)ssh_user_uid(uc);
+            if (au_user_mask(ssh_user_name(uc), &au_info.ai_mask) < 0)
+              {
+                SSH_DEBUG(2, ("ssh_user_become: aa_user_mask failed for "
+                              "``%s'' (auditing not configured for system "
+                              "nor user)", ssh_user_name(uc)));
+              }
+            else if (setaudit(&au_info) < 0)
+              {
+                SSH_DEBUG(2, ("ssh_user_become: setauid %d: %s",
+                              (int)ssh_user_uid(uc), strerror(errno)));
+              }
+          }
+      }
+#endif /* HAVE_LIBBSM */
+      
 #ifdef HAVE_SIA
+#ifdef SSHDIST_SESSION_SIA
       /* Temporarily switch to the desired uid.  We'll permanently switch
          in ssh_user_become_real(). */
       if (seteuid(ssh_user_uid(uc)) < 0)
@@ -1445,6 +1538,7 @@ Boolean ssh_user_become(SshUser uc)
                         (int)ssh_user_uid(uc)));
           return FALSE;
         }
+#endif /* SSHDIST_SESSION_SIA */
 #else /* HAVE_SIA */
 #ifdef HAVE_SETLUID
   /* Set login uid, if we have setluid(). */
@@ -1469,30 +1563,6 @@ Boolean ssh_user_become(SshUser uc)
           return FALSE;
         }
 #endif /* HAVE_SIA */
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-#endif /* SSHDIST_SESSION_SIA */
     }
   
 #ifdef KERBEROS
@@ -1503,10 +1573,14 @@ Boolean ssh_user_become(SshUser uc)
       krb5_ccache ccache;
       krb5_error_code kerr;
       char ccname[256];
+      char ccenvname[256], *env_ccache;
+
+      ssh_snprintf(ccname, sizeof(ccname), "FILE:/tmp/krb5cc_p%s%d",
+                   uc->krb_local_username, getpid());
+
+      /* Construct the environment variable name to be used below */
+      ssh_snprintf(ccenvname, sizeof(ccenvname),"KRB5CCNAME=%s",ccname);
 
-      /* Construct and resolve the name of the credentials cache file. */
-      snprintf(ccname, sizeof(ccname), "FILE:/tmp/krb5cc_%ld",
-               (long)ssh_user_uid(uc));
       kerr = krb5_cc_resolve(uc->kcontext, ccname, &ccache);
       if (kerr)
         {
@@ -1532,6 +1606,18 @@ Boolean ssh_user_become(SshUser uc)
 
       /* Ok, we have successfully stored the TGT in the ccache. */
       SSH_DEBUG(2, ("TGT successfully stored in ccache"));
+      
+      /* Put the Cache name into the environment */
+      SSH_DEBUG(2, ("Placing ticket cache %s into environment", ccenvname));
+      env_ccache = ssh_xstrdup(ccenvname);
+      putenv(env_ccache);
+
+#if 0
+      /* This is the spot to setup for DCE/DFS or AFS if possible */
+      /* Map the kerberos principal to a local user name. */
+      kerr = krb5_dfs_pag(uc->kcontext, 1,  uc->krb_user, uc->krb_local_username);
+      SSH_DEBUG(2, ("krb_5dfs_pag returned %x", kerr));
+#endif
 
     krb_fail:
       /* Free the ccache object. */
@@ -1614,8 +1700,8 @@ Boolean ssh_user_become_real(SshUser uc,
                       (char *)ssh_user_name(uc), (char *)ttyname, 0, NULL,
                       NULL, SIA_BEU_REALLOGIN) != SIASUCCESS)
     {
-      SSH_DEBUG(2, ("ssh_user_become_real: sia_become_user() failed for user %d.",
-                    (int)ssh_user_uid(uc)));
+      SSH_DEBUG(2, ("ssh_user_become_real: sia_become_user() failed for "
+                    "user %d.", (int)ssh_user_uid(uc)));
       return FALSE;
     }
   /* Our sia calls are complete, so we can finally close any extra
@@ -1767,6 +1853,7 @@ SshGroup *ssh_user_get_groups(SshUser uc)
         }
 
       endgrent();
+      groups[count] = NULL;
       uc->groups = groups;
 #else /* HAVE_GETGRENT */
       /* If we don't have getgrent(), we have to settle for the
