@@ -40,20 +40,19 @@ Boolean ssh_user_free(SshUser uc, Boolean undo);
 /* Returns TRUE if logging in as the specified user is allowed. */
 Boolean ssh_user_login_is_allowed(SshUser uc);
 
+/* Record that login failed (systems often have their own ways to
+   count unsuccesful logins, and calling this is the way to do it
+   machine-independently). This will update the 'login_allowed' field,
+   so that ssh_user_login_is_allowed() will return correct value.
+
+   XXX currently only called by ssh2 with password auth. How about
+   others?  */
+void ssh_user_record_login_failure(SshUser uc, const char *remote_host);
+
 /* Returns TRUE if login is allowed with the given local password. */
-#ifdef SSHDIST_SESSION_SIA
-#ifdef HAVE_SIA
 Boolean ssh_user_validate_local_password(SshUser uc,
                                          const char *password,
                                          const char *remote_host);
-#else /* HAVE_SIA */
-Boolean ssh_user_validate_local_password(SshUser uc,
-                                         const char *password);
-#endif /* HAVE_SIA */
-
-
-
-#endif /* SSHDIST_SESSION_SIA */
 
 /* Returns TRUE if the user's password needs to be changed. */
 Boolean ssh_user_password_must_be_changed(SshUser uc,
@@ -189,6 +188,35 @@ void ssh_user_record_logout(pid_t pid, const char *ttyname);
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
