@@ -15,6 +15,7 @@ In particular, this updates:
 
 */
 
+#ifndef VXWORKS
 #define SSH_ALLOW_CPLUSPLUS_KEYWORDS
 #include "sshsessionincludes.h"
 #ifdef HAVE_UTIL_H
@@ -47,6 +48,13 @@ In particular, this updates:
 # include <login.h>
 #endif /* STRUCT_LASTLOG_DEFINED_IN_LOGIN_H */
 
+#ifdef _HPUX_SOURCE
+#undef ctime
+#include <sys/types.h>
+#include <hpsecurity.h>
+#include <prot.h>
+#endif /* _HPUX_SOURCE */
+
 /* Prototypes for those systems, where they are missing... */
 #ifndef HAVE_PROTO_LOGIN
 void login(const struct utmp *);
@@ -75,17 +83,17 @@ SshTime ssh_user_get_last_login_time(SshUser user,
   int fd;
 
 #ifdef _PATH_LASTLOG
-  snprintf(lastlogfile, sizeof(lastlogfile),
-           "%.200s/%.200s", _PATH_LASTLOG, ssh_user_name(user));
-#else
+  ssh_snprintf(lastlogfile, sizeof(lastlogfile),
+               "%.200s/%.200s", _PATH_LASTLOG, ssh_user_name(user));
+#else /* _PATH_LASTLOG */
 #ifdef LASTLOG_FILE
-  snprintf(lastlogfile, sizeof(lastlogfile),
-           "%.200s/%.200s", LASTLOG_FILE, ssh_user_name(user));
-#else
-  snprintf(lastlogfile, sizeof(lastlogfile),
-           "%.200s/%.200s", SSH_LASTLOG, ssh_user_name(user));
-#endif
-#endif
+  ssh_snprintf(lastlogfile, sizeof(lastlogfile),
+               "%.200s/%.200s", LASTLOG_FILE, ssh_user_name(user));
+#else /* LASTLOG_FILE */
+  ssh_snprintf(lastlogfile, sizeof(lastlogfile),
+               "%.200s/%.200s", SSH_LASTLOG, ssh_user_name(user));
+#endif /* LASTLOG_FILE */
+#endif /* _PATH_LASTLOG */
 
   strcpy(hostbuf, "");
 
@@ -103,9 +111,9 @@ SshTime ssh_user_get_last_login_time(SshUser user,
   strncpy(hostbuf, ll.ll_host, hostbufsize);
   hostbuf[hostbufsize - 1] = 0;
   return ll.ll_time;
-  
-#else /* HAVE_LASTLOG_H || HAVE_LASTLOG */
 
+#else /* HAVE_LASTLOG_H || HAVE_LASTLOG */
+  strcpy(hostbuf, "");
   return 0;
 
 #endif /* HAVE_LASTLOG_H || HAVE_LASTLOG */
@@ -117,74 +125,72 @@ SshTime ssh_user_get_last_login_time(SshUser user,
                                      char *hostbuf,
                                      unsigned int hostbufsize)
 {
-#if defined(HAVE_LASTLOG_H) || defined(HAVE_LASTLOG)
+  strcpy(hostbuf, "");
+#ifdef HAVE_USERSEC_H
+  {
+    char *lasthost;
+    int lasttime;
+    if (setuserdb(S_READ) < 0)
+      return 0;
+    if (getuserattr((char *)ssh_user_name(user), S_LASTTIME,
+                    &lasttime, SEC_INT) < 0)
+      {
+        enduserdb();
+        return 0;
+      }
+    if (getuserattr((char *)ssh_user_name(user), S_LASTHOST,
+                    &lasthost, SEC_CHAR) < 0)
+      {
+        enduserdb();
+        return 0;
+      }
+    if (hostbufsize > strlen(lasthost) + 1)
+      hostbufsize = strlen(lasthost) + 1;
+    strncpy(hostbuf, lasthost, hostbufsize);
+    hostbuf[hostbufsize - 1] = 0;
+    if (enduserdb() < 0)
+      return 0;
+    return lasttime;
+  }
+#endif /* HAVE_USERSEC_H */
 
-  struct lastlog ll;
-  char *lastlog;
-  int fd;
+#if defined(HAVE_LASTLOG_H) || defined(HAVE_LASTLOG)
+  {
+    struct lastlog ll;
+    char *lastlog;
+    int fd;
 
 #ifdef _PATH_LASTLOG
-  lastlog = _PATH_LASTLOG;
-#else
+    lastlog = _PATH_LASTLOG;
+#else /* _PATH_LASTLOG */
 #ifdef LASTLOG_FILE
-  lastlog = LASTLOG_FILE;
-#else
-  lastlog = SSH_LASTLOG;
-#endif
-#endif
+    lastlog = LASTLOG_FILE;
+#else /* LASTLOG_FILE */
+    lastlog = SSH_LASTLOG;
+#endif /* LASTLOG_FILE */
+#endif /* _PATH_LASTLOG */
 
-  strcpy(hostbuf, "");
-
-  fd = open(lastlog, O_RDONLY);
-  if (fd < 0)
-    return 0;
-  lseek(fd, (off_t)((long)ssh_user_uid(user) * sizeof(ll)), 0);
-  if (read(fd, &ll, sizeof(ll)) != sizeof(ll))
-    {
-      close(fd);
+    fd = open(lastlog, O_RDONLY);
+    if (fd < 0)
       return 0;
-    }
-  close(fd);
-  if (hostbufsize > sizeof(ll.ll_host) + 1)
-    hostbufsize = sizeof(ll.ll_host) + 1;
-  strncpy(hostbuf, ll.ll_host, hostbufsize);
-  hostbuf[hostbufsize - 1] = 0;
-  return ll.ll_time;
-
+    lseek(fd, (off_t)((long)ssh_user_uid(user) * sizeof(ll)), 0);
+    if (read(fd, &ll, sizeof(ll)) != sizeof(ll))
+      {
+        close(fd);
+        return 0;
+      }
+    close(fd);
+    if (hostbufsize > sizeof(ll.ll_host) + 1)
+      hostbufsize = sizeof(ll.ll_host) + 1;
+    strncpy(hostbuf, ll.ll_host, hostbufsize);
+    hostbuf[hostbufsize - 1] = 0;
+    return ll.ll_time;
+  }
 #else /* HAVE_LASTLOG_H || HAVE_LASTLOG */
 
-#ifdef HAVE_USERSEC_H
-
-  char *lasthost;
-  int lasttime;
-  if (setuserdb(S_READ) < 0)
-    return 0;
-  if (getuserattr((char *)ssh_user_name(user), S_LASTTIME,
-                  &lasttime, SEC_INT) < 0)
-    {
-      enduserdb();
-      return 0;
-    }
-  if (getuserattr((char *)ssh_user_name(user), S_LASTHOST,
-                  &lasthost, SEC_CHAR) < 0)
-    {
-      enduserdb();
-      return 0;
-    }
-  strncpy(hostbuf, lasthost, hostbufsize);
-  hostbuf[hostbufsize - 1] = 0;
-  if (enduserdb() < 0)
-    return 0;
-  return lasttime;
-
-#else /* HAVE_USERSEC_H */
-  
   /* XXX getting last login time on this platform is not supported. */
-  strcpy(hostbuf, "");
   return 0;
 
-#endif /* HAVE_USERSEC_H */
-
 #endif /* HAVE_LASTLOG_H || HAVE_LASTLOG */
 }
 #endif /* LASTLOG_IS_DIR */
@@ -259,21 +265,21 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
     }
   else
     memset(&u.ut_addr, 0, sizeof(u.ut_addr));
-# endif
+# endif /* HAVE_ADDR_IN_UTMP */
   /* Figure out the file names. */
 # ifdef _PATH_UTMP
   utmp = _PATH_UTMP;
   wtmp = _PATH_WTMP;
-# else
+# else /* _PATH_UTMP */
 #  ifdef UTMP_FILE
   utmp = UTMP_FILE;
   wtmp = WTMP_FILE;
-#  else
+#  else /* UTMP_FILE */
   utmp = SSH_UTMP;
   wtmp = SSH_WTMP;
-#  endif
-# endif
-  
+#  endif /* UTMP_FILE */
+# endif /* _PATH_UTMP */
+
 # ifdef HAVE_LIBUTIL_LOGIN
   login(&u);
 # else /* HAVE_LIBUTIL_LOGIN */
@@ -306,7 +312,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
 #   endif /* ultrix || NeXT */
         lseek(fd, (off_t)(n*sizeof(u)), 0);
         if (write(fd, &u, sizeof(u)) != sizeof(u))
-          ssh_warning("Could not write to %.100s: %.100s", 
+          ssh_warning("Could not write to %.100s: %.100s",
                       utmp, strerror(errno));
       } else
 #  endif /* HAVE_TTYSLOT */
@@ -319,7 +325,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
             {
               lseek(fd, offset, 0);
               if (write(fd, &u, sizeof(u)) != sizeof(u))
-                ssh_warning("Could not append to %.100s: %.100s", 
+                ssh_warning("Could not append to %.100s: %.100s",
                             utmp, strerror(errno));
               break;
             }
@@ -331,7 +337,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
             {
               lseek(fd, offset, 0);
               if (write(fd, &u, sizeof(u)) != sizeof(u))
-                ssh_warning("Could not write to %.100s: %.100s", 
+                ssh_warning("Could not write to %.100s: %.100s",
                             utmp, strerror(errno));
               break;
             }
@@ -376,7 +382,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
     strncpy(ux.ut_id, ttyname + 8, sizeof(ux.ut_id)); /* /dev/ttyq99 -> q99 */
 # else /* __sgi || SCO || linux */
     if (sizeof(ux.ut_id) > 4)
-      { 
+      {
         strncpy(ux.ut_id, ttyname + 5, sizeof(ux.ut_id));
       }
     else
@@ -384,25 +390,25 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
         char buf[20];
 #  ifdef HAVE_MINOR
         struct stat st;
-        
+
         buf[0] = 0;
         if (stat(ttyname, &st) == 0) {
           /* allow for 1000 /dev/pts devices */
-          snprintf(buf, sizeof (buf), "P%03d", (int)minor(st.st_rdev));
+          ssh_snprintf(buf, sizeof (buf), "P%03d", (int)minor(st.st_rdev));
         }
         strncpy(ux.ut_id, buf, sizeof(ux.ut_id));
 #  else /* HAVE_MINOR */
         /* if we don't have minor, we just dig out the last <= three letters
            from ttyname. */
-        
+
         size_t ttyname_len = strlen(ttyname);
         if(ttyname_len > 3)
           {
-            snprintf(buf, sizeof (buf), "P%s", &ttyname[ttyname_len - 3]);
+            ssh_snprintf(buf, sizeof (buf), "P%s", &ttyname[ttyname_len - 3]);
           }
         else
           {
-            snprintf(buf, sizeof (buf), "P%s", ttyname);
+            ssh_snprintf(buf, sizeof (buf), "P%s", ttyname);
           }
 #  endif /* HAVE_MINOR */
       }
@@ -410,9 +416,9 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
 # ifdef HAVE_GETTIMEOFDAY
 #  ifdef HAVE_NO_TZ_IN_GETTIMEOFDAY
     gettimeofday(&ux.ut_tv);
-#  else
+#  else /* HAVE_NO_TZ_IN_GETTIMEOFDAY */
     gettimeofday(&ux.ut_tv, NULL);
-#  endif
+#  endif /* HAVE_NO_TZ_IN_GETTIMEOFDAY */
 # else /* HAVE_GETTIMEOFDAY */
     ux.ut_tv.tv_sec = ssh_time();
     ux.ut_tv.tv_usec = 0;
@@ -436,10 +442,10 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
      * automatically if we don't manage to, for some odd reason
      */
     if (user == NULL)
-        modutx(&ux);
+      modutx(&ux);
     else
-        makeutx(&ux);
-# else
+      makeutx(&ux);
+# else /* HAVE_MAKEUTX */
     pututxline(&ux);
 # if defined(HAVE_UPDWTMPX) && defined(WTMPX_FILE)
     updwtmpx(WTMPX_FILE, &ux);
@@ -449,17 +455,35 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
   }
 #endif /* HAVE_UTMPX_H */
 
-#if defined(HAVE_LASTLOG_H) || defined(HAVE_LASTLOG)
+#ifdef HAVE_LOGINSUCCESS
 
-# ifdef _PATH_LASTLOG
+  /* We don't need to manually update the lastlog file, done below,
+     as the loginsuccess function does that for us */
+  if (user != NULL)
+    {
+      char *msg;
+      if (loginsuccess(ssh_user_name(user), host, ttyname, &msg) < 0)
+        ssh_warning("loginsuccess failed: %.100s", strerror(errno));
+
+      /* XXX - memory leak here. need to free() the msg */
+      return;
+    }
+
+#endif /* HAVE_LOGINSUCCESS */
+
+#ifndef HAVE_USERSEC_H
+
+# if defined(HAVE_LASTLOG_H) || defined(HAVE_LASTLOG)
+
+#  ifdef _PATH_LASTLOG
   lastlog = _PATH_LASTLOG;
-# else
-#  ifdef LASTLOG_FILE
+#  else /* _PATH_LASTLOG */
+#   ifdef LASTLOG_FILE
   lastlog = LASTLOG_FILE;
-#  else
+#   else /* LASTLOG_FILE */
   lastlog = SSH_LASTLOG;
-#  endif
-# endif
+#   endif /* LASTLOG_FILE */
+#  endif /* _PATH_LASTLOG */
 
   /* Update lastlog unless actually recording a logout. */
   if (user != NULL) /* only on login ... */
@@ -472,26 +496,26 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
       ll.ll_time = ssh_time();
       strncpy(ll.ll_line, ttyname + 5, sizeof(ll.ll_line));
       strncpy(ll.ll_host, host, sizeof(ll.ll_host));
-# ifdef LASTLOG_IS_DIR
-      snprintf(lastlogfile, 
-               sizeof (lastlogfile), 
-               "%.100s/%.100s", 
-               lastlog,
-               ssh_user_name(user));
+#  ifdef LASTLOG_IS_DIR
+      ssh_snprintf(lastlogfile,
+                   sizeof (lastlogfile),
+                   "%.100s/%.100s",
+                   lastlog,
+                   ssh_user_name(user));
       fd = open(lastlogfile, O_WRONLY | O_CREAT, 0644);
       if (fd >= 0)
         {
           if (write(fd, &ll, sizeof(ll)) != sizeof(ll))
-            ssh_warning("Could not write %.100s: %.100s", 
+            ssh_warning("Could not write %.100s: %.100s",
                         lastlogfile, strerror(errno));
           close(fd);
-        } 
-      else 
+        }
+      else
         {
           ssh_warning("Could not open %.100s: %.100s",
                       lastlogfile, strerror(errno));
         }
-# else /* LASTLOG_IS_DIR */
+#  else /* LASTLOG_IS_DIR */
       fd = open(lastlog, O_RDWR);
       if (fd >= 0)
         {
@@ -501,11 +525,10 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
                         lastlog, strerror(errno));
           close(fd);
         }
-# endif /* LASTLOG_IS_DIR */
+#  endif /* LASTLOG_IS_DIR */
     }
-#endif /* HAVE_LASTLOG_H || HAVE_LASTLOG */
-
-#ifdef HAVE_USERSEC_H
+# endif /* HAVE_LASTLOG_H || HAVE_LASTLOG */
+#else /* HAVE_USERSEC_H */
 
   if (user != NULL) /* only on login ... */
     {
@@ -520,11 +543,11 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
         ssh_warning("putuserattr S_LASTTIME failed: %.100s", strerror(errno));
       if (putuserattr((char *)ssh_user_name(user),
                       S_LASTTTY, (void *)(ttyname + 5), SEC_CHAR) < 0)
-        ssh_warning("putuserattr S_LASTTTY %.900s failed: %.100s", 
+        ssh_warning("putuserattr S_LASTTTY %.900s failed: %.100s",
                     ttyname, strerror(errno));
       if (putuserattr((char *)ssh_user_name(user),
                       S_LASTHOST, (void *)host, SEC_CHAR) < 0)
-        ssh_warning("putuserattr S_LASTHOST %.900s failed: %.100s", 
+        ssh_warning("putuserattr S_LASTHOST %.900s failed: %.100s",
                     host, strerror(errno));
       if (putuserattr((char *)ssh_user_name(user), 0, NULL, SEC_COMMIT) < 0)
         ssh_warning("putuserattr SEC_COMMIT failed: %.100s", strerror(errno));
@@ -532,8 +555,30 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
         ssh_warning("enduserdb failed: %.100s", strerror(errno));
     }
 #endif /* HAVE_USERSEC_H */
-}
   
+#ifdef _HPUX_SOURCE
+  if (iscomsec() && user != NULL) /* only on login... */
+    {
+      /* System is in trusted mode */
+      struct pr_passwd save_pr, *pr = getprpwnam((char *)ssh_user_name(user));
+      if (pr)
+        {
+          save_pr = *pr;
+          pr = &save_pr;  /* Point to a copy, not the original */
+
+          pr->uflg.fg_nlogins = 1;
+          pr->ufld.fd_nlogins = 0;
+
+          pr->uflg.fg_slogin = 1;
+          pr->ufld.fd_slogin = ssh_time();
+
+          putprpwnam((char *)ssh_user_name(user), pr);
+        }
+      endprpwent();
+    }
+#endif /* _HPUX_SOURCE */
+}
+
 /* Records that the user on the tty has logged out. */
 
 void ssh_user_record_logout(pid_t pid, const char *ttyname)
@@ -544,5 +589,6 @@ void ssh_user_record_logout(pid_t pid, const char *ttyname)
     logwtmp((char *)line, "", "");
 #else /* HAVE_LIBUTIL_LOGIN */
   ssh_user_record_login(NULL, pid, ttyname, "", "");
-#endif /* HAVE_LIBUTIL_LOGIN */  
+#endif /* HAVE_LIBUTIL_LOGIN */
 }
+#endif /* VXWORKS */
