@@ -4,7 +4,7 @@
 
   Author: Antti Huima <huima@ssh.fi>
 
-  Copyright (c) 1999, 2000 SSH Communications Security, Finland
+  Copyright (c) 1999, 2000, 2001 SSH Communications Security, Finland
   All rights reserved.
 
   Created Wed Sep  8 17:41:05 1999.
@@ -15,6 +15,12 @@
 #include "sshadt_i.h"
 #include "sshdebug.h"
 
+#ifdef _KERNEL
+#include "sshadt_array.h"           /* Because of SSH_ADT_ARRAY */
+#include "sshadt_priority_queue.h"  /* Because of SSH_ADT_PRIORITY_QUEUE */
+#include "sshadt_priority_heap.h"   /* Because of SSH_ADT_PRIORITY_HEAP */
+#endif /* _KERNEL */
+
 #define SSH_DEBUG_MODULE "SshADT"
 
 /******************************************************** Default callbacks. */
@@ -60,17 +66,21 @@ SshUInt32 ssh_adt_default_hash(const void *obj, void *context)
   return (SshUInt32)obj;
 }
 
+
 /**************************************** Initialize and destroy containers. */
 
 static void set_default_values(SshADTStandardFields *f)
 {
-  f->app_methods.compare = ssh_adt_default_compare;
-  f->app_methods.copy = ssh_adt_default_copy;
-  f->app_methods.duplicate = ssh_adt_default_duplicate;
-  f->app_methods.destr = ssh_adt_default_destroy;
-  f->app_methods.init = ssh_adt_default_init;
-  f->app_methods.hash = ssh_adt_default_hash;
-  f->app_methods.context = NULL;
+  f->app_methods.compare       = ssh_adt_default_compare;
+  f->app_methods.copy          = ssh_adt_default_copy;
+  f->app_methods.duplicate     = ssh_adt_default_duplicate;
+  f->app_methods.destr         = ssh_adt_default_destroy;
+  f->app_methods.init          = ssh_adt_default_init;
+  f->app_methods.hash          = ssh_adt_default_hash;
+  f->app_methods.cleanup       = NULL;
+  f->app_methods.map_attach    = NULL;
+  f->app_methods.map_detach    = NULL;
+  f->app_methods.context       = NULL;
 }
 
 static Boolean init_toplevel_container(SshADTContainer c,
@@ -95,30 +105,62 @@ static Boolean init_toplevel_container(SshADTContainer c,
         {
         case SSH_ADT_CONTEXT:
           ptr->f.app_methods.context = va_arg(args, void *);
+          SSH_DEBUG(9, ("Registered callback context @%p.",
+                        ptr->f.app_methods.context));
           break;
 
         case SSH_ADT_COMPARE:
           ptr->f.app_methods.compare = va_arg(args, SshADTCompareFunc);
+          SSH_DEBUG(9, ("Registered compare callback @%p.",
+                        ptr->f.app_methods.compare));
           break;
 
         case SSH_ADT_COPY:
           ptr->f.app_methods.copy = va_arg(args, SshADTCopyFunc);
+          SSH_DEBUG(9, ("Registered copy callback @%p.",
+                        ptr->f.app_methods.copy));
           break;
 
         case SSH_ADT_DUPLICATE:
           ptr->f.app_methods.duplicate = va_arg(args, SshADTDuplicateFunc);
+          SSH_DEBUG(9, ("Registered duplicate callback @%p.",
+                        ptr->f.app_methods.duplicate));
           break;
 
         case SSH_ADT_DESTROY:
           ptr->f.app_methods.destr = va_arg(args, SshADTDestroyFunc);
-          break;
-
-        case SSH_ADT_INIT:
-          ptr->f.app_methods.init = va_arg(args, SshADTInitFunc);
+          SSH_DEBUG(9, ("Registered destroy callback @%p.",
+                        ptr->f.app_methods.destr));
           break;
 
         case SSH_ADT_HASH:
           ptr->f.app_methods.hash = va_arg(args, SshADTHashFunc);
+          SSH_DEBUG(9, ("Registered hash callback @%p.",
+                        ptr->f.app_methods.hash));
+          break;
+
+        case SSH_ADT_INIT:
+          ptr->f.app_methods.init = va_arg(args, SshADTInitFunc);
+          SSH_DEBUG(9, ("Registered init callback @%p.",
+                        ptr->f.app_methods.init));
+          break;
+
+        case SSH_ADT_MAP_ATTACH:
+          ptr->f.app_methods.map_attach = va_arg(args, SshADTMapAttachFunc);
+          SSH_DEBUG(9, ("Registered map_attach callback @%p.",
+                        ptr->f.app_methods.map_attach));
+          break;
+
+        case SSH_ADT_MAP_DETACH:
+          ptr->f.app_methods.map_detach = va_arg(args, SshADTMapDetachFunc);
+          SSH_DEBUG(9, ("Registered map_detach callback @%p.",
+                        ptr->f.app_methods.map_detach));
+          break;
+
+        case SSH_ADT_CLEANUP:
+          ptr->f.app_methods.cleanup = va_arg(args, SshADTCleanupFunc);
+          SSH_DEBUG(9, ("Registered cleanup callback @%p.",
+                        ptr->f.app_methods.cleanup));
           break;
 
         case SSH_ADT_SIZE:
@@ -126,14 +168,6 @@ static Boolean init_toplevel_container(SshADTContainer c,
           ptr->f.default_object_size = va_arg(args, size_t);
           break;
 
-        case SSH_ADT_FASTALLOC:
-          (void)va_arg(args, int);
-          break;
-
-        case SSH_ADT_SIZE_HINT:
-          (void)va_arg(args, long);
-          break;
-
         case SSH_ADT_HEADER:
           ptr->flags |= SSH_ADT_FLAG_CONTAINED_HEADER;
           ptr->f.header_offset = va_arg(args, SshUInt32);
@@ -145,7 +179,10 @@ static Boolean init_toplevel_container(SshADTContainer c,
     }
 
 #ifdef _KERNEL
-  SSH_ASSERT(ptr->flags & SSH_ADT_FLAG_CONTAINED_HEADER);
+  SSH_ASSERT((ptr->flags & SSH_ADT_FLAG_CONTAINED_HEADER) ||
+             (pars.type == SSH_ADT_ARRAY) ||
+             (pars.type == SSH_ADT_PRIORITY_QUEUE) ||
+             (pars.type == SSH_ADT_PRIORITY_HEAP));
   SSH_ASSERT(!(ptr->flags & SSH_ADT_FLAG_ALLOCATE));
 #endif
 
@@ -159,22 +196,31 @@ SshADTContainer ssh_adt_create_generic(SshADTContainerType type, ...)
   va_list args;
   SshADTContainer c;
 
-  if (!(c = SSH_MEM_ALLOC(sizeof(*c))))
+  if (!(c = ssh_malloc(sizeof(*c))))
     return NULL;
 
   va_start(args, type);
-  if (init_toplevel_container(c, type, args) == FALSE)
-    return NULL;
-  va_end(args);
 
-  return c;
+  if (init_toplevel_container(c, type, args) == FALSE)
+    {
+      ssh_free(c);
+      va_end(args);
+      return NULL;
+    }
+  else
+    {
+      va_end(args);
+      return c;
+    }
 }
 
 void ssh_adt_destroy(SshADTContainer container)
 {
-  SSH_PRECOND(container);
+  if (!container)
+    return;
+
   (*(container->static_data->methods.destr))(container);
-  SSH_MEM_FREE(container);
+  ssh_free(container);
 }
 
 #if 0
@@ -192,7 +238,7 @@ void ssh_adt_uninit(SshADTContainer container)
 {
   SSH_ADT_CALL_DESTROY_HOOK(container, destr);
   SSH_ADT_CALL(container, FALSE, container_uninit, (container));
-  SSH_MEM_FREE(container->hooks);
+  ssh_free(container->hooks);
 }
 #endif
 
@@ -237,8 +283,8 @@ void ssh_adt_genlist_remove(void **listptr, void *object, int next_ptr_offset)
 
 int ssh_adt_initialize_hooks(SshADTContainer container)
 {
-  SSH_MEM_FREE(container->hooks);
-  if (!(container->hooks = SSH_MEM_ALLOC(sizeof(*container->hooks))))
+  ssh_free(container->hooks);
+  if (!(container->hooks = ssh_malloc(sizeof(*container->hooks))))
     return 1;
 
   container->hooks->insert      = NULL;
@@ -252,7 +298,7 @@ int ssh_adt_initialize_hooks(SshADTContainer container)
 
 void ssh_adt_uninitialize_hooks(SshADTContainer container)
 {
-  SSH_MEM_FREE(container->hooks);
+  ssh_free(container->hooks);
   container->hooks = NULL;
 }
 
@@ -524,7 +570,7 @@ SshADTHandle ssh_adt_get_handle_to(SshADTContainer container,
 }
 
 SshADTHandle ssh_adt_get_handle_to_equal(SshADTContainer container,
-                                         SshADTHandle object)
+                                         void *object)
 {
   SSH_ADT_ASSERT_CONTAINER;
   if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
@@ -577,12 +623,15 @@ void *ssh_adt_detach(SshADTContainer container, SshADTHandle handle)
 }
 
 /* This is called from inside the container implementations when
-   objects are detached just prior to destroying them. So do not
-   assert for externally allocated objects. */
+   objects are detached just prior to destroying them.  In this case
+   we need not require that objects are external because no user will
+   ever get hold on them.  (See the sshadt_std_i.h::SSH_ADT_STD_DELETE
+   macro.)  */
 void *ssh_adt_detach_i(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.detach);
   return ssh_adt_detach__(container, handle);
 }
 
@@ -666,31 +715,47 @@ SshADTHandle ssh_adt_enumerate_next(SshADTContainer container,
   return ssh_adt_enumerate_next__(container, handle);
 }
 
+void *ssh_adt_map_lookup(SshADTContainer container, SshADTHandle handle)
+{
+  SSH_ADT_ASSERT_CONTAINER;
+  SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.map_lookup);
+  return ssh_adt_map_lookup__(container, handle);
+}
+
+void ssh_adt_map_attach(SshADTContainer container,
+                        SshADTHandle handle, void *obj)
+{
+  SSH_ADT_ASSERT_CONTAINER;
+  SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.map_attach);
+  ssh_adt_map_attach__(container, handle, obj);
+}
+
+
 #endif /* !SSH_ADT_WITH_MACRO_INTERFACE */
 
 /************************************************* duplication of containers */
 
 /* This creates a new container from an old one, independently of the
-   latters class.  It is not implemented as a method, so it should
-   work on all new container classes immediately.  It simply walks
-   through an enumeration loop and inserts all elements to the default
-   location.
+   latters class.  It simply walks through an enumeration loop and
+   inserts all elements to the default location.  It might work on new
+   container classes immediately.
 
    The hooks and callbacks are copied as function pointers, so doing
-   weird stuff with static variables might lead to unexpected results.
-   Callback context is set to NULL, hook contexts are shared.  */
+   weird stuff with static variables will probably not work.  Callback
+   and hook contexts need to be NULL in the duplicated container.  */
 
 
 #define FAIL(a)                         \
 do                                      \
 {                                       \
  SSH_DEBUG(3, ("failed: %s.", a));      \
- SSH_MEM_FREE(c2);                      \
+ ssh_free(c2);                      \
  return NULL;                           \
 }                                       \
 while(0)
 
-
 SshADTContainer ssh_adt_duplicate_container(SshADTContainer c1)
 {
   SshADTContainer c2;
@@ -698,7 +763,7 @@ SshADTContainer ssh_adt_duplicate_container(SshADTContainer c1)
   SshADTHandle h;
 
   /* init a new container */
-  if (!(c2 = SSH_MEM_ALLOC(sizeof(*c2))))
+  if (!(c2 = ssh_malloc(sizeof(*c2))))
     return NULL;
 
   /* init pars (the structure is defined in sshadt_i.h) */
@@ -709,9 +774,9 @@ SshADTContainer ssh_adt_duplicate_container(SshADTContainer c1)
   /* the instance flags */
   pars.flags = c1->flags;
 
-  /* callbacks and memory management setting (context is always set to NULL) */
+  /* callbacks and memory management setting */
   pars.f = c1->f;
-  pars.f.app_methods.context = NULL;
+  SSH_ASSERT(pars.f.app_methods.context == NULL);
 
   /* call the init function of the appropriate type */
   if ((*(((SshADTStaticData *) pars.type)->methods.container_init))(c2, &pars)
@@ -719,13 +784,14 @@ SshADTContainer ssh_adt_duplicate_container(SshADTContainer c1)
     FAIL("create");
 
   /* hooks */
-
-  c2->hooks = c1->hooks;
-  /* c2->hooks->reallocated_ctx = NULL; */
-  /* c2->hooks->destr_ctx = NULL; */
+  if (c1->hooks)
+    {
+      SSH_ASSERT(c1->hooks->reallocated_ctx == NULL);
+      SSH_ASSERT(c1->hooks->destr_ctx == NULL);
+      c2->hooks = c1->hooks;
+    }
 
   /* copy elements */
-
   if ((h = ssh_adt_enumerate_start(c1)) != SSH_ADT_INVALID)
     {
       do {
@@ -733,13 +799,13 @@ SshADTContainer ssh_adt_duplicate_container(SshADTContainer c1)
           {
             void *o = ssh_adt_get(c1, h);
             SshADTHandle b = ssh_adt_put(c2, o);
-            if (b == SSH_ADT_INVALID) FAIL("insert_liballoc");
+            if (b == SSH_ADT_INVALID) FAIL("insert, liballoc");
           }
         else                                        /* voidptr */
           {
             void *o = ssh_adt_get(c1, h);
             SshADTHandle h = ssh_adt_duplicate(c2, o);
-            if (h == SSH_ADT_INVALID) FAIL("insert_voidptr");
+            if (h == SSH_ADT_INVALID) FAIL("insert, voidptr");
           }
       }
       while ((h = ssh_adt_enumerate_next(c1, h)) != SSH_ADT_INVALID);
