@@ -14,17 +14,24 @@
 #ifndef SSHADT_H_INCLUDED
 #define SSHADT_H_INCLUDED
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* The fast macro interface is used unless DEBUG_LIGHT is defined. */
 
+
 #ifndef DEBUG_LIGHT
 #define SSH_ADT_WITH_MACRO_INTERFACE
-#endif
+#endif /* DEBUG_LIGHT */
+
+
 
 #define SSHADT_INSIDE_SSHADT_H
 
 /* Generic header type. */
 typedef struct {
-  void *ptr[4];
+  void *ptr[5];
 } SshADTHeaderStruct;
 
 typedef struct SshADTContainerRec *SshADTContainer;
@@ -58,6 +65,25 @@ typedef void (* SshADTInitFunc)(void *obj, size_t size, void *context);
 
 /* Function type to calculate the hash of a user-supplied data item. */
 typedef SshUInt32 (* SshADTHashFunc)(const void *obj, void *context);
+
+/* Function type for notifying map values when they are referenced.
+   To be more precise, this callback is invoked iff map_attach binds
+   an value (== image) to a key in a container.  */
+typedef void (* SshADTMapAttachFunc)(void *value, void *context);
+                                     
+/* Function type for notifying map values when a key is deleted.  More
+   precisely, this function is called when an value is overwritten by
+   a new value attached to an object by map_attach, or if a key is
+   deleted from a container.  If a key is only detached, the image
+   remains alive.  */
+typedef void (* SshADTMapDetachFunc)(void *value, void *context);
+
+/* Clean up the container context.  This is called when the container
+   is destroyed, after all elements have been removed from the
+   container.  */
+typedef void (* SshADTCleanupFunc)(void *ctx);
+
+
 
 typedef enum {
   /* Argument type identifier           C type of the next argument
@@ -84,6 +110,15 @@ typedef enum {
   /* Set the init func. */
   SSH_ADT_INIT,                         /* SshADTInitFunc */
 
+  /* Set the map insert notification func. */
+  SSH_ADT_MAP_ATTACH,                   /* SshADTMapAttachFunc */
+
+  /* Set the map insert notification func. */
+  SSH_ADT_MAP_DETACH,                   /* SshADTMapDetachFunc */
+
+  /* Set the cleanup func. */
+  SSH_ADT_CLEANUP,                      /* SshADTCleanupFunc */
+
   /* Tell the ADT library to use a header inside the objects instead
      of allocating auxiliary data outside the objects. The next
      argument must be the offset (size_t) to an SshADTHeader field. */
@@ -93,19 +128,6 @@ typedef enum {
      automatic object allocation on. */
   SSH_ADT_SIZE,                         /* size_t */
 
-  /* Tell the ADT library to allocate objects by using the
-     sshfastalloc utility. Two restrictions: 1) SSH_ADT_SIZE must be
-     set also. 2) If you use SSH_ADT_FASTALLOC you cannot allocate
-     objects that are not of the default size. Thus, ssh_adt_realloc
-     is not available and ssh_adt_alloc does not work with non-default
-     size. */
-  SSH_ADT_FASTALLOC,                    /* int */
-
-  /* Give a hint about the maximum number of objects that will be inserted
-     to the container. This can be used e.g. to guess the initial size
-     of a hash table. */
-  SSH_ADT_SIZE_HINT,                    /* long */
-
   /* end of args marker. */
   SSH_ADT_ARGS_END                      /* None */
 } SshADTArgumentType;
@@ -117,10 +139,11 @@ typedef enum {
 
 typedef void *SshADTContainerType;
 
-/* Invalid handle. */
-
+/* Invalid handle.  (Never ever redefine this to be something
+   different from NULL.  It will never work.)  */
 #define SSH_ADT_INVALID   NULL
-
+
+
 /******************************************** Handling containers as a whole */
 
 /* Generic container allocation. This can return NULL if memory
@@ -314,9 +337,11 @@ void ssh_adt_delete_object(SshADTContainer container,
 /************************************* Resizing internally allocated objects */
 
 /* Reallocate a object inside a container. */
+
 void *ssh_adt_realloc(SshADTContainer container, void *object,
                       size_t new_size);
 
+
 /* Return the default object size. */
 size_t ssh_adt_default_size(SshADTContainer container);
 
@@ -335,6 +360,15 @@ SshADTHandle ssh_adt_enumerate_start(SshADTContainer container);
 SshADTHandle ssh_adt_enumerate_next(SshADTContainer container,
                                     SshADTHandle handle);
 
+/************************************************* Generic mapping functions */
+
+/* Lookup the value of key `handle' in a mapping container.  */
+void *ssh_adt_map_lookup(SshADTContainer container, SshADTHandle handle);
+
+/* Add a new value to a mapping with key `handle'.  */
+void ssh_adt_map_attach(SshADTContainer container,
+                        SshADTHandle handle, void *obj);
+
 /************************************************ Other container operations */
 
 /* Create a clone of container.  This is not a proper class method,
@@ -352,4 +386,8 @@ SshADTContainer ssh_adt_duplicate_container(SshADTContainer container);
 
 #undef SSHADT_INSIDE_SSHADT_H
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* SSHADT_H_INCLUDED */
