@@ -4,7 +4,7 @@
 
   Author: Antti Huima <huima@ssh.fi>
 
-  Copyright (c) 1999, 2000 SSH Communications Security, Finland
+  Copyright (c) 1999, 2000, 2001 SSH Communications Security, Finland
   All rights reserved.
 
   Created Thu Oct 21 13:35:14 1999.
@@ -20,7 +20,7 @@
 
 #define SSH_DEBUG_MODULE "SshADTArray"
 
-#define ROOT     ((SshADTArrayRoot *)(c->container_specific))
+#define ROOT(c)     ((SshADTArrayRoot *)(c->container_specific))
 
 /* We need to add and subtract one because otherwise the index zero
    would actually correspond to SSH_ADT_INVALID, which would be
@@ -30,14 +30,20 @@
 
 static Boolean init(SshADTContainer c)
 {
-/* Handles are of type (void *)idx. Ugly, but who cares. */
+  /* Handles are of type (void *)idx.  */
   SSH_VERIFY(sizeof(void *) >= sizeof(unsigned int));
+
   /* The dynamic array type has zero-sized headers so they should
      not be `contained'. */
-  SSH_ASSERT(!(c->flags & SSH_ADT_FLAG_CONTAINED_HEADER));
-  c->container_specific = SSH_MEM_ALLOC(sizeof(*ROOT));
-  ROOT->array = NULL;
-  ROOT->array_size = 0;
+  SSH_ASSERT(!(c->flags & SSH_ADT_FLAG_CONTAINED_HEADER) &&
+             "See documentation (array specifics section).");
+
+  if ((c->container_specific = ssh_malloc(sizeof(*ROOT(c)))) == NULL)
+    return FALSE;
+
+  ROOT(c)->array = NULL;
+  ROOT(c)->array_size = 0;
+
   return TRUE;
 }
 
@@ -45,9 +51,9 @@ static Boolean init(SshADTContainer c)
 static void clear(SshADTContainer c)
 {
   int i;
-  for (i = 0; i < ROOT->array_size; i++)
+  for (i = 0; i < ROOT(c)->array_size; i++)
     {
-      if (ROOT->array[i] != NULL)
+      if (ROOT(c)->array[i] != NULL)
         ssh_adt_delete(c, UINTTOPTR(i));
     }
 }
@@ -57,7 +63,7 @@ static void pq_clear(SshADTContainer c)
 {
   SshADTHandle h = UINTTOPTR(0);
 
-  while (c->aux.num_objects > 0)
+  while (c->num_objects > 0)
     {
       ssh_adt_delete(c, h);
     }
@@ -70,15 +76,15 @@ SSH_ADT_STD_INIT(container_init, init(c))
 static void uninit(SshADTContainer c)
 {
   clear(c);
-  SSH_MEM_FREE(ROOT->array);
-  SSH_MEM_FREE(ROOT);
+  ssh_free(ROOT(c)->array);
+  ssh_free(ROOT(c));
 }
 
 static void pq_uninit(SshADTContainer c)
 {
   pq_clear(c);
-  SSH_MEM_FREE(ROOT->array);
-  SSH_MEM_FREE(ROOT);
+  ssh_free(ROOT(c)->array);
+  ssh_free(ROOT(c));
 }
 
 /* $$METHOD(array, destr) */
@@ -89,43 +95,52 @@ SSH_ADT_STD_DESTROY(pq_destr, pq_uninit(c);)
 
 /* After calling initialize_cell(c, n), the nth (0-based) cell is
    valid and empty. The array is expanded if necessary, and if there
-   was an object at the nth cell, the object has been deleted. */
-static void initialize_cell(SshADTContainer c, unsigned int idx)
+   was an object at the nth cell, the object has been deleted.
+
+   In usermode or if enough memory is available, initialize_cell
+   always succeeds and returns TRUE.  In kernel mode and if
+   ssh_calloc returns NULL, the new cell is not allocated and
+   FALSE is returned. */
+static Boolean initialize_cell(SshADTContainer c, unsigned int idx)
 {
   size_t i, old_size, new_size;
   void **array;
 
-  if (ROOT->array_size <= idx)
+  if (ROOT(c)->array_size <= idx)
     {
-      old_size = ROOT->array_size;
-      new_size = (idx + (idx / 4) + 1);
-      ROOT->array = ssh_xrealloc(ROOT->array, new_size *
-                                 sizeof(ROOT->array[0]));
-      array = ROOT->array;
+      old_size = ROOT(c)->array_size;
+      new_size = idx + (idx / 4) + 1;
+
+      if ((array =
+           ssh_realloc(ROOT(c)->array,
+                       old_size * sizeof(ROOT(c)->array[0]),
+                       new_size * sizeof(ROOT(c)->array[0]))) == NULL)
+        return FALSE;
+
+      ROOT(c)->array = array;
+
       for (i = old_size; i < new_size ; i++)
-        {
-          array[i] = NULL;
-        } 
-      ROOT->array_size = new_size;
-      return;
+        array[i] = NULL;
+
+      ROOT(c)->array_size = new_size;
+      return TRUE;
     }
   else
     {
-      if (ROOT->array[idx] != NULL)
-        {
-          ssh_adt_delete(c, UINTTOPTR(idx));
-        }
+      if (ROOT(c)->array[idx] != NULL)
+        ssh_adt_delete(c, UINTTOPTR(idx));
     }
+
+  return TRUE;
 }
 
-static unsigned int empty_idx(SshADTContainer c,
-                              SshADTAbsoluteLocation location)
+static Boolean empty_idx(SshADTContainer c,
+                         SshADTAbsoluteLocation location,
+                         unsigned int *idx)
 {
-  unsigned int idx;
   SSH_ASSERT(SSH_ADT_IS_INDEX(location));
-  idx = SSH_ADT_GET_INDEX(location);
-  initialize_cell(c, idx);
-  return idx;
+  *idx = SSH_ADT_GET_INDEX(location);
+  return initialize_cell(c, *idx);
 }
 
 /* $$METHOD(array, insert_to) */
@@ -135,9 +150,12 @@ static SshADTHandle insert_to(SshADTContainer c,
 {
   unsigned int idx;
   SshADTHandle h;
-  idx = empty_idx(c, location);
-  ROOT->array[idx] = object;
-  c->aux.num_objects++;
+
+  if (!empty_idx(c, location, &idx))
+    return SSH_ADT_INVALID;
+
+  ROOT(c)->array[idx] = object;
+  c->num_objects++;
   h = UINTTOPTR(idx);
   SSH_ADT_CALL_HOOK(c, insert, h);
   return h;
@@ -153,15 +171,15 @@ static void percolate_up(SshADTContainer c, int idx)
     {
       parent = (idx - 1) >> 1;
       SSH_ADT_CALL_APP_MANDATORY(c, compare,
-                                 (ROOT->array[idx],
-                                  ROOT->array[parent],
+                                 (ROOT(c)->array[idx],
+                                  ROOT(c)->array[parent],
                                   SSH_ADT_APPCTX(c)),
                                  cmp);
       SSH_DEBUG(9, ("Comp %d %d == %d\n", idx, parent, cmp));
       if (cmp >= 0) return;
-      tmp = ROOT->array[parent];
-      ROOT->array[parent] = ROOT->array[idx];
-      ROOT->array[idx] = tmp;
+      tmp = ROOT(c)->array[parent];
+      ROOT(c)->array[parent] = ROOT(c)->array[idx];
+      ROOT(c)->array[idx] = tmp;
       idx = parent;
     }
 }
@@ -174,13 +192,13 @@ static void percolate_down(SshADTContainer c, int idx)
 
   while (left = (idx << 1) + 1,
          right = left + 1,
-         left < c->aux.num_objects)
+         left < c->num_objects)
     {
-      if (right < c->aux.num_objects)
+      if (right < c->num_objects)
         {
           SSH_ADT_CALL_APP_MANDATORY(c, compare,
-                                     (ROOT->array[left],
-                                      ROOT->array[right],
+                                     (ROOT(c)->array[left],
+                                      ROOT(c)->array[right],
                                       SSH_ADT_APPCTX(c)),
                                      child_cmp);
         }
@@ -191,14 +209,14 @@ static void percolate_down(SshADTContainer c, int idx)
       if (child_cmp < 0) child = left; else child = right;
 
       SSH_ADT_CALL_APP_MANDATORY(c, compare,
-                                 (ROOT->array[idx],
-                                  ROOT->array[child],
+                                 (ROOT(c)->array[idx],
+                                  ROOT(c)->array[child],
                                   SSH_ADT_APPCTX(c)),
                                  cmp);
       if (cmp <= 0) return;
-      tmp = ROOT->array[child];
-      ROOT->array[child] = ROOT->array[idx];
-      ROOT->array[idx] = tmp;
+      tmp = ROOT(c)->array[child];
+      ROOT(c)->array[child] = ROOT(c)->array[idx];
+      ROOT(c)->array[idx] = tmp;
 
       idx = child;
     }
@@ -213,20 +231,24 @@ static SshADTHandle pq_insert_to(SshADTContainer c,
 
   SSH_ASSERT(location == SSH_ADT_DEFAULT);
 
-  idx = c->aux.num_objects; /* Add to the end. */
-  initialize_cell(c, idx);
-  ROOT->array[idx] = object;
-  c->aux.num_objects++;
+  idx = c->num_objects; /* Add to the end. */
+  if (!initialize_cell(c, idx))
+    return SSH_ADT_INVALID;
+
+  ROOT(c)->array[idx] = object;
+  c->num_objects++;
   percolate_up(c, idx);
-  return SSH_ADT_INVALID;       /* No handle actually. */
+  return UINTTOPTR(0);
 }
 
 /* $$METHOD(array, get_handle_to_location) */
 static SshADTHandle get_handle_to_location(SshADTContainer c,
                                            SshADTAbsoluteLocation location)
 {
-  SshADTHandle h = UINTTOPTR(SSH_ADT_GET_INDEX(location));
-  return h;
+  unsigned int idx = SSH_ADT_GET_INDEX(location);
+
+  if (idx >= ROOT(c)->array_size) return SSH_ADT_INVALID;
+  else return UINTTOPTR(idx);
 }
 
 /* $$METHOD(pq, get_handle_to_location) */
@@ -234,9 +256,9 @@ static SshADTHandle pq_get_handle_to_location(SshADTContainer c,
                                               SshADTAbsoluteLocation location)
 {
   SSH_ASSERT(location == SSH_ADT_DEFAULT);
-  if (ROOT->array[0] == NULL)
-    return SSH_ADT_INVALID;
-  return UINTTOPTR(0);
+
+  if (c->num_objects == 0) return SSH_ADT_INVALID;
+  else return UINTTOPTR(0);
 }
 
 /* $$METHOD(array, alloc_n_to) */
@@ -247,9 +269,13 @@ static SshADTHandle alloc_n_to(SshADTContainer c,
   unsigned int idx;
   SshADTHandle h;
   void *newp;
-  idx = empty_idx(c, location);
-  ROOT->array[idx] = (newp = SSH_MEM_ALLOC(size));
-  c->aux.num_objects++;
+
+  if (!empty_idx(c, location, &idx))
+    return SSH_ADT_INVALID;
+  if ((ROOT(c)->array[idx] = newp = ssh_malloc(size)) == NULL)
+    return SSH_ADT_INVALID;
+
+  c->num_objects++;
   h = UINTTOPTR(idx);
   SSH_ADT_CALL_APP(c, init, (newp, size, SSH_ADT_APPCTX(c)));
   SSH_ADT_CALL_HOOK(c, insert, h);
@@ -265,9 +291,13 @@ static SshADTHandle put_n_to(SshADTContainer c,
   unsigned int idx;
   SshADTHandle h;
   void *newp;
-  idx = empty_idx(c, location);
-  ROOT->array[idx] = (newp = SSH_MEM_ALLOC(size));
-  c->aux.num_objects++;
+
+  if (!empty_idx(c, location, &idx))
+    return SSH_ADT_INVALID;
+  if ((ROOT(c)->array[idx] = newp = ssh_malloc(size)) == NULL)
+    return SSH_ADT_INVALID;
+
+  c->num_objects++;
   h = UINTTOPTR(idx);
   SSH_ADT_CALL_APP(c, copy, (newp, size, object, SSH_ADT_APPCTX(c)));
   SSH_ADT_CALL_HOOK(c, insert, h);
@@ -285,15 +315,17 @@ static SshADTHandle pq_alloc_n_to(SshADTContainer c,
 
   SSH_ASSERT(location == SSH_ADT_DEFAULT);
 
-  idx = c->aux.num_objects;
-  initialize_cell(c, idx);
+  idx = c->num_objects;
+  if (!initialize_cell(c, idx))
+    return SSH_ADT_INVALID;
+  if ((ROOT(c)->array[idx] = newp = ssh_malloc(size)) == NULL)
+    return SSH_ADT_INVALID;
 
-  ROOT->array[idx] = (newp = SSH_MEM_ALLOC(size));
-  c->aux.num_objects++;
+  c->num_objects++;
   h = UINTTOPTR(idx);
   SSH_ADT_CALL_APP(c, init, (newp, size, SSH_ADT_APPCTX(c)));
   percolate_up(c, idx);
-  return SSH_ADT_INVALID;
+  return h;
 }
 
 /* $$METHOD(pq, put_n_to) */
@@ -308,15 +340,17 @@ static SshADTHandle pq_put_n_to(SshADTContainer c,
 
   SSH_ASSERT(location == SSH_ADT_DEFAULT);
 
-  idx = c->aux.num_objects;
-  initialize_cell(c, idx);
+  idx = c->num_objects;
+  if (!initialize_cell(c, idx))
+    return SSH_ADT_INVALID;
+  if ((ROOT(c)->array[idx] = newp = ssh_malloc(size)) == NULL)
+    return SSH_ADT_INVALID;
 
-  ROOT->array[idx] = (newp = SSH_MEM_ALLOC(size));
-  c->aux.num_objects++;
+  c->num_objects++;
   h = UINTTOPTR(idx);
   SSH_ADT_CALL_APP(c, copy, (newp, size, object, SSH_ADT_APPCTX(c)));
   percolate_up(c, idx);
-  return SSH_ADT_INVALID;
+  return h;
 }
 
 /* $$METHOD(array, get) */
@@ -324,8 +358,8 @@ static SshADTHandle pq_put_n_to(SshADTContainer c,
 static void *get(SshADTContainer c, SshADTHandle h)
 {
   unsigned int idx = PTRTOUINT(h);
-  SSH_ASSERT(idx < ROOT->array_size);
-  return ROOT->array[idx];
+  SSH_ASSERT(idx < ROOT(c)->array_size);
+  return ROOT(c)->array[idx];
 }
 
 /* $$METHOD(array, num_objects) */
@@ -333,12 +367,13 @@ static void *get(SshADTContainer c, SshADTHandle h)
 SSH_ADT_STD_NUM_OBJECTS(num_objects)
 
 /* $$METHOD(array, get_handle_to) */
+/* $$METHOD(pq,    get_handle_to) */
 static SshADTHandle get_handle_to(SshADTContainer c, void *object)
 {
   unsigned int i;
-  for (i = 0; i < ROOT->array_size; i++)
+  for (i = 0; i < ROOT(c)->array_size; i++)
     {
-      if (ROOT->array[i] == object)
+      if (ROOT(c)->array[i] == object)
         return UINTTOPTR(i);
     }
   return SSH_ADT_INVALID;
@@ -348,9 +383,8 @@ static SshADTHandle get_handle_to(SshADTContainer c, void *object)
 /* $$METHOD(pq, enumerate_start) */
 static SshADTHandle enum_start(SshADTContainer c)
 {
-  SshADTHandle h = UINTTOPTR(0);
-  if (ROOT->array_size == 0) return SSH_ADT_INVALID;
-  return h;
+  if (c->num_objects == 0) return SSH_ADT_INVALID;
+  else return UINTTOPTR(0);
 }
 
 /* $$METHOD(array, enumerate_next) */
@@ -359,8 +393,8 @@ static SshADTHandle enum_next(SshADTContainer c, SshADTHandle h)
   unsigned int idx = PTRTOUINT(h);
   SSH_ASSERT(h != SSH_ADT_INVALID);
   idx++;
-  if (idx >= ROOT->array_size) return SSH_ADT_INVALID;
-  return UINTTOPTR(idx);
+  if (idx >= ROOT(c)->array_size) return SSH_ADT_INVALID;
+  else return UINTTOPTR(idx);
 }
 
 /* $$METHOD(pq, enumerate_next) */
@@ -369,9 +403,8 @@ static SshADTHandle pq_enum_next(SshADTContainer c, SshADTHandle h)
   unsigned int idx = PTRTOUINT(h);
   SSH_ASSERT(h != SSH_ADT_INVALID);
   idx++;
-  if (idx >= ROOT->array_size) return SSH_ADT_INVALID;
-  if (ROOT->array[idx] == NULL) return SSH_ADT_INVALID;
-  return UINTTOPTR(idx);
+  if (idx >= c->num_objects) return SSH_ADT_INVALID;
+  else return UINTTOPTR(idx);
 }
 
 /* $$METHOD(array, detach) */
@@ -381,15 +414,14 @@ static void *detach(SshADTContainer c, SshADTHandle h)
   unsigned int idx = PTRTOUINT(h);
 
   SSH_ASSERT(h != SSH_ADT_INVALID);
-  SSH_ASSERT(idx < ROOT->array_size);
-  SSH_ASSERT(ROOT->array[idx] != NULL);
+  SSH_ASSERT(idx < ROOT(c)->array_size);
 
   SSH_ADT_CALL_HOOK(c, detach, h);
 
-  c->aux.num_objects--;
+  c->num_objects--;
 
-  object = ROOT->array[idx];
-  ROOT->array[idx] = NULL;
+  object = ROOT(c)->array[idx];
+  ROOT(c)->array[idx] = NULL;
   return object;
 }
 
@@ -400,17 +432,17 @@ static void *pq_detach(SshADTContainer c, SshADTHandle h)
   unsigned int idx = PTRTOUINT(h);
 
   SSH_ASSERT(h != SSH_ADT_INVALID);
-  SSH_ASSERT(idx == 0);
-  SSH_ASSERT(idx < ROOT->array_size);
-  SSH_ASSERT(ROOT->array[idx] != NULL);
+  SSH_ASSERT(idx < ROOT(c)->array_size);
 
   SSH_ADT_CALL_HOOK(c, detach, h);
-  
-  c->aux.num_objects--;
-  object = ROOT->array[idx];
-  ROOT->array[idx] = ROOT->array[c->aux.num_objects];
-  ROOT->array[c->aux.num_objects] = NULL;
+
+  object = ROOT(c)->array[idx];
+
+  c->num_objects--;
+  ROOT(c)->array[idx] = ROOT(c)->array[c->num_objects];
+  ROOT(c)->array[c->num_objects] = NULL;
   percolate_down(c, idx);
+
   return object;
 }
 
@@ -418,7 +450,7 @@ static void *pq_detach(SshADTContainer c, SshADTHandle h)
 /* $$METHOD(pq, reallocate) */
 static void *reallocate(SshADTContainer c, void *object, size_t new_size)
 {
-  ssh_fatal("Realloc does not work with arrays now.");
+  ssh_fatal("Realloc does not work with arrays.  Please complain.");
   return NULL;
 }
 
@@ -427,14 +459,14 @@ static void *reallocate(SshADTContainer c, void *object, size_t new_size)
 static void delet(SshADTContainer c, SshADTHandle handle)
 {
   void *object = ssh_adt_detach_i(c, handle);
-  SSH_ADT_CALL_APP(c, destr, (object, SSH_ADT_APPCTX(c)));  
+  SSH_ADT_CALL_APP(c, destr, (object, SSH_ADT_APPCTX(c)));
   if (c->flags & SSH_ADT_FLAG_ALLOCATE)
     {
-      SSH_MEM_FREE(object);
+      ssh_free(object);
     }
 }
 
-SshADTStaticData ssh_adt_array_static_data = 
+SshADTStaticData ssh_adt_array_static_data =
 {
   {
     /* $$METHODS(array) */
@@ -461,6 +493,8 @@ SshADTStaticData ssh_adt_array_static_data =
     reallocate, /* reallocate */
     detach, /* detach */
     delet, /* delet */
+    NULL, /* map_lookup */
+    NULL, /* map_attach */
     /* $$ENDMETHODS */
   },
   0,
@@ -484,7 +518,7 @@ SshADTStaticData ssh_adt_pq_static_data =
     pq_put_n_to, /* put_n_to */
     get, /* get */
     num_objects, /* num_objects */
-    NULL, /* get_handle_to */
+    get_handle_to, /* get_handle_to */
     pq_get_handle_to_location, /* get_handle_to_location */
     NULL, /* next */
     NULL, /* previous */
@@ -494,6 +528,8 @@ SshADTStaticData ssh_adt_pq_static_data =
     reallocate, /* reallocate */
     pq_detach, /* detach */
     delet, /* delet */
+    NULL, /* map_lookup */
+    NULL, /* map_attach */
     /* $$ENDMETHODS */
   },
   0,
@@ -501,10 +537,5 @@ SshADTStaticData ssh_adt_pq_static_data =
 };
 
 
-#ifdef _KERNEL
-const SshADTContainerType ssh_adt_array_type = NULL;
-const SshADTContainerType ssh_adt_priority_queue_type = NULL;
-#else  /* _KERNEL */
 const SshADTContainerType ssh_adt_array_type = &ssh_adt_array_static_data;
 const SshADTContainerType ssh_adt_priority_queue_type = &ssh_adt_pq_static_data;
-#endif  /* _KERNEL */
