@@ -4,7 +4,7 @@
 
   Author: Antti Huima <huima@ssh.fi>
 
-  Copyright (c) 1999 SSH Communications Security, Finland
+  Copyright (c) 1999, 2001 SSH Communications Security, Finland
   All rights reserved.
 
   Created Fri Sep 24 15:28:22 1999.
@@ -27,7 +27,7 @@ typedef struct {
 static void hook_domain_unmap(SshADTHandle h, void *ctx)
 {
   SshADTAssociationContext *a = ctx;
-  SshADTHandle image = ssh_adt_map_map(a->domain, h);
+  SshADTHandle image = ssh_adt_map_lookup(a->domain, h);
 
   if (image != SSH_ADT_INVALID)
     {
@@ -38,35 +38,35 @@ static void hook_domain_unmap(SshADTHandle h, void *ctx)
 static void hook_range_unmap2(SshADTHandle h, void *ctx)
 {
   SshADTAssociationContext *a = ctx;
-  SshADTHandle image = ssh_adt_map_map(a->range, h);
+  SshADTHandle image = ssh_adt_map_lookup(a->range, h);
 
   if (image != SSH_ADT_INVALID)
     {
-      ssh_adt_map_set_map(a->domain, image, SSH_ADT_INVALID);
+      ssh_adt_map_attach(a->domain, image, SSH_ADT_INVALID);
     }
 }
 
 static void hook_domain_unmap2(SshADTHandle h, void *ctx)
 {
   SshADTAssociationContext *a = ctx;
-  SshADTHandle image = ssh_adt_map_map(a->domain, h);
+  SshADTHandle image = ssh_adt_map_lookup(a->domain, h);
 
   if (image != SSH_ADT_INVALID)
     {
-      ssh_adt_map_set_map(a->range, image, SSH_ADT_INVALID);
+      ssh_adt_map_attach(a->range, image, SSH_ADT_INVALID);
     }
 }
 
 static void hook_domain_detach(SshADTHandle h, void *ctx)
 {
   SshADTAssociationContext *a = ctx;
-  ssh_adt_map_set_map(a->domain, h, SSH_ADT_INVALID);
+  ssh_adt_map_attach(a->domain, h, SSH_ADT_INVALID);
 }
 
 static void hook_range_detach(SshADTHandle h, void *ctx)
 {
   SshADTAssociationContext *a = ctx;
-  ssh_adt_map_set_map(a->range, h, SSH_ADT_INVALID);
+  ssh_adt_map_attach(a->range, h, SSH_ADT_INVALID);
 }
 
 static void hook_destroy(void *ctx)
@@ -79,13 +79,13 @@ static void hook_domain_map2(SshADTHandle h, void *ctx)
 {
   SshADTAssociationContext *a = ctx;
   SshADTHandle image, revimage;
-  image = ssh_adt_map_map(a->domain, h);
+  image = ssh_adt_map_lookup(a->domain, h);
   if (image != SSH_ADT_INVALID)
     {
-      revimage = ssh_adt_map_map(a->range, image);
+      revimage = ssh_adt_map_lookup(a->range, image);
       if (revimage != h)
         {
-          ssh_adt_map_set_map(a->range, image, h);
+          ssh_adt_map_attach(a->range, image, h);
         }
     }
 }
@@ -94,22 +94,23 @@ static void hook_range_map2(SshADTHandle h, void *ctx)
 {
   SshADTAssociationContext *a = ctx;
   SshADTHandle image, revimage;
-  image = ssh_adt_map_map(a->range, h);
+  image = ssh_adt_map_lookup(a->range, h);
   if (image != SSH_ADT_INVALID)
     {
-      revimage = ssh_adt_map_map(a->domain, image);
+      revimage = ssh_adt_map_lookup(a->domain, image);
       if (revimage != h)
         {
-          ssh_adt_map_set_map(a->domain, image, h);
+          ssh_adt_map_attach(a->domain, image, h);
         }
     }
 }
 
-void ssh_adt_associate_unimap(SshADTContainer domain, SshADTContainer range)
+Boolean ssh_adt_associate_unimap(SshADTContainer domain, SshADTContainer range)
 {
   SshADTAssociationContext *ctx;
 
-  ctx = SSH_MEM_ALLOC(sizeof(*ctx));
+  if (!(ctx = ssh_malloc(sizeof(*ctx))))
+    return FALSE;
 
   ctx->domain = domain;
   ctx->range  = range;
@@ -128,13 +129,16 @@ void ssh_adt_associate_unimap(SshADTContainer domain, SshADTContainer range)
 
   range->hooks->destr = hook_destroy;
   range->hooks->destr_ctx = ctx;
+
+  return TRUE;
 }
 
-void ssh_adt_associate_bimap(SshADTContainer domain, SshADTContainer range)
+Boolean ssh_adt_associate_bimap(SshADTContainer domain, SshADTContainer range)
 {
   SshADTAssociationContext *ctx;
 
-  ctx = SSH_MEM_ALLOC(sizeof(*ctx));
+  if (!(ctx = ssh_malloc(sizeof(*ctx))))
+    return FALSE;
 
   ctx->domain = domain;
   ctx->range  = range;
@@ -151,7 +155,6 @@ void ssh_adt_associate_bimap(SshADTContainer domain, SshADTContainer range)
   domain->hooks->detach = hook_domain_detach;
   domain->hooks->detach_ctx = ctx;
 
-
   range->hooks->unmap = hook_range_unmap2;
   range->hooks->unmap_ctx = ctx;
 
@@ -161,18 +164,19 @@ void ssh_adt_associate_bimap(SshADTContainer domain, SshADTContainer range)
   range->hooks->detach = hook_range_detach;
   range->hooks->detach_ctx = ctx;
 
-
   domain->hooks->destr = hook_destroy;
   domain->hooks->destr_ctx = ctx;
 
   range->hooks->destr = hook_destroy;
   range->hooks->destr_ctx = ctx;
+
+  return TRUE;
 }
 
 void ssh_adt_unassociate(SshADTContainer c1, SshADTContainer c2)
 {
   /* Delete the association context. */
-  SSH_MEM_FREE(c1->hooks->destr_ctx);
+  ssh_free(c1->hooks->destr_ctx);
 
   ssh_adt_uninitialize_hooks(c1);
   ssh_adt_uninitialize_hooks(c2);
