@@ -14,6 +14,9 @@
 #ifndef SSH_ADT_BAG_H_INCLUDED
 #define SSH_ADT_BAG_H_INCLUDED
 
+#include "sshadt.h"
+
+
 extern const SshADTContainerType ssh_adt_bag_type;
 
 #define SSH_ADT_BAG (ssh_adt_bag_type)
@@ -23,4 +26,11 @@ extern const SshADTContainerType ssh_adt_bag_type;
 Boolean ssh_adt_bag_changed(SshADTContainer bag,
                             SshADTHandle object_with_changed_contents);
 
+/* Use this instead of SshADTHeaderStruct if you want to save 8 bytes
+   of memory per header.  */
+typedef struct {
+  Boolean a; void *b, *c;
+} SshADTBagHeaderStruct;
+
+
 #endif /* SSH_ADT_BAG_H_INCLUDED */
