@@ -15,6 +15,8 @@
 #ifndef SSHADT_CONV_I_H_INCLUDED
 #define SSHADT_CONV_I_H_INCLUDED
 
+#include "sshadt.h"
+
 
 /******************************************** Convenience callbacks: General */
 
@@ -34,7 +36,7 @@ int ssh_adt_callback_compare_str(const void *obj1, const void *obj2, void *ctx);
 void *ssh_adt_callback_duplicate_str(const void *obj, void *ctx);
 
 /* Iteration over a string with some shifts and additions. */
-unsigned long ssh_adt_callback_hash_str(const void *obj, void *ctx);
+SshUInt32 ssh_adt_callback_hash_str(const void *obj, void *ctx);
 
 
 /******************************************* Convenience callbacks: Integers */
@@ -43,8 +45,8 @@ unsigned long ssh_adt_callback_hash_str(const void *obj, void *ctx);
 int ssh_adt_callback_compare_int(const void *obj1, const void *obj2,
                                  void *context);
 
-/* returns `*(unsigned long *)obj' */
-unsigned long ssh_adt_callback_hash_int(const void *obj, void *ctx);
+/* returns `*(SshUInt32 *)obj' */
+SshUInt32 ssh_adt_callback_hash_int(const void *obj, void *ctx);
 
 
 
