@@ -14,82 +14,41 @@
 #ifndef SSHADT_I_H_INCLUDED
 #define SSHADT_I_H_INCLUDED
 
-/* Object layout.
-
-   1. Internally allocated object without inlined header.
-
-   +---------+
-   |Header   | <====== HANDLE
-   +---------+ -----------------------
-   |Object   | <====== OBJECT        | Visible to application
-   |         |                       |
-   +---------+
-
-   The whole object is allocated by the library. Handles are pointers
-   to the headers.
-
-
-   2. External object with an inlined header.
-
-   +--------+
-   |Object  | <====== OBJECT
-   ++-------+
-   ||Header | <====== HANDLE
-   ++-------+
-   |        |
-   +--------+
-
-   The whole object is given from the outside. Handles are pointers to
-   the internal headers.
-
-
-   3. External object without inlined header.
-                
-                 +------+
-                 |Object| ---------+
-                 +------+   |   +--V-----+
-   HANDLE ======>|Node  |   |   |Object  | <===== OBJECT
-                 +------+   |   +--------+
-                            |
-                            |----> Visible to application
-
-   Node is allocated by the library, object is given from the outside.
-   Handles are pointers to the node data part.
-
-   When this layout is used, the object pointer must be the FIRST
-   FIELD in the node.
-
-
-   4. Internally allocated object with inlined header.
-
-   +--------+ -----------------------
-   |Object  | <====== OBJECT        | Visible to application
-   ++-------+                       |
-   ||Header | <====== HANDLE
-   ++-------+
-   |        |
-   +--------+
-
-*/
-   
 
 #include "sshadt.h"
 
 /* It is possible that sshadt_structs.h does not get included from
-   sshadt.h, so include it here anyway. */
-
+   sshadt.h, so just try again. */
 #define SSHADT_INSIDE_SSHADT_H
 #include "sshadt_structs.h"
 #undef SSHADT_INSIDE_SSHADT_H
 
-#include "sshfastalloc.h"
+/* workaround for macro optimization bug in MSC (by siren@ssh.fi) */
+#if _MSC_VER == 1200
+#pragma optimize( "", off )
+#endif
 
-#define SSH_ADT_FLAG_REFCOUNT           0x0001
+/* Memory modes and object layout. */
+
+/* done by the library (ON) or by the caller (OFF) */
 #define SSH_ADT_FLAG_ALLOCATE           0x0002
+
+/* header is inlined (ON) or maintained by the library (OFF) */
 #define SSH_ADT_FLAG_CONTAINED_HEADER   0x0004
+
+/* If invisible library-maintained headers and user-allocated memory
+   (void pointer semantics) are chosen, this flag is high.  In this
+   case the behaviour of the mappings between objects and handles is
+   quite different.  In particular, objects can NOT be mapped to
+   handles by efficient pointer arithmetics.  Only handle->object can
+   be done in O(1).  (See sshadt_std_i.h and the documentation for
+   ssh_adt_get_handle_to.)
+
+   In e.g. the list implementation, we have two data types Node and
+   ENode.  ENode is short for `extra node', meaning that the ENode
+   thing is present if and only if this flag is high. */
 #define SSH_ADT_FLAG_NEED_EXTRA_NODES   0x0008
-#define SSH_ADT_FLAG_PROXY_CONTAINER    0x0010
-#define SSH_ADT_FLAG_FASTALLOC          0x0020
+
 
 /* Hooks. */
 
@@ -113,35 +72,31 @@ struct ssh_adt_hooks {
   void *destr_ctx;
 };
 
-typedef struct ssh_adt_hooks SshADTHooks;
+typedef struct ssh_adt_hooks SshADTHooksStruct;
 
 #define SSH_ADT_ARGS1(x) , x
 #define SSH_ADT_ARGS2(x, y) , x, y
 #define SSH_ADT_ARGS0() 
 
-#define SSH_ADT_CALL_HOOK_GENERIC(container, hook_name, args)   \
-do                                                              \
-{                                                               \
-  SshADTHooks *__h = container->hooks;                          \
-  if (__h == NULL) break;                                       \
-  if (__h->hook_name == NULL) break;                            \
-                                                                \
-  (*(__h->hook_name)) args;                                     \
-}                                                               \
+#define SSH_ADT_CALL_HOOK_GENERIC(container, hook_name, args)                 \
+do                                                                            \
+{                                                                             \
+  SshADTHooksStruct *__h = container->hooks;                                  \
+  if (__h == NULL) break;                                                     \
+  if (__h->hook_name == NULL) break;                                          \
+                                                                              \
+  (*(__h->hook_name)) args;                                                   \
+}                                                                             \
 while(0)
 
-#define SSH_ADT_CALL_HOOK_0(container, hook_name) \
-SSH_ADT_CALL_HOOK_GENERIC(container, hook_name, (__h->hook_name ## _ctx))
-
-#define SSH_ADT_CALL_HOOK_1(container, hook_name, a) \
+#define SSH_ADT_CALL_HOOK(container, hook_name, a) \
 SSH_ADT_CALL_HOOK_GENERIC(container, hook_name, (a, __h->hook_name ## _ctx))
 
-#define SSH_ADT_CALL_HOOK_2(container, hook_name, a, b) \
-SSH_ADT_CALL_HOOK_GENERIC(container, hook_name, (a, b, __h->hook_name ## _ctx))
+#define SSH_ADT_CALL_DESTROY_HOOK(container, hook_name) \
+SSH_ADT_CALL_HOOK_GENERIC(container, hook_name, (__h->hook_name ## _ctx))
 
-#define SSH_ADT_CALL_REALLOC_HOOK(c,h,a,b) SSH_ADT_CALL_HOOK_2(c,h,a,b)
-#define SSH_ADT_CALL_HOOK(c,h,a) SSH_ADT_CALL_HOOK_1(c,h,a)
-#define SSH_ADT_CALL_DESTROY_HOOK(c,h) SSH_ADT_CALL_HOOK_0(c,h)
+#define SSH_ADT_CALL_REALLOC_HOOK(container, hook_name, a, b) \
+SSH_ADT_CALL_HOOK_GENERIC(container, hook_name, (a, b, __h->hook_name ## _ctx))
 
 int ssh_adt_initialize_hooks(SshADTContainer container);
 void ssh_adt_uninitialize_hooks(SshADTContainer container);
@@ -179,12 +134,12 @@ typedef struct ssh_adt_container_pars {
 void ssh_adt_genlist_add(void **listptr, void *object, int next_ptr_offset);
 void ssh_adt_genlist_remove(void **listptr, void *object, int next_ptr_offset);
 
-#define SSH_ADT_GENLIST_ADD(object_type, next_field, object, list_ptr)  \
-do                                                                      \
-{                                                                       \
-  ssh_adt_genlist_add(list_ptr, object,                                 \
-                      SSH_ADT_OFFSET_OF(object_type, next_field));      \
-}                                                                       \
+#define SSH_ADT_GENLIST_ADD(object_type, next_field, object, list_ptr)        \
+do                                                                            \
+{                                                                             \
+  ssh_adt_genlist_add(list_ptr, object,                                       \
+                      SSH_ADT_OFFSET_OF(object_type, next_field));            \
+}                                                                             \
 while(0)
 
 #define SSH_ADT_GENLIST_REMOVE(object_type, next_field, object, list_ptr)     \
@@ -195,14 +150,14 @@ do                                                                            \
 }                                                                             \
 while(0)
 
-#define SSH_ADT_GENLIST_MOVE(object_type, next_field, object, from, to) \
-do                                                                      \
-{                                                                       \
-  ssh_adt_genlist_add(list_ptr, object,                                 \
-                      SSH_ADT_OFFSET_OF(object_type, next_field));      \
-  ssh_adt_genlist_remove(list_ptr, object,                              \
-                         SSH_ADT_OFFSET_OF(object_type, next_field));   \
-}                                                                       \
+#define SSH_ADT_GENLIST_MOVE(object_type, next_field, object, from, to)       \
+do                                                                            \
+{                                                                             \
+  ssh_adt_genlist_add(list_ptr, object,                                       \
+                      SSH_ADT_OFFSET_OF(object_type, next_field));            \
+  ssh_adt_genlist_remove(list_ptr, object,                                    \
+                         SSH_ADT_OFFSET_OF(object_type, next_field));         \
+}                                                                             \
 while(0)
 
 #define SSH_ADT_CALL_APP(container, method, args)                                  \
@@ -210,12 +165,12 @@ do
 {                                                                                  \
   if (container->f.app_methods.method != NULL)                                     \
     {                                                                              \
-      SSH_DEBUG(6, ("Invoking callback @%p.", container->f.app_methods.method));   \
+      SSH_DEBUG(9, ("Invoking callback @%p.", container->f.app_methods.method));   \
       (*(container->f.app_methods.method))args;                                    \
-      SSH_DEBUG(6, ("Callback @%p returned.", container->f.app_methods.method));   \
+      SSH_DEBUG(9, ("Callback @%p returned.", container->f.app_methods.method));   \
     }                                                                              \
   else                                                                             \
-    SSH_DEBUG(1, ("Reference to non-existent callback (doing nothing)."));         \
+    SSH_DEBUG(9, ("Reference to non-existent callback (doing nothing)."));         \
 }                                                                                  \
 while(0)
 
@@ -223,15 +178,23 @@ while(0)
 do                                                                                 \
 {                                                                                  \
   SSH_ASSERT(container->f.app_methods.method);                                     \
-  SSH_DEBUG(6, ("Invoking callback @%p.", container->f.app_methods.method));       \
+  SSH_DEBUG(9, ("Invoking callback @%p.", container->f.app_methods.method));       \
   result = (*(container->f.app_methods.method))args;                               \
-  SSH_DEBUG(6, ("Callback @%p returned.", container->f.app_methods.method));       \
+  SSH_DEBUG(9, ("Callback @%p returned.", container->f.app_methods.method));       \
 }                                                                                  \
 while(0)
 
 #define SSH_ADT_APPCTX(container) \
   (container->f.app_methods.context)
 
+
+
+#ifdef MSC_REPLACE_SOME_SUSPISIOUS_MACROS  /* XXX */
+void *SSH_ADT_OBJECT_AT_NODE(void *n);
+SshADTHandle SSH_ADT_HANDLE_FROM_OBJECT(SshADTContainer c, void *o);
+void *SSH_ADT_OBJECT_FROM_HANDLE(SshADTContainer c, SshADTHandle h);
+
+#else  /* MSC_REPLACE_SOME_SUSPISIOUS_MACROS */
 #define SSH_ADT_OBJECT_AT_NODE(n) \
   (*((void **)(((unsigned char *)n) - sizeof(void *))))
 
@@ -244,9 +207,14 @@ while(0)
 #define SSH_ADT_OBJECT_FROM_HANDLE(c, h) \
 ((void *)(((unsigned char *) h) - ((c)->f.header_offset)))
 
+#endif  /* MSC_REPLACE_SOME_SUSPISIOUS_MACROS */
+
+
+
 #define SSH_ADT_HASH_OBJECT(c, obj, result) \
 SSH_ADT_CALL_APP_MANDATORY(c, hash, (obj, SSH_ADT_APPCTX(c)), result)
 
+
 /* And then for something different, namely the registration of
    handles can be used internally for implementing e.g. foreach. */
 
@@ -281,4 +249,25 @@ void *ssh_adt_alloc_i(SshADTContainer container, size_t size);
 void *ssh_adt_detach_i(SshADTContainer container, SshADTHandle handle);
 #endif
 
+/* Default callbacks need to be available in at least one case (lists
+   don't like user-provided hash callbacks and launch a warning,
+   because those cannot possibly be accessed anywhere).  */
+int ssh_adt_default_compare(const void *obj1, const void *obj2,
+                            void *context);
+
+void *ssh_adt_default_duplicate(void *obj, void *context);
+
+void ssh_adt_default_copy(void *dst, size_t d_size,
+                          const void *src, void *context);
+
+void ssh_adt_default_destroy(void *obj, void *context);
+
+void ssh_adt_default_init(void *obj, size_t size, void *context);
+
+void ssh_adt_no_init(void *data, void *context);
+
+SshUInt32 ssh_adt_default_hash(const void *obj, void *context);
+
+
+
 #endif /* SSHADT_I_H_INCLUDED */
