@@ -4,7 +4,7 @@
 
   Author: Antti Huima <huima@ssh.fi>
 
-  Copyright (c) 1999 SSH Communications Security, Finland
+  Copyright (c) 1999, 2001 SSH Communications Security, Finland
   All rights reserved.
 
   Created Thu Sep  9 11:57:24 1999.
@@ -19,29 +19,21 @@
 
 #define SSH_DEBUG_MODULE "SshADTList"
 
-#define ROOT     ((SshADTListRoot *)(c->container_specific))
-#define NODE(x)  ((SshADTListNode *)x)
-
-static void *make_node(SshADTContainer c)
-{
-  void *ptr;
-  if (!(ptr = SSH_MEM_ALLOC(sizeof(SshADTListENode))))
-    return NULL;
-  return ptr;
-}
-
-static void free_node(SshADTContainer c, void *node)
-{
-  SSH_DEBUG(99, ("freeing node %p.", node));
-  SSH_MEM_FREE(node);
-  SSH_DEBUG(99, ("ok"));
-}
+#define ROOT(c)     ((SshADTListRootStruct *)(c->container_specific))
+#define NODE(x)     ((SshADTListNodeStruct *)(x))
 
 static Boolean init(SshADTContainer c)
 {
-  if (!(c->container_specific = SSH_MEM_ALLOC(sizeof(*ROOT))))
+  if (!(c->container_specific = ssh_malloc(sizeof(*ROOT(c)))))
     return FALSE;
-  ROOT->first_node = ROOT->last_node = NULL;
+  ROOT(c)->first_node = ROOT(c)->last_node = NULL;
+
+  if(c->f.app_methods.hash != ssh_adt_default_hash)
+    {
+      SSH_DEBUG(0, ("*** You have provided a hash callback for a list."));
+      SSH_DEBUG(0, ("*** Please report to maintainer."));
+    }
+
   return TRUE;
 }
 
@@ -51,9 +43,9 @@ SSH_ADT_STD_INIT(container_init, init(c))
 
 static void uninit(SshADTContainer c)
 {
-  while (ROOT->first_node != NULL)
+  while (ROOT(c)->first_node != NULL)
     ssh_adt_delete_from(c, SSH_ADT_BEGINNING);
-  SSH_MEM_FREE(ROOT);
+  ssh_free(ROOT(c));
 }
 
 /* $$METHOD(list, destr) */
@@ -62,25 +54,25 @@ SSH_ADT_STD_DESTROY(destr, uninit(c);)
 /* $$METHOD(list, clear) */
 static void clear(SshADTContainer c)
 {
-  while (ROOT->first_node != NULL)
+  while (ROOT(c)->first_node != NULL)
     ssh_adt_delete_from(c, SSH_ADT_BEGINNING);
 }
 
 #if 0
-static void sanity_dump(SshADTListRoot *r)
+static void sanity_dump(SshADTListRootStruct *r)
 {
-  SshADTListNode *n = r->first_node;
+  SshADTListNodeStruct *n = r->first_node;
   while (n != NULL)
     {
-      fprintf(stderr, "%p next:%p prev:%p\n", 
+      fprintf(stderr, "%p next:%p prev:%p\n",
               n, n->next, n->prev);
       n = n->next;
     }
 }
 
-static void sanity_check(SshADTListRoot *r)
+static void sanity_check(SshADTListRootStruct *r)
 {
-  SshADTListNode *n = r->first_node;
+  SshADTListNodeStruct *n = r->first_node;
   while (n != NULL)
     {
       if ((n->prev != NULL && n->prev->next != n)
@@ -103,21 +95,21 @@ static void sanity_check(SshADTListRoot *r)
 /* $$METHOD(list, num_objects) */
 SSH_ADT_STD_NUM_OBJECTS(num_objects)
 
-static void insert_at_beginning(SshADTListRoot *r,
-                                SshADTListNode *n)
+static void insert_at_beginning(SshADTListRootStruct *r,
+                                SshADTListNodeStruct *n)
 {
   SSH_ADT__LIST_INSERT_TO_BEGINNING(r, n);
 }
 
-static void insert_at_end(SshADTListRoot *r,
-                          SshADTListNode *n)
+static void insert_at_end(SshADTListRootStruct *r,
+                          SshADTListNodeStruct *n)
 {
   SSH_ADT__LIST_INSERT_TO_END(r, n);
 }
 
-static Boolean insert_prior_to_node(SshADTListRoot *r,
-                                    SshADTListNode *p,
-                                    SshADTListNode *n)
+static Boolean insert_prior_to_node(SshADTListRootStruct *r,
+                                    SshADTListNodeStruct *p,
+                                    SshADTListNodeStruct *n)
 {
   if (n == NULL)
     return FALSE;
@@ -137,9 +129,9 @@ static Boolean insert_prior_to_node(SshADTListRoot *r,
   return TRUE;
 }
 
-static Boolean insert_after_node(SshADTListRoot *r,
-                                 SshADTListNode *p,
-                                 SshADTListNode *n)
+static Boolean insert_after_node(SshADTListRootStruct *r,
+                                 SshADTListNodeStruct *p,
+                                 SshADTListNodeStruct *n)
 {
   if (n == NULL)
     return FALSE;
@@ -165,9 +157,9 @@ static Boolean my_insert_relative(SshADTContainer c, SshADTRelativeLocation l,
   switch (l)
     {
     case SSH_ADT_BEFORE:
-      return (insert_prior_to_node(ROOT, where, newp));
+      return (insert_prior_to_node(ROOT(c), where, newp));
     case SSH_ADT_AFTER:
-      return (insert_after_node(ROOT, where, newp));
+      return (insert_after_node(ROOT(c), where, newp));
     default:
       SSH_NOTREACHED;
       return FALSE;
@@ -177,7 +169,7 @@ static Boolean my_insert_relative(SshADTContainer c, SshADTRelativeLocation l,
 /* $$METHOD(list, insert_at) */
 SSH_ADT_STD_INSERT_AT(insert_relative,
                       my_insert_relative(c, location, handle, h),
-                      __handle = make_node(c);)
+                      __handle = ssh_malloc(sizeof(SshADTListENodeStruct));)
 
 
 /* return the handle of the nth object in a list.  returns
@@ -185,7 +177,7 @@ SSH_ADT_STD_INSERT_AT(insert_relative,
  * 0). */
 static SshADTHandle get_nth(SshADTContainer c, int location)
 {
-  SshADTListNode *n;
+  SshADTListNodeStruct *n;
 
   if (location >= num_objects(c) || location < 0)
     {
@@ -193,7 +185,7 @@ static SshADTHandle get_nth(SshADTContainer c, int location)
       return SSH_ADT_INVALID;
     }
 
-  n = ROOT->first_node;
+  n = ROOT(c)->first_node;
   SSH_ASSERT(n != NULL);
 
   while(location--) {
@@ -208,19 +200,21 @@ static Boolean my_insert_absolute(SshADTContainer c, SshADTAbsoluteLocation l,
                                   SshADTHandle newp)
 {
   SshADTHandle where;
-          
+
+  /* (SshADTHandle == SshADTListNodeStruct *) */
+
   if (newp == NULL)
     return FALSE;
 
   switch (l)
     {
     case SSH_ADT_BEGINNING:
-      insert_at_beginning(ROOT, newp);
+      insert_at_beginning(ROOT(c), newp);
       return TRUE;
 
     case SSH_ADT_END:
     case SSH_ADT_DEFAULT:
-      insert_at_end(ROOT, newp);
+      insert_at_end(ROOT(c), newp);
       return TRUE;
 
     default:
@@ -230,7 +224,7 @@ static Boolean my_insert_absolute(SshADTContainer c, SshADTAbsoluteLocation l,
           /* catch special case `append to end of list' */
           if (SSH_ADT_UNINDEX(l) == num_objects(c))
             {
-              insert_at_end(ROOT, newp);
+              insert_at_end(ROOT(c), newp);
               return TRUE;
             }
           else
@@ -241,7 +235,7 @@ static Boolean my_insert_absolute(SshADTContainer c, SshADTAbsoluteLocation l,
         }
       else
         {
-          insert_prior_to_node(ROOT, where, newp);
+          insert_prior_to_node(ROOT(c), where, newp);
           return TRUE;
         }
     }
@@ -250,7 +244,7 @@ static Boolean my_insert_absolute(SshADTContainer c, SshADTAbsoluteLocation l,
 /* $$METHOD(list, insert_to) */
 SSH_ADT_STD_INSERT_TO(insert_absolute,
                       my_insert_absolute(c, location, h),
-                      __handle = make_node(c);)
+                      __handle = ssh_malloc(sizeof(SshADTListENodeStruct));)
 
 /* $$METHOD(list, alloc_n_at) */
 SSH_ADT_STD_ALLOC_N_AT(alloc_n_at,
@@ -273,7 +267,7 @@ SSH_ADT_STD_GET(get)
 
 static SshADTHandle find_node(SshADTContainer c, void *object)
 {
-  SshADTListNode *n = ROOT->first_node;
+  SshADTListNodeStruct *n = ROOT(c)->first_node;
 
   while (n != NULL)
     {
@@ -296,13 +290,13 @@ static SshADTHandle get_location(SshADTContainer c,
   switch (location)
     {
     case SSH_ADT_BEGINNING:
-      if (ROOT->first_node == NULL) return SSH_ADT_INVALID;
-      return ROOT->first_node;
+      if (ROOT(c)->first_node == NULL) return SSH_ADT_INVALID;
+      return ROOT(c)->first_node;
 
     case SSH_ADT_END:
     case SSH_ADT_DEFAULT:
-      if (ROOT->last_node == NULL) return SSH_ADT_INVALID;
-      return ROOT->last_node;
+      if (ROOT(c)->last_node == NULL) return SSH_ADT_INVALID;
+      return ROOT(c)->last_node;
 
     default:
       return get_nth(c, SSH_ADT_UNINDEX(location));
@@ -312,7 +306,7 @@ static SshADTHandle get_location(SshADTContainer c,
 /* $$METHOD(list, enumerate_start) */
 static SshADTHandle enum_start(SshADTContainer c)
 {
-  return ROOT->first_node;
+  return ROOT(c)->first_node;
 }
 
 /* $$METHOD(list, enumerate_next) */
@@ -332,9 +326,9 @@ static SshADTHandle to_previous(SshADTContainer c, SshADTHandle h)
   return h;
 }
 
-static void detach_at_beginning(SshADTListRoot *r)
+static void detach_at_beginning(SshADTListRootStruct *r)
 {
-  SshADTListNode *n;
+  SshADTListNodeStruct *n;
   n = r->first_node;
 
   if (r->last_node == r->first_node)
@@ -351,9 +345,9 @@ static void detach_at_beginning(SshADTListRoot *r)
     }
 }
 
-static void detach_at_end(SshADTListRoot *r)
+static void detach_at_end(SshADTListRootStruct *r)
 {
-  SshADTListNode *n;
+  SshADTListNodeStruct *n;
   n = r->last_node;
 
   if (r->last_node == r->first_node)
@@ -370,8 +364,8 @@ static void detach_at_end(SshADTListRoot *r)
     }
 }
 
-static void detach_at(SshADTListRoot *r,
-                      SshADTListNode *n)
+static void detach_at(SshADTListRootStruct *r,
+                      SshADTListNodeStruct *n)
 {
   SSH_PRECOND(r);
   SSH_PRECOND(n);
@@ -389,15 +383,32 @@ static void detach_at(SshADTListRoot *r,
 }
 
 /* $$METHOD(list, detach) */
-SSH_ADT_STD_DETACH(detach, detach_at(ROOT, handle);, free_node(c, node);)
+SSH_ADT_STD_DETACH(detach, detach_at(ROOT(c), handle);, ssh_free(node);)
+
+/* $$METHOD(list, get_handle_to_equal) */
+static SshADTHandle to_equal(SshADTContainer c, void *object)
+{
+  SshADTHandle h;
+  int result;
+
+  for (h = ROOT(c)->first_node; h != SSH_ADT_INVALID; h = NODE(h)->next)
+    {
+      SSH_ADT_STD_COMPARE_H_O(c, h, object, result);
+
+      if (!result)
+        return h;
+    }
+
+  return SSH_ADT_INVALID;
+}
 
 #ifndef _KERNEL
 static void reallocated(SshADTContainer c, SshADTHandle old, SshADTHandle newp)
 {
-  SshADTListNode *n = newp;
-  if (n->prev == NULL) ROOT->first_node = n;
+  SshADTListNodeStruct *n = newp;
+  if (n->prev == NULL) ROOT(c)->first_node = n;
   else n->prev->next = n;
-  if (n->next == NULL) ROOT->last_node = n;
+  if (n->next == NULL) ROOT(c)->last_node = n;
   else n->next->prev = n;
 }
 #endif  /* _KERNEL */
@@ -432,13 +443,15 @@ SshADTStaticData ssh_adt_list_static_data =
     to_previous, /* previous */
     enum_start, /* enumerate_start */
     enum_next, /* enumerate_next */
-    NULL, /* get_handle_to_equal */
+    to_equal, /* get_handle_to_equal */
     reallocate, /* reallocate */
     detach, /* detach */
     delet, /* delet */
+    NULL, /* map_lookup */
+    NULL, /* map_attach */
     /* $$ENDMETHODS */
   },
-  sizeof(SshADTListNode),
+  sizeof(SshADTListNodeStruct),
   0
 };
 
@@ -471,25 +484,26 @@ const SshADTContainerType ssh_adt_list_type = &ssh_adt_list_static_data;
 
 void ssh_adt_list_sort(SshADTContainer c)
 {
-  SshADTListNode *table[64];    /* Enough to sort around 2^64 entries... */
-  SshADTListNode *cursor;
-  SshADTListNode *t1, *t2, *t, *tail;
+  SshADTListNodeStruct *table[64];    /* Enough to sort around 2^64
+                                       * entries... */
+  SshADTListNodeStruct *cursor;
+  SshADTListNodeStruct *t1, *t2, *t, *tail;
   int i;
   int result;
 
   SSH_ASSERT(c->static_data == ssh_adt_list_type);
 
-  cursor = ROOT->first_node;
+  cursor = ROOT(c)->first_node;
 
   if (cursor == NULL || cursor->next == NULL) return;
-  
+
   /* Clear the table. */
   for (i = 0; i < 64; i++)
     table[i] = NULL;
 
   while (cursor != NULL)
     {
-      SshADTListNode *op1, *op2;
+      SshADTListNodeStruct *op1, *op2;
 
       op1 = cursor;
       op2 = cursor->next;
@@ -500,8 +514,8 @@ void ssh_adt_list_sort(SshADTContainer c)
           SSH_ADT_STD_COMPARE(c, op1, op2, result);
           if (result > 0)
             {
-              SshADTListNode *t;
-              
+              SshADTListNodeStruct *t;
+
               /* Swap. */
               t = op2; op2 = op1; op1 = t;
 
@@ -531,9 +545,9 @@ void ssh_adt_list_sort(SshADTContainer c)
           t1 = table[i];
           t2 = op1;
           op1 = t1;
-          
-          MERGE(op1); 
-          
+
+          MERGE(op1);
+
           table[i] = NULL;
 
           SSH_ASSERT(i < 64);
@@ -549,9 +563,9 @@ void ssh_adt_list_sort(SshADTContainer c)
           t1 = table[i];
           t2 = cursor;
           cursor = t1;
-          
+
           MERGE(cursor);
-          
+
           table[i] = NULL;
         }
       else
@@ -563,12 +577,12 @@ void ssh_adt_list_sort(SshADTContainer c)
 
   /* `cursor' now points to the first element. */
 
-  ROOT->first_node = cursor;
+  ROOT(c)->first_node = cursor;
   cursor->prev = NULL;
   while ((t1 = cursor->next) != NULL)
     {
       t1->prev = cursor;
       cursor = t1;
     }
-  ROOT->last_node = cursor;
+  ROOT(c)->last_node = cursor;
 }
