@@ -4,7 +4,7 @@
 
   Author: Antti Huima <huima@ssh.fi>
 
-  Copyright (c) 1999, 2000 SSH Communications Security, Finland
+  Copyright (c) 1999-2001 SSH Communications Security, Finland
   All rights reserved.
 
   Created Thu Sep  9 15:41:50 1999.
@@ -14,50 +14,73 @@
 #include "sshincludes.h"
 #include "sshadt_i.h"
 #include "sshadt_map.h"
+#include "sshadt_bag.h"
 #include "sshadt_map_i.h"
 #include "sshadt_bag_i.h"
 #include "sshadt_std_i.h"
 #include "sshdebug.h"
 
+
+#define TEMPORARY_DEBUGGING_CODE   /* XXX code will be removed after. */
+
+
 #define SSH_DEBUG_MODULE "SshADTMap"
 
-#define SSH_ADT_MAP_INITIAL_SLOTS 11
+/* how big is the array initially */
+#define SSH_ADT_MAP_INITIAL_SLOTS   11
+
+/* How many objects must be in one slot before rehashing the array?
+   if (objects * X > slots), double the array size.  The array size is
+   never decreased. */
+#define SSH_ADT_MAP_RATIO            2
+
+/* convenience macros */
+#define ROOT(c)     ((SshADTMapRoot *)(c->container_specific))
+#define NODE(x)     ((SshADTMapNode *)(x))
 
-#define ROOT ((SshADTMapRoot *)(c->container_specific))
-#define NODE(x) ((SshADTMapNode *)x)
 
 static void *make_node(SshADTContainer c)
 {
   SshADTMapENode *e;
-  if (!(e = SSH_MEM_ALLOC(sizeof(*e))))
+  if (!(e = ssh_malloc(sizeof(*e))))
     return SSH_ADT_INVALID;
   return e;
 }
 
 static void free_node(SshADTContainer c, void *node)
 {
-  SSH_MEM_FREE(node);
+  ssh_free(node);
 }
 
 static Boolean init_to_size(SshADTMapRoot *r, int size)
 {
   int i;
-  r->array_size = size;
-  if (!(r->nodes = SSH_MEM_ALLOC(sizeof(r->nodes[0]) * r->array_size)))
+  SshADTMapNode **nodes;
+
+  if (!(nodes = ssh_malloc(sizeof(r->nodes[0]) * size)))
     return FALSE;
+
+  r->array_size = size;
+  r->nodes = nodes;
+
   for (i = 0; i < r->array_size; i++)
-    {
-      r->nodes[i] = NULL;
-    }
+    r->nodes[i] = NULL;
+
   return TRUE;
 }
 
 static Boolean init(SshADTContainer c)
 {
-  if (!(c->container_specific = SSH_MEM_ALLOC(sizeof(SshADTMapRoot))))
+  if (!(c->container_specific = ssh_malloc(sizeof(SshADTMapRoot))))
     return FALSE;
-  init_to_size(ROOT, SSH_ADT_MAP_INITIAL_SLOTS);
-  ROOT->num_objects = 0;
+
+  if (!init_to_size(ROOT(c), SSH_ADT_MAP_INITIAL_SLOTS))
+    {
+      ssh_free(c->container_specific);
+      c->container_specific = NULL;
+      return FALSE;
+    }
+  ROOT(c)->num_objects = 0;
   return TRUE;
 }
 
@@ -70,11 +93,11 @@ SSH_ADT_STD_INIT(container_init, init(c))
 static void clear(SshADTContainer c)
 {
   int i;
-  for (i = 0; i < ROOT->array_size; i++)
+  for (i = 0; i < ROOT(c)->array_size; i++)
     {
-      while (ROOT->nodes[i] != NULL)
+      while (ROOT(c)->nodes[i] != NULL)
         {
-          ssh_adt_delete(c, ROOT->nodes[i]);
+          ssh_adt_delete(c, ROOT(c)->nodes[i]);
         }
     }
   SSH_DEBUG(9, ("map cleared."));
@@ -83,8 +106,8 @@ static void clear(SshADTContainer c)
 static void uninit(SshADTContainer c)
 {
   clear(c);
-  SSH_MEM_FREE(ROOT->nodes);
-  SSH_MEM_FREE(c->container_specific);
+  ssh_free(ROOT(c)->nodes);
+  ssh_free(c->container_specific);
 }
 
 /* $$METHOD(map, destr) */
@@ -94,11 +117,11 @@ SSH_ADT_STD_DESTROY(destr, uninit(c);)
 static SshADTHandle next_rib_start(SshADTContainer c, int start_idx)
 {
   int i;
-  for (i = start_idx; i < ROOT->array_size; i++)
+  for (i = start_idx; i < ROOT(c)->array_size; i++)
     {
-      if (ROOT->nodes[i] != NULL)
+      if (ROOT(c)->nodes[i] != NULL)
         {
-          return ROOT->nodes[i];
+          return ROOT(c)->nodes[i];
         }
     }
   return SSH_ADT_INVALID;
@@ -118,8 +141,8 @@ static SshADTHandle forward(SshADTContainer c, SshADTHandle h)
   if (d->is_last_in_rib)
     {
       SshADTMapNode **ptr = d->u.rib_start;
-      int idx = ((unsigned char *)ptr - (unsigned char *)(ROOT->nodes))
-        / (sizeof(ROOT->nodes[0]));
+      int idx = ((unsigned char *)ptr - (unsigned char *)(ROOT(c)->nodes))
+        / (sizeof(ROOT(c)->nodes[0]));
       return next_rib_start(c, idx + 1);
     }
   else
@@ -136,40 +159,32 @@ static SshADTHandle enum_next(SshADTContainer c, SshADTHandle h)
   return forward(c, h);
 }
 
-static void rehash(SshADTContainer c, size_t new_size);
 
-static Boolean insert(SshADTContainer c, SshADTMapNode *n)
-{
-  unsigned long hash_value;
+/* changing the array size, rehashing and insertion are closely
+   entangled. */
 
-  if (n == NULL)  /* e.g., if the duplicate callback returns it. */
-    return FALSE;
-
-  SSH_ADT_STD_HASH(c, n, hash_value);
-  hash_value %= (ROOT->array_size);
-
-  if (ROOT->nodes[hash_value] == NULL)
-    {
-      n->is_last_in_rib = TRUE;
-      n->u.rib_start = &(ROOT->nodes[hash_value]);
-      ROOT->nodes[hash_value] = n;
-    }
-  else
-    {
-      n->is_last_in_rib = FALSE;
-      n->u.next = ROOT->nodes[hash_value];
-      ROOT->nodes[hash_value] = n;
-    }
-
-  ROOT->num_objects++;
-
-  if (ROOT->num_objects * 2 > ROOT->array_size)
-    {
-      rehash(c, ROOT->array_size * 2 + 1);
-    }
-
-  return TRUE;
-}
+#define INSERT_WITHOUT_REHASH(c, n)                                           \
+do                                                                            \
+{                                                                             \
+  SshUInt32 hash_value;                                                       \
+                                                                              \
+  SSH_ADT_STD_HASH(c, n, hash_value);                                         \
+  hash_value %= (ROOT(c)->array_size);                                        \
+                                                                              \
+  if (ROOT(c)->nodes[hash_value] == NULL)                                     \
+    {                                                                         \
+      n->is_last_in_rib = TRUE;                                               \
+      n->u.rib_start = &(ROOT(c)->nodes[hash_value]);                         \
+      ROOT(c)->nodes[hash_value] = n;                                         \
+    }                                                                         \
+  else                                                                        \
+    {                                                                         \
+      n->is_last_in_rib = FALSE;                                              \
+      n->u.next = ROOT(c)->nodes[hash_value];                                 \
+      ROOT(c)->nodes[hash_value] = n;                                         \
+    }                                                                         \
+}                                                                             \
+while(0)
 
 static void rehash(SshADTContainer c, size_t new_size)
 {
@@ -178,49 +193,65 @@ static void rehash(SshADTContainer c, size_t new_size)
   int old_size;
   int i;
 
-  SSH_DEBUG(6, ("Rehashing, old size %ld, new size %ld, %d objects.",
-                (long)ROOT->array_size, (long)new_size, ROOT->num_objects));
+  SSH_DEBUG(6, ("Rehashing, old size %ld, new size %ld.",
+                (long)ROOT(c)->array_size,
+                (long)new_size));
 
-  old = ROOT->nodes;
-  old_size = ROOT->array_size;
-  init_to_size(ROOT, new_size);
+  old = ROOT(c)->nodes;
+  old_size = ROOT(c)->array_size;
 
-  ROOT->num_objects = 0;
+  /* If we cannot resize, we'll just use the old hash. Performance
+     will degrade, though. */
+  if (!init_to_size(ROOT(c), new_size))
+    return;
 
+  /* move objects */
   for (i = 0; i < old_size; i++)
     {
       n = old[i];
       if (n == NULL) continue;
       while (1)
         {
-          SSH_DEBUG(99, ("i=%d (%d) [%p].", i, ROOT->num_objects, n));
+          SSH_DEBUG(99, ("i=%d (%d) [%p].", i, ROOT(c)->num_objects, n));
           if (n->is_last_in_rib)
             {
-              insert(c, n);
+              INSERT_WITHOUT_REHASH(c, n);
               break;
             }
           else
             {
               next = n->u.next;
-              insert(c, n);
+              INSERT_WITHOUT_REHASH(c, n);
               n = next;
             }
         }
     }
-  SSH_MEM_FREE(old);
 
-  ROOT->array_size = new_size;
+  /* clean up */
+  ssh_free(old);
+}
 
-  SSH_DEBUG(6, ("Done rehash, %d objects present.", ROOT->num_objects));
+static Boolean insert(SshADTContainer c, SshADTMapNode *n)
+{
+  if (n == NULL)  /* E.g. if the duplicate callback returns it. */
+    return FALSE;
+
+  INSERT_WITHOUT_REHASH(c, n);
+  ROOT(c)->num_objects++;
+
+  if (ROOT(c)->num_objects * SSH_ADT_MAP_RATIO > ROOT(c)->array_size)
+    rehash(c, ROOT(c)->array_size * 2 + 1);
+
+  return TRUE;
 }
 
 static Boolean my_insert_absolute(SshADTContainer c,
                                   SshADTAbsoluteLocation location,
-                                  SshADTHandle object)
+                                  SshADTHandle h)
 {
   SSH_ASSERT(location == SSH_ADT_DEFAULT);
-  NODE(object)->image = NULL;
-  return (insert(c, object));
+  NODE(h)->image = NULL;
+  return (insert(c, h));
 }
 
 /* $$METHOD(map, insert_to) */
@@ -250,25 +281,25 @@ SSH_ADT_STD_NUM_OBJECTS(num_objects)
 static SshADTHandle find_node(SshADTContainer c, void *object)
 {
   SshADTMapNode *i;
-  unsigned long hash_value;
+  SshUInt32 hash_value;
   void *result;
 
   SSH_ADT_HASH_OBJECT(c, object, hash_value);
-  hash_value %= ROOT->array_size;
-  if (ROOT->nodes[hash_value] == NULL) return SSH_ADT_INVALID;
+  hash_value %= ROOT(c)->array_size;
+  if (ROOT(c)->nodes[hash_value] == NULL) return SSH_ADT_INVALID;
 
-  for (i = ROOT->nodes[hash_value]; ; i = i->u.next)
+  for (i = ROOT(c)->nodes[hash_value]; ; i = i->u.next)
     {
       SSH_ADT_STD_GET_OBJECT_FROM_HANDLE(c, i, result);
+
       if (result == object)
-        {
-          return i;
-        }
+        return (SshADTHandle)i;
 
       if (i->is_last_in_rib)
-        break;
+        return SSH_ADT_INVALID;
     }
-  return SSH_ADT_INVALID;
+
+  SSH_NOTREACHED;
 }
 
 /* $$METHOD(map, get_handle_to) */
@@ -280,19 +311,19 @@ SSH_ADT_STD_GET_HANDLE_TO(get_handle_to, handle = find_node(c, object);)
 static SshADTHandle to_equal(SshADTContainer c, void *object)
 {
   SshADTMapNode *i;
-  unsigned long hash_value;
+  SshUInt32 hash_value;
   int result;
 
   SSH_ADT_HASH_OBJECT(c, object, hash_value);
-  hash_value %= ROOT->array_size;
-  if (ROOT->nodes[hash_value] == NULL) return SSH_ADT_INVALID;
+  hash_value %= ROOT(c)->array_size;
+  if (ROOT(c)->nodes[hash_value] == NULL) return SSH_ADT_INVALID;
 
-  for (i = ROOT->nodes[hash_value]; ; i = i->u.next)
+  for (i = ROOT(c)->nodes[hash_value]; ; i = i->u.next)
     {
       SSH_ADT_STD_COMPARE_H_O(c, i, object, result);
 
       if (!result)
-        return i;
+        return (SshADTHandle)i;
 
       if (i->is_last_in_rib)
         return SSH_ADT_INVALID;
@@ -303,15 +334,12 @@ static SshADTHandle to_equal(SshADTContainer c, void *object)
 
 static Boolean my_detach(SshADTContainer c, SshADTHandle handle)
 {
-  SshADTMapNode *target;
-  SshADTMapNode *prior;
-  target = handle;
+  SshADTMapNode *target = handle;  /* what we want to remove */
+  SshADTMapNode *prior = handle;   /* the list predecessor of target */
 
   /* We need to find the node prior to that being detached and change
      it. This is a muddy algorithm. */
 
-  prior = target;
-
   while (1)
     {
       /* If `prior' does not point to the last item in the chain... */
@@ -342,10 +370,9 @@ static Boolean my_detach(SshADTContainer c, SshADTHandle handle)
           prior = prior->u.next;
           continue;             /* Continue scanning. */
         }
-      else
+      else  /* if `prior' is the last item in the chain... */
         {
-          /* When `prior' IS the last item in the chain. */
-          /* If `target' is actually the first one in the chain... */
+          /* if `target' is actually the first one in the chain... */
           if (*(prior->u.rib_start) == target)
             {
               /* If `target' is also the last... */
@@ -369,9 +396,9 @@ static Boolean my_detach(SshADTContainer c, SshADTHandle handle)
         }
     }
 
-  ROOT->num_objects--;
+  ROOT(c)->num_objects--;
 
-  return TRUE;  /* my_detach never fails */
+  return TRUE;  /* my_detach never fails, it rather explodes... :) */
 }
 
 /* $$METHOD(map, detach) */
@@ -403,11 +430,18 @@ SSH_ADT_STD_REALLOC(reallocate, realloc_prepare(c, oldh);,
 /* $$METHOD(bag, delet) */
 SSH_ADT_STD_DELETE(delet)
 
+/* $$METHOD(map, map_attach) */
+SSH_ADT_STD_MAP_ATTACH(map_attach, NODE(handle)->image)
+
+/* $$METHOD(map, map_lookup) */
+SSH_ADT_STD_MAP_LOOKUP(map_lookup, NODE(handle)->image)
+
+
 SshADTStaticData ssh_adt_map_static_data =
 {
   {
     /* $$METHODS(map) */
-    /* DO NOT EDIT THIS, edit METHODS.h and
+    /* DO NOT EDIT THIS, edit METHODS.h and 
        the method implementations above instead. */
     container_init, /* container_init */
     clear, /* clear */
@@ -430,6 +464,8 @@ SshADTStaticData ssh_adt_map_static_data =
     reallocate, /* reallocate */
     detach, /* detach */
     delet, /* delet */
+    map_lookup, /* map_lookup */
+    map_attach, /* map_attach */
     /* $$ENDMETHODS */
   },
   sizeof(SshADTMapNode),
@@ -442,7 +478,7 @@ SshADTStaticData ssh_adt_bag_static_data =
 {
   {
     /* $$METHODS(bag) */
-    /* DO NOT EDIT THIS, edit METHODS.h and
+    /* DO NOT EDIT THIS, edit METHODS.h and 
        the method implementations above instead. */
     container_init, /* container_init */
     clear, /* clear */
@@ -465,6 +501,8 @@ SshADTStaticData ssh_adt_bag_static_data =
     reallocate, /* reallocate */
     detach, /* detach */
     delet, /* delet */
+    NULL, /* map_lookup */
+    NULL, /* map_attach */
     /* $$ENDMETHODS */
   },
   sizeof(SshADTMapNode),
@@ -476,70 +514,12 @@ const SshADTContainerType ssh_adt_bag_type = &ssh_adt_bag_static_data;
 
 /*************************************************************** Interface I */
 
-void ssh_adt_map_set_map(SshADTContainer c, SshADTHandle h,
-                         void *image)
-{
-  SSH_ADT_CALL_HOOK(c, unmap, h);
-  NODE(h)->image = image;
-  SSH_ADT_CALL_HOOK(c, map, h);
-}
-
-void *ssh_adt_map_map(SshADTContainer c, SshADTHandle h)
-{
-  return NODE(h)->image;
-}
-
 Boolean ssh_adt_map_changed(SshADTContainer c, SshADTHandle h)
 {
-  return (detach (c, h) && insert(c, h));
+  return (detach(c, h) && insert(c, h));
 }
 
 Boolean ssh_adt_bag_changed(SshADTContainer c, SshADTHandle h)
 {
-  return (detach (c, h) && insert(c, h));
-}
-
-
-/************************************************************** Interface II */
-
-SshADTHandle ssh_adt_xmap_add(SshADTContainer c, void *key, void *value)
-{
-  SshADTHandle h;
-  SSH_ASSERT(!(ssh_adt_xmap_exists(c, key)));
-
-  if (c->flags & SSH_ADT_FLAG_ALLOCATE)  /* liballoc */
-    h = ssh_adt_put(c, key);
-  else  /* voidptr */
-    h = ssh_adt_duplicate(c, key);
-
-  SSH_ASSERT(ssh_adt_xmap_exists(c, key));
-  ssh_adt_map_set_map(c, h, value);
-  return h;
-}
-
-void ssh_adt_xmap_remove(SshADTContainer c, void *key)
-{
-  SshADTHandle h = ssh_adt_get_handle_to_equal(c, key);
-  if (h != SSH_ADT_INVALID) ssh_adt_delete(c, h);
-}
-
-void ssh_adt_xmap_set(SshADTContainer c, void *key, void *value)
-{
-  SshADTHandle h = ssh_adt_get_handle_to_equal(c, key);
-  SSH_ASSERT(h != SSH_ADT_INVALID);
-  ssh_adt_map_set_map(c, h, value);
-}
-
-void *ssh_adt_xmap_get(SshADTContainer c, void *key)
-{
-  SshADTHandle h = ssh_adt_get_handle_to_equal(c, key);
-  if (h == SSH_ADT_INVALID)
-    return NULL;
-  return ssh_adt_map_map(c, h);
-}
-
-Boolean ssh_adt_xmap_exists(SshADTContainer c, void *key)
-{
-  SshADTHandle h = ssh_adt_get_handle_to_equal(c, key);
-  return (h != SSH_ADT_INVALID);
+  return (detach(c, h) && insert(c, h));
 }
