@@ -4,7 +4,7 @@
 
   Author: Antti Huima <huima@ssh.fi>
 
-  Copyright (c) 1999-2000 SSH Communications Security, Finland
+  Copyright (c) 1999-2001 SSH Communications Security, Finland
   All rights reserved.
 
   Created Tue Oct 19 16:02:07 1999.
@@ -14,6 +14,8 @@
 #ifndef SSHADT_STD_I_H_INCLUDED
 #define SSHADT_STD_I_H_INCLUDED
 
+#include "sshadt.h"
+
 
 #define SSH_ADT_STD_COMPARE(container, h1, h2, result)                        \
 do                                                                            \
@@ -46,6 +48,16 @@ do                                                                            \
 }                                                                             \
 while(0)
 
+/* A handle is mapped to an object in two different ways, depending on
+   the active memory allocation mode.  (See the list implementation
+   for an easy example.)
+
+   If extra nodes are present that contain the header structure, move
+   backwards from the header structure (ENodeStruct) by one pointer in
+   order to get to the object's base address (Node *) (it is required
+   that this ENode structure has the form `{ void *obj, HeaderStruct
+   header }').  If not, move backwards by header_offset.  */
+
 #define SSH_ADT_STD_GET_OBJECT_FROM_HANDLE(container, handle, result)         \
 do                                                                            \
 {                                                                             \
@@ -70,11 +82,11 @@ do                                                                            \
     {                                                                         \
       unsigned char *__ptr;                                                   \
       SSH_ADT_STD_GET_OBJECT_FROM_HANDLE((container), (handle), __ptr);       \
-      SSH_MEM_FREE(__ptr);                                                    \
+      ssh_free(__ptr);                                                        \
     }                                                                         \
   else                                                                        \
     {                                                                         \
-      SSH_MEM_FREE(handle);                                                   \
+      ssh_free(handle);                                                       \
     }                                                                         \
 }                                                                             \
 while(0)
@@ -94,18 +106,21 @@ do                                                                            \
   unsigned char *__ptr;                                                       \
   if (!((container)->flags & SSH_ADT_FLAG_CONTAINED_HEADER))                  \
     {                                                                         \
-      __ptr = SSH_MEM_ALLOC((size) +                                          \
+      __ptr = ssh_xmalloc((size) +                                            \
                           (container)->static_data->internal_header_size);    \
       result = __ptr - (container)->f.header_offset;                          \
     }                                                                         \
   else                                                                        \
     {                                                                         \
-      result = SSH_MEM_ALLOC(size);                                           \
+      result = ssh_xmalloc(size);                                             \
     }                                                                         \
 }                                                                             \
 while(0)
 #endif /* _KERNEL */
 
+/* Construct a handle from an object.  See comment in sshadt_i.h for
+   the SSH_ADT_FLAG_NEED_EXTRA_NODES flag.  */
+
 #define SSH_ADT_STD_MAKE_HANDLE(container, object, block, result)             \
 do                                                                            \
 {                                                                             \
@@ -115,7 +130,7 @@ do                                                                            \
       SSH_ADT_PROTECT(block);                                                 \
       if (__handle != NULL)                                                   \
         {                                                                     \
-          *(((void **)__handle)) = object;                                    \
+          *((void **)__handle) = object;                                      \
           result = (unsigned char *)__handle + sizeof(void *);                \
         }                                                                     \
       else                                                                    \
@@ -152,7 +167,7 @@ static Boolean name(SshADTContainer c, SshADTContainerPars *pars)             \
         -((SshInt32)(c->static_data->internal_header_size));                  \
     }                                                                         \
                                                                               \
-  c->aux.num_objects = 0;                                                     \
+  c->num_objects = 0;                                                         \
   return (block);                                                             \
 }
 
@@ -161,8 +176,9 @@ static void name(SshADTContainer c)                                           \
 {                                                                             \
   SSH_ADT_CALL_DESTROY_HOOK(c, destr);                                        \
   SSH_ADT_PROTECT(block);                                                     \
+  SSH_ADT_CALL_APP(c, cleanup, SSH_ADT_APPCTX(c));                            \
   if (c->flags & (SSH_ADT_FLAG_CONTAINED_HEADER | SSH_ADT_FLAG_ALLOCATE))     \
-    SSH_MEM_FREE(c->hooks);                                                   \
+    ssh_free(c->hooks);                                                       \
 }
 
 #define SSH_ADT_STD_INSERT_AT(name, block, make_node_block)                   \
@@ -183,7 +199,7 @@ static SshADTHandle name(SshADTContainer c,                                   \
       SSH_DEBUG(6, ("inserting %p failed.", object));                         \
       return SSH_ADT_INVALID;                                                 \
     }                                                                         \
-  c->aux.num_objects++;                                                       \
+  c->num_objects++;                                                           \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -197,15 +213,15 @@ static SshADTHandle name(SshADTContainer c,                                   \
   SSH_ADT_STD_MAKE_HANDLE(c, object, make_node_block, h);                     \
   if (h == SSH_ADT_INVALID)                                                   \
     {                                                                         \
-      SSH_DEBUG(6, ("inserting %p failed.", object));                         \
+      SSH_DEBUG(6, ("inserting %p failed (alloc).", object));                 \
       return SSH_ADT_INVALID;                                                 \
     }                                                                         \
-  if (block == FALSE)                                                         \
+  if (!(block))                                                               \
     {                                                                         \
-      SSH_DEBUG(6, ("inserting %p failed.", object));                         \
+      SSH_DEBUG(6, ("inserting %p failed (init).", object));                  \
       return SSH_ADT_INVALID;                                                 \
     }                                                                         \
-  c->aux.num_objects++;                                                       \
+  c->num_objects++;                                                           \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -223,7 +239,7 @@ static SshADTHandle name(SshADTContainer c,                                   \
   SSH_ADT_CALL_APP(c, init, (newp, size, SSH_ADT_APPCTX(c)));                 \
   h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                    \
   SSH_ADT_PROTECT(block);                                                     \
-  c->aux.num_objects++;                                                       \
+  c->num_objects++;                                                           \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -240,7 +256,7 @@ static SshADTHandle name(SshADTContainer c,                                   \
   SSH_ADT_CALL_APP(c, init, (newp, size, SSH_ADT_APPCTX(c)));                 \
   h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                    \
   SSH_ADT_PROTECT(block);                                                     \
-  c->aux.num_objects++;                                                       \
+  c->num_objects++;                                                           \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -260,7 +276,7 @@ static SshADTHandle name(SshADTContainer c,                                   \
                              SSH_ADT_APPCTX(c)));                             \
   h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                    \
   SSH_ADT_PROTECT(block);                                                     \
-  c->aux.num_objects++;                                                       \
+  c->num_objects++;                                                           \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -279,7 +295,7 @@ static SshADTHandle name(SshADTContainer c,                                   \
                                      SSH_ADT_APPCTX(c)));                     \
   h = SSH_ADT_HANDLE_FROM_OBJECT(c, newp);                                    \
   SSH_ADT_PROTECT(block);                                                     \
-  c->aux.num_objects++;                                                       \
+  c->num_objects++;                                                           \
   SSH_ADT_CALL_HOOK(c, insert, h);                                            \
   return h;                                                                   \
 }
@@ -295,7 +311,7 @@ static void *name(SshADTContainer c, SshADTHandle handle)                     \
 #define SSH_ADT_STD_NUM_OBJECTS(name)                                         \
 static size_t name(SshADTContainer c)                                         \
 {                                                                             \
-  return c->aux.num_objects;                                                  \
+  return c->num_objects;                                                      \
 }
 
 #define SSH_ADT_STD_GET_HANDLE_TO(name, block)                                \
@@ -308,7 +324,10 @@ static SshADTHandle name(SshADTContainer c,                                   \
       SSH_ADT_PROTECT(block);                                                 \
       return handle;                                                          \
     }                                                                         \
-  return SSH_ADT_HANDLE_FROM_OBJECT(c, object);                               \
+  else                                                                        \
+    {                                                                         \
+      return SSH_ADT_HANDLE_FROM_OBJECT(c, object);                           \
+    }                                                                         \
 }
 
 #ifdef _KERNEL
@@ -319,6 +338,7 @@ static void *name(SshADTContainer c, void *object, size_t new_size)           \
   return NULL;                                                                \
 }
 #else /* _KERNEL */
+/* XXX: this is definitely wrong. Old size is not known here */
 #define SSH_ADT_STD_REALLOC(name, block1, block2)                             \
 static void *name(SshADTContainer c, void *object, size_t new_size)           \
 {                                                                             \
@@ -329,15 +349,15 @@ static void *name(SshADTContainer c, void *object, size_t new_size)           \
   SSH_ADT_PROTECT(block1);                                                    \
   if (c->flags & SSH_ADT_FLAG_CONTAINED_HEADER)                               \
     {                                                                         \
-      ptr = SSH_MEM_REALLOC(object, NULL, new_size);                          \
+      ptr = ssh_realloc(object, 0, new_size);                                 \
       newh = SSH_ADT_HANDLE_FROM_OBJECT(c, (void *)ptr);                      \
     }                                                                         \
   else                                                                        \
     {                                                                         \
       ptr = object;                                                           \
       ptr -= c->static_data->internal_header_size;                            \
-      ptr = SSH_MEM_REALLOC(ptr, NULL, new_size +                             \
-                            c->static_data->internal_header_size);            \
+      ptr = ssh_realloc(ptr, 0, new_size +                                    \
+                        c->static_data->internal_header_size);                \
       newh = ptr;                                                             \
     }                                                                         \
   SSH_ADT_PROTECT(block2);                                                    \
@@ -363,22 +383,63 @@ static void *name(SshADTContainer c, SshADTHandle handle)                     \
       SSH_ADT_PROTECT(destroy_node_block);                                    \
     }                                                                         \
                                                                               \
-  c->aux.num_objects--;                                                       \
+  c->num_objects--;                                                           \
   return object;                                                              \
 }
 
 #define SSH_ADT_STD_DELETE(name)                                              \
 static void name(SshADTContainer container, SshADTHandle handle)              \
 {                                                                             \
-  void *object = NULL;                                                        \
+  void *object = ssh_adt_detach_i(container, handle);                         \
                                                                               \
-  object = ssh_adt_detach_i(container, handle);                               \
   SSH_ADT_CALL_APP(container, destr, (object, SSH_ADT_APPCTX(container)));    \
+                                                                              \
+  /* For mappings, the CALL_APP macros are not used here because the          \
+     conditions for calling are slightly different.  Also observe that        \
+     the detaching operation above invalidated the handle, but did not        \
+     free it.  The mapping remains untouched, and the image has to be         \
+     freed manually later.  */                                                \
+  if (container->f.app_methods.map_detach != NULL)                            \
+    {                                                                         \
+      void *old_image = ssh_adt_map_lookup(container, handle);                \
+      if (old_image != NULL)                                                  \
+        (*(container->f.app_methods.map_detach))                              \
+          (old_image, SSH_ADT_APPCTX(container));                             \
+    }                                                                         \
+                                                                              \
   if (container->flags & SSH_ADT_FLAG_ALLOCATE)                               \
     {                                                                         \
       SSH_ADT_STD_FREE_I(container, handle);                                  \
     }                                                                         \
-}                                                                             \
+}
+
+#define SSH_ADT_STD_MAP_ATTACH(name, image_location)                          \
+static void name(SshADTContainer container,                                   \
+                 SshADTHandle handle, void *new_image)                        \
+{                                                                             \
+  if (container->f.app_methods.map_detach != NULL)                            \
+    {                                                                         \
+      void *old_image = ssh_adt_map_lookup(container, handle);                \
+      if (old_image != NULL)                                                  \
+        (*(container->f.app_methods.map_detach))                              \
+          (old_image, SSH_ADT_APPCTX(container));                             \
+    }                                                                         \
+                                                                              \
+  if (container->f.app_methods.map_attach != NULL)                            \
+    if (new_image != NULL)                                                    \
+      (*(container->f.app_methods.map_attach))                                \
+        (new_image, SSH_ADT_APPCTX(container));                               \
+                                                                              \
+  SSH_ADT_CALL_HOOK(container, unmap, handle);                                \
+  image_location = new_image;                                                 \
+  SSH_ADT_CALL_HOOK(container, map, handle);                                  \
+}
+
+#define SSH_ADT_STD_MAP_LOOKUP(name, image_location)                          \
+static void *name(SshADTContainer container, SshADTHandle handle)             \
+{                                                                             \
+  return image_location;                                                      \
+}
 
 #define SSH_ADT_STD_NOTHING do { } while(0);
 
