@@ -14,6 +14,10 @@
 #ifndef SSHADT_STRMAP_H_INCLUDED
 #define SSHADT_STRMAP_H_INCLUDED
 
+#include "sshadt.h"
+#include "sshadt_map.h"
+
+
 SshADTContainer ssh_adt_create_strmap(void);
 
 SshADTHandle ssh_adt_strmap_add(SshADTContainer c, const char *key,
