@@ -3,7 +3,7 @@
   Author: Antti Huima <huima@ssh.fi>
           Tatu Ylonen <ylo@ssh.fi>
 
-  Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1996-2001 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Displaying debugging, warning, and fatal error messages.
@@ -14,7 +14,7 @@
 #include "sshincludes.h"
 #include "sshdebug.h"
 #include "sshmatch.h"
-#include "ssheprintf.h"
+#include "sshdsprintf.h"
 
 /* Be very careful in calling debugging output from the debugging
    module itself. Careless introduction of debugging constructs
@@ -106,6 +106,10 @@ typedef struct SshDebugModuleLevelRec {
   int level;
 } *SshDebugModuleLevel;
 
+/* Prototypes */
+
+static void ssh_debug_cleanup_items(void);
+
 /* The global debugging level.  This is used when no per-module level
    can be found for a particular module. */
 int ssh_debug_global_level = 0;
@@ -138,15 +142,14 @@ static int ssh_debug_indent_level = 0;
 
 char *ssh_debug_format(const char *format, ...)
 {
-  char buf[SSH_DEBUG_BUFFER_SIZE];
-
   va_list args;
+  char *p;
+
   va_start(args, format);
-  ssh_evsnprintf(buf, sizeof(buf), format, args);
+  ssh_dvsprintf(&p, format, args);
   va_end(args);
 
-  return
-    SSH_MEM_STRDUP(buf);
+  return p;
 }
 
 /* Returns TRUE if debugging has been enabled for the given module
@@ -195,8 +198,8 @@ void ssh_debug_set_global_level(unsigned int level)
     {
       dl = ssh_debug_module_levels;
       ssh_debug_module_levels = dl->next;
-      SSH_MEM_FREE(dl->module);
-      SSH_MEM_FREE(dl);
+      ssh_free(dl->module);
+      ssh_free(dl);
     }
 }
 
@@ -210,12 +213,12 @@ void ssh_debug_set_module_level(const char *module, unsigned int level)
 
   /* XXX: How to fail on memory failure? Well, it's only debug
      level. Nothing fatal, I hope? */
-  if (!(dl = SSH_MEM_ALLOC(sizeof(*dl))))
+  if (!(dl = ssh_malloc(sizeof(*dl))))
     return;
 
-  if (!(dl->module = SSH_MEM_STRDUP(module)))
+  if (!(dl->module = ssh_strdup(module)))
     {
-      SSH_MEM_FREE(dl);
+      ssh_free(dl);
       return;
     }
   dl->level = level;
@@ -223,6 +226,31 @@ void ssh_debug_set_module_level(const char *module, unsigned int level)
   ssh_debug_module_levels = dl;
 }
 
+/* Uninitialize the debug module to its initial state and free all
+   allocated memory. Call this before exiting the program. */
+
+void ssh_debug_uninit(void)
+{
+  ssh_debug_clear_modules();
+  ssh_debug_cleanup_items();
+}
+
+/* Stops debugging the modules. You can activate the modules
+   again with ssh_debug_set_level_string function. */
+
+void ssh_debug_clear_modules(void)
+{
+  SshDebugModuleLevel p, n;
+
+  for (p = ssh_debug_module_levels; p != NULL; p = n)
+    {
+      n = p->next;
+      ssh_free(p->module);
+      ssh_free(p);
+    }
+  ssh_debug_module_levels = NULL;
+}
+
 /* Sets debugging levels as specified by the string.  The string
    is a comma-separated list of level assignments of the following format:
        pattern=level
@@ -238,8 +266,15 @@ void ssh_debug_set_level_string(const char *string)
   Boolean error;
 
 #ifndef DEBUG_LIGHT
-  ssh_warning("Development-time debugging not compiled in.");
-  ssh_warning("To enable, configure with --enable-debug and recompile.");
+  {
+    static int warned = 0;
+    if (!warned)
+      {
+        ssh_warning("Development-time debugging not compiled in.");
+        ssh_warning("To enable, configure with --enable-debug and recompile.");
+        warned = 1;
+      }
+  }
 #endif /* !DEBUG_LIGHT */
 
   while (*string)
@@ -321,14 +356,14 @@ void ssh_debug_set_level_string(const char *string)
             }
           else
             {
-              name = SSH_MEM_ALLOC(name_len + 1);
+              name = ssh_malloc(name_len + 1);
 
               if (name)
                 {
                   memcpy(name, name_start, name_len);
                   name[name_len] = '\0';
                   ssh_debug_set_module_level(name, (unsigned int)level_value);
-                  SSH_MEM_FREE(name);
+                  ssh_free(name);
                 }
             }
         }
@@ -349,10 +384,12 @@ void ssh_debug_hexdump(size_t offset, const unsigned char *data,
   int c;
   char buf[100];
 
+  if (data == NULL) buf_siz = 0;
+
   for (i = 0; i < buf_siz; i += 0x10)
     {
-      snprintf(buf, sizeof(buf),
-               "%08x: ", (unsigned int)(i + offset));
+      ssh_snprintf(buf, sizeof(buf),
+                   "%08x: ", (unsigned int)(i + offset));
 
       jmax = buf_siz - i;
       jmax = jmax > 16 ? 16 : jmax;
@@ -360,56 +397,229 @@ void ssh_debug_hexdump(size_t offset, const unsigned char *data,
       for (j = 0; j < jmax; j++)
         {
           if ((j % 2) == 1)
-            snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
-                     "%02x ", (unsigned int)data[i+j]);
+            ssh_snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
+                         "%02x ", (unsigned int)data[i+j]);
           else
-            snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
-                     "%02x", (unsigned int)data[i+j]);
+            ssh_snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
+                         "%02x", (unsigned int)data[i+j]);
         }
       for (; j < 16; j++)
         {
           if ((j % 2) == 1)
-            snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
-                     "   ");
+            ssh_snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
+                         "   ");
           else
-            snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
-                     "  ");
+            ssh_snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf),
+                         "  ");
         }
 
-      snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), " ");
+      ssh_snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), " ");
       for (j = 0; j < jmax; j++)
         {
           c = data[i+j];
           c = c < 32 || c >= 127 ? '.' : c;
-          snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "%c", c);
+          ssh_snprintf(buf + strlen(buf), sizeof(buf) - strlen(buf), "%c", c);
         }
       ssh_debug("%s", buf);
     }
 }
 
+#ifndef KERNEL
+/*
+ *  Waits until given output file descriptor allows
+ *  to be written on again.
+ *
+ */
+
+#if 0
+static int
+ssh_debug_wait_fd_readable(unsigned int filedes, unsigned long wait,
+                           size_t *reason)
+{
+  struct timeval tv;
+  fd_set fdset;
+  int ret;
+
+  /* Initialize the file descriptor set. */
+  FD_ZERO(&fdset);
+  FD_SET(filedes, &fdset);
+
+  /* set timeout values */
+  memset((void *)&tv, 0, sizeof(tv));
+
+  /* select returns 0 if timeout, 1 if input available, -1 if error. */
+  errno = 0;
+
+  if (wait > 0)
+    {
+      tv.tv_sec = wait / 1000000;
+      tv.tv_usec = wait % 1000000;
+      ret = select(filedes + 1, &fdset, NULL, NULL, &tv);
+    }
+  else
+    {
+      ret = select(filedes + 1, &fdset, NULL, NULL, NULL);
+    }
+
+  if (ret > 0)
+    {
+      SSH_ASSERT(FD_ISSET(filedes, &fdset));
+    }
+  else
+    {
+      if (ret < 0 && reason)
+        *reason = errno;
+    }
+  return ret;
+}
+#endif /* 0 */
+
+static int
+ssh_debug_wait_fd_writable(unsigned int filedes, unsigned long wait,
+                           size_t *reason)
+{
+  struct timeval tv;
+  fd_set fdset;
+  int ret;
+
+  /* Initialize the file descriptor set. */
+  FD_ZERO(&fdset);
+  FD_SET(filedes, &fdset);
+
+  /* set timeout values */
+  memset((void *)&tv, 0, sizeof(tv));
+
+  /* select returns 0 if timeout, 1 if input available, -1 if error. */
+  errno = 0;
+
+  if (wait > 0)
+    {
+      tv.tv_sec = (long)(wait / 1000000);
+      tv.tv_usec = (long)(wait % 1000000);
+      ret = select(filedes + 1, NULL, &fdset, NULL, &tv);
+    }
+  else
+    {
+      ret = select(filedes + 1, NULL, &fdset, NULL, NULL);
+    }
+
+  if (ret > 0)
+    {
+      SSH_ASSERT(FD_ISSET(filedes, &fdset));
+    }
+  else
+    {
+      if (ret < 0 && reason)
+        *reason = errno;
+    }
+  return ret;
+}
+
+static int
+ssh_debug_stream_unbuffer(FILE *stream)
+{
+  fflush(stream);
+  setvbuf(stream, NULL, _IONBF, 0);
+
+  return 0;
+}
+
+
+static int
+ssh_debug_set_stream_unbuffered(FILE *stream)
+{
+  ssh_debug_stream_unbuffer(stdout);
+  ssh_debug_stream_unbuffer(stderr);
+  ssh_debug_stream_unbuffer(stream);
+
+  return 0;
+}
+
+/*
+ *  Tries to write the whole string in 'buf'
+ *  to the stream. Appends a newline if requested by boolean cr.
+ *
+ */
+void
+ssh_debug_output_i(FILE *stream, char *buf, Boolean cr)
+{
+  size_t len;
+  size_t c;
+  size_t reason;
+  int fd;
+  size_t r;
+  int result;
+  len = strlen(buf);
+  c = 0;
+  fd = fileno(stream);
+
+  for (;;)
+    {
+      errno = 0;
+      r = fwrite(buf + c, 1, len - c, stream);
+      c += r;
+      if (c == len)
+        {
+          if (cr != TRUE)
+            {
+              /* We have managed to write everything */
+              return;
+            }
+          cr = FALSE;
+          c = 0;
+          buf = "\n";
+          len = 1;
+          continue;
+        }
+      SSH_ASSERT(errno != 0);
+
+      if (errno != EAGAIN &&
+#ifdef EWOULDBLOCK
+          errno != EWOULDBLOCK &&
+#endif /* EWOULDBLOCK */
+          errno != EINTR)
+        break;
+
+      do
+        {
+          result = ssh_debug_wait_fd_writable(fd, 0, &reason);
+        } while (result == -1 && reason == EINTR);
+      SSH_ASSERT(result == 1);
+    }
+}
+#endif /* !KERNEL */
+
 /* Outputs a warning message. */
 
 void ssh_warning(const char *fmt, ...)
 {
   va_list va;
   char buf[SSH_DEBUG_BUFFER_SIZE];
+  static int initd = 0;
 
   /* Format the message. */
   va_start(va, fmt);
-  ssh_evsnprintf(buf, sizeof(buf), fmt, va);
+  ssh_vsnprintf(buf, sizeof(buf), fmt, va);
   va_end(va);
 
+  if (!initd)
+    {
+      initd = 1;
+#ifndef KERNEL
+      ssh_debug_set_stream_unbuffered(stderr);
+#endif /* not KERNEL */
+    }
+
   /* Send the message to the callback registered for warning messages,
      or use default handling. */
   if (ssh_debug_warning_callback)
     {
-    (*ssh_debug_warning_callback)(buf, ssh_debug_error_context);
+      (*ssh_debug_warning_callback)(buf, ssh_debug_error_context);
     }
   else
     {
 # ifndef KERNEL
-      fprintf(stderr, "%s\n", buf);
-      fflush(stderr);
+      ssh_debug_output_i(stderr, buf, TRUE);
 # endif /* KERNEL */
     }
 }
@@ -420,23 +630,32 @@ void ssh_debug(const char *fmt, ...)
 {
   va_list va;
   char buf[SSH_DEBUG_BUFFER_SIZE];
+  static int initd = 0;
 
   /* Format the message. */
   va_start(va, fmt);
-  ssh_evsnprintf(buf, sizeof(buf), fmt, va);
+  ssh_vsnprintf(buf, sizeof(buf), fmt, va);
   va_end(va);
 
+  if (!initd)
+    {
+      initd = 1;
+#ifndef KERNEL
+      ssh_debug_set_stream_unbuffered(stderr);
+#endif /* not KERNEL */
+    }
+
+
   /* Send the message to the registered callback for debug messages,
      or use default handling. */
   if (ssh_debug_debug_callback)
     {
-    (*ssh_debug_debug_callback)(buf, ssh_debug_error_context);
+      (*ssh_debug_debug_callback)(buf, ssh_debug_error_context);
     }
   else
     {
 # ifndef KERNEL
-      fprintf(stderr, "%s\n", buf);
-      fflush(stderr);
+      ssh_debug_output_i(stderr, buf, TRUE);
 # endif /* KERNEL */
     }
 }
@@ -453,12 +672,21 @@ void ssh_fatal(const char *fmt, ...)
 {
   va_list va;
   char buf[SSH_DEBUG_BUFFER_SIZE];
+  static int initd = 0;
 
   /* Format the message. */
   va_start(va, fmt);
-  ssh_evsnprintf(buf, sizeof(buf), fmt, va);
+  ssh_vsnprintf(buf, sizeof(buf), fmt, va);
   va_end(va);
 
+  if (!initd)
+    {
+      initd = 1;
+#ifndef KERNEL
+      ssh_debug_set_stream_unbuffered(stderr);
+#endif /* not KERNEL */
+    }
+
   /* Send it to the callback, or do default handling if no callback has
      been specified. */
   if (ssh_debug_fatal_callback)
@@ -468,7 +696,7 @@ void ssh_fatal(const char *fmt, ...)
   else
     {
 # ifndef KERNEL
-      fprintf(stderr, "%s\n", buf);
+      ssh_debug_output_i(stderr, buf, TRUE);
       fflush(stderr);
 # endif /* KERNEL */
     }
@@ -570,7 +798,7 @@ void ssh_log_event(SshLogFacility facility, SshLogSeverity severity,
 
   /* Format the message. */
   va_start(va, fmt);
-  ssh_evsnprintf(buf, sizeof(buf), fmt, va);
+  ssh_vsnprintf(buf, sizeof(buf), fmt, va);
   va_end(va);
 
   /* If a callback has been set, use it to send the message. */
@@ -858,9 +1086,9 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
     case '!':
       c1 = parse_atomic_condition(argptr);
       if (c1 == NULL) return NULL;
-      if (!(c2 = SSH_MEM_ALLOC(sizeof(*c2))))
+      if (!(c2 = ssh_malloc(sizeof(*c2))))
         {
-          SSH_MEM_FREE(c1);
+          ssh_free(c1);
           return NULL;
         }
       c2->type = SSH_DEBUG_FORMAT_COND_NEG;
@@ -868,7 +1096,7 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
       return c2;
 
     case '*':
-      if (!(c1 = SSH_MEM_ALLOC(sizeof(*c1))))
+      if (!(c1 = ssh_malloc(sizeof(*c1))))
           return NULL;
       c1->type = SSH_DEBUG_FORMAT_COND_MAGIC;
       c1->arg.number = -2;
@@ -881,14 +1109,14 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
       if (!ssh_debug_parse_arg(*argptr, argptr, &cond_arg))
         return NULL;
 
-      if (!(c1 = SSH_MEM_ALLOC(sizeof(*c1))))
+      if (!(c1 = ssh_malloc(sizeof(*c1))))
       return NULL;
 
-      if (!(c1->arg.string = SSH_MEM_STRDUP(cond_arg)))
-      {
-        SSH_MEM_FREE(c1);
-        return NULL;
-      }
+      if (!(c1->arg.string = ssh_strdup(cond_arg)))
+        {
+          ssh_free(c1);
+          return NULL;
+        }
       switch (c)
         {
         case 'm': c1->type = SSH_DEBUG_FORMAT_COND_MODNAME; break;
@@ -903,7 +1131,7 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
     case '=':
       if (!ssh_debug_parse_arg(*argptr, argptr, &cond_arg))
         return NULL;
-      if (!(c1 = SSH_MEM_ALLOC(sizeof(*c1))))
+      if (!(c1 = ssh_malloc(sizeof(*c1))))
       return NULL;
 
       c1->type = (c == '<' ? SSH_DEBUG_FORMAT_COND_LEVELLT :
@@ -912,7 +1140,7 @@ static SshDebugCondition parse_atomic_condition(char **argptr)
       c1->arg.number = atoi(cond_arg);
       if (c1->arg.number < 0 || c1->arg.number > 1000)
       {
-        SSH_MEM_FREE(c1);
+        ssh_free(c1);
         return NULL;
       }
       return c1;
@@ -946,14 +1174,14 @@ static SshDebugCondition parse_condition(char **argptr)
       c2 = parse_condition(argptr);
       if (c2 == NULL)
         {
-          SSH_MEM_FREE(c2);
+          ssh_free(c2);
           return NULL;
         }
 
-      if (!(c3 = SSH_MEM_ALLOC(sizeof(*c3))))
+      if (!(c3 = ssh_malloc(sizeof(*c3))))
         {
-          SSH_MEM_FREE(c1);
-          SSH_MEM_FREE(c2);
+          ssh_free(c1);
+          ssh_free(c2);
           return NULL;
         }
 
@@ -1006,24 +1234,33 @@ static void ssh_debug_parse_format(void)
       fmt_string = (char *)ssh_debug_default_format;
     }
 
-  /* XXX: How to react on memory allocation failure? It's just debug,
-     so we opt just to bail out */
-  if (!(fmt_string = SSH_MEM_STRDUP(fmt_string)))
-    return;
+  /* This is just debugging so it is not too severe if we run out of
+     memory. */
+  fmt_string = ssh_strdup(fmt_string);
+  if (fmt_string == NULL)
+    {
+      ssh_warning("Out of memory while parsing debug format string");
+      return;
+    }
 
   ref = fmt_string;
 
   while (*fmt_string != '\0')
     {
-      /* XXX: How to handle out-of-memory? We break out of the fmt
-         loop here */
-      if (!(item = SSH_MEM_ALLOC(sizeof(*item))))
-        break;
+      /* If we run out of memory, we just break out of the loop and
+         ignore the rest of the debug format. */
+      item = ssh_malloc(sizeof(*item));
+      if (item == NULL)
+        {
+          ssh_warning("Out of memory while parsing debug format string");
+          break;
+        }
 #ifdef WANT_COMPLEX_FORMATS
       item->align_right = 0;
       item->minwidth = 0;
       item->maxwidth = 1000;
 #endif
+      item->arg = NULL;
 
       if (*fmt_string == '%') /* escape starts */
         {
@@ -1119,7 +1356,7 @@ static void ssh_debug_parse_format(void)
               if (temp == NULL)
 #ifdef HAVE_GETHOSTNAME
                 {
-                  if ((item->arg = SSH_MEM_ALLOC(60)) != NULL)
+                  if ((item->arg = ssh_malloc(60)) != NULL)
                     {
                       if (gethostname(item->arg, 60) < 0)
                         {
@@ -1133,11 +1370,11 @@ static void ssh_debug_parse_format(void)
                 }
 #else /* HAVE_GETHOSTNAME */
                 {
-                  item->arg = SSH_MEM_STRDUP("<host unknown>");
+                  item->arg = ssh_strdup("<host unknown>");
                 }
 #endif /* HAVE_GETHOSTNAME */
               else
-                item->arg = SSH_MEM_STRDUP(temp);
+                item->arg = ssh_strdup(temp);
               break;
 
               /* `%E(var)' expands to the value of the environment
@@ -1154,12 +1391,13 @@ static void ssh_debug_parse_format(void)
               temp = (char *)getenv(arg);
               if (temp == NULL)
                 {
-                  if ((item->arg = SSH_MEM_ALLOC(strlen(arg) + 10)) != NULL)
-                    snprintf(item->arg, strlen(arg) + 10, "<%s unset>", arg);
+                  if ((item->arg = ssh_malloc(strlen(arg) + 10)) != NULL)
+                    ssh_snprintf(item->arg, strlen(arg) + 10,
+                                 "<%s unset>", arg);
                 }
               else
                 {
-                  item->arg = SSH_MEM_STRDUP(temp);
+                  item->arg = ssh_strdup(temp);
                 }
 #else /* HAVE_GETENV */
               ssh_warning("getenv() is not available for debugging output, "
@@ -1217,7 +1455,7 @@ static void ssh_debug_parse_format(void)
                   || ssh_debug_format_wrapindent > 1000)
                 goto failure;
 
-              SSH_MEM_FREE(item);
+              ssh_free(item);
               continue;
 
               break;
@@ -1281,7 +1519,7 @@ static void ssh_debug_parse_format(void)
                   goto failure;
               item->type = SSH_DEBUG_FORMAT_TERMCONTROL;
               item->numarg = -1;
-              item->arg = SSH_MEM_STRDUP(arg);
+              item->arg = ssh_strdup(arg);
               break;
 
               /* '%c(n)' will cause the debug module to sleep n
@@ -1336,14 +1574,14 @@ static void ssh_debug_parse_format(void)
 
             case '%':
               item->type = SSH_DEBUG_FORMAT_LITERAL;
-              item->arg = SSH_MEM_STRDUP("%");
+              item->arg = ssh_strdup("%");
               break;
 
             default:
               ssh_warning("Invalid or unsupported "
                           "debug format control character '%c'.", c);
               item->type = SSH_DEBUG_FORMAT_LITERAL;
-              if ((item->arg = SSH_MEM_ALLOC(2)) != NULL)
+              if ((item->arg = ssh_malloc(2)) != NULL)
                 {
                   item->arg[0] = c;
                   item->arg[1] = '\0';
@@ -1370,7 +1608,7 @@ static void ssh_debug_parse_format(void)
               c = *fmt_string;
               *fmt_string = '\0';
               item->type = SSH_DEBUG_FORMAT_LITERAL;
-              item->arg = SSH_MEM_STRDUP(temp);
+              item->arg = ssh_strdup(temp);
               *fmt_string = c;
             }
         }
@@ -1386,7 +1624,7 @@ static void ssh_debug_parse_format(void)
       *(pptr) = item;
       pptr = &(item->next);
     }
-  SSH_MEM_FREE(ref);
+  ssh_free(ref);
 
  /* succesful end of parsing */
 
@@ -1407,17 +1645,13 @@ failure: /* something went wrong... give a warning and fall
                 "SSH_DEBUG_FMT environment variable.");
 #endif
 
+  /* Do not leak item. */
+  ssh_free(item);
+
   /* Release if there was anything previously */
-  while (ssh_debug_format_items != NULL)
-    {
-      SshDebugFormatItem item = ssh_debug_format_items;
-      ssh_debug_format_items = item->next;
+  ssh_debug_cleanup_items();
 
-      SSH_MEM_FREE(item->arg);
-      SSH_MEM_FREE(item);
-    }
-
-  ssh_debug_format_items = SSH_MEM_ALLOC(sizeof(*ssh_debug_format_items));
+  ssh_debug_format_items = ssh_malloc(sizeof(*ssh_debug_format_items));
 
   /* XXX: How to recover from memory allocation failure here? */
   if (!ssh_debug_format_items)
@@ -1425,6 +1659,7 @@ failure: /* something went wrong... give a warning and fall
 
   ssh_debug_format_items->type = SSH_DEBUG_FORMAT_MSG;
   ssh_debug_format_items->next = NULL;
+  ssh_debug_format_items->arg = NULL;
 
 #ifdef WANT_COMPLEX_FORMATS
 
@@ -1435,7 +1670,7 @@ failure: /* something went wrong... give a warning and fall
   ssh_debug_format_wrapcol = 0;
 #endif /* WANT_COMPLEX_FORMATS */
 
-  SSH_MEM_FREE(ref);
+  ssh_free(ref);
 
   return;
 }
@@ -1459,7 +1694,7 @@ static void ssh_debug_parse_format(void)
 
   for (i = 8; i >= 0; i--)
     {
-      if (!(item = SSH_MEM_ALLOC(sizeof(*item))))
+      if (!(item = ssh_malloc(sizeof(*item))))
         return;
 
 #ifdef WANT_COMPLEX_FORMATS
@@ -1481,7 +1716,7 @@ static void ssh_debug_parse_format(void)
 
         case 1:
           item->type = SSH_DEBUG_FORMAT_LITERAL;
-          item->arg = SSH_MEM_STRDUP("/");
+          item->arg = ssh_strdup("/");
           break;
 
         case 2:
@@ -1489,7 +1724,7 @@ static void ssh_debug_parse_format(void)
 
         case 3:
           item->type = SSH_DEBUG_FORMAT_LITERAL;
-          item->arg = SSH_MEM_STRDUP(":");
+          item->arg = ssh_strdup(":");
           break;
 
         case 4:
@@ -1499,7 +1734,7 @@ static void ssh_debug_parse_format(void)
 
         case 5:
           item->type = SSH_DEBUG_FORMAT_LITERAL;
-          item->arg = SSH_MEM_STRDUP("/");
+          item->arg = ssh_strdup("/");
           break;
 
         case 6:
@@ -1508,7 +1743,7 @@ static void ssh_debug_parse_format(void)
 #endif
         case 7:
           item->type = SSH_DEBUG_FORMAT_LITERAL;
-          item->arg = SSH_MEM_STRDUP(": ");
+          item->arg = ssh_strdup(": ");
           break;
 
         case 8:
@@ -1523,6 +1758,22 @@ static void ssh_debug_parse_format(void)
 
 #endif /* WANT_DEBUG_PARSER */
 
+/* cleanup debug format items */
+
+static void ssh_debug_cleanup_items(void)
+{
+  SshDebugFormatItem item;
+
+  while (ssh_debug_format_items != NULL)
+    {
+      item = ssh_debug_format_items;
+      ssh_debug_format_items = item->next;
+
+      ssh_free(item->arg);
+      ssh_free(item);
+    }
+}
+
 /* The procedure for evaluating conditionals is compiled only
    if WANT_COMPLEX_FORMATS is defined. */
 
@@ -1654,9 +1905,9 @@ void ssh_debug_output(int level,
       ssh_debug_parse_format();
       if (ssh_debug_format_items == NULL) /* still? */
         {
-          ssh_fatal("Internal error in ssh_debug_output "
+          ssh_warning("Internal error in ssh_debug_output "
                     "(ssh_debug_format_items == NULL).");
-          /* exited */
+          return;
         }
     }
 
@@ -1705,7 +1956,7 @@ void ssh_debug_output(int level,
 #ifdef WANT_COMPLEX_FORMATS
         case SSH_DEBUG_FORMAT_IGNORE:
           {
-            SSH_MEM_FREE(msg);
+            ssh_free(msg);
             return;
           }
 
@@ -1767,38 +2018,38 @@ void ssh_debug_output(int level,
 #ifdef HAVE_LOCALTIME
 
         case SSH_DEBUG_FORMAT_MINUTES:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
-                   current_time->minute);
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
+                       current_time->minute);
           break;
 
         case SSH_DEBUG_FORMAT_SECONDS:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
-                   current_time->second);
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
+                       current_time->second);
           break;
 
         case SSH_DEBUG_FORMAT_DAY:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
-                   current_time->monthday);
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
+                       current_time->monthday);
           break;
 
         case SSH_DEBUG_FORMAT_YEAR:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%04d",
-                   (int)current_time->year);
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%04d",
+                       (int)current_time->year);
           break;
 
         case SSH_DEBUG_FORMAT_MONTH:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
-                   current_time->month + 1);
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
+                       current_time->month + 1);
           break;
 
         case SSH_DEBUG_FORMAT_HOURS:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
-                   current_time->hour);
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%02d",
+                       current_time->hour);
           break;
 
         case SSH_DEBUG_FORMAT_ORDINAL:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d",
-                   ssh_debug_msg_number);
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d",
+                       ssh_debug_msg_number);
           break;
 #endif
           /* Date constructs cannot be incorporated into the debug
@@ -1807,17 +2058,17 @@ void ssh_debug_output(int level,
 
 #ifdef HAVE_GETPID
         case SSH_DEBUG_FORMAT_PID:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", getpid()); break;
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", getpid()); break;
 #endif /* HAVE_GETPID */
 
 #ifdef HAVE_GETUID
         case SSH_DEBUG_FORMAT_UID:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", getuid()); break;
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", getuid()); break;
 #endif /* HAVE_GETUID */
 
 #ifdef HAVE_GETEUID
         case SSH_DEBUG_FORMAT_EUID:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", geteuid()); break;
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", geteuid()); break;
 #endif /* HAVE_GETEUID */
 
           /* Similarly, the get*id constructs cannot be entered into
@@ -1829,7 +2080,7 @@ void ssh_debug_output(int level,
           break;
 
         case SSH_DEBUG_FORMAT_LEVEL:
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", level);
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", level);
           break;
 
         case SSH_DEBUG_FORMAT_MODULE:
@@ -1842,7 +2093,7 @@ void ssh_debug_output(int level,
 
         case SSH_DEBUG_FORMAT_LINE:
           NO_SPLIT;
-          snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", line);
+          ssh_snprintf(temp, SSH_DEBUG_TMP_BUF_SIZE, "%d", line);
           break;
 
         case SSH_DEBUG_FORMAT_FUNC:
@@ -1917,7 +2168,8 @@ void ssh_debug_output(int level,
 
         default:
 #ifdef  KERNEL
-          ssh_warning("Internal error in ssh_debug_output (unknown iter->type=%d).", iter->type);
+          ssh_warning("Internal error in ssh_debug_output (unknown "
+                      "iter->type=%d).", iter->type);
           fragment = NULL;
           break;
 #else
@@ -2101,7 +2353,7 @@ void ssh_debug_output(int level,
 
   /* Free the message and print the formatted version out */
 
-  SSH_MEM_FREE(msg);
+  ssh_free(msg);
   ssh_debug("%s", buf);
 
 #ifdef WANT_COMPLEX_FORMATS
