@@ -63,7 +63,7 @@ void ssh_debug_change_indentation(int change);
    <debug type define> is one of the defines below (see DEBUG LEVELS).
    It is mapped into the debug level this message belongs to (zero
    most commonly viewed). <format> and <args> are passed to
-   ssh_esnprintf. See DEBUG LEVELS below for details.
+   ssh_snprintf. See DEBUG LEVELS below for details.
 
    ssh_debug_initialize [see sshdmod.h] must be called in the
    beginning of the application.
@@ -159,23 +159,25 @@ void ssh_debug_change_indentation(int change);
 #define SSH_DEBUG_COMPILE_TIME_MAX_LEVEL 999999
 #endif
 
+/* True if tracing of `level'  messages is enabled for this module. */
+#define SSH_TRACE_ENABLED(level)                        \
+  ((SSH_DEBUG_COMPILE_TIME_MAX_LEVEL == 999999 ||       \
+    (level) <= SSH_DEBUG_COMPILE_TIME_MAX_LEVEL) &&     \
+   ssh_debug_enabled(SSH_DEBUG_MODULE, (level)))
+
 /* Outputs a debug message.  This macro is always compiled into the binary. */
-#define SSH_TRACE(level, varcall)                                             \
-do                                                                            \
-{                                                                             \
-  if (SSH_DEBUG_COMPILE_TIME_MAX_LEVEL == 999999 ||                           \
-      (level) <= SSH_DEBUG_COMPILE_TIME_MAX_LEVEL)                            \
-    {                                                                         \
-      if (ssh_debug_enabled(SSH_DEBUG_MODULE, (level)))                       \
-        {                                                                     \
-          ssh_debug_output((level), __FILE__, __LINE__, SSH_DEBUG_MODULE,     \
-                           SSH_DEBUG_FUNCTION,                                \
-                           ssh_debug_format varcall);                         \
-                                                                              \
-        }                                                                     \
-    }                                                                         \
-}                                                                             \
-while (0)
+#define SSH_TRACE(level, varcall)                                       \
+  do                                                                    \
+  {                                                                     \
+    if (SSH_TRACE_ENABLED(level))                                       \
+      {                                                                 \
+        ssh_debug_output((level), __FILE__, __LINE__, SSH_DEBUG_MODULE, \
+                         SSH_DEBUG_FUNCTION,                            \
+                         ssh_debug_format varcall);                     \
+                                                                        \
+      }                                                                 \
+  }                                                                     \
+  while (0)
 
 /* Outputs a debug message with hex dump.  This macro is always compiled
    into the binary.
@@ -185,21 +187,18 @@ while (0)
                      ("Buffer (%d bytes):", sizeof(buf)),
                      buf, sizeof(buf)); */
 
-#define SSH_TRACE_HEXDUMP(level, varcall, buf, len)                           \
-do                                                                            \
-{                                                                             \
-  if (SSH_DEBUG_COMPILE_TIME_MAX_LEVEL == 999999 ||                           \
-      (level) <= SSH_DEBUG_COMPILE_TIME_MAX_LEVEL)                            \
-    {                                                                         \
-      if (ssh_debug_enabled(SSH_DEBUG_MODULE, (level))) {                     \
-        ssh_debug_output((level), __FILE__, __LINE__, SSH_DEBUG_MODULE,       \
-                         SSH_DEBUG_FUNCTION,                                  \
-                         ssh_debug_format varcall);                           \
-        ssh_debug_hexdump(0, (buf), (len));                                   \
-      }                                                                       \
-    }                                                                         \
-}                                                                             \
-while (0)
+#define SSH_TRACE_HEXDUMP(level, varcall, buf, len)                     \
+  do                                                                    \
+  {                                                                     \
+    if (SSH_TRACE_ENABLED(level))                                       \
+      {                                                                 \
+        ssh_debug_output((level), __FILE__, __LINE__, SSH_DEBUG_MODULE, \
+                         SSH_DEBUG_FUNCTION,                            \
+                         ssh_debug_format varcall);                     \
+        ssh_debug_hexdump(0, (buf), (len));                             \
+      }                                                                 \
+  }                                                                     \
+  while (0)
 
 /* Check assertions. SSH_PRECOND, SSH_POSTCOND, SSH_ASSERT, SSH_INVARIANT
    and SSH_NOTREACHED are compiled only if DEBUG_LIGHT is defined.
@@ -232,9 +231,11 @@ while (0)
 #define SSH_DEBUG(level, varcall) SSH_TRACE((level), varcall)
 #define SSH_DEBUG_HEXDUMP(level, varcall, buf, len) \
      SSH_TRACE_HEXDUMP((level), varcall, (buf), (len))
+#define SSH_DEBUG_ENABLED(level) SSH_TRACE_ENABLED(level)
 #else
 #define SSH_DEBUG(level, varcall) do {} while (0)
 #define SSH_DEBUG_HEXDUMP(level, varcall, buf, len) do {} while (0)
+#define SSH_DEBUG_ENABLED(level) 0
 #endif
 
 /* DEBUG_HEAVY is compiled in only if DEBUG_HEAVY is defined. */
@@ -242,9 +243,11 @@ while (0)
 #define SSH_HEAVY_DEBUG(level, varcall) SSH_TRACE((level), varcall)
 #define SSH_HEAVY_DEBUG_HEXDUMP(level, varcall, buf, len) \
      SSH_TRACE_HEXDUMP((level), varcall, (buf), (len))
+#define SSH_HEAVY_DEBUG_ENABLED(level) SSH_TRACE_ENABLED(level)
 #else
 #define SSH_HEAVY_DEBUG(level, varcall) do {} while (0)
 #define SSH_HEAVY_DEBUG_HEXDUMP(level, varcall, buf, len) do {} while (0)
+#define SSH_HEAVY_DEBUG_ENABLED(level) 0
 #endif
 
 /* SSH_DEBUG_INDENT is compiled in only if DEBUG_LIGHT is defined. */
@@ -274,6 +277,14 @@ void ssh_debug_set_level_string(const char *string);
 /* Sets the debugging level for all modules. */
 void ssh_debug_set_global_level(unsigned int level);
 
+/* Uninitialize the debug module to its initial state and free all
+   allocated memory. Call this before exiting the program. */
+void ssh_debug_uninit(void);
+
+/* Stops debugging the modules. You can activate the modules
+   again with ssh_debug_set_level_string function. */
+void ssh_debug_clear_modules(void);
+
 /* Sets the formatting string. If `override' is TRUE the string set
    will override one defined in UNIX environment. If `override' is
    FALSE the version from environment will have preference when set.
@@ -456,6 +467,9 @@ void ssh_log_get_callback(SshLogCallback *log_cb_return,
 /* Software malfunctions */
 #define SSH_D_ERROR  0
 
+/* Software failure, but caused by a packet coming from network */
+#define SSH_D_NETFAULT 3
+
 /* Data formatted incorrectly coming from a network or other outside source */
 #define SSH_D_NETGARB 3
 
