@@ -24,6 +24,16 @@ void ssh_getopt_init_data(SshGetOptData data)
   return;
 }
 
+#ifdef VXWORKS
+void ssh_getopt_restart(void)
+{
+  memset(&ssh_getopt_default_data, 0, sizeof(struct SshGetOptDataRec));
+  ssh_getopt_default_data.err = 1;
+  ssh_getopt_default_data.ind = 1;
+  ssh_getopt_default_data.current = "";
+}
+#endif /* VXWORKS */
+
 static int ssh_str_is_number(char *str)
 {
   if (!str)
@@ -117,6 +127,10 @@ int ssh_getopt(int argc, char **argv, const char *ostr, SshGetOptData data)
               data->arg_num = 1;
               data->arg_val = atoi((data->arg));
             }
+          else
+            {
+              data->arg_num = 0;
+            }
         }
       else if (!optional_arg && argc > ++(data->ind))
         {
@@ -127,6 +141,10 @@ int ssh_getopt(int argc, char **argv, const char *ostr, SshGetOptData data)
               data->arg_num = 1;
               data->arg_val = atoi(data->arg);
             }
+          else
+            {
+              data->arg_num = 0;
+            }
         }
       else
         {
