@@ -5,7 +5,7 @@ Authors: Timo J. Rinne <tri@ssh.fi>
          Tero Mononen <tmo@ssh.fi>
          Sami Lehtinen <sjl@ssh.fi> (junior member of the group)
 
-Copyright (C) 1999, 2000 SSH Communications Security Oy, Espoo, Finland
+Copyright (C) 1999, 2000, 2001 SSH Communications Security Oy, Espoo, Finland
 All rights reserved.
 
 Calendar time retrieval and manipulation.
@@ -185,14 +185,14 @@ char *ssh_time_string(SshTime input_time)
   char temp[100];
 
   ssh_calendar_time(input_time, calendar, FALSE);
-  snprintf(temp, sizeof(temp), "%04d%02d%02d%02d%02d%02d",
-           (int)calendar->year,
-           (int)calendar->month + 1,
-           (int)calendar->monthday,
-           (int)calendar->hour,
-           (int)calendar->minute,
-           (int)calendar->second);
-  return ssh_xstrdup(temp);
+  ssh_snprintf(temp, sizeof(temp), "%04d%02d%02d%02d%02d%02d",
+               (int)calendar->year,
+               (int)calendar->month + 1,
+               (int)calendar->monthday,
+               (int)calendar->hour,
+               (int)calendar->minute,
+               (int)calendar->second);
+  return ssh_strdup(temp);
 }
 
 char *ssh_readable_time_string(SshTime input_time, Boolean local_time)
@@ -209,27 +209,27 @@ char *ssh_readable_time_string(SshTime input_time, Boolean local_time)
     }
   else if (calendar->utc_offset > 0)
     {
-      snprintf(zoneid, sizeof (zoneid), " +%02d%02d",
-               (int)((calendar->utc_offset / 3600) % 100),
-               (int)((calendar->utc_offset / 60) % 60));
+      ssh_snprintf(zoneid, sizeof (zoneid), " +%02d%02d",
+                   (int)((calendar->utc_offset / 3600) % 100),
+                   (int)((calendar->utc_offset / 60) % 60));
     }
   else
     {
-      snprintf(zoneid, sizeof (zoneid), " -%02d%02d",
-               (int)(((- calendar->utc_offset) / 3600) % 100),
-               (int)(((- calendar->utc_offset) / 60) % 60));
+      ssh_snprintf(zoneid, sizeof (zoneid), " -%02d%02d",
+                   (int)(((- calendar->utc_offset) / 3600) % 100),
+                   (int)(((- calendar->utc_offset) / 60) % 60));
     }
 
-  snprintf(temp, sizeof(temp), "%s %s %02d %04d %02d:%02d:%02d%s",
-           abbr_day[calendar->weekday % 7],
-           abbr_month[calendar->month % 12],
-           (int)calendar->monthday,
-           (int)calendar->year,
-           (int)calendar->hour,
-           (int)calendar->minute,
-           (int)calendar->second,
-           zoneid);
-  return ssh_xstrdup(temp);
+  ssh_snprintf(temp, sizeof(temp), "%s %s %02d %04d %02d:%02d:%02d%s",
+               abbr_day[calendar->weekday % 7],
+               abbr_month[calendar->month % 12],
+               (int)calendar->monthday,
+               (int)calendar->year,
+               (int)calendar->hour,
+               (int)calendar->minute,
+               (int)calendar->second,
+               zoneid);
+  return ssh_strdup(temp);
 }
 
 /* Returns seconds that local timezone is east from the UTC meridian
@@ -304,13 +304,26 @@ void ssh_get_local_timezone(SshTime tv,
         *utc_offset = (SshInt32)(tm->__tm_gmtoff__);
     }
   else
-   {
-     if (utc_offset != NULL)
-       *utc_offset = (SshInt32)0;
-   }
+    {
+      if (utc_offset != NULL)
+        *utc_offset = (SshInt32)0;
+    }
 #else /* HAVE_OLD_TM_GMTOFF_IN_STRUCT_TM */
+#ifdef HAVE_EXTERNAL_TIMEZONE
+  if((timezone >= (-50400))  && (timezone <= 50400))
+    {
+      if(utc_offset != NULL)
+        *utc_offset = (SshInt32) - timezone;
+    }
+  else
+    {
+      if (utc_offset != NULL)
+        *utc_offset = (SshInt32)0;
+    }
+#else /* HAVE_EXTERNAL_TIMEZONE */
   if (utc_offset != NULL)
     *utc_offset = (SshInt32)0;
+#endif /* HAVE_EXTERNAL_TIMEZONE */
 #endif /* HAVE_OLD_TM_GMTOFF_IN_STRUCT_TM */
 #endif /* HAVE_TM_GMTOFF_IN_STRUCT_TM */
 #ifdef HAVE_TM_ISDST_IN_STRUCT_TM
@@ -336,4 +349,193 @@ void ssh_get_local_timezone(SshTime tv,
 #endif /* ! defined (USE_SSH_INTERNAL_LOCALTIME) && defined (HAVE_LOCALTIME) */
 }
 
+/* Convert SshCalendarTime to SshTime. If the dst is set to TRUE then daylight
+   saving time is assumed to be set, if dst field is set to FALSE then it is
+   assumed to be off. It if it is set to -1 then the function tries to find out
+   if the dst was on or off at the time given.
+
+   Weekday and yearday fields are ignored in the conversion, but filled with
+   approriate values during the conversion. All other values are normalized to
+   their normal range during the conversion.
+
+   If the local_time is set to TRUE then dst and utc_offset values
+   are ignored.
+
+   If the time cannot be expressed as SshTime this function returns FALSE,
+   otherwise returns TRUE. */
+Boolean ssh_make_time(SshCalendarTime calendar_time, SshTime *time_return,
+                      Boolean local_time)
+{
+  SshCalendarTimeStruct test_time;
+  SshTime estimate;
+
+  SSH_DEBUG(SSH_D_MIDSTART, ("Original time is %04d-%02d-%02d %02d:%02d:%02d",
+                             calendar_time->year,
+                             calendar_time->month + 1,
+                             calendar_time->monthday,
+                             calendar_time->hour,
+                             calendar_time->minute,
+                             calendar_time->second));
+
+  /* Normalize values first */
+  while (calendar_time->second > 59)
+    {
+      SSH_DEBUG(SSH_D_UNCOMMON, ("Seconds too large, adjusting %d",
+                                 calendar_time->second));
+      calendar_time->second -= 60;
+      calendar_time->minute++;
+    }
+  while (calendar_time->minute > 59)
+    {
+      SSH_DEBUG(SSH_D_UNCOMMON, ("Minutes too large, adjusting %d",
+                                 calendar_time->minute));
+      calendar_time->minute -= 60;
+      calendar_time->hour++;
+    }
+  while (calendar_time->hour > 23)
+    {
+      SSH_DEBUG(SSH_D_UNCOMMON, ("Hours too large, adjusting %d",
+                                 calendar_time->hour));
+      calendar_time->hour -= 24;
+      calendar_time->monthday++;
+    }
+  do {
+    int days_per_month;
+
+    while (calendar_time->month > 12)
+      {
+        SSH_DEBUG(SSH_D_UNCOMMON, ("Month too large, adjusting %d",
+                                   calendar_time->month));
+        calendar_time->month -= 12;
+        calendar_time->year++;
+      }
+    days_per_month = monthdays[calendar_time->month] +
+      ((calendar_time->month == 1 &&
+        SSH_IS_LEAP_YEAR(calendar_time->year)) ? 1 : 0);
+    if (calendar_time->monthday > days_per_month)
+      {
+        SSH_DEBUG(SSH_D_UNCOMMON, ("Month day too large, adjusting %d",
+                                   calendar_time->monthday));
+        calendar_time->monthday -= days_per_month;
+        calendar_time->month++;
+      }
+    else if (calendar_time->monthday == 0)
+      {
+        SSH_DEBUG(SSH_D_UNCOMMON, ("Month day zero, adjusting %d",
+                                   calendar_time->monthday));
+        if (calendar_time->month == 0)
+          {
+            calendar_time->month = 12;
+            calendar_time->year--;
+          }
+        else
+          {
+            calendar_time->month--;
+          }
+        calendar_time->monthday = monthdays[calendar_time->month] +
+          ((calendar_time->month == 1 &&
+            SSH_IS_LEAP_YEAR(calendar_time->year)) ? 1 : 0);
+      }
+    else
+      {
+        break;
+      }
+  } while (1);
+
+  SSH_DEBUG(SSH_D_LOWOK, ("Adjusted time is %04d-%02d-%02d %02d:%02d:%02d",
+                          calendar_time->year,
+                          calendar_time->month + 1,
+                          calendar_time->monthday,
+                          calendar_time->hour,
+                          calendar_time->minute,
+                          calendar_time->second));
+
+  /* Calculate estimate */
+  estimate = calendar_time->monthday - 1 +
+    30 * calendar_time->month +
+    365 * (calendar_time->year - 1970) +
+    ((calendar_time->year - 1970) / 4);
+  estimate *= 24;
+  estimate += calendar_time->hour;
+  estimate *= 60;
+  estimate += calendar_time->minute;
+  estimate *= 60;
+  estimate += calendar_time->second;
+
+  do {
+    SSH_DEBUG(SSH_D_LOWOK, ("Estimate is %ld", (unsigned long) estimate));
+    ssh_calendar_time(estimate, &test_time, FALSE);
+    SSH_DEBUG(SSH_D_LOWOK, ("Compare time is %04d-%02d-%02d %02d:%02d:%02d",
+                            test_time.year,
+                            test_time.month + 1,
+                            test_time.monthday,
+                            test_time.hour,
+                            test_time.minute,
+                            test_time.second));
+
+    if (test_time.year == calendar_time->year &&
+        test_time.month == calendar_time->month &&
+        test_time.monthday == calendar_time->monthday &&
+        test_time.hour == calendar_time->hour &&
+        test_time.minute == calendar_time->minute &&
+        test_time.second == calendar_time->second)
+      break;
+    if (test_time.year == calendar_time->year &&
+        test_time.month == calendar_time->month &&
+        test_time.monthday == calendar_time->monthday)
+      {
+        if (test_time.hour != calendar_time->hour)
+          estimate += (calendar_time->hour - test_time.hour) * 3600;
+        if (test_time.minute != calendar_time->minute)
+          estimate += (calendar_time->minute - test_time.minute) * 60;
+        if (test_time.hour != calendar_time->hour)
+          estimate += (calendar_time->second - test_time.second);
+        continue;
+      }
+    if (test_time.year != calendar_time->year)
+      {
+        estimate += (calendar_time->year - test_time.year) * 365 * 86400;
+        continue;
+      }
+    if (test_time.month != calendar_time->month)
+      {
+        estimate += (calendar_time->month - test_time.month) * 28 * 86400;
+        continue;
+      }
+    if (test_time.monthday != calendar_time->monthday)
+      {
+        estimate += (calendar_time->monthday - test_time.monthday) * 86400;
+        continue;
+      }
+    ssh_fatal("Internal error in ssh_make_time");
+  } while (1);
+
+  if (local_time)
+    {
+      SshInt32 utc_offset;
+      Boolean dst;
+
+      ssh_get_local_timezone(estimate, &utc_offset, &dst);
+      if (utc_offset != calendar_time->utc_offset && calendar_time->dst == -1)
+        {
+          SSH_DEBUG(SSH_D_UNCOMMON, ("Utc offset in input does not match "
+                                     "current system timezone. Dst rules "
+                                     "might be incorrect"));
+        }
+      SSH_DEBUG(SSH_D_LOWOK, ("Adding timezone informartion %d seconds",
+                              calendar_time->utc_offset));
+      estimate += calendar_time->utc_offset;
+      if (calendar_time->dst == TRUE ||
+          (calendar_time->dst == -1 &&
+           dst))
+        estimate += 3600;
+    }
+  SSH_DEBUG(SSH_D_MIDSTART, ("Result is %ld", (unsigned long) estimate));
+  *time_return =  estimate;
+  calendar_time->yearday = test_time.yearday;
+  calendar_time->weekday = test_time.weekday;
+  return TRUE;
+}
+
+
 /* eof (sshtime.c) */
