@@ -4,7 +4,7 @@ sshincludes.h
 
 Author: Tatu Ylonen <ylo@cs.hut.fi>
 
-Copyright (c) 1995-2000 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
+Copyright (c) 1995-2001 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
                    All rights reserved
 
 Created: Mon Jan 15 10:36:06 1996 ylo
@@ -21,12 +21,6 @@ Common include files for various platforms.
 
 #if defined(KERNEL) || defined(_KERNEL)
 
-/* In the kernel mode, we must use ssh_k*alloc() routines. */
-#define SSH_MEM_ALLOC(s)            ssh_kmalloc((s))
-#define SSH_MEM_FREE(p)             ssh_kfree((p))
-#define SSH_MEM_REALLOC(p, o, s)    ssh_krealloc((p), (o), (s))
-#define SSH_MEM_STRDUP(s)           ssh_kstrdup((s))
-
 #ifdef WINNT
 #include "sshincludes_ntddk.h"
 #elif WIN95
@@ -43,14 +37,6 @@ Common include files for various platforms.
 
 #else /* KERNEL || _KERNEL */
 
-#include "sshmalloc.h"
-
-/* In the user mode, we must use ssh_x*alloc() routines. */
-#define SSH_MEM_ALLOC(s)            ssh_xmalloc((s))
-#define SSH_MEM_FREE(p)             ssh_xfree((p))
-#define SSH_MEM_REALLOC(p, o, s)    ssh_xrealloc((p), (s))
-#define SSH_MEM_STRDUP(s)           ssh_xstrdup((s))
-
 #if defined(WIN32)
 # include "sshincludes_win32.h"
 #elif macintosh
@@ -63,24 +49,44 @@ Common include files for various platforms.
 # endif /* VXWORKS */
 #endif /* WIN32 */
 
+#ifdef SSH_INCLUDE_ENGINE_ALLOC_H
+#include "engine_alloc.h"
+#endif /* SSH_INCLUDE_ENGINE_ALLOC_H */
+#include "sshmalloc.h"
 #endif /* KERNEL || _KERNEL */
 
 
 /* Common (operating system independent) stuff below */
 
+#include "sshsnprintf.h"
+
+#ifndef SSH_ALLOW_SYSTEM_SPRINTFS
+
 /* The sprintf and vsprintf functions are FORBIDDEN in all SSH code.
    This is for security reasons - they are the source of way too many
    security bugs.  Instead, we guarantee the existence of snprintf and
-   vsnprintf.  These MUST be used instead. */
+   ssh_vsnprintf.  These MUST be used instead. */
 #ifdef sprintf
 # undef sprintf
 #endif
-#define sprintf ssh_fatal(SPRINTF_IS_FORBIDDEN_USE_SNPRINTF_INSTEAD)
+#define sprintf ssh_fatal(SPRINTF_IS_FORBIDDEN_USE_SSH_SNPRINTF_INSTEAD)
 
 #ifdef vsprintf
 # undef vsprintf
 #endif
-#define vsprintf ssh_fatal(VSPRINTF_IS_FORBIDDEN_USE_VSNPRINTF_INSTEAD)
+#define vsprintf ssh_fatal(VSPRINTF_IS_FORBIDDEN_USE_SSH_VSNPRINTF_INSTEAD)
+
+#ifdef snprintf
+# undef snprintf
+#endif
+#define snprintf ssh_fatal(SNPRINTF_IS_FORBIDDEN_USE_SSH_SNPRINTF_INSTEAD)
+
+#ifdef vsnprintf
+# undef vsnprintf
+#endif
+#define vsnprintf ssh_fatal(VSNPRINTF_IS_FORBIDDEN_USE_SSH_VSNPRINTF_INSTEAD)
+
+#endif /* !SSH_ALLOW_SYSTEM_SPRINTFS */
 
 #ifdef index
 # undef index
@@ -153,7 +159,7 @@ Common include files for various platforms.
 #ifdef mktime
 # undef mktime
 #endif
-#define mktime ssh_fatal(MKTIME_IS_FORBIDDEN)
+#define mktime ssh_fatal(MKTIME_IS_FORBIDDEN_USE_SSH_MAKE_TIME_INSTEAD)
 
 /* Conditionals for various OS & compilation environments */
 
@@ -191,6 +197,89 @@ Common include files for various platforms.
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -224,9 +313,11 @@ Common include files for various platforms.
 #endif /* WINDOWS */
 #endif /* SSH_CODE_SEGMENT */
 
-#ifndef SSH_UID_ROOT
-#define SSH_UID_ROOT 0
-#endif /* SSH_UID_ROOT */
+/* Define UID_ROOT to be the user id for root (normally zero, but different
+   e.g. on Amiga). */
+#ifndef UID_ROOT
+#define UID_ROOT 0
+#endif /* UID_ROOT */
 
 #ifdef SSHDIST_ZLIB
 /* Define zlib to have ssh prefix, so we will not care if there is another zlib
@@ -266,4 +357,21 @@ Common include files for various platforms.
 #define voidp                   ssh_z_voidp
 #endif /* SSHDIST_ZLIB */
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 #endif /* SSHINCLUDES_H */
