@@ -10,7 +10,7 @@
         IP related functions and definitions.
 
   Copyright:
-        Copyright (c) 1998-2000 SSH Communications Security, Finland
+        Copyright (c) 1998-2001 SSH Communications Security, Finland
         All rights reserved
 */
 
@@ -23,6 +23,8 @@
 
 /* The address string of the SSH_IPADDR_ANY. */
 const char *ssh_ipaddr_any = "*** SSH_IPADDR_ANY ***";
+const char *ssh_ipaddr_any_ipv4 = "0.0.0.0";
+const char *ssh_ipaddr_any_ipv6 = "0::0";
 
 /* Mapping between protocol name and doi protocol number */
 const SshKeywordStruct ssh_ip_protocol_id_keywords[] =
@@ -142,6 +144,25 @@ const SshKeywordStruct ssh_ip_protocol_id_keywords[] =
   { "pgm", SSH_IPPROTO_PGM },
   { "0hop", SSH_IPPROTO_0HOP },
   { "l2tp", SSH_IPPROTO_L2TP },
+  { "ddx", SSH_IPPROTO_DDX },
+  { "iatp", SSH_IPPROTO_IATP },
+  { "stp", SSH_IPPROTO_STP },
+  { "srp", SSH_IPPROTO_SRP },
+  { "uti", SSH_IPPROTO_UTI },
+  { "smp", SSH_IPPROTO_SMP },
+  { "sm", SSH_IPPROTO_SM },
+  { "ptp", SSH_IPPROTO_PTP},
+  { "isis over ipv4", SSH_IPPROTO_ISISIPV4 },
+  { "fire", SSH_IPPROTO_FIRE },
+  { "crtp", SSH_IPPROTO_CRTP },
+  { "crudp", SSH_IPPROTO_CRUDP },
+  { "sscopmce", SSH_IPPROTO_SSCOPMCE },
+  { "iplt", SSH_IPPROTO_IPLT },
+  { "sps", SSH_IPPROTO_SPS },
+  { "pipe", SSH_IPPROTO_PIPE },
+  { "sctp", SSH_IPPROTO_SCTP },
+  { "fc", SSH_IPPROTO_FC },
+  { "rsvp-e2e-ignore", SSH_IPPROTO_RSVP_E2E_IGNORE },
   { "reserved", SSH_IPPROTO_RESERVED },
   { NULL, 0 }
 };
@@ -355,7 +376,7 @@ void ssh_ipaddr_set_bits(SshIpAddr result, SshIpAddr ip,
       else
         {
           SSH_ASSERT(keep_bits - 8 * i < 8);
-          result->addr_data[i] &= (0xff << (8 - keep_bits - 8 * i));
+          result->addr_data[i] &= (0xff << (8 - (keep_bits - 8 * i)));
           if (value)
             result->addr_data[i] |= (0xff >> (keep_bits - 8 * i));
         }
@@ -419,10 +440,44 @@ static Boolean ssh_ipaddr_ipv4_parse(unsigned char *data, const char *str)
     {
       if (i != 0)
         {
-          if (*str == '.' && *(str + 1) != '.')
-            str++;
+          if (!*str)
+            {
+              switch (i)
+                {
+                case 1:
+                  /* Single zero means 0.0.0.0.
+                     Other single digit address is invalid. */
+                  if (data[0] == 0)
+                    {
+                      data[1] = data[2] = data[3] = 0;
+                      return TRUE;
+                    }
+                  return FALSE;
+
+                case 2:
+                  /* 1.2 -> 1.0.0.2 */
+                  data[3] = data[1];
+                  data[1] = data[2] = 0;
+                  return TRUE;
+
+                case 3:
+                  /* 1.2.3 -> 1.2.0.3 */
+                  data[3] = data[2];
+                  data[2] = 0;
+                  return TRUE;
+
+                default:
+                  SSH_NOTREACHED;
+                }
+            }
+          else if (*str == '.' && *(str + 1) != '.')
+            {
+              str++;
+            }
           else
-            return FALSE;
+            {
+              return FALSE;
+            }
         }
       for (value = 0; *str >= '0' && *str <= '9'; str++)
         {
@@ -431,7 +486,8 @@ static Boolean ssh_ipaddr_ipv4_parse(unsigned char *data, const char *str)
             return FALSE;
         }
 
-      if (!(!*str || *str == '.' || (*str >= '0' && *str <= '9')))
+      if ((*str && *str != '.' && !(*str >= '0' && *str <= '9')) ||
+          (!*str && i == 0))
           return FALSE;
 
       if (data)
@@ -631,21 +687,13 @@ Boolean ssh_ipaddr_parse_with_mask(SshIpAddr ip,
 
   if (mask == NULL)
     {
-#ifdef _KERNEL
-      dup = ssh_kstrdup(str);
-#else
-      dup = ssh_xstrdup(str);
-#endif
+      dup = ssh_strdup(str);
       if (!dup)
         return FALSE;
 
       if ((cp = strchr(dup, '/')) == NULL)
         {
-#ifdef _KERNEL
-          ssh_kfree(dup);
-#else
-          ssh_xfree(dup);
-#endif
+          ssh_free(dup);
           return FALSE;
         }
       str = dup;
@@ -698,11 +746,7 @@ Boolean ssh_ipaddr_parse_with_mask(SshIpAddr ip,
     }
 
   if (dup != NULL)
-#ifdef _KERNEL
-    ssh_kfree(dup);
-#else
-    ssh_xfree(dup);
-#endif
+    ssh_free(dup);
 
   return ret;
 }
@@ -715,8 +759,8 @@ static void ssh_ipaddr_ipv4_print(const unsigned char *data,
 {
   char largebuf[64];
 
-  snprintf(largebuf, sizeof(largebuf), "%d.%d.%d.%d",
-           data[0], data[1], data[2], data[3]);
+  ssh_snprintf(largebuf, sizeof(largebuf), "%d.%d.%d.%d",
+               data[0], data[1], data[2], data[3]);
 
   strncpy(buf, largebuf, buflen);
   buf[buflen - 1] = '\0';
@@ -768,7 +812,8 @@ static void ssh_ipaddr_ipv6_print(const unsigned char *data,
         }
       else
         {
-          snprintf(cp, 20, "%x", (unsigned int) SSH_GET_16BIT(data + i * 2));
+          ssh_snprintf(cp, 20, "%x",
+                       (unsigned int) SSH_GET_16BIT(data + i * 2));
           cp += strlen(cp);
 
           if (i + 1 < 8)
@@ -803,8 +848,8 @@ char *ssh_ipaddr_print_with_mask(const SshIpAddr ip, char *buf, size_t buflen)
   else
     ssh_ipaddr_ipv6_print(ip->addr_data, buf, buflen);
 
-  snprintf(buf + strlen(buf), buflen - strlen(buf) - 1, "/%d",
-           (unsigned int) ip->mask_len);
+  ssh_snprintf(buf + strlen(buf), buflen - strlen(buf) - 1, "/%d",
+               (unsigned int) ip->mask_len);
 
   return buf;
 }
@@ -904,7 +949,7 @@ int ssh_ipaddr_render(char *buf, int buf_size, int precision, void *datum)
   int len;
 
   if (ip->type == SSH_IP_TYPE_NONE)
-    snprintf(buf, buf_size, "<none>");
+    ssh_snprintf(buf, buf_size, "<none>");
   else if (SSH_IP_ADDR_LEN(ip) * 8 == SSH_IP_MASK_LEN(ip))
     (void) ssh_ipaddr_print(ip, buf, buf_size);
   else
@@ -912,6 +957,26 @@ int ssh_ipaddr_render(char *buf, int buf_size, int precision, void *datum)
 
   len = strlen(buf);
 
+  if (len == buf_size)
+    return len + 1;
+  if (precision >= 0)
+    if (len > precision)
+      len = precision;
+
+  return len;
+}
+
+int ssh_ipaddr4_uint32_render(char *buf, int buf_size, int precision,
+                              void *datum)
+{
+  int len;
+  unsigned char tmp[4];
+
+  SSH_PUT_32BIT(tmp, (SshUInt32) datum);
+  ssh_ipaddr_ipv4_print(tmp, buf, buf_size);
+
+  len = strlen(buf);
+
   if (precision >= 0)
     if (len > precision)
       len = precision;
@@ -919,6 +984,24 @@ int ssh_ipaddr_render(char *buf, int buf_size, int precision, void *datum)
   return len;
 }
 
+int ssh_ipaddr6_byte16_render(char *buf, int buf_size, int precision,
+                              void *datum)
+{
+  int len;
+
+  ssh_ipaddr_ipv6_print((unsigned char*) datum, buf, buf_size);
+
+  len = strlen(buf);
+
+  if (precision >= 0)
+    if (len > precision)
+      len = precision;
+
+  return len;
+}
+
+
+
 /* Renders an IP protocol name */
 int ssh_ipproto_render(char *buf, int buf_size, int precision, void *datum)
 {
@@ -928,12 +1011,12 @@ int ssh_ipproto_render(char *buf, int buf_size, int precision, void *datum)
   for (i = 0; ssh_ip_protocol_id_keywords[i].name; i++)
     if (proto == ssh_ip_protocol_id_keywords[i].code)
       {
-        snprintf(buf, buf_size, "%s", ssh_ip_protocol_id_keywords[i].name);
+        ssh_snprintf(buf, buf_size, "%s", ssh_ip_protocol_id_keywords[i].name);
         break;
       }
 
   if (!ssh_ip_protocol_id_keywords[i].name)
-    snprintf(buf, buf_size, "(unknown %u)", proto);
+    ssh_snprintf(buf, buf_size, "(unknown %u)", proto);
 
   len = strlen(buf);
 
@@ -964,7 +1047,7 @@ int ssh_ipmask_render(char *buf, int buf_size, int precision, void *datum)
       if (ip->addr_data[i] & (1 << j))
         bits++;
 
-  snprintf(buf, buf_size, "%d", bits);
+  ssh_snprintf(buf, buf_size, "%d", bits);
   len = strlen(buf);
 
   if (precision >= 0)
@@ -973,3 +1056,55 @@ int ssh_ipmask_render(char *buf, int buf_size, int precision, void *datum)
 
   return len;
 }
+
+/* Rendering function for Ethernet MAC addresses. */
+int ssh_etheraddr_render(char *buf, int buf_size, int precision, void *datum)
+{
+  int len;
+  unsigned char *hw_addr = datum;
+
+  ssh_snprintf(buf, buf_size, "%02x:%02x:%02x:%02x:%02x:%02x",
+               hw_addr[0], hw_addr[1], hw_addr[2],
+               hw_addr[3], hw_addr[4], hw_addr[5]);
+
+  len = strlen(buf);
+
+  if (precision >= 0 && len > precision)
+    len = precision;
+
+  return len;
+}
+
+/* Check if ipv6 address is just an ipv4 address mapped into ipv6 mask. */
+
+Boolean ssh_inet_addr_is_ip6_mapped_ip4(SshIpAddr ip_addr)
+{
+  if (! SSH_IP_IS6(ip_addr))
+    return FALSE;
+  SSH_ASSERT(ip_addr->mask_len == 128);
+  return ((ip_addr->addr_data[0] == 0x0) &&
+          (ip_addr->addr_data[1] == 0x0) &&
+          (ip_addr->addr_data[2] == 0x0) &&
+          (ip_addr->addr_data[3] == 0x0) &&
+          (ip_addr->addr_data[4] == 0x0) &&
+          (ip_addr->addr_data[5] == 0x0) &&
+          (ip_addr->addr_data[6] == 0x0) &&
+          (ip_addr->addr_data[7] == 0x0) &&
+          (ip_addr->addr_data[8] == 0x0) &&
+          (ip_addr->addr_data[9] == 0x0) &&
+          (ip_addr->addr_data[10] == 0xff) &&
+          (ip_addr->addr_data[11] == 0xff));
+}
+
+/* Convert if ipv6 mapped ipv4 address to an ipv4 address, if possible. */
+
+Boolean ssh_inet_convert_ip6_mapped_ip4_to_ip4(SshIpAddr ip_addr)
+{
+  if (! ssh_inet_addr_is_ip6_mapped_ip4(ip_addr))
+    return FALSE;
+  memcpy(ip_addr->addr_data, &(ip_addr->addr_data[12]), 4);
+  memset(&(ip_addr->addr_data[4]), 0, 12);
+  ip_addr->mask_len = 32;
+  ip_addr->type = SSH_IP_TYPE_IPV4;
+  return TRUE;
+}
