@@ -13,7 +13,7 @@
         contains definitions specific to various media types (e.g. ethernet).
 
   Copyright:
-        Copyright (c) 1998-2000 SSH Communications Security, Finland
+        Copyright (c) 1998-2001 SSH Communications Security, Finland
         All rights reserved
 */
 
@@ -28,8 +28,15 @@
    to IPv4 and IPv6 addresses, if supported by the system. */
 #define SSH_IPADDR_ANY ssh_ipaddr_any
 
+/* Special IP address strings that can be given for TCP and UDP
+   listeners to bind only to local IP addresses of requested type. */
+#define SSH_IPADDR_ANY_IPV4 ssh_ipaddr_any_ipv4
+#define SSH_IPADDR_ANY_IPV6 ssh_ipaddr_any_ipv6
+
 /* Constant placeholder for the SSH_IPADDR_ANY address string. */
 extern const char *ssh_ipaddr_any;
+extern const char *ssh_ipaddr_any_ipv4;
+extern const char *ssh_ipaddr_any_ipv6;
 
 /* Predicate to check whether the address string `addr' is an
    SSH_IPADDR_ANY. */
@@ -38,6 +45,17 @@ extern const char *ssh_ipaddr_any;
    && ((addr) == SSH_IPADDR_ANY \
        || strcmp((addr), SSH_IPADDR_ANY) == 0))
 
+/* Predicates to check whether the address string `addr' is an
+   SSH_IPADDR_ANY_IPV4 or SSH_IPADDR_ANY_IPV6. */
+#define SSH_IS_IPADDR_ANY_IPV4(addr) \
+  ((addr)                       \
+   && ((addr) == SSH_IPADDR_ANY_IPV4 \
+       || strcmp((addr), SSH_IPADDR_ANY_IPV4) == 0))
+#define SSH_IS_IPADDR_ANY_IPV6(addr) \
+  ((addr)                       \
+   && ((addr) == SSH_IPADDR_ANY_IPV6 \
+       || strcmp((addr), SSH_IPADDR_ANY_IPV6) == 0))
+
 /* IP protocol names to keywords definitions */
 extern const SshKeywordStruct ssh_ip_protocol_id_keywords[];
 
@@ -158,6 +176,25 @@ typedef enum {
   SSH_IPPROTO_PGM = 113,     /* PGM Reliable Transport */
   SSH_IPPROTO_0HOP = 114,    /* Any 0-hop protocol [IANA] */
   SSH_IPPROTO_L2TP = 115,    /* Layer Two Tunneling [Aboba] */
+  SSH_IPPROTO_DDX = 116,     /* D-II Data Exchange (DDX) [Worley]  */
+  SSH_IPPROTO_IATP = 117,    /* Interactive Agent Transfer Protocol [Murphy] */
+  SSH_IPPROTO_STP = 118,     /* Schedule Transfer Protocol [JMP] */
+  SSH_IPPROTO_SRP = 119,     /* SpectraLink Radio Protocol [Hamilton] */
+  SSH_IPPROTO_UTI = 120,     /* UTI [Lothberg] */
+  SSH_IPPROTO_SMP = 121,     /* Simple Message Protocol [Ekblad] */
+  SSH_IPPROTO_SM = 122,      /* SM [Crowcroft] */
+  SSH_IPPROTO_PTP = 123,     /* Performance Transparency Protocol [Welzl] */
+  SSH_IPPROTO_ISISIPV4 = 124,/* ISIS over IPv4 [Przygienda] */
+  SSH_IPPROTO_FIRE = 125,    /* FIRE [Partridge] */
+  SSH_IPPROTO_CRTP = 126,    /* Combat Radio Transport Protocol [Sautter] */
+  SSH_IPPROTO_CRUDP = 127,   /* Combat Radio User Datagram [Sautter] */
+  SSH_IPPROTO_SSCOPMCE = 128,/* SSCOPMCE [Waber] */
+  SSH_IPPROTO_IPLT = 129,    /* IPLT [Hollbach] */
+  SSH_IPPROTO_SPS = 130,     /* Secure Packet Shield [McIntosh] */
+  SSH_IPPROTO_PIPE = 131,    /* Private IP Encapsulation within IP [Petri] */
+  SSH_IPPROTO_SCTP = 132,    /* Stream Control Transmission Protocol[Stewart]*/
+  SSH_IPPROTO_FC = 133,      /* Fibre Channel [Rajagopal] */
+  SSH_IPPROTO_RSVP_E2E_IGNORE = 134, /* RSVP-E2E-IGNORE [RFCXXXX] */
   SSH_IPPROTO_RESERVED = 255 /* Reserved [IANA] */
 } SshInetIPProtocolID;
 
@@ -227,12 +264,12 @@ typedef enum {
 
   /* ICMP6 informational types */
   SSH_ICMP6_TYPE_ECHOREQUEST = 128,     /* Echo request */
-  SSH_ICMP6_TYPE_ECHOREPLY = 129,        /* Echo reply */
+  SSH_ICMP6_TYPE_ECHOREPLY = 129,       /* Echo reply */
 
   /* From RFC2461 */
-  SSH_ICMP6_TYPE_ROUTER_SOLICITATION = 133,      /* Router solicitation */
-  SSH_ICMP6_TYPE_ROUTER_ADVERTISEMENT = 134,     /* Router advertisement */
-  SSH_ICMP6_TYPE_NEIGHBOR_SOLICITATION = 135,   /* Neighbor Solicitation */
+  SSH_ICMP6_TYPE_ROUTER_SOLICITATION = 133,   /* Router solicitation */
+  SSH_ICMP6_TYPE_ROUTER_ADVERTISEMENT = 134,  /* Router advertisement */
+  SSH_ICMP6_TYPE_NEIGHBOR_SOLICITATION = 135, /* Neighbor Solicitation */
   SSH_ICMP6_TYPE_NEIGHBOR_ADVERTISEMENT = 136 /* Neighbor Advertisement */
 } SshInetIPIcmp6Type;
 
@@ -269,10 +306,8 @@ typedef enum {
 
 /*************************** Auxiliary functions ****************************/
 
-/* Determines whether the given string is a valid numeric IP address.
-   (This currently only works for IPv4 addresses, but might be changed
-   in future to accept also IPv6 addresses on systems that support
-   them. */
+/* Determines whether the given string is a valid numeric IP address
+   (either v4 or v6 address). */
 Boolean ssh_inet_is_valid_ip_address(const char *address);
 
 /* Compares two IP addresses, and returns <0 if address1 is smaller
@@ -297,6 +332,10 @@ Boolean ssh_inet_compare_netmask(const char *nets, const char *ip);
 Boolean ssh_inet_strtobin(const char *ip_address, unsigned char *out_buffer,
                           size_t *out_buffer_len_in_out);
 
+/******** Protocol specifiers to be used in protocol selection masks ********/
+
+#define SSH_IP_TYPE_MASK_IP4    0x00000001
+#define SSH_IP_TYPE_MASK_IP6    0x00000002
 
 /***************************** SshIpAddr stuff ******************************/
 
@@ -308,10 +347,23 @@ typedef enum {
 
 typedef struct SshIpAddrRec
 {
-    SshIpAddrType               type;
+  /* Note: All fields of this data structure are private, and should not
+     be accessed except using the macros and functions defined in this
+     header.  They should never be accessed directly; the internal definition
+     of this structure is subject to change without notice. */
 
-    SshUInt32                   mask_len;
-    unsigned char               addr_data[16];
+  /* For optimised mask comparison routine _addr_data has to be 32-bit
+     aligned so it can be read as words on machines requiring
+     alignment */
+  union {
+    unsigned char _addr_data[16];
+    SshUInt32 _addr_align;
+  } addr_union;
+
+#define addr_data addr_union._addr_data
+
+  SshUInt8 type;
+  SshUInt8 mask_len;
 } *SshIpAddr, SshIpAddrStruct;
 
 #define SSH_IP_DEFINED(ip_addr) ((ip_addr)->type != SSH_IP_TYPE_NONE)
@@ -325,9 +377,9 @@ typedef struct SshIpAddrRec
       ? (16)                            \
       : 0))
 
-#define SSH_IP_UNDEFINE(IPADDR)                     \
-do {                                            \
-  (IPADDR)->type = SSH_IP_TYPE_NONE;            \
+#define SSH_IP_UNDEFINE(IPADDR)         \
+do {                                    \
+  (IPADDR)->type = SSH_IP_TYPE_NONE;    \
 } while (0)
 
 #define __SSH_IP_MASK_DECODE(IPADDR,TYPE,BYTES,BYTELEN,MASKLEN) \
@@ -365,29 +417,45 @@ do {                                            \
 #define SSH_IP6_MASK_ENCODE(ip_addr,bytes,mask) \
   __SSH_IP_MASK_ENCODE(ip_addr,SSH_IP_TYPE_IPV6,bytes,16,mask)
 
-#define __SSH_IP_TO_INT(addr_data)              \
-  (((unsigned long) (addr_data)[0] << 24)       \
-   | ((unsigned long) (addr_data)[1] << 16)     \
-   | ((unsigned long) (addr_data)[2] << 8)      \
-   | (unsigned long) (addr_data)[3])
+#define SSH_IP_DECODE(ip_addr,addr_buf,addr_len)        \
+do {                                                    \
+  if (addr_len == 4)                                    \
+    SSH_IP4_DECODE(ip_addr,addr_buf);                   \
+  else if (addr_len == 16)                              \
+    SSH_IP6_DECODE(ip_addr,addr_buf);                   \
+  else                                                  \
+    SSH_IP_UNDEFINE(ip_addr);                           \
+} while(0)
 
-#define SSH_IP4_TO_INT(ip_addr) __SSH_IP_TO_INT((ip_addr)->addr_data)
+#define SSH_IP_ENCODE(ip_addr,addr_buf,addr_len)        \
+do {                                                    \
+  if ((ip_addr) == NULL)                                \
+    (addr_len) = 0;                                     \
+  else                                                  \
+    {                                                   \
+      addr_len = SSH_IP_ADDR_LEN(ip_addr);              \
+      if (SSH_IP_IS4(ip_addr))                          \
+        SSH_IP4_ENCODE(ip_addr,addr_buf);               \
+      else if (SSH_IP_IS6(ip_addr))                     \
+        SSH_IP6_ENCODE(ip_addr, addr_buf);              \
+    }                                                   \
+} while(0)
 
-#define SSH_INT_TO_IP4(ip_addr, num)                                    \
-  do {                                                                  \
-    (ip_addr)->type = SSH_IP_TYPE_IPV4;                                 \
-    (ip_addr)->mask_len = 32;                                           \
-    (ip_addr)->addr_data[0] = (unsigned char) (((num) >> 24) & 0xff);   \
-    (ip_addr)->addr_data[1] = (unsigned char) (((num) >> 16) & 0xff);   \
-    (ip_addr)->addr_data[2] = (unsigned char) (((num) >>  8) & 0xff);   \
-    (ip_addr)->addr_data[3] = (unsigned char) (((num) >>  0) & 0xff);   \
-  } while (0)
+#define SSH_IP4_TO_INT(ip_addr) SSH_GET_32BIT((ip_addr)->addr_data)
 
-#define SSH_IP6_WORD0_TO_INT(ip_addr) __SSH_IP_TO_INT((ip_addr)->addr_data)
-#define SSH_IP6_WORD1_TO_INT(ip_addr) __SSH_IP_TO_INT((ip_addr)->addr_data + 4)
-#define SSH_IP6_WORD2_TO_INT(ip_addr) __SSH_IP_TO_INT((ip_addr)->addr_data + 8)
-#define SSH_IP6_WORD3_TO_INT(ip_addr) \
-  __SSH_IP_TO_INT((ip_addr)->addr_data + 12)
+#define SSH_INT_TO_IP4(ip_addr, num)            \
+do                                              \
+  {                                             \
+    (ip_addr)->type = SSH_IP_TYPE_IPV4;         \
+    (ip_addr)->mask_len = 32;                   \
+    SSH_PUT_32BIT((ip_addr)->addr_data, (num)); \
+  }                                             \
+while (0)
+
+#define SSH_IP6_WORD0_TO_INT(ip_addr) SSH_GET_32BIT((ip_addr)->addr_data)
+#define SSH_IP6_WORD1_TO_INT(ip_addr) SSH_GET_32BIT((ip_addr)->addr_data + 4)
+#define SSH_IP6_WORD2_TO_INT(ip_addr) SSH_GET_32BIT((ip_addr)->addr_data + 8)
+#define SSH_IP6_WORD3_TO_INT(ip_addr) SSH_GET_32BIT((ip_addr)->addr_data + 12)
 
 #define SSH_IP4_BYTE1(ip_addr) ((ip_addr)->addr_data[0])
 #define SSH_IP4_BYTE2(ip_addr) ((ip_addr)->addr_data[1])
@@ -425,7 +493,7 @@ do {                                            \
 
 #define SSH_IP_MASK_EQUAL(ip1, ip2) ssh_ipaddr_mask_equal(ip1, ip2)
 #define SSH_IP_WITH_MASK_EQUAL(ip1, ip2,mask) \
-  ssh_ipaddr_equal_with_mask(ip1, ip2,mask)
+  ssh_ipaddr_with_mask_equal(ip1, ip2,mask)
 
 #define SSH_IP_HASH(ip_addr) ssh_ipaddr_hash(ip_addr)
 
@@ -449,13 +517,16 @@ do {                                            \
    && (ip_addr)->addr_data[2] == 0xff   \
    && (ip_addr)->addr_data[3] == 0xff)
 
-/* Returns TRUE if the given IP address is a multicast address between
-   224.0.0.0 and 239.255.255.255, inclusive. */
+/* Returns TRUE if the given IP address is a multicast address. */
 #define SSH_IP_IS_MULTICAST(ip_addr)    \
   (SSH_IP_IS6(ip_addr) ?                \
    ((ip_addr)->addr_data[0] == 0xff) :  \
    ((ip_addr)->addr_data[0] >= 0xe0 && (ip_addr)->addr_data[0] <= 0xef))
 
+/* Return TRUE if the IPv6 address `ip_addr' is a multicast
+   address. */
+#define SSH_IP6_IS_MULTICAST(ip_addr) (SSH_IP6_BYTE1(ip_addr) = 0xff)
+
 /* Returns TRUE if the given IP address is a loopback address. */
 #define SSH_IP_IS_LOOPBACK(ip_addr)                             \
   (SSH_IP_IS6(ip_addr)                                          \
@@ -469,6 +540,31 @@ do {                                            \
   (SSH_IP_IS6((ip_addr))                \
    && (((SSH_IP6_WORD0_TO_INT((ip_addr))) & 0xffc00000) == 0xfe800000))
 
+/* Returns TRUE if the given IP address is an IPv6 site-local
+   address. */
+#define SSH_IP6_IS_SITE_LOCAL(ip_addr)  \
+  (SSH_IP_IS6((ip_addr))                \
+   && (((SSH_IP6_WORD0_TO_INT((ip_addr))) & 0xffc00000) == 0xfec00000))
+
+/* IPv6 multicast address scopes. */
+
+#define SSH_IP6_MC_SCOPE(ip_addr) (SSH_IP6_BYTE2(ip_addr) & 0x0f)
+
+#define SSH_IP6_IS_MC_NODE_LOCAL(ip_addr)       \
+(SSH_IP6_IS_MULTICAST(ip_addr) && (SSH_IP6_MC_SCOPE(ip_addr) == 0x01))
+
+#define SSH_IP6_IS_MC_LINK_LOCAL(ip_addr)       \
+(SSH_IP6_IS_MULTICAST(ip_addr) && (SSH_IP6_MC_SCOPE(ip_addr) == 0x02))
+
+#define SSH_IP6_IS_MC_SITE_LOCAL(ip_addr)       \
+(SSH_IP6_IS_MULTICAST(ip_addr) && (SSH_IP6_MC_SCOPE(ip_addr) == 0x05))
+
+#define SSH_IP6_IS_MC_ORG_LOCAL(ip_addr)        \
+(SSH_IP6_IS_MULTICAST(ip_addr) && (SSH_IP6_MC_SCOPE(ip_addr) == 0x08))
+
+#define SSH_IP6_IS_MC_GLOBAL(ip_addr)   \
+(SSH_IP6_IS_MULTICAST(ip_addr) && (SSH_IP6_MC_SCOPE(ip_addr) == 0x0e))
+
 
 /******************** Definitions for IP(4/6) packets ***********************/
 
@@ -595,7 +691,8 @@ do {                                            \
 /******************** Upper level protocols TCP and UDP *********************/
 
 /* TCP header length, and offsets to header and pseudo-header */
-#define SSH_TCP_HEADER_LEN 20
+#define SSH_TCPH_HDRLEN 20
+#define SSH_TCP_HEADER_LEN SSH_TCPH_HDRLEN
 
 #define SSH_TCPH_OFS_SRCPORT            0
 #define SSH_TCPH_OFS_DSTPORT            2
@@ -645,7 +742,8 @@ do {                                            \
 /********************* Definitions for IPv4 UDP packets *********************/
 
 /* UDP header length, field offsets and pseudo-header offsets */
-#define SSH_UDP_HEADER_LEN 8
+#define SSH_UDPH_HDRLEN 8
+#define SSH_UDP_HEADER_LEN SSH_UDPH_HDRLEN
 
 #define SSH_UDPH_OFS_SRCPORT            0
 #define SSH_UDPH_OFS_DSTPORT            2
@@ -869,28 +967,48 @@ do {                                            \
 void ssh_ipaddr_set_bits(SshIpAddr result, SshIpAddr ip,
                          unsigned int keep_bits, unsigned int value);
 
-void ssh_ipaddr_merge_bits (SshIpAddr result, SshIpAddr left_ip,
-                            unsigned int bits, SshIpAddr right_ip);
+/* Merges the two IP addresses, so that leftmost `bits' bits are from left_ip,
+   and the remaining bits from right_ip. */
+void ssh_ipaddr_merge_bits(SshIpAddr result, SshIpAddr left_ip,
+                           unsigned int bits, SshIpAddr right_ip);
 
 /* Parses an IP address from the string to the internal representation. */
 Boolean ssh_ipaddr_parse(SshIpAddr ip, const char *str);
 Boolean ssh_ipaddr_parse_with_mask(SshIpAddr ip, const char *str,
                                    const char *mask);
 
+/* Check if ipv6 address is just an ipv4 address mapped into ipv6 mask. */
+Boolean ssh_inet_addr_is_ip6_mapped_ip4(SshIpAddr ip_addr);
+
+/* Convert if ipv6 mapped ipv4 address to an ipv4 address, if possible. */
+Boolean ssh_inet_convert_ip6_mapped_ip4_to_ip4(SshIpAddr ip_addr);
+
 /* Prints the IP address into the buffer in string format.  If the buffer
    is too short, the address is truncated.  This returns `buf'. */
 char *ssh_ipaddr_print(const SshIpAddr ip, char *buf, size_t buflen);
 char *ssh_ipaddr_print_with_mask(const SshIpAddr ip, char *buf, size_t buflen);
 
-/* Rendering function (for ssh_e*printf %@ format) for IP addresses */
-int ssh_ipaddr_render(char *buf, int buf_size, int precision, void * datum);
+/* Rendering function (for ssh_e*printf %@ format) for IP
+   addresses. Datum is SshIpAddr. */
+int ssh_ipaddr_render(char *buf, int buf_size, int precision, void *datum);
 
-/* Rendering function for IP protocol numbers. */
-int ssh_ipproto_render(char *buf, int buf_size, int precision, void * datum);
+/* Renders an IPv4 address. Datum is SshUInt32 value. */
+int ssh_ipaddr4_uint32_render(char *buf, int buf_size, int precision,
+                              void *datum);
 
-/* Rendering function for IP address masks. */
-int ssh_ipmask_render(char *buf, int buf_size, int precision, void * datum);
+/* Renders an IPv6 address. Datum is unsigned char[16] array. */
+int ssh_ipaddr6_byte16_render(char *buf, int buf_size, int precision,
+                              void *datum);
 
+/* Rendering function for IP protocol numbers. Datum is SshUInt32. */
+int ssh_ipproto_render(char *buf, int buf_size, int precision, void *datum);
+
+/* Rendering function for IP address masks. Datum is SshIpAddr*/
+int ssh_ipmask_render(char *buf, int buf_size, int precision, void *datum);
+
+/* Rendering function for Ethernet MAC addresses. Datum is unsigned
+ * char[6] array. */
+int ssh_etheraddr_render(char *buf, int buf_size, int precision, void *datum);
 
 /*************************** Internal definitions ***************************/
 
