@@ -6,7 +6,7 @@
         Santeri Paavolainen <santtu@ssh.fi>
 
   Copyright:
-        Copyright (c) 1999 SSH Communications Security, Finland
+        Copyright (c) 1999, 2001 SSH Communications Security, Finland
         All rights reserved
 
 */
@@ -15,18 +15,18 @@
 #include "sshencode.h"
 #include "sshinetencode.h"
 
-size_t ssh_encode_ipaddr_array (unsigned char * buf, size_t bufsize,
-                                const SshIpAddr ip)
+size_t ssh_encode_ipaddr_array(unsigned char *buf, size_t bufsize,
+                               const SshIpAddr ip)
 {
   size_t got;
 
   if (ip->type == SSH_IP_TYPE_NONE)
     got = ssh_encode_array(buf, bufsize,
-                           SSH_FORMAT_CHAR, ip->type,
+                           SSH_FORMAT_CHAR, (unsigned int) ip->type,
                            SSH_FORMAT_END);
   else
     if ((got = ssh_encode_array(buf, bufsize,
-                                SSH_FORMAT_CHAR, ip->type,
+                                SSH_FORMAT_CHAR, (unsigned int) ip->type,
                                 SSH_FORMAT_UINT32, ip->mask_len,
                                 SSH_FORMAT_DATA,
                                 ip->addr_data, SSH_IP_ADDR_LEN(ip),
@@ -37,8 +37,8 @@ size_t ssh_encode_ipaddr_array (unsigned char * buf, size_t bufsize,
   return got;
 }
 
-size_t ssh_encode_ipaddr_array_alloc (unsigned char ** buf_return,
-                                      const SshIpAddr ip)
+size_t ssh_encode_ipaddr_array_alloc(unsigned char **buf_return,
+                                     const SshIpAddr ip)
 {
   size_t req, got;
 
@@ -50,22 +50,14 @@ size_t ssh_encode_ipaddr_array_alloc (unsigned char ** buf_return,
   if (buf_return == NULL)
     return req;
 
-#ifdef _KERNEL
-  if (!(*buf_return = ssh_kmalloc(req)))
+  if ((*buf_return = ssh_malloc(req)) == NULL)
     return 0;
-#else
-  *buf_return = ssh_xmalloc(req);
-#endif /* _KERNEL */
 
   got = ssh_encode_ipaddr_array(*buf_return, req, ip);
 
   if (got != req)
     {
-#ifdef _KERNEL
-      ssh_kfree(*buf_return);
-#else
-      ssh_xfree(*buf_return);
-#endif /* _KERNEL */
+      ssh_free(*buf_return);
       *buf_return = NULL;
       return 0;
     }
@@ -73,31 +65,36 @@ size_t ssh_encode_ipaddr_array_alloc (unsigned char ** buf_return,
   return got;
 }
 
-size_t ssh_decode_ipaddr_array (const unsigned char * buf,
-                                size_t len,
-                                SshIpAddr ip)
+size_t ssh_decode_ipaddr_array(const unsigned char *buf, size_t len,
+                               SshIpAddr ip)
 {
   size_t point, got;
+  SshUInt32 mask_len;
+  unsigned int type;
 
   point = 0;
 
   if ((got = ssh_decode_array(buf + point, len - point,
-                              SSH_FORMAT_CHAR, &ip->type,
+                              SSH_FORMAT_CHAR, &type,
                               SSH_FORMAT_END)) != 1)
       return 0;
 
+  ip->type = type;
+
   point += got;
 
   if (ip->type == SSH_IP_TYPE_NONE)
     return point;
 
   if ((got = ssh_decode_array(buf + point, len - point,
-                              SSH_FORMAT_UINT32, &ip->mask_len,
+                              SSH_FORMAT_UINT32, &mask_len,
                               SSH_FORMAT_DATA,
                                 ip->addr_data, SSH_IP_ADDR_LEN(ip),
                               SSH_FORMAT_END)) != (4 + SSH_IP_ADDR_LEN(ip)))
       return 0;
 
+  ip->mask_len = mask_len;
+
   point += got;
 
   /* Sanity check */
@@ -108,20 +105,22 @@ size_t ssh_decode_ipaddr_array (const unsigned char * buf,
 }
 
 #if !(defined(_KERNEL) || defined(KERNEL))
-size_t ssh_encode_ipaddr_buffer (SshBuffer buffer, const SshIpAddr ip)
+size_t ssh_encode_ipaddr_buffer(SshBuffer buffer, const SshIpAddr ip)
 {
   size_t got;
   unsigned char tmpbuf[1 + 4 + 16];
 
   if ((got = ssh_encode_ipaddr_array(tmpbuf, sizeof(tmpbuf), ip)) == 0)
-      return FALSE;
+    return FALSE;
 
-  ssh_buffer_append(buffer, tmpbuf, got);
+  if (ssh_buffer_append(buffer, tmpbuf, got) == SSH_BUFFER_OK)
+    return TRUE;
+  else
+    return FALSE;
 
-  return TRUE;
 }
 
-size_t ssh_ipaddr_decode_buffer (SshBuffer buffer, SshIpAddr ip)
+size_t ssh_ipaddr_decode_buffer(SshBuffer buffer, SshIpAddr ip)
 {
   size_t got;
 
