@@ -18,6 +18,7 @@ Listeners with these streams are identified with file names.
 #define SSHLOCALSTREAM_H
 
 #include "sshstream.h"
+#include "sshoperation.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -57,10 +58,14 @@ void ssh_local_destroy_listener(SshLocalListener listener);
    will be colled when the connection is complete or has failed. If
    the connection is successful, an SshStream object is created and
    passed to the callback.  If connecting fails, NULL is passed to the
-   callback as the stream. */
-void ssh_local_connect(const char *path,
-                       SshLocalCallback callback,
-                       void *context);
+   callback as the stream.
+
+   Returns an operation handle, which is non NULL if the connect is
+   asynchronous. Connection can be aborted by calling 
+   ssh_operation_abort, for the returned handle. */
+SshOperationHandle ssh_local_connect(const char *path,
+                                     SshLocalCallback callback,
+                                     void *context);
 #ifdef __cplusplus
 }
 #endif
