@@ -3,27 +3,26 @@
 Author: Tatu Ylonen <ylo@ssh.fi>
         Antti Huima <huima@ssh.fi>
 
-Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
+Copyright (C) 1996-2001 SSH Communications Security Oy, Espoo, Finland
 All rights reserved.
 
 Interface to sockets.
 
 */
 
-/*
- * $Id: sshtcp.c,v 1.4 2000/09/27 02:22:22 kivinen Exp $
- * $Log: sshtcp.c,v $
- * $EndLog$
- */
-
 #include "sshincludes.h"
 #include "sshstream.h"
 #include "sshtcp.h"
+#include "sshinet.h"
 #include "sshbuffer.h"
 #include "sshsocks.h"
+#ifdef SSHDIST_UTIL_URL
 #include "sshurl.h"
+#endif /* SSHDIST_UTIL_URL */
 #include "sshtimeouts.h"
 
+#define SSH_DEBUG_MODULE "SshTcp"
+
 typedef enum
 {
   CONNECT_STATE_HOST_LOOKUP,
@@ -44,6 +43,7 @@ typedef struct {
   char *host_addresses;                 /* addresses for the host to connect */
   const char *next_address;             /* next address to try */
   unsigned int host_port;               /* port to connect on the host */
+  SshUInt32 protocol_mask;              /* protocols used in connect */
 
   /* User callback. */
   SshTcpCallback user_callback;
@@ -60,7 +60,7 @@ typedef struct {
   const char *socks_next_address;       /* next address to try */
   unsigned int socks_port;              /* socks port */
   char *user_name;                      /* user requesting connection */
-  SshBuffer socks_buf;                 /* Socks buffer */
+  SshBuffer socks_buf;                  /* Socks buffer */
 
   /* An open stream to either the socks server or the final destination. */
   SshStream stream;
@@ -94,6 +94,18 @@ void ssh_socket_destroy_connect_context(ConnectContext c);
    Destroys the context. */
 void ssh_socket_connect_final(ConnectContext c, SshStream stream,
                               SshIpError error);
+
+/* Remove addresses that don't match the protocol match from the
+   address list.  Overwrites the original list.  Can also return an
+   empty string. */
+void ssh_remove_non_matching_addresses_from_list(char *address,
+                                                 SshUInt32 protocol_mask);
+
+/* Return TRUE, if address os of type that is specified in the
+   protocol mask. */
+Boolean ssh_address_type_matches_protocol_mask(char *address,
+                                               SshUInt32 protocol_mask);
+
 /* Connection timed out */
 void ssh_tcp_connect_timed_out(void *context)
 {
@@ -118,7 +130,7 @@ void ssh_tcp_connect_aborted(void *context)
 
    Note that the callback may be called either during this
    call or some time later.
-   
+
    Returns SshOperationHandle that can be used to abort the tcp open.
 
    The `host_name_or_address' argument may be a numeric IP address or a
@@ -136,14 +148,32 @@ SshOperationHandle ssh_tcp_connect(const char *host_name_or_address,
 {
   ConnectContext c;
 
-  c = ssh_xmalloc(sizeof(*c));
+  c = ssh_malloc(sizeof(*c));
+  if (c == NULL)
+    {
+      (*callback)(SSH_IP_FAILURE, NULL, context);
+      return NULL;
+    }
+
   memset(c, 0, sizeof(*c));
 
-  c->host_name = ssh_xstrdup(host_name_or_address);
+  c->host_name = ssh_strdup(host_name_or_address);
   c->host_port = ssh_tcp_get_port_by_service(port_or_service, "tcp");
   c->host_addresses = NULL;
   c->next_address = NULL;
 
+  if (c->host_name == NULL || c->host_port == 0)
+    {
+      (*callback)(SSH_IP_FAILURE, NULL, context);
+      ssh_socket_destroy_connect_context(c);
+      return NULL;
+    }
+
+  if (params && (params->protocol_mask != 0))
+    c->protocol_mask = params->protocol_mask;
+  else
+    c->protocol_mask = (~0);
+
   c->user_callback = callback;
   c->user_context = context;
   if (params && params->connection_timeout != 0)
@@ -164,34 +194,49 @@ SshOperationHandle ssh_tcp_connect(const char *host_name_or_address,
   c->state = CONNECT_STATE_HOST_LOOKUP;
 
   /* Initialize socks-related data. */
+#ifdef SSHDIST_UTIL_URL
   if (params &&
       params->socks_server_url != NULL &&
       strcmp(params->socks_server_url, "") != 0)
     {
       char *scheme, *port;
-
-      ssh_url_parse_and_decode(params->socks_server_url, &scheme,
-                               &(c->socks_host), &port,
-                               &(c->user_name), NULL, &(c->socks_exceptions));
-
-      if (scheme != NULL && strcmp(scheme, "socks") != 0)
-        ssh_warning("Socks server scheme not socks");
-      if (scheme != NULL)
-        ssh_xfree(scheme);
-
-      if (c->socks_host != NULL)
+      
+      if (ssh_url_parse_and_decode(params->socks_server_url, &scheme,
+                                   &(c->socks_host), &port,
+                                   &(c->user_name), NULL, &(c->socks_exceptions)) ||
+          ssh_url_parse_relaxed_and_decode(params->socks_server_url, &scheme,
+                                           &(c->socks_host), &port,
+                                           &(c->user_name), NULL, &(c->socks_exceptions)))
         {
-          c->socks_buf = ssh_buffer_allocate();
-          c->socks_addresses = NULL;
-          if (port == NULL || strcmp(port, "") == 0)
-            c->socks_port = 1080; /* The standard socks port. */
-          else
-            c->socks_port = ssh_tcp_get_port_by_service(port, "tcp");
+          if (scheme != NULL && strcmp(scheme, "socks") != 0)
+            ssh_warning("Socks server scheme not socks");
+          if (scheme != NULL)
+            ssh_xfree(scheme);
+          
+          if (c->socks_host != NULL)
+            {
+              if ((c->socks_buf = ssh_buffer_allocate()) == NULL)
+                {
+                  (*callback)(SSH_IP_FAILURE, NULL, context);
+                  ssh_socket_destroy_connect_context(c);
+                  return NULL;
+                }
+              c->socks_addresses = NULL;
+              if (port == NULL || strcmp(port, "") == 0)
+                c->socks_port = 1080; /* The standard socks port. */
+              else
+                c->socks_port = ssh_tcp_get_port_by_service(port, "tcp");
+            }
+          if (port != NULL)
+            ssh_xfree(port);
+        }
+      else
+        {
+          ssh_warning("Socks server URL is malformed.");
         }
-      if (port != NULL)
-        ssh_xfree(port);
     }
   else
+#endif /* SSHDIST_UTIL_URL */
     c->socks_host = NULL;
 
   c->upper_handle = NULL;
@@ -269,6 +314,73 @@ void ssh_socket_connect_final(ConnectContext c, SshStream stream,
     }
 }
 
+/* Remove addresses that don't match the protocol match from the
+   address list.  Overwrites the original list.  Can also return an
+   empty string. */
+
+void ssh_remove_non_matching_addresses_from_list(char *address,
+                                                 SshUInt32 protocol_mask)
+{
+  char *na, *cur, *next;
+
+  na = ssh_xmalloc(strlen(address) + 1);
+  na[0] = '\0';
+  cur = address;
+  while (cur != NULL)
+    {
+      next = strchr(cur, ',');
+      if (next)
+        *next = '\0';
+      if (ssh_address_type_matches_protocol_mask(cur, protocol_mask))
+        {
+          if (na[0] != '\0')
+            {
+              strcat(na, ",");
+            }
+          strcat(na, cur);
+        }
+      if (next)
+        {
+          cur = next + 1;
+          *next = ',';
+        }
+      else
+        {
+          cur = NULL;
+        }
+    }
+  SSH_DEBUG(5, ("Original address list = \"%s\"", address));
+  strcpy(address, na);
+  ssh_xfree(na);
+  SSH_DEBUG(5, ("Fixed address list = \"%s\"", address));
+  return;
+}
+
+/* Return TRUE, if address os of type that is specified in the
+   protocol mask. */
+
+Boolean ssh_address_type_matches_protocol_mask(char *address,
+                                               SshUInt32 protocol_mask)
+{
+  SshIpAddrStruct ipaddr;
+  Boolean pr;
+  char *next;
+
+  next = strchr(address, ',');
+  if (next)
+    *next = '\0';
+  pr = ssh_ipaddr_parse(&ipaddr, address);
+  if (next)
+    *next = ',';
+  if (! pr)
+    return FALSE;
+  if (SSH_IP_IS6(&ipaddr) && (protocol_mask & SSH_IP_TYPE_MASK_IP6))
+    return TRUE;
+  if (SSH_IP_IS4(&ipaddr) && (protocol_mask & SSH_IP_TYPE_MASK_IP4))
+    return TRUE;
+  return FALSE;
+}
+
 /* Increments the attempt count, and returns true if we have used all of
    our attempts.  If that happens, this will call the user callback
    with the error, and destroy the context; the caller should just return. */
@@ -290,8 +402,26 @@ void ssh_socket_connect_host_lookup_done(SshIpError error,
                                          void *context)
 {
   ConnectContext c = (ConnectContext)context;
+  char *addrs = NULL;
 
   c->handle = NULL;
+  if (error == SSH_IP_OK)
+    {
+      if ((addrs = ssh_strdup(result)) == NULL)
+        {
+          error = SSH_IP_FAILURE;
+        }
+      else
+        {
+          ssh_remove_non_matching_addresses_from_list(addrs, c->protocol_mask);
+          if (strlen(addrs) == 0)
+            {
+              ssh_xfree(addrs);
+              addrs = NULL;
+              error = SSH_IP_NO_ADDRESS;
+            }
+        }
+    }
   if (error != SSH_IP_OK)
     {
       if (ssh_socket_failure(c, error))
@@ -302,8 +432,7 @@ void ssh_socket_connect_host_lookup_done(SshIpError error,
       return;
     }
 
-  /* Save the lookup result. */
-  c->host_addresses = ssh_xstrdup(result);
+  c->host_addresses = addrs;
   c->next_address = c->host_addresses;
 
   /* Enter the next state. */
@@ -322,6 +451,7 @@ void ssh_socket_connect_socks_lookup_done(SshIpError error,
                                           void *context)
 {
   ConnectContext c = (ConnectContext)context;
+
   c->handle = NULL;
   if (error != SSH_IP_OK)
     {
@@ -334,26 +464,48 @@ void ssh_socket_connect_socks_lookup_done(SshIpError error,
     }
 
   /* Save the lookup result. */
-  c->socks_addresses = ssh_xstrdup(result);
+  if ((c->socks_addresses = ssh_strdup(result)) == NULL)
+    {
+      if (ssh_socket_failure(c, error))
+        return;
+    }
+
   c->socks_next_address = (char *) c->socks_addresses;
 
   /* Enter the next state. */
   if (c->socks_exceptions)
     {
       char *next;
-      next = strchr(c->host_addresses, ',');
+      SshIpAddrStruct ipaddr;
+
+      next = strchr(c->next_address, ',');
       if (next)
         *next = '\0';
-      if (ssh_inet_compare_netmask(c->socks_exceptions,
-                                   c->host_addresses))
-        c->state = CONNECT_STATE_HOST_CONNECT;
+
+      if (! ssh_ipaddr_parse(&ipaddr, c->next_address))
+        {
+          c->state = CONNECT_STATE_HOST_CONNECT;
+        }
+      else if (SSH_IP_IS6(&ipaddr))
+        {
+          c->state = CONNECT_STATE_HOST_CONNECT;
+        }
+      else if (ssh_inet_compare_netmask(c->socks_exceptions,
+                                        c->next_address))
+        {
+          c->state = CONNECT_STATE_HOST_CONNECT;
+        }
       else
-        c->state = CONNECT_STATE_SOCKS_CONNECT;
+        {
+          c->state = CONNECT_STATE_SOCKS_CONNECT;
+        }
       if (next)
         *next = ',';
     }
   else
-    c->state = CONNECT_STATE_SOCKS_CONNECT;
+    {
+      c->state = CONNECT_STATE_SOCKS_CONNECT;
+    }
   ssh_socket_connect_step(c);
 }
 
@@ -380,11 +532,20 @@ void ssh_socket_host_connect_done(SshIpError error,
             return;
           c->next_address = c->host_addresses;
         }
+
       /* Try connecting again. */
-      h = ssh_socket_low_connect(c->next_address, c->host_port,
-                                 ssh_socket_host_connect_done, (void *)c);
-      if (h)
-        c->handle = h;
+      if (c->socks_host)
+        {
+          c->state = CONNECT_STATE_SOCKS_LOOKUP;
+          ssh_socket_connect_step(c);
+        }
+      else
+        {
+          h = ssh_socket_low_connect(c->next_address, c->host_port,
+                                     ssh_socket_host_connect_done, (void *)c);
+          if (h)
+            c->handle = h;
+        }
       return;
     }
 
@@ -469,7 +630,9 @@ void ssh_socket_socks_connect_done(SshIpError error,
     {
       /* Get next address. */
       if (strchr(c->socks_next_address, ','))
-        c->socks_next_address = strchr(c->socks_next_address, ',') + 1;
+        {
+          c->socks_next_address = strchr(c->socks_next_address, ',') + 1;
+        }
       else
         { /* At end of list; consider it as a failure. */
           if (ssh_socket_failure(c, error))
@@ -494,7 +657,7 @@ void ssh_socket_socks_connect_done(SshIpError error,
   socksinfo.socks_version_number = 4;
   socksinfo.command_code = SSH_SOCKS_COMMAND_CODE_CONNECT;
   socksinfo.ip = (char *) c->next_address;
-  snprintf(host_port, sizeof(host_port), "%d", c->host_port);
+  ssh_snprintf(host_port, sizeof(host_port), "%d", c->host_port);
   socksinfo.port = host_port;
   socksinfo.username = c->user_name;
 
@@ -502,10 +665,21 @@ void ssh_socket_socks_connect_done(SshIpError error,
   ret = ssh_socks_client_generate_open(c->socks_buf, &socksinfo);
   if (ret != SSH_SOCKS_SUCCESS)
     {
-      if (ret == SSH_SOCKS_ERROR_INVALID_ARGUMENT)
-        ssh_socket_connect_final(c, NULL, SSH_IP_NO_ADDRESS);
+      if (strchr(c->next_address, ','))
+        {
+          c->stream = NULL;
+          ssh_stream_destroy(stream);
+          c->next_address = strchr(c->next_address, ',') + 1;
+          c->state = CONNECT_STATE_SOCKS_LOOKUP;
+          ssh_socket_connect_step(c);
+        }
       else
-        ssh_socket_connect_final(c, NULL, SSH_IP_FAILURE);
+        {
+          if (ret == SSH_SOCKS_ERROR_INVALID_ARGUMENT)
+            ssh_socket_connect_final(c, NULL, SSH_IP_NO_ADDRESS);
+          else
+            ssh_socket_connect_final(c, NULL, SSH_IP_FAILURE);
+        }
       return;
     }
 
@@ -579,7 +753,10 @@ restart:
         {
           unsigned char *p;
 
-          ssh_buffer_append_space(c->socks_buf, &p, 1);
+          if (ssh_buffer_append_space(c->socks_buf, &p, 1) != SSH_BUFFER_OK)
+            {
+              goto socks_fail;
+            }
           len = ssh_stream_read(c->stream, p, 1);
           if (len == 0)
             { /* Premature EOF received. */
