@@ -11,8 +11,181 @@ Interface to sockets.
 */
 
 /*
- * $Id: sshtcp.h,v 1.4 2000/11/22 13:23:31 tmo Exp $
- * $Log: sshtcp.h,v $
+ * $Id: sshtcp.h,v 1.7 2001/06/14 21:47:38 tri Exp $
+ * $Log: sshtcp.h,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -66,6 +239,7 @@ typedef void (*SshTcpCallback)(SshIpError error,
 /* Parameters to the ssh_tcp_connect function. To get default values just
    memset the structure before giving it the the ssh_tcp_connect function. */
 typedef struct SshTcpConnectParamsRec {
+#ifdef SSHDIST_UTIL_URL
   char *socks_server_url;       /* Socks server url for going out through
                                    firewalls. URL specifies the SOCKS host,
                                    port, username, and socks network
@@ -73,6 +247,7 @@ typedef struct SshTcpConnectParamsRec {
                                    connection will be made without SOCKS. If
                                    port is not given in the url, the default
                                    SOCKS port (1080) will be used.  */
+#endif /* SSHDIST_UTIL_URL */
   SshUInt32 connection_attempts; /* Number of connection attempts before giving
                                     up. (Some systems appear to spuriously fail
                                     connections wihtout apparent reason, and
@@ -85,6 +260,13 @@ typedef struct SshTcpConnectParamsRec {
                                    the connect operation fails. If this is zero
                                    then we use the timeouts defined by the
                                    operating system. */ 
+  SshUInt32 protocol_mask;      /* Use given protocol(s) to make connection.
+                                   Default value is zero, which means that
+                                   any protocol can be used.  To limit
+                                   protocols, SSH_IP_TYPE_MASK_IP4 or
+                                   SSH_IP_TYPE_MASK_IP6 or bitwise or
+                                   arbitrary `bitwise or' of the 
+                                   SSH_IP_TYPE_MASK_* can be specified. */
 } *SshTcpConnectParams, SshTcpConnectParamsStruct;
 
 /* Opens a connection to the specified host, and calls the callback
