@@ -5,7 +5,7 @@ sshpacketimpl.c
 Author: Tatu Ylonen <ylo@ssh.fi>
         Tero Kivinen <kivinen@ssh.fi>
 
-Copyright (c) 1998 SSH Communications Security, Finland
+Copyright (c) 1998, 2001 SSH Communications Security, Finland
                    All rights reserved
 
 This module implements a SshStream that sends/receives packets.  This has
@@ -50,19 +50,19 @@ typedef struct SshPacketImplRec {
 
   /* This flag is TRUE if ssh_packet_impl_can_send has returned FALSE. */
   Boolean send_blocked;
-  
+
   /* SshBuffer for outgoing data. */
   SshBufferStruct outgoing;
 
   /* SshBuffer for formatting outgoing packet. */
   SshBufferStruct outgoing_packet;
-  
+
   /* Outgoing EOF. */
   Boolean outgoing_eof;
 
   /* Shortcircuit stream.  This is NULL if shortcircuiting not in effect. */
   SshStream shortcircuit_stream;
-  
+
   /* Callbacks to the actual protocol implementation code. */
 
   SshPacketReceiveProc received_packet;
@@ -160,7 +160,7 @@ int ssh_packet_impl_read(void *context, unsigned char *buf, size_t size)
 {
   SshPacketImpl up = (SshPacketImpl)context;
   size_t len;
-  
+
   /* Compute the number of bytes we can transmit. */
   len = ssh_buffer_len(&up->outgoing);
   if (len > size)
@@ -182,7 +182,7 @@ int ssh_packet_impl_read(void *context, unsigned char *buf, size_t size)
           return -1;
         }
     }
-  
+
   /* Move data to the caller's buffer. */
   memcpy(buf, ssh_buffer_ptr(&up->outgoing), len);
   ssh_buffer_consume(&up->outgoing, len);
@@ -190,11 +190,11 @@ int ssh_packet_impl_read(void *context, unsigned char *buf, size_t size)
   /* Wake up the sender if appropriate. */
   if (ssh_buffer_len(&up->outgoing) == 0)
     ssh_packet_impl_restart_send(up);
-  
+
   return len;
 }
 
-/* This function is called when the upper layer writes to the stream. 
+/* This function is called when the upper layer writes to the stream.
    Note that there are essentially two very different cases: an entire
    packet is received at once, and a partial packet is received.  */
 
@@ -221,7 +221,7 @@ normal:
       /* If already processing a partial packet, continue it now. */
       if (ssh_buffer_len(&up->incoming) > 0)
         goto partial;
-      
+
       /* If only partial packet available, do special proccessing. */
       if (size - offset < 4)
         goto partial;  /* Need partial packet processing. */
@@ -229,7 +229,7 @@ normal:
 
       if (size - offset < 4 + payload_len)
         goto partial;  /* Need partial packet processing. */
-      
+
       /* The entire packet is available; pass it to the callback. */
       if (up->received_packet)
         (*up->received_packet)((SshPacketType)buf[offset + 4],
@@ -254,7 +254,13 @@ partial:
       len = 4 - len;
       if (size - offset < len)
         len = size - offset;
-      ssh_buffer_append(&up->incoming, buf + offset, len);
+      if (ssh_buffer_append(&up->incoming, buf + offset, len)
+          != SSH_BUFFER_OK)
+        {
+          (*up->received_eof)(up->context);
+          return 0;
+        }
+
       offset += len;
     }
   if (ssh_buffer_len(&up->incoming) < 4)
@@ -268,7 +274,12 @@ partial:
   len = 4 + payload_len - ssh_buffer_len(&up->incoming);
   if (len > size - offset)
     len = size - offset;
-  ssh_buffer_append(&up->incoming, buf + offset, len);
+  if (ssh_buffer_append(&up->incoming, buf + offset, len)
+      != SSH_BUFFER_OK)
+    {
+      (*up->received_eof)(up->context);
+      return 0;
+    }
   offset += len;
 
   /* If some data still not available, return. */
@@ -278,10 +289,10 @@ partial:
   /* The entire packet is now in buffer. */
   ucp = ssh_buffer_ptr(&up->incoming);
   if (up->received_packet)
-    (*up->received_packet)((SshPacketType)ucp[4], ucp + 5, 
+    (*up->received_packet)((SshPacketType)ucp[4], ucp + 5,
                            payload_len - 1,
                            up->context);
-  
+
   /* Clear the incoming partial packet buffer and resume normal processing. */
   ssh_buffer_clear(&up->incoming);
   goto normal;
@@ -299,7 +310,7 @@ void ssh_packet_impl_output_eof(void *context)
       ssh_stream_output_eof(up->shortcircuit_stream);
       return;
     }
-  
+
   /* Mark that we have received EOF. */
   up->incoming_eof = TRUE;
 
@@ -350,7 +361,7 @@ void ssh_packet_impl_destroy(void *context)
   ssh_cancel_timeouts(ssh_packet_impl_signal_output_proc, (void *)up);
   ssh_cancel_timeouts(ssh_packet_impl_signal_input_proc, (void *)up);
   ssh_cancel_timeouts(ssh_packet_impl_signal_send_proc, (void *)up);
-  
+
   /* Uninitialize the buffers. */
   ssh_buffer_uninit(&up->outgoing);
   ssh_buffer_uninit(&up->outgoing_packet);
@@ -364,7 +375,7 @@ void ssh_packet_impl_destroy(void *context)
 
 /* Methods table for the stream. */
 
-const SshStreamMethodsStruct ssh_packet_impl_methods =
+SshStreamMethodsStruct ssh_packet_impl_methods =
 {
   ssh_packet_impl_read,
   ssh_packet_impl_write,
@@ -402,7 +413,7 @@ SshStream ssh_packet_impl_create(SshPacketReceiveProc received_packet,
 {
   SshPacketImpl up;
   SshStream stream;
-  
+
   /* Allocate and initialize the context. */
   up = ssh_xcalloc(1, sizeof(*up));
   ssh_buffer_init(&up->incoming);
@@ -414,7 +425,7 @@ SshStream ssh_packet_impl_create(SshPacketReceiveProc received_packet,
   up->up_write_blocked = FALSE;
   up->up_read_blocked = FALSE;
   up->send_blocked = TRUE; /* Cause a callback immediately. */
-  
+
   /* Save the callback functions. */
   up->received_packet = received_packet;
   up->received_eof = received_eof;
@@ -428,13 +439,13 @@ SshStream ssh_packet_impl_create(SshPacketReceiveProc received_packet,
   /* Cause the send callback to be called if non-NULL.  Note that it isn't
      called until from the bottom of the event loop. */
   ssh_packet_impl_restart_send(up);
-  
+
   /* Wrap it into a stream. */
   stream = ssh_stream_create(&ssh_packet_impl_methods, (void *)up);
 
   /* Enable receives. */
   ssh_packet_impl_can_receive(stream, TRUE);
-  
+
   return stream;
 }
 
@@ -517,8 +528,8 @@ Boolean ssh_packet_impl_can_send(SshStream up_stream)
     up->send_blocked = TRUE;
 
   return status;
-}    
-  
+}
+
 /* Sends a packet to the stream, encoding the payload of the packet
    as specified for ssh_encode_buffer_va. */
 
@@ -543,17 +554,22 @@ void ssh_packet_impl_send_encode_va(SshStream up_stream,
   if (ssh_buffer_len(&up->outgoing) + ssh_buffer_len(&up->outgoing_packet) >=
       BUFFER_MAX_SIZE)
     {
-      ssh_debug("ssh_packet_impl_send_encode_va: flow control problems; outgoing packet dropped.");
+      ssh_debug("ssh_packet_impl_send_encode_va: "
+                "flow control problems; outgoing packet dropped.");
       return;
     }
 
   /* Append the packet to the outgoing buffer. */
-  ssh_buffer_append(&up->outgoing, ssh_buffer_ptr(&up->outgoing_packet),
-                ssh_buffer_len(&up->outgoing_packet));
+  if (ssh_buffer_append(&up->outgoing,
+                        ssh_buffer_ptr(&up->outgoing_packet),
+                        ssh_buffer_len(&up->outgoing_packet)) != SSH_BUFFER_OK)
+    {
+      return;
+    }
 
   /* Restart reads by upper level. */
   ssh_packet_impl_restart_input(up);
-  
+
   /* Sanity check that we didn't exceed max buffer size. */
   if (ssh_buffer_len(&up->outgoing) > BUFFER_MAX_SIZE)
     ssh_debug("ssh_packet_impl_send: buffer max size exceeded: size %ld",
@@ -586,8 +602,8 @@ void ssh_packet_impl_send(SshStream up_stream, SshPacketType type,
                           const unsigned char *data, size_t len)
 {
   ssh_packet_impl_send_encode(up_stream, type,
-                           SSH_FORMAT_DATA, data, len,
-                           SSH_FORMAT_END);
+                              SSH_FORMAT_DATA, data, len,
+                              SSH_FORMAT_END);
 }
 
 /* INTERNAL FUNCTION - not to be called from applications.  This
@@ -612,7 +628,7 @@ void ssh_packet_impl_shortcircuit_now(SshStream up_stream,
 
   /* We currently require there to be no partial incoming packet. */
   SSH_ASSERT(ssh_buffer_len(&up->incoming) == 0);
-  
+
   /* If it is non-NULL, make it use application callbacks directly. */
   if (down_stream)
     ssh_stream_set_callback(down_stream, up->up_callback, up->up_context);
