@@ -4,7 +4,7 @@ sshpacketwrapper.c
 
 Author: Tatu Ylonen <ylo@ssh.fi>
 
-Copyright (c) 1998 SSH Communications Security, Finland
+Copyright (c) 1998, 2001 SSH Communications Security, Finland
                    All rights reserved
 
 This module implements a wrapper around SshStream for sending/receiving
@@ -24,7 +24,7 @@ callbacks, making it easy to do packet-based communications over a SshStream.
 
 #define SSH_DEBUG_MODULE "SshPacketWrapper"
 
-#define ALLOW_AFTER_BUFFER_FULL         (10000 + 5)
+#define ALLOW_AFTER_BUFFER_FULL         (65536 + 5)
 #define BUFFER_MAX_SIZE                 (65536*20)
 
 
@@ -130,6 +130,8 @@ Boolean ssh_packet_wrapper_output(SshPacketWrapper down)
       /* Consume written data. */
       ssh_buffer_consume(&down->outgoing, len);
 
+      SSH_DEBUG(7, ("Wrote %ld bytes of data.", len));
+
       /* We've done something, so return TRUE. */
       return_value = TRUE;
     }
@@ -206,9 +208,14 @@ Boolean ssh_packet_wrapper_input(SshPacketWrapper down)
          data_to_read - data_read bytes). */
       if (data_read < 4)
         {
-          /* Packet header not yet in buffer.  Read only header. */
+          /* Packet header not yet in buffer.  Read only header if we can
+             make space for it. If not, return. */
           data_to_read = 4;
-          ssh_buffer_append_space(&down->incoming, &ptr, 4 - data_read);
+          if (ssh_buffer_append_space(&down->incoming, &ptr, 4 - data_read)
+              != SSH_BUFFER_OK)
+            {
+              return return_value;
+            }
         }
       else
         {
@@ -221,7 +228,7 @@ Boolean ssh_packet_wrapper_input(SshPacketWrapper down)
                           "invalid packet received: len %ld "
                           "closing the offending input channel",
                           (long)data_to_read);
-              
+
               down->incoming_eof = TRUE;
               /* Pass the EOF to the application callback. */
               down->cannot_destroy = TRUE;
@@ -231,9 +238,16 @@ Boolean ssh_packet_wrapper_input(SshPacketWrapper down)
               ssh_packet_wrapper_destroy(down);
               return FALSE;
             }
-          SSH_ASSERT(data_to_read > data_read);
-          ssh_buffer_append_space(&down->incoming, &ptr,
-                                  data_to_read - data_read);
+          SSH_ASSERT(data_to_read >= data_read);
+          if (ssh_buffer_append_space(&down->incoming,
+                                      &ptr,
+                                      data_to_read - data_read)
+              != SSH_BUFFER_OK)
+            {
+              /* Assume upper protocol recovers/terminates if
+                 malformed packet is received. */
+              return FALSE;
+            }
         }
 
       /* Keep reading until entire packet read, or no more data available. */
@@ -283,7 +297,7 @@ Boolean ssh_packet_wrapper_input(SshPacketWrapper down)
                   ssh_warning("ssh_packet_wrapper_input: "
                               "invalid packet received: len %ld "
                               "closing the offending input channel.",
-                              (long)data_to_read); 
+                              (long)data_to_read);
                   down->incoming_eof = TRUE;
                   /* Pass the EOF to the application callback. */
                   down->cannot_destroy = TRUE;
@@ -295,8 +309,15 @@ Boolean ssh_packet_wrapper_input(SshPacketWrapper down)
                 }
 
               if (data_to_read > data_read)
-                ssh_buffer_append_space(&down->incoming, &ptr,
-                                    data_to_read - data_read);
+                {
+                  if (ssh_buffer_append_space(&down->incoming,
+                                              &ptr,
+                                              data_to_read - data_read)
+                      != SSH_BUFFER_OK)
+                    {
+                      return FALSE;
+                    }
+                }
             }
           else
             data_read += ret;
@@ -523,13 +544,15 @@ Boolean ssh_packet_wrapper_can_send(SshPacketWrapper down)
 /* Sends a packet to the underlying stream.  The payload will be encoded as
    specified for ssh_encode_buffer_va. */
 
-void ssh_packet_wrapper_send_encode_va(SshPacketWrapper down,
-                                       SshPacketType type,
-                                       va_list va)
+Boolean ssh_packet_wrapper_send_encode_va(SshPacketWrapper down,
+                                          SshPacketType type,
+                                          va_list va)
 {
   /* Format the packet in a separate buffer. */
   ssh_buffer_clear(&down->outgoing_packet);
-  ssh_packet_encode_va(&down->outgoing_packet, type, va);
+
+  if (ssh_packet_encode_va(&down->outgoing_packet, type, va) == 0)
+    return FALSE;
 
   /* Check that we don't overflow maximum buffer size.  Drop the packet
      if we would. */
@@ -538,30 +561,39 @@ void ssh_packet_wrapper_send_encode_va(SshPacketWrapper down,
     {
       ssh_debug("ssh_packet_wrapper_send_encode_va: flow control problems; "
                 "outgoing packet dropped.");
-      return;
+
+      return FALSE;
     }
 
   /* Append the packet to the outgoing buffer. */
-  ssh_buffer_append(&down->outgoing, ssh_buffer_ptr(&down->outgoing_packet),
-                    ssh_buffer_len(&down->outgoing_packet));
+  if (ssh_buffer_append(&down->outgoing,
+                        ssh_buffer_ptr(&down->outgoing_packet),
+                        ssh_buffer_len(&down->outgoing_packet))
+      != SSH_BUFFER_OK)
+    return FALSE;
 
   /* Reset the callback to ensure that our callback gets called. */
   ssh_stream_set_callback(down->stream, ssh_packet_wrapper_callback,
                           (void *)down);
+
+  return TRUE;
 }
 
 /* Sends a packet to the underlying stream.  The payload will be encoded as
    specified for ssh_encode_buffer. */
 
-void ssh_packet_wrapper_send_encode(SshPacketWrapper down,
-                                    SshPacketType type,
-                                    ...)
+Boolean ssh_packet_wrapper_send_encode(SshPacketWrapper down,
+                                       SshPacketType type,
+                                       ...)
 {
   va_list va;
+  Boolean status;
 
   va_start(va, type);
-  ssh_packet_wrapper_send_encode_va(down, type, va);
+  status = ssh_packet_wrapper_send_encode_va(down, type, va);
   va_end(va);
+
+  return status;
 }
 
 /* Sends a packet to the underlying stream.  The packet may actually
@@ -569,12 +601,12 @@ void ssh_packet_wrapper_send_encode(SshPacketWrapper down,
    sequence.  The application should use ssh_packet_wrapper_can_send and
    the `can_send' callback to implement flow control. */
 
-void ssh_packet_wrapper_send(SshPacketWrapper down, SshPacketType type,
+Boolean ssh_packet_wrapper_send(SshPacketWrapper down, SshPacketType type,
                          const unsigned char *data, size_t len)
 {
-  ssh_packet_wrapper_send_encode(down, type,
-                                 SSH_FORMAT_DATA, data, len,
-                                 SSH_FORMAT_END);
+  return ssh_packet_wrapper_send_encode(down, type,
+                                        SSH_FORMAT_DATA, data, len,
+                                        SSH_FORMAT_END);
 }
 
 /* Set callbacks (and context) to new values. */
@@ -615,7 +647,8 @@ void ssh_packet_shortcircuit(SshStream packet_stream,
 #if 0 /* the packet is still in wrapper->incoming when we call the callback */
   /* Sanity check: there must not be data in incoming buffer. */
   if (ssh_buffer_len(&wrapper->incoming) != 0)
-    ssh_fatal("ssh_packet_shortcircuit: incoming data in buffer; not set in packet callback");
+    ssh_fatal("ssh_packet_shortcircuit: incoming data in buffer; not set "
+              "in packet callback");
 #endif /* 0 */
 
   /* If there is no data to drain, shortcircuit output now. */
