@@ -2,10 +2,12 @@
 
 sshfilterstream.c
 
-Author: Tatu Ylonen <ylo@ssh.fi>
+Authors: Tatu Ylonen <ylo@ssh.fi>
+         Tomi Salo <ttsalo@ssh.com>
+         Sami Lehtinen <sjl@ssh.com>
 
-Copyright (c) 1997 SSH Communications Security, Finland
-                   All rights reserved
+Copyright (c) 1997,2000 SSH Communications Security, Finland
+                        All rights reserved
 
 An object for filtering a data stream bidirectionally.  The object
 allows callbacks to modify and filter data bidirectionally, and allow
@@ -19,7 +21,7 @@ them to disconnect the connection.
 
 #define SSH_DEBUG_MODULE "SshFilterStream"
 
-typedef struct SshStreamFilterRec
+struct SshStreamFilterRec
 {
   /* The underlying stream whose data we are filtering.  The "to" direction is
      data being written to this stream, and the "from" direction is data
@@ -61,7 +63,7 @@ typedef struct SshStreamFilterRec
 
   /* Set to TRUE if the upper level calls output_eof. */
   Boolean to_eof_received;
-  
+
   /* Set to TRUE if the stream has been disconnected. */
   Boolean disconnected;
 
@@ -90,15 +92,34 @@ typedef struct SshStreamFilterRec
 
   /* Set to TRUE if the stream has been scheduled for deletion. */
   Boolean deleted;
-} *SshStreamFilter;
+};
 
-/* Predeclaration. */
+typedef enum {
+  SSH_STREAM_FILTER_INBOUND,
+  SSH_STREAM_FILTER_OUTBOUND
+} SshStreamFilterDataDir;
+
+typedef struct SshStreamFilterRec *SshStreamFilter;
+
+/* Predeclarations. */
 static void actual_destroy(SshStreamFilter sf);
+void ssh_stream_filter_get_inbound(void *internal_context, SshBuffer *data_ret,
+                                   size_t *offset_ret, Boolean *eof_received_ret);
+void ssh_stream_filter_get_outbound(void *internal_context, SshBuffer *data_ret,
+                                    size_t *offset_ret, Boolean *eof_received_ret);
+void ssh_stream_filter_complete_inbound(void *internal_context, SshInt32 operation);
+void ssh_stream_filter_complete_outbound(void *internal_context, SshInt32 operation);
+void ssh_stream_filter_accept(SshStreamFilter sf, SshStreamFilterDataDir dir, int op);
+
+/* Called when the underlying stream wants to notify us. */
+void ssh_stream_filter_callback(SshStreamNotification op, void *context);
+
 
 /* Called from a generated event, this calls the callback registered for
    this stream with INPUT_AVAILABLE notification. */
 
-void ssh_stream_filter_read_upcall(void *context)
+void
+ssh_stream_filter_read_upcall(void *context)
 {
   SshStreamFilter sf = (SshStreamFilter)context;
 
@@ -109,7 +130,8 @@ void ssh_stream_filter_read_upcall(void *context)
 /* Called from a generated event, this calls the callback registered for
    this stream with CAN_OUTPUT notification. */
 
-void ssh_stream_filter_write_upcall(void *context)
+void
+ssh_stream_filter_write_upcall(void *context)
 {
   SshStreamFilter sf = (SshStreamFilter)context;
 
@@ -120,11 +142,13 @@ void ssh_stream_filter_write_upcall(void *context)
 /* Schedules a call to our stream callback with INPUT_AVAILABLE, but only
    if reads are blocked (i.e., our read function has returned -1). */
 
-void ssh_stream_filter_wake_up_reads(SshStreamFilter sf)
+void
+ssh_stream_filter_wake_up_reads(SshStreamFilter sf)
 {
   if (!sf->read_blocked && !sf->deleted)
     return;
 
+  SSH_DEBUG(5, ("Scheduling ssh_stream_filter_read_upcall timeout"));
   ssh_register_timeout(0L, 0L, ssh_stream_filter_read_upcall, (void *)sf);
   sf->read_blocked = FALSE;
 }
@@ -132,7 +156,8 @@ void ssh_stream_filter_wake_up_reads(SshStreamFilter sf)
 /* Schedules a call to our stream callback with CAN_OUTPUT, but only
    if writes are blocked (i.e., our write function has returned -1). */
 
-void ssh_stream_filter_wake_up_writes(SshStreamFilter sf)
+void
+ssh_stream_filter_wake_up_writes(SshStreamFilter sf)
 {
   if (!sf->write_blocked && !sf->deleted)
     return;
@@ -145,7 +170,8 @@ void ssh_stream_filter_wake_up_writes(SshStreamFilter sf)
    in both directions, the filter functions will not be called again,
    and no more data will be transmitted. */
 
-void ssh_stream_filter_disconnect_now(SshStreamFilter sf)
+void
+ssh_stream_filter_disconnect_now(SshStreamFilter sf)
 {
   sf->disconnected = TRUE;
   ssh_stream_output_eof(sf->stream);
@@ -155,7 +181,8 @@ void ssh_stream_filter_disconnect_now(SshStreamFilter sf)
 
 /* Tries to write data to the underlying stream. */
 
-void ssh_stream_filter_try_write(SshStreamFilter sf)
+void
+ssh_stream_filter_try_write(SshStreamFilter sf)
 {
   int len;
   
@@ -170,6 +197,7 @@ void ssh_stream_filter_try_write(SshStreamFilter sf)
                              sf->to_accepted_bytes);
       if (len == -1)
         return;
+      
       if (len == 0)
         return;
       sf->to_accepted_bytes -= len;
@@ -203,7 +231,8 @@ void ssh_stream_filter_try_write(SshStreamFilter sf)
    again.  This may shortcircuit immediately, or may arrange for
    shortcircuit to happen when all data has been transmitted. */
 
-void ssh_stream_filter_shortcircuit_now(SshStreamFilter sf)
+void
+ssh_stream_filter_shortcircuit_now(SshStreamFilter sf)
 {
   /* Mark that shortcircuit has been requested. */
   sf->shortcircuit_requested = TRUE;
@@ -232,13 +261,74 @@ void ssh_stream_filter_shortcircuit_now(SshStreamFilter sf)
     ssh_stream_filter_try_write(sf);
 }
 
+/* Calls the filter in the "to" direction. */
+
+void
+ssh_stream_filter_call_to_filter(SshStreamFilter sf)
+{
+  /* Call filter or accept everything */
+  SSH_ASSERT(sf->to_accepted_bytes <= ssh_buffer_len(&sf->to_buffer));
+  if (sf->to_filter)
+    (*sf->to_filter)(sf->context, ssh_stream_filter_get_inbound,
+                     ssh_stream_filter_complete_inbound,
+                     sf);
+  else
+    {
+      int len = ssh_buffer_len(&sf->to_buffer) -
+        sf->to_accepted_bytes;
+
+      if (len > 0)
+        ssh_stream_filter_accept(sf, SSH_STREAM_FILTER_INBOUND, len);
+      else
+        ssh_stream_filter_accept(sf, SSH_STREAM_FILTER_INBOUND,
+                                 SSH_FILTER_HOLD);
+    }
+}
+
+/* Calls the filter in the "from" direction */
+
+void
+ssh_stream_filter_call_from_filter(SshStreamFilter sf)
+{
+  /* Call the filter or accept everything straight away. */
+  SSH_ASSERT(sf->from_accepted_bytes <= ssh_buffer_len(&sf->from_buffer));
+  if (sf->from_filter)
+    (*sf->from_filter)(sf->context, ssh_stream_filter_get_outbound,
+                       ssh_stream_filter_complete_outbound,
+                       sf);
+  else
+    {
+      int len = ssh_buffer_len(&sf->from_buffer) -
+        sf->from_accepted_bytes;
+
+      if (len > 0)
+        ssh_stream_filter_accept(sf, SSH_STREAM_FILTER_OUTBOUND, len);
+      else
+        ssh_stream_filter_accept(sf, SSH_STREAM_FILTER_OUTBOUND,
+                                 SSH_FILTER_HOLD);
+    }
+}
+
+void
+ssh_stream_filter_call_to_filter_timeout(void *context)
+{
+  ssh_stream_filter_call_to_filter((SshStreamFilter)context);
+}
+
+void
+ssh_stream_filter_call_from_filter_timeout(void *context)
+{
+  ssh_stream_filter_call_from_filter((SshStreamFilter)context);
+}
+  
 /* Tries to read data from the underlying stream. */
 
-void ssh_stream_filter_try_read(SshStreamFilter sf)
+void
+ssh_stream_filter_try_read(SshStreamFilter sf)
 {
   char buf[1024];
-  int len, op;
-  Boolean eof_got = FALSE;
+  int len;
+  Boolean max_buffer_size_reached = FALSE;
   
   /* If disconnected or already shortcircuiting, just return. */
   if (sf->disconnected || sf->shortcircuit_requested)
@@ -247,7 +337,11 @@ void ssh_stream_filter_try_read(SshStreamFilter sf)
   /* If already too much data buffered, don't read any more. */
   SSH_ASSERT(ssh_buffer_len(&sf->from_buffer) <= sf->max_buffer_size);
   if (ssh_buffer_len(&sf->from_buffer) >= sf->max_buffer_size)
-    return;
+    {
+      SSH_DEBUG(4, ("Buffer has reached max_buf_size (%ld bytes)",
+                    sf->max_buffer_size));
+      return;
+    }
 
   for (;;)
     {
@@ -256,75 +350,39 @@ void ssh_stream_filter_try_read(SshStreamFilter sf)
       if (len > sizeof(buf))
         len = sizeof(buf);
       if (len == 0)
-        break;
+        {
+          max_buffer_size_reached = TRUE;
+          break;
+        }
 
       /* Try to read data from the stream. */
       len = ssh_stream_read(sf->stream, (unsigned char *) buf, len);
       if (len < 0)
         break;
+      
       if (len == 0)
         {
           sf->from_eof_received = TRUE;
-          eof_got = TRUE;
           break;
         }
       ssh_buffer_append(&sf->from_buffer, (unsigned char *) buf, len);
     }
 
-  /* Call the filter. */
-  SSH_ASSERT(sf->from_accepted_bytes <= ssh_buffer_len(&sf->from_buffer));
-  if (sf->from_filter)
-    op = (*sf->from_filter)(&sf->from_buffer, sf->from_accepted_bytes,
-                            sf->from_eof_received, sf->context);
-  else
-    op = ssh_buffer_len(&sf->from_buffer) - sf->from_accepted_bytes;
+  ssh_stream_filter_call_from_filter(sf);
 
-  /* First handle the case that we accepted a non-zero number of bytes. */
-  if (op > 0)
+  if (max_buffer_size_reached)
     {
-      /* We accepted some bytes. */
-      sf->from_accepted_bytes += op;
-
-      /* Wake up reads if they are blocked. */
-      ssh_stream_filter_wake_up_reads(sf);
-      return;
-    }
-
-  /* Process special return values. */
-  switch (op)
-    {
-    case SSH_FILTER_HOLD:
-      /* Gather more data and continue then.
-         Note: this is equivalent to accepting zero bytes. */
-      if (sf->from_accepted_bytes == 0 &&
-          ssh_buffer_len(&sf->from_buffer) == sf->max_buffer_size)
-        ssh_fatal("ssh_stream_filter_try_read: "
-                  "SSH_FILTER_HOLD returned, but buffer already full.");
-
-      /* Must wake up reads if we got EOF. */
-      if (eof_got)
-        {
-          ssh_stream_filter_wake_up_reads(sf);
-        }
-      break;
-      
-    case SSH_FILTER_DISCONNECT:
-      ssh_stream_filter_disconnect_now(sf);
-      break;
-      
-    case SSH_FILTER_SHORTCIRCUIT:
-      ssh_stream_filter_shortcircuit_now(sf);
-      break;
-      
-    default:
-      ssh_fatal("ssh_stream_filter_try_read: filter returned bad op %d",
-                op);
+      SSH_DEBUG(4, ("Waking up read from down, because ssh_stream_read "
+                    "didn't return -1"));
+      ssh_stream_set_callback(sf->stream,
+                              ssh_stream_filter_callback, (void *)sf);
     }
 }
 
 /* This is called when the underlying stream wants to notify us. */
 
-void ssh_stream_filter_callback(SshStreamNotification op, void *context)
+void
+ssh_stream_filter_callback(SshStreamNotification op, void *context)
 {
   SshStreamFilter sf = (SshStreamFilter)context;
 
@@ -349,58 +407,167 @@ void ssh_stream_filter_callback(SshStreamNotification op, void *context)
     }
 }
 
-/* Calls the filter in the "to" direction. */
+/* This function accepts (a part of) the data. It is called through the
+   completion functions. */
 
-void ssh_stream_filter_call_to_filter(SshStreamFilter sf)
+void
+ssh_stream_filter_accept(SshStreamFilter sf,
+                         SshStreamFilterDataDir dir,
+                         int op)
 {
-  int op;
-
-  /* Call filter. */
-  SSH_ASSERT(sf->to_accepted_bytes <= ssh_buffer_len(&sf->to_buffer));
-  if (sf->to_filter)
-    op = (*sf->to_filter)(&sf->to_buffer, sf->to_accepted_bytes,
-                          sf->to_eof_received, sf->context);
-  else
-    op = ssh_buffer_len(&sf->to_buffer) - sf->to_accepted_bytes;
-
-  /* First handle the case that we accepted a non-zero number of bytes. */
-  if (op > 0)
+  if (dir == SSH_STREAM_FILTER_INBOUND)
     {
-      /* We accepted some bytes. */
-      sf->to_accepted_bytes += op;
+      /* First handle the case that we accepted a non-zero number of bytes. */
+      if (op > 0)
+        {
+          /* We accepted some bytes. */
+          sf->to_accepted_bytes += op;          
 
-      /* Try writing to the stream. */
-      ssh_stream_filter_try_write(sf);
-      return;
+          if (op >= 0)
+            {
+              /* This is accept-op, so schedule the filter proc to be
+                 called again. */
+              ssh_register_timeout(0, 0,
+                                   ssh_stream_filter_call_to_filter_timeout,
+                                   sf);
+            }
+
+          /* Try writing to the stream. */
+          ssh_stream_filter_try_write(sf);
+          return;
+        }
+      
+      /* Process special return values. */
+      switch (op)
+        {
+        case SSH_FILTER_HOLD:
+          /* Gather more data and continue then.
+             Note: this not is equivalent to accepting zero bytes. */
+          if (sf->to_accepted_bytes == 0 &&
+              ssh_buffer_len(&sf->to_buffer) >= sf->max_buffer_size)
+            ssh_fatal("ssh_stream_filter_call_to_filter: SSH_FILTER_HOLD "
+                      "returned, but buffer already full.");
+          break;
+          
+        case SSH_FILTER_DISCONNECT:
+          ssh_stream_filter_disconnect_now(sf);
+          break;
+          
+        case SSH_FILTER_SHORTCIRCUIT:
+          ssh_stream_filter_shortcircuit_now(sf);
+          break;
+          
+        default:
+          ssh_fatal("ssh_stream_filter_call_to_filter: filter returned bad "
+                    "op %d", op);
+        }
     }
-
-  /* Process special return values. */
-  switch (op)
+  else
     {
-    case SSH_FILTER_HOLD:
-      /* Gather more data and continue then.
-         Note: this is equivalent to accepting zero bytes. */
-      if (sf->to_accepted_bytes == 0 &&
-          ssh_buffer_len(&sf->to_buffer) == sf->max_buffer_size)
-        ssh_fatal("ssh_stream_filter_call_to_filter: SSH_FILTER_HOLD returned, but buffer already full.");
-      break;
+      /* First handle the case that we accepted a non-zero number of bytes. */
+      if (op > 0)
+        {
+          /* We accepted some bytes. */
+          sf->from_accepted_bytes += op;
+        }
       
-    case SSH_FILTER_DISCONNECT:
-      ssh_stream_filter_disconnect_now(sf);
-      break;
+      if (op >= 0)
+        {
+          /* Wake up reads if they are blocked. */
+          ssh_stream_filter_wake_up_reads(sf);
+
+          /* This is accept-op, so schedule the filter proc to be
+             called again. */
+          ssh_register_timeout(0, 0,
+                               ssh_stream_filter_call_from_filter_timeout, sf);
+          return;
+        }
       
-    case SSH_FILTER_SHORTCIRCUIT:
-      ssh_stream_filter_shortcircuit_now(sf);
-      break;
-      
-    default:
-      ssh_fatal("ssh_stream_filter_call_to_filter: filter returned bad op %d",
-                op);
+      /* Process special return values. */
+      switch (op)
+        {
+        case SSH_FILTER_HOLD:
+          /* Gather more data and continue then.  Note: this is not
+             equivalent to accepting zero bytes. */
+          if (sf->from_accepted_bytes == 0 &&
+              ssh_buffer_len(&sf->from_buffer) >= sf->max_buffer_size)
+            ssh_fatal("ssh_stream_filter_try_read: SSH_FILTER_HOLD "
+                      "returned, but buffer already full.");
+          
+          /* Must wake up reads if we got EOF. */
+          if (sf->from_eof_received)
+            {
+              ssh_stream_filter_wake_up_reads(sf);
+            }
+          break;
+          
+        case SSH_FILTER_DISCONNECT:
+          ssh_stream_filter_disconnect_now(sf);
+          break;
+          
+        case SSH_FILTER_SHORTCIRCUIT:
+          ssh_stream_filter_shortcircuit_now(sf);
+          break;
+          
+        default:
+          ssh_fatal("ssh_stream_filter_try_read: filter returned bad op %d",
+                    op);
+        }
     }
 }
 
-static void actual_destroy(SshStreamFilter sf)
+/* Returns the buffer and offset */
+
+void
+ssh_stream_filter_get_inbound(void *internal_context,
+                              SshBuffer *data_ret,
+                              size_t *offset_ret,
+                              Boolean *eof_received_ret)
 {
+  SshStreamFilter sf = (SshStreamFilter)internal_context;
+  *data_ret = &(sf->to_buffer);
+  *offset_ret = sf->to_accepted_bytes;
+  *eof_received_ret = sf->to_eof_received;
+}
+
+void
+ssh_stream_filter_get_outbound(void *internal_context,
+                               SshBuffer *data_ret,
+                               size_t *offset_ret,
+                               Boolean *eof_received_ret)
+{
+  SshStreamFilter sf = (SshStreamFilter)internal_context;
+  *data_ret = &(sf->from_buffer);
+  *offset_ret = sf->from_accepted_bytes;
+  *eof_received_ret = sf->from_eof_received;
+}
+
+/* Completes the filtering of a piece of data */
+
+void
+ssh_stream_filter_complete_outbound(void *internal_context,
+                                    SshInt32 operation)
+{
+  ssh_stream_filter_accept((SshStreamFilter)internal_context,
+                           SSH_STREAM_FILTER_OUTBOUND, operation);
+}
+
+void
+ssh_stream_filter_complete_inbound(void *internal_context,
+                                   SshInt32 operation)
+{
+  ssh_stream_filter_accept((SshStreamFilter)internal_context,
+                           SSH_STREAM_FILTER_INBOUND, operation);
+}
+
+
+static void
+actual_destroy(SshStreamFilter sf)
+{
+  /* Call the user destroy function if supplied. */
+  if (sf->destroy)
+    (*sf->destroy)(sf->context);
+
   /* Destroy the underlying stream. */
   ssh_stream_destroy(sf->stream);
   sf->stream = NULL;
@@ -409,17 +576,20 @@ static void actual_destroy(SshStreamFilter sf)
   ssh_buffer_uninit(&sf->to_buffer);
   ssh_buffer_uninit(&sf->from_buffer);
 
-  /* Call the user destroy function if supplied. */
-  if (sf->destroy)
-    (*sf->destroy)(sf->context);
-
+  /* Cancel pending timeouts (the context is about to be destroyed) */
+  ssh_cancel_timeouts(ssh_stream_filter_call_to_filter_timeout, (void *)sf);
+  ssh_cancel_timeouts(ssh_stream_filter_call_from_filter_timeout, (void *)sf);
+  ssh_cancel_timeouts(ssh_stream_filter_read_upcall, (void *)sf);
+  ssh_cancel_timeouts(ssh_stream_filter_write_upcall, (void *)sf);
+  
   /* Free the context. */
   memset(sf, 'F', sizeof(*sf));
   ssh_xfree(sf);
 }
 
 /* Called when the filter stream is destroyed. */
-void ssh_stream_filter_destroy(void *context)
+void
+ssh_stream_filter_destroy(void *context)
 {
   SshStreamFilter sf = (SshStreamFilter)context;
 
@@ -451,7 +621,8 @@ void ssh_stream_filter_destroy(void *context)
 
 /* Called when the application reads from the filter stream. */
 
-int ssh_stream_filter_read(void *context, unsigned char *buf, size_t size)
+int
+ssh_stream_filter_read(void *context, unsigned char *buf, size_t size)
 {
   SshStreamFilter sf = (SshStreamFilter)context;
   size_t len;
@@ -500,8 +671,9 @@ int ssh_stream_filter_read(void *context, unsigned char *buf, size_t size)
 
 /* Processes a write from up. */
 
-int ssh_stream_filter_write(void *context, const unsigned char *buf,
-                            size_t size)
+int
+ssh_stream_filter_write(void *context, const unsigned char *buf,
+                        size_t size)
 {
   SshStreamFilter sf = (SshStreamFilter)context;
   size_t len;
@@ -551,7 +723,8 @@ int ssh_stream_filter_write(void *context, const unsigned char *buf,
 
 /* Processes EOF from up. */
 
-void ssh_stream_filter_output_eof(void *context)
+void
+ssh_stream_filter_output_eof(void *context)
 {
   SshStreamFilter sf = (SshStreamFilter)context;
 
@@ -579,8 +752,9 @@ void ssh_stream_filter_output_eof(void *context)
 
 /* Sets the stream callback. */
 
-void ssh_stream_filter_set_callback(void *context, SshStreamCallback callback,
-                                    void *callback_context)
+void
+ssh_stream_filter_set_callback(void *context, SshStreamCallback callback,
+                               void *callback_context)
 {
   SshStreamFilter sf = (SshStreamFilter)context;
 
@@ -602,7 +776,7 @@ void ssh_stream_filter_set_callback(void *context, SshStreamCallback callback,
 
 /* Stream methods table for filter streams. */
 
-const SshStreamMethodsStruct ssh_stream_filter_methods =
+SshStreamMethodsStruct ssh_stream_filter_methods =
 {
   ssh_stream_filter_read,
   ssh_stream_filter_write,
@@ -630,12 +804,13 @@ const SshStreamMethodsStruct ssh_stream_filter_methods =
      `context'            context argument to pass to the functions.
    The filter functions are not allowed to directly destroy the stream. */
 
-SshStream ssh_stream_filter_create(SshStream stream,
-                                   size_t max_buffer_size,
-                                   SshFilterProc to_stream_filter,
-                                   SshFilterProc from_stream_filter,
-                                   void (*destroy)(void *context),
-                                   void *context)
+SshStream
+ssh_stream_filter_create(SshStream stream,
+                         size_t max_buffer_size,
+                         SshFilterProc to_stream_filter,
+                         SshFilterProc from_stream_filter,
+                         void (*destroy)(void *context),
+                         void *context)
 {
   SshStreamFilter sf;
 
@@ -658,6 +833,5 @@ SshStream ssh_stream_filter_create(SshStream stream,
   ssh_stream_set_callback(stream, ssh_stream_filter_callback, (void *)sf);
 
   /* Wrap the context into a stream. */
-  return ssh_stream_create(&ssh_stream_filter_methods, (void *)sf);
+  return ssh_stream_create((SshStreamMethodsStruct *)&ssh_stream_filter_methods, (void *)sf);
 }
-
