@@ -12,8 +12,84 @@
 */
 
 /*
- * $Id: sshmiscstring.c,v 1.8 2000/09/14 23:39:52 sjl Exp $
- * $Log: sshmiscstring.c,v $
+ * $Id: sshmiscstring.c,v 1.16 2001/09/04 07:11:01 vsuontam Exp $
+ * $Log: sshmiscstring.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -112,7 +188,7 @@ char *ssh_format_number(char *buffer, size_t len, SshUInt64 number, int scale)
 
   if (number < scale)
     {
-      snprintf(buffer, len, "%d", (int) number);
+      ssh_snprintf(buffer, len, "%d", (int) number);
       return buffer;
     }
   while (number >= 1000)
@@ -123,13 +199,13 @@ char *ssh_format_number(char *buffer, size_t len, SshUInt64 number, int scale)
     }
   if (num < 995 * scale / 100)
     {
-      d = ((num * 100 / scale) + 5) / 10;
-      snprintf(buffer, len, "%d.%d%c", d / 10, d % 10, *scale_str);
+      d = (int)(((num * 100 / scale) + 5) / 10);
+      ssh_snprintf(buffer, len, "%d.%d%c", d / 10, d % 10, *scale_str);
     }
   else
     {
-      d = ((num * 10 / scale) + 5) / 10;
-      snprintf(buffer, len, "%d%c", d, *scale_str);
+      d = (int)(((num * 10 / scale) + 5) / 10);
+      ssh_snprintf(buffer, len, "%d%c", d, *scale_str);
     }
   return buffer; 
 }
@@ -143,21 +219,104 @@ char *ssh_format_number(char *buffer, size_t len, SshUInt64 number, int scale)
 char *ssh_format_time(char *buffer, size_t len, SshTime t)
 {
   if (t < 60 * 60 * 24)
-    snprintf(buffer, len, "%02d:%02d:%02d",
-             (int)(t / 60 / 60), (int)((t / 60) % 60),(int) (t % 60));
+    ssh_snprintf(buffer, len, "%02d:%02d:%02d",
+                 (int)(t / 60 / 60), (int)((t / 60) % 60),(int) (t % 60));
   else
     if (t < 60 * 60 * 24 * 100)
-      snprintf(buffer, len, "%d+%02d:%02d",
-               (int) (t / 24 / 60 / 60), (int)((t / 60 / 60) % 24),
-               (int)((t / 60) % 60));
+      ssh_snprintf(buffer, len, "%d+%02d:%02d",
+                   (int) (t / 24 / 60 / 60), (int)((t / 60 / 60) % 24),
+                   (int)((t / 60) % 60));
     else
       if (t / (60 * 60 * 24) < 100000)
-        snprintf(buffer, len, "%d+%02d",
-                 (int) (t / 24 / 60 / 60), (int)((t / 60 / 60) % 24));
+        ssh_snprintf(buffer, len, "%d+%02d",
+                     (int) (t / 24 / 60 / 60), (int)((t / 60 / 60) % 24));
       else
-        snprintf(buffer, len, "%d",
-                 (int)(t / 24 / 60 / 60));
+        ssh_snprintf(buffer, len, "%d",
+                     (int)(t / 24 / 60 / 60));
   return buffer;
 }
 
+
+
+/* Returns an item inside brackets. For "(foo(test(bar())))" returns
+   "foo(test(bar()))". */
+static char *
+ssh_mstr_get_next_item(const char *str)
+{
+  int c = 0;
+  char *ptr, *start, *r;
+
+  ptr = start = (char *)str;
+
+  do {
+    if (*ptr == '(') c++;
+    if (*ptr == ')' && --c == 0) break;
+  } while(*(++ptr) && c > 0);
+  
+  r = ssh_malloc(ptr - start);
+  if (r)
+    {
+      memcpy(r, start + 1, ptr - start - 1);
+      r[ptr - start - 1] = '\0';
+    }
+  return r;
+}
+
+
+/* Get the data from a string. Component identifies the data which to get. 
+   The source string is assumed to be in format 
+   "component1(component1_data), component2(component2_data)". 
+
+   Returns NULL, if the component is not found in the string and an empty 
+   string, if the component is empty. */
+char *ssh_str_get_component_data(const char *source, const char *component)
+{
+  const char *s = (char *)source, *c = (char *)component;
+  int count = 0, c_len;
+
+  if (source == NULL ||
+      component == NULL)
+    return NULL;
+
+  c_len = strlen(component);
+
+  while (*s)
+    {
+      if (*s++ == *c)
+        {
+          if (++count == c_len) 
+            {
+              if (*s == '(') break;
+              c = (char *)component;
+              count = 0;
+            }
+          else c++;
+        }
+      else if (count)
+        {
+          s--; 
+          c = (char *)component;
+          count = 0;
+        }
+    }
+
+  if (*s == '\0') return NULL;
+  /* Now s points to the opening bracket of the compoent. */
+  return ssh_mstr_get_next_item(s);
+}
+
+/* Free an array of strings. The strings of the array are freed individually
+ * using ssh_xfree and the list is freed at last. 
+ */
+void ssh_str_array_free(char **list, SshUInt32 num_items)
+{
+  for (; num_items > 0; num_items--)
+    ssh_xfree(list[num_items - 1]);
+ 
+  ssh_xfree(list);
+
+}
+
+
 /* eof (sshmiscstring.c) */
+
