@@ -14,7 +14,7 @@
 #ifndef SSHDIRECTORY_H
 #define SSHDIRECTORY_H
 
-#include <sshincludes.h>
+#include "sshincludes.h"
 
 /*
  * Types and definitions.
@@ -66,4 +66,8 @@ const char *ssh_directory_file_name(SshDirectoryHandle directory);
 
 
 
+
+
+
+
 #endif /* not SSHDIRECTORY_H */
