@@ -7,25 +7,119 @@
 /*
  *        Program: sshutil
  *        $Source: /ssh/CVS/src/lib/sshutil/sshsysutil/sshfileio.c,v $
- *        $Author: toka $
+ *        $Author: ttsalo $
  *
  *        Creation          : 11:03 Oct  9 1998 kivinen
  *        Last Modification : 04:32 Nov 13 1998 kivinen
- *        Last check in     : $Date: 2000/09/15 15:49:57 $
- *        Revision number   : $Revision: 1.4 $
+ *        Last check in     : $Date: 2001/10/25 14:58:23 $
+ *        Revision number   : $Revision: 1.10 $
  *        State             : $State: Exp $
  *        Version           : 1.107
+ *        
  *
  *        Description       : Read and write file from and to the disk
  *                            in various formats.
  *
- *        $Log: sshfileio.c,v $
+ *        $Log: sshfileio.c,v $ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
  *        $EndLog$
  */
 
 #include "sshincludes.h"
 #include "sshdebug.h"
+
+#ifdef SSHDIST_UTIL_BASE64
 #include "sshbase64.h"
+#endif /* SSHDIST_UTIL_BASE64 */
 
 #define SSH_BUFFER_LEN 1024
 
@@ -42,7 +136,10 @@ Boolean ssh_read_file(const char *file_name,
 
   /* Read the file */
   if (file_name == NULL || strcmp(file_name, "-") == 0)
-    fp = stdin;
+    {
+      fp = stdin;
+      file_name = NULL;
+    }
   else
     fp = fopen(file_name, "rb");
 
@@ -75,6 +172,7 @@ Boolean ssh_read_file(const char *file_name,
   return TRUE;
 }
 
+#ifdef SSHDIST_UTIL_BASE64
 /* Read base 64 encoded file from the disk. Return mallocated buffer and the
    size of the buffer. If the reading of file failes return FALSE. If the file
    name is NULL or "-" then read from the stdin. */
@@ -82,67 +180,26 @@ Boolean ssh_read_file_base64(const char *file_name, unsigned char **buf,
                              size_t *buf_len)
 {
   unsigned char *tmp, *cp;
-  size_t len, i, end, start, header, inside, skip;
+  size_t len, start, end;
 
-  if (!ssh_read_file(file_name, buf, buf_len))
+  if (!ssh_read_file(file_name, &tmp, &len))
     return FALSE;
 
-  tmp = *buf;
-  len = *buf_len;
-
-  /* Remove all before and after headers. */
-  for (i = 0, skip = 0, end = 0, start = 0, header = 0, inside = 0;
-       i < len; i++)
-    {
-      switch (tmp[i])
-        {
-        case '-':
-          if (skip)
-            break;
-
-          if (inside)
-            end = i;
-          header = 1;
-          inside ^= 1;
-          skip = 1;
-          break;
-        case '\n':
-        case '\r':
-          if (header)
-            {
-              header = 0;
-              if (inside)
-                start = i + 1;
-            }
-          skip = 0;
-          break;
-        default:
-          break;
-        }
-    }
-  if (end == 0 && start == 0)
-    {
-      start = 0;
-      end = len;
-    }
-  if (end == start)
+  if (ssh_base64_remove_headers(tmp, len, &start, &end) == FALSE)
     {
       ssh_xfree(tmp);
       return FALSE;
     }
-
-  if (end <= start)
-    {
-      ssh_xfree(tmp);
-      return FALSE;
-    }
-
+  
   cp = ssh_base64_remove_whitespace(tmp + start, end - start);
-  *buf = ssh_base64_to_buf(cp, buf_len);
+  *buf = ssh_base64_to_buf(cp, &len);
+  *buf_len = len;
+
   ssh_xfree(cp);
   ssh_xfree(tmp);
   return TRUE;
 }
+#endif /* SSHDIST_UTIL_BASE64 */
 
 /* Read hexl encoded file from the disk. Return mallocated buffer and the size
    of the buffer. If the reading of file failes return FALSE. If the file name
@@ -154,12 +211,9 @@ Boolean ssh_read_file_hexl(const char *file_name, unsigned char **buf,
   size_t len, i = 0;
   int state, l = 0;
 
-  if (!ssh_read_file(file_name, buf, buf_len))
+  if (!ssh_read_file(file_name, &tmp, &len))
     return FALSE;
 
-  tmp = *buf;
-  len = *buf_len;
-
   *buf = ssh_xmalloc(len + 1);
   *buf_len = 0;
 
@@ -221,8 +275,10 @@ Boolean ssh_read_gen_file(const char *file_name,
     return ssh_read_file(file_name, buf, buf_len);
   if (file_name[1] == 'b')
     return ssh_read_file(file_name + 3, buf, buf_len);
+#ifdef SSHDIST_UTIL_BASE64
   if (file_name[1] == 'p')
     return ssh_read_file_base64(file_name + 3, buf, buf_len);
+#endif /* SSHDIST_UTIL_BASE64 */
   if (file_name[1] == 'h')
     return ssh_read_file_hexl(file_name + 3, buf, buf_len);
   ssh_warning("Unknown file format given to ssh_read_gen_file");
@@ -261,6 +317,7 @@ Boolean ssh_write_file(const char *file_name,
   return TRUE;
 }
 
+#ifdef SSHDIST_UTIL_BASE64
 /* Write base 64 encoded file to the disk. If the write fails retuns FALSE. If
    the file name is NULL or "-" then write to the stdout */
 Boolean ssh_write_file_base64(const char *file_name,
@@ -270,7 +327,7 @@ Boolean ssh_write_file_base64(const char *file_name,
                               size_t buf_len)
 {
   FILE *fp;
-  char *tmp;
+  char *tmp = NULL;
   size_t len, i;
 
   tmp = (char *) ssh_buf_to_base64(buf, buf_len);
@@ -279,12 +336,18 @@ Boolean ssh_write_file_base64(const char *file_name,
 
   /* Write the file */
   if (file_name == NULL || strcmp(file_name, "-") == 0)
-    fp = stdout;
+    {
+      fp = stdout;
+      file_name = NULL;
+    }
   else
     fp = fopen(file_name, "w");
 
   if (fp == NULL)
-    return FALSE;
+    {
+      ssh_xfree(tmp);
+      return FALSE;
+    }
 
   if (begin)
     if (fprintf(fp, "%s\n", begin) < 0)
@@ -304,12 +367,15 @@ Boolean ssh_write_file_base64(const char *file_name,
       goto error;
   if (file_name)
     fclose(fp);
+  ssh_xfree(tmp);
   return TRUE;
 error:
   if (file_name)
     fclose(fp);
+  ssh_xfree(tmp);
   return FALSE;
 }
+#endif /* SSHDIST_UTIL_BASE64 */
 
 /* Write hexl encoded file to the disk. If the write fails retuns FALSE. If the
    file name is NULL or "-" then write to the stdout */
@@ -322,7 +388,10 @@ Boolean ssh_write_file_hexl(const char *file_name,
 
   /* Write the file */
   if (file_name == NULL || strcmp(file_name, "-") == 0)
-    fp = stdout;
+    {
+      fp = stdout;
+      file_name = NULL;
+    }
   else
     fp = fopen(file_name, "w");
 
@@ -405,8 +474,10 @@ Boolean ssh_write_gen_file(const char *file_name,
     return ssh_write_file(file_name, buf, buf_len);
   if (file_name[1] == 'b')
     return ssh_write_file(file_name + 3, buf, buf_len);
+#ifdef SSHDIST_UTIL_BASE64
   if (file_name[1] == 'p')
     return ssh_write_file_base64(file_name + 3, begin, end, buf, buf_len);
+#endif /* SSHDIST_UTIL_BASE64 */
   if (file_name[1] == 'h')
     return ssh_write_file_hexl(file_name + 3, buf, buf_len);
   ssh_warning("Unknown file format given to ssh_read_gen_file");
