@@ -15,8 +15,91 @@ maximum core dump size.
 */
 
 /*
- * $Id: sshsignals.c,v 1.3 2000/09/14 23:39:55 sjl Exp $
- * $Log: sshsignals.c,v $
+ * $Id: sshsignals.c,v 1.7 2001/05/29 14:18:56 sjl Exp $
+ * $Log: sshsignals.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -46,19 +129,9 @@ static Boolean eloop_used_in_signals = FALSE;
 
 static RETSIGTYPE ssh_signals_fatal_signal_handler(int sig)
 {
-  ssh_fatal("Received signal %d.", sig);
-  exit(255);
-}
-
-#if 0
-static void ssh_signals_eloop_fatal_signal_handler(int sig, void *ctx);
-static void ssh_signals_eloop_fatal_signal_handler(int sig, void *ctx)
-{
-  /* XXX logging */
   fprintf(stderr, "\nReceived signal %d. (no core)\n", sig);
   exit(255);
 }
-#endif
 
 static Boolean ssh_sig_terminal(int sig)
 {
@@ -94,11 +167,14 @@ static Boolean ssh_sig_terminal(int sig)
 #ifdef SIGINFO
       case SIGINFO:
 #endif
-#if defined(SIGFREEZE)
+#ifdef SIGFREEZE
       case SIGFREEZE:
 #endif
-#if defined(SIGTHAW)
+#ifdef SIGTHAW
       case SIGTHAW:
+#endif
+#ifdef SIGPROF
+    case SIGPROF:
 #endif
         return FALSE;
       
@@ -137,7 +213,7 @@ void ssh_signals_prevent_core(Boolean use_eloop, void *ctx)
                                 ctx);
           else
 #endif
-            signal(sig, ssh_signals_fatal_signal_handler);
+            (void)signal(sig, ssh_signals_fatal_signal_handler);
         }
     }
 
