@@ -4,7 +4,7 @@ t-encode.c
 
 Author: Tatu Ylonen <ylo@ssh.fi>
 
-Copyright (c) 1997 SSH Communications Security, Finland
+Copyright (c) 1997, 2001 SSH Communications Security, Finland
                    All rights reserved
 
 Tests for the ssh_encode/decode functions.
@@ -18,13 +18,13 @@ Tests for the ssh_encode/decode functions.
 #include "sshincludes.h"
 #include "sshencode.h"
 
-#define SSH_DEBUG_MODULE "Main"
+#define SSH_DEBUG_MODULE "t-encode"
 
 SshBuffer buffer;
 
-/* This encodes data into a buffer using the given format, and compares the
-   result against the expected value.  This also checks that data at the
-   beginning of the buffer is not altered. */
+/* This encodes data into a buffer using the given format, and
+   compares the result against the expected value.  This also checks
+   that data at the beginning of the buffer is not altered. */
 
 void encode_case(const char *name, const char *expect,
                  size_t expect_len, ...)
@@ -32,7 +32,7 @@ void encode_case(const char *name, const char *expect,
   size_t len, i, bytes;
   unsigned char ch, *cp;
   va_list va;
-  
+
   ssh_buffer_clear(buffer);
   len = rand() % 100;
   ch = rand();
@@ -41,12 +41,14 @@ void encode_case(const char *name, const char *expect,
 
   va_start(va, expect_len);
   bytes = ssh_encode_buffer_va(buffer, va);
+
   if (bytes != expect_len || ssh_buffer_len(buffer) != len + expect_len)
     ssh_fatal("test_encode: %s: unexpected length %d vs. %d",
               name, bytes, len + expect_len);
   cp = ssh_buffer_ptr(buffer);
   if (memcmp(expect, cp + len, expect_len) != 0)
     ssh_fatal("test_encode: %s: mismatch", name);
+
   for (i = 0; i < len; i++)
     if (cp[i] != ch)
       ssh_fatal("test_encode: %s: beginning corrupted", name);
@@ -92,16 +94,16 @@ void decode_case_int(SshEncodingFormat fmt, unsigned int value)
 
 void decode_case_bool(SshEncodingFormat fmt, Boolean value)
 {
-  Boolean bool;
+  Boolean boo;
   size_t bytes;
 
   bytes = ssh_buffer_len(buffer);
   if (bytes != ssh_decode_array(ssh_buffer_ptr(buffer), ssh_buffer_len(buffer),
                                       fmt, NULL, SSH_FORMAT_END))
     ssh_fatal("decode_case_bool: NULL decode bad len");
-  if (bytes != ssh_decode_buffer(buffer, fmt, &bool, SSH_FORMAT_END))
+  if (bytes != ssh_decode_buffer(buffer, fmt, &boo, SSH_FORMAT_END))
     ssh_fatal("decode_case_bool: bad returned len");
-  if (bool != value)
+  if (boo != value)
     ssh_fatal("decode_case_bool: bad value");
   if (ssh_buffer_len(buffer) > 0)
     ssh_fatal("decode_case_bool: data left");
@@ -145,23 +147,6 @@ void decode_case_data(SshEncodingFormat fmt, const char *value, size_t valuelen)
 
 void test_encode(void)
 {
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
   encode_case("uint32_str 0", "\0\0\0\0", 4,
               SSH_FORMAT_UINT32_STR, NULL, 0, SSH_FORMAT_END);
   decode_case_str(SSH_FORMAT_UINT32_STR, "", 0);
@@ -171,38 +156,6 @@ void test_encode(void)
   encode_case("uint32_str 5", "\0\0\0\5ABCDE", 9,
               SSH_FORMAT_UINT32_STR, "ABCDEFGHIJK", 5, SSH_FORMAT_END);
   decode_case_str(SSH_FORMAT_UINT32_STR, "ABCDE", 5);
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
   encode_case("uint32 0x12345678", "\22\64\126\170", 4,
               SSH_FORMAT_UINT32, (SshUInt32) 0x12345678, SSH_FORMAT_END);
   decode_case_int(SSH_FORMAT_UINT32, (SshUInt32) 0x12345678);
@@ -232,24 +185,12 @@ void test_encode(void)
 
 void test_empty_decode(void)
 {
-
-
-
-
-
-
   if (ssh_decode_array((const unsigned char *) "", 0,
                        SSH_FORMAT_UINT32_STR, NULL, NULL,
                        SSH_FORMAT_END) != 0)
     ssh_fatal("test_empty_decode failed");
-
-
-
-
-
-
   if (ssh_decode_array((const unsigned char *) "", 0,
-                       SSH_FORMAT_UINT32, NULL, 
+                       SSH_FORMAT_UINT32, NULL,
                        SSH_FORMAT_END) != 0)
     ssh_fatal("test_empty_decode failed");
   if (ssh_decode_array((const unsigned char *) "", 0,
@@ -264,10 +205,6 @@ void test_empty_decode(void)
                        SSH_FORMAT_DATA, NULL, NULL,
                        SSH_FORMAT_END) != 0)
     ssh_fatal("test_empty_decode failed");
-  if (ssh_decode_array((const unsigned char *)"", 0,
-                       SSH_FORMAT_MP_INT, NULL,
-                       SSH_FORMAT_END) != 0)
-    ssh_fatal("test_empty_decode failed");
 }
 
 void test_random_decode(void)
@@ -277,17 +214,9 @@ void test_random_decode(void)
 
   for (i = 0; i < sizeof(buf); i++)
     buf[i] = rand();
-  
-
-
-
 
   ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_UINT32_STR, NULL, NULL,
                          SSH_FORMAT_END);
-
-
-
-
   ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_UINT32, NULL,
                          SSH_FORMAT_END);
   ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_CHAR, NULL,
@@ -296,9 +225,7 @@ void test_random_decode(void)
                          SSH_FORMAT_END);
   ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_DATA, NULL, NULL,
                          SSH_FORMAT_END);
-  ssh_decode_array(buf, sizeof(buf), SSH_FORMAT_MP_INT, NULL,
-                         SSH_FORMAT_END);
-} 
+}
 
 void test_functions_parse_compound(SshBuffer buffer, ...)
 {
@@ -314,86 +241,65 @@ void test_functions(int foo, ...)
 {
   va_list va;
   unsigned char *cp;
-  
+
   ssh_buffer_clear(buffer);
 
-
-
-
-
-
-
-
   ssh_buffer_clear(buffer);
   va_start(va, foo);
   if (ssh_encode_buffer_va(buffer, va) != 2)
     ssh_fatal("test_functions: ssh_encode_buffer_va error");
+  va_end(va);
   if (memcmp(ssh_buffer_ptr(buffer), "\100\103", 2) != 0)
     ssh_fatal("test_functions: ssh_encode_buffer_va data error");
 
-
-
-
-
-
-
-
-
-
-
-
-
-
+  va_start(va, foo);
   if (ssh_encode_array_alloc_va(NULL, va) != 2)
     ssh_fatal("test_function: ssh_encode_array_alloc_va NULL error");
+  va_end(va);
+
+  va_start(va, foo);
   if (ssh_encode_array_alloc_va(&cp, va) != 2)
     ssh_fatal("test_functions: ssh_encode_array_alloc_va error");
+  va_end(va);
   if (memcmp(cp, "\100\103", 2) != 0)
     ssh_fatal("test_functions: ssh_encode_array_alloc_va data error");
   ssh_xfree(cp);
+}
 
+static void test1(void)
+{
+  unsigned char *t1, *t2;
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+  buffer = ssh_buffer_allocate();
+  ssh_encode_buffer(buffer,
+    SSH_FORMAT_CHAR, NULL,
+    SSH_FORMAT_DATA,       "sikapantteri", strlen("sikapantteri"),
+    SSH_FORMAT_UINT32_STR, "sikapantteri", strlen("sikapantteri"),
+    SSH_FORMAT_UINT32_STR, "sikapantteri", strlen("sikapantteri"),
+    SSH_FORMAT_END);
+  ssh_decode_buffer(buffer,
+                    SSH_FORMAT_CHAR, NULL,
+                    SSH_FORMAT_DATA, NULL, strlen("sikapantteri"),
+                    SSH_FORMAT_UINT32_STR, &t1, NULL,
+                    SSH_FORMAT_UINT32_STR, &t2, NULL,
+                    SSH_FORMAT_END);
+  ssh_xfree(t2);
+  ssh_xfree(t1);
 }
 
 int main()
 {
   int pass;
 
+  test1();
   for (pass = 0; pass < 1000; pass++)
     {
       buffer = ssh_buffer_allocate();
       test_encode();
       test_empty_decode();
       test_random_decode();
-
-
-
       ssh_buffer_free(buffer);
     }
 
   return 0;
 }
-
