@@ -154,16 +154,16 @@ int main(int argc, char **argv)
 
   BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0, 0x0000,
           "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00");
-  BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0, 0xffff,
+  BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0, 0xffffU,
           "\x00\xff\xff\x00\x00\x00\x00\x00\x00\x00");
-  BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0, 0x1234,
+  BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0, 0x1234U,
           "\x00\x34\x12\x00\x00\x00\x00\x00\x00\x00");
 
   BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0xff, 0x0000,
           "\xff\x00\x00\xff\xff\xff\xff\xff\xff\xff");
-  BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0xff, 0xffff,
+  BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0xff, 0xffffU,
           "\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff");
-  BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0xff, 0x1234,
+  BUFTEST(SSH_PUT_16BIT_LSB_FIRST, 0xff, 0x1234U,
           "\xff\x34\x12\xff\xff\xff\xff\xff\xff\xff");
 
   BUFTEST(SSH_PUT_24BIT, 0, 0x00000000U,
