@@ -7,19 +7,48 @@
 /*
  *        Program: util
  *        $Source: /ssh/CVS/src/lib/sshutil/tests/t-miscstring.c,v $
- *        $Author: kivinen $
+ *        $Author: vsuontam $
  *
  *        Creation          : 15:40 Dec 30 1999 kivinen
  *        Last Modification : 02:33 May 11 2000 kivinen
- *        Last check in     : $Date: 2000/05/10 23:42:57 $
- *        Revision number   : $Revision: 1.6 $
+ *        Last check in     : $Date: 2001/08/29 12:38:17 $
+ *        Revision number   : $Revision: 1.8 $
  *        State             : $State: Exp $
  *        Version           : 1.192
+ *        
  *
  *        Description       : Test program for misc string functions.
  *
  *
- *        $Log: t-miscstring.c,v $
+ *        $Log: t-miscstring.c,v $ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
  *        $EndLog$
  */
 
@@ -610,5 +639,37 @@ int main(int argc, char **argv)
              "") != 0)
     ssh_fatal("ssh_format_time failed: %s", buffer);
 
+  p = ssh_str_get_component_data("component(test1)", "component");
+  if (p == NULL || 
+      strcmp(p, "test1") != 0)
+    ssh_fatal("ssh_str_get_component_data failed");
+  ssh_xfree(p);
+
+  p = ssh_str_get_component_data("component()", "component");
+  if (p == NULL || 
+      strcmp(p, "") != 0)
+    ssh_fatal("ssh_str_get_component_data failed");
+  ssh_xfree(p);
+
+  p = ssh_str_get_component_data("componentboo(test1)", "component");
+  if (p != NULL)
+    ssh_fatal("ssh_str_get_component_data failed");
+
+  p = ssh_str_get_component_data("component(data())", "component");
+  if (p == NULL || 
+      strcmp(p, "data()") != 0)
+    ssh_fatal("ssh_str_get_component_data failed");
+  ssh_xfree(p);
+
+  p = ssh_str_get_component_data("component(data(data()))", "component");
+  if (p == NULL || 
+      strcmp(p, "data(data())") != 0)
+    ssh_fatal("ssh_str_get_component_data failed");
+  ssh_xfree(p);
+
+
+
+
+
   exit(0);
 }
