@@ -3,28 +3,62 @@
  * Author: Tero Kivinen <kivinen@iki.fi>
  *                               Jukka Aittokallio <jai@ssh.fi>
  *
- * Copyright (c) 2000 SSH Communications Security Oy <info@ssh.fi>
+ * Copyright (c) 2000-2001 SSH Communications Security Oy <info@ssh.fi>
  */
 /*
  *        Program: Util Lib
  *        $Source: /ssh/CVS/src/lib/sshutil/tests/t-threads.c,v $
- *        $Author: kivinen $
+ *        $Author: ksr $
  *
  *        Creation          : 15:01 Feb 23 2000 kivinen
  *        Last Modification : 22:11 Jun 14 2000 kivinen
- *        Last check in     : $Date: 2000/06/14 19:12:21 $
- *        Revision number   : $Revision: 1.6 $
+ *        Last check in     : $Date: 2001/10/22 16:12:07 $
+ *        Revision number   : $Revision: 1.10 $
  *        State             : $State: Exp $
  *        Version           : 1.210
+ *        
  *
  *        Description       : Multithread timeouts support test program
  *
  *
- *        $Log: t-threads.c,v $
+ *        $Log: t-threads.c,v $ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
+ *        
  *        $EndLog$
  */
 
-#include "sshconf.h"
+#include "sshincludes.h"
 
 #ifdef _XOPEN_SOURCE_EXTENDED
 #undef _XOPEN_SOURCE_EXTENDED
@@ -78,7 +112,7 @@ SshMutex debug_mutex;
 
 struct SshThreadRec {
   HANDLE handle;
-  DWORD thread_id;  
+  DWORD thread_id;
   SshForkedFunc func;
   void *func_context;
 };
@@ -113,7 +147,7 @@ void wait_thread(SshThread thread)
 {
   WaitForSingleObject(thread->handle, INFINITE);
   CloseHandle(thread->handle);
-  ssh_xfree(thread);      
+  ssh_xfree(thread);
 }
 
 #endif
@@ -127,7 +161,7 @@ void wait_thread(SshThread thread)
    printf x, \
    ssh_mutex_unlock(debug_mutex), 0)
 #else /* DEBUG_HEAVY */
-#define D(x) 
+#define D(x)
 #endif /* DEBUG_HEAVY */
 
 struct SshThreadRec {
@@ -215,7 +249,7 @@ void *func1(void *context)
   ssh_mutex_unlock(mutex1);
   D(("In thread 1, Unlocking mutex 3\n"));
   ssh_mutex_unlock(mutex3);
-  
+
   D(("In thread 1, All done, inserting timeout, thread = %p\n", thread1));
   ssh_register_threaded_timeout(0, 0, call_wait_thread, thread1);
   D(("In thread 1, Exiting\n"));
@@ -291,7 +325,7 @@ void start_program(void *context)
   ssh_mutex_unlock(mutex2_start);
 }
 
-char *program; 
+char *program;
 
 void usage(void)
 {
@@ -367,7 +401,7 @@ void main(int argc, char **argv)
   D(("Staring event loop\n"));
   ssh_event_loop_run();
   D(("Event loop run returned\n"));
-  
+
   D(("Destroying mutex 1, 2, and 3\n"));
   ssh_mutex_destroy(mutex1);
   ssh_mutex_destroy(mutex1_start);
@@ -383,7 +417,7 @@ void main(int argc, char **argv)
 }
 
 #else  /* HAVE_THREADS */
-int main(int argc, char *argv)
+int main(int argc, char **argv)
 {
   printf("No thread support included, testing ignored\n");
   return 0;
