@@ -12,10 +12,12 @@ Created: Fri Apr 23 07:58:12 1999 tri
 */
 
 #include "sshincludes.h"
+#include "sshrand.h"
 
 void check_time(SshTime tv, int year, int month, int day)
 {
-  struct SshCalendarTimeRec ct[1];
+  struct SshCalendarTimeRec ct[1], mt[1];
+  SshTime cv;
 
   ssh_calendar_time(tv, ct, FALSE);
   if ((ct->year != year) || 
@@ -28,6 +30,24 @@ void check_time(SshTime tv, int year, int month, int day)
               year, month, day);
       exit(1);
     }
+  *mt = *ct;
+  if (!ssh_make_time(mt, &cv, FALSE))
+    ssh_fatal("ssh_make_time fails");
+
+  if (cv != tv)
+    ssh_fatal("ssh_make_time returns different time than given "
+              "to ssh_calendar_time");
+  if (ct->year != mt->year ||
+      ct->month != mt->month ||
+      ct->monthday != mt->monthday ||
+      ct->hour != mt->hour ||
+      ct->minute != mt->minute ||
+      ct->second != mt->second ||
+      ct->weekday != mt->weekday ||
+      ct->yearday != mt->yearday ||
+      ct->dst != mt->dst ||
+      ct->utc_offset != mt->utc_offset)
+    ssh_fatal("Time values are different in struct");
   return;
 }
 
@@ -35,8 +55,50 @@ int main()
 {
   int i, j, k, d, y, p;
   char *a, *b, *first, *last, *rfirst, *rlast;
-  SshTime t;
-  struct SshCalendarTimeRec ct[1];
+  SshTime t, t2;
+  struct SshCalendarTimeRec ct[1], mt[1];
+
+  ssh_debug_set_level_string("*=6");
+
+  ssh_rand_seed(1234);
+  for(i = 0; i < 1000; i++)
+    {
+      ct->year = 1900 + ssh_rand() % 200;
+      ct->month = ssh_rand() % 12;
+      ct->monthday = 1 + ssh_rand() % 28;
+      ct->hour = ssh_rand() % 24;
+      ct->minute = ssh_rand() % 60;
+      ct->second = ssh_rand() % 60;
+      ct->utc_offset = ((ssh_rand() % 96) - 48) * 900;
+      ct->dst = (ssh_rand() % 3) - 1;
+      if (!ssh_make_time(ct, &t, FALSE))
+        ssh_fatal("ssh_make_time failed");
+      ssh_calendar_time(t, mt, FALSE);
+      if (ct->year != mt->year ||
+          ct->month != mt->month ||
+          ct->monthday != mt->monthday ||
+          ct->hour != mt->hour ||
+          ct->minute != mt->minute ||
+          ct->second != mt->second ||
+          ct->weekday != mt->weekday ||
+          ct->yearday != mt->yearday)
+        ssh_fatal("Time values are different in struct");
+      if (!ssh_make_time(mt, &t2, TRUE))
+        ssh_fatal("ssh_make_time failed");
+      if (ct->year != mt->year ||
+          ct->month != mt->month ||
+          ct->monthday != mt->monthday ||
+          ct->hour != mt->hour ||
+          ct->minute != mt->minute ||
+          ct->second != mt->second ||
+          ct->weekday != mt->weekday ||
+          ct->yearday != mt->yearday)
+        ssh_fatal("Time values are different in struct with local time");
+      if (mt->dst)
+        t2 -= 3600;
+      if (t2 - t != mt->utc_offset)
+        ssh_fatal("Localtime vs gmtime values does not match!");
+    }
 
   t = ssh_time();
   a = ssh_time_string(t);
