@@ -10,14 +10,57 @@ Test time measurement.
 */
 
 /*
- * $Id: t-timemeasure.c,v 1.9 1999/05/04 02:20:32 kivinen Exp $
- * $Log: t-timemeasure.c,v $
+ * $Id: t-timemeasure.c,v 1.11 2001/04/24 18:29:57 fis Exp $
+ * $Log: t-timemeasure.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
 #include "sshincludes.h"
 #include "sshtimemeasure.h"
 
+#define SSH_DEBUG_MODULE "SshTimeMeasureTest"
+
 /* 
  * START, STOP, RESET and INTERMEDIATE macros are context dependent
  * and expect that there is a double variable `rv' in which operation
@@ -121,6 +164,8 @@ Test time measurement.
                 (printf("Intermediate timer %s (%.12f years (anomalistic)).\n", \
                           #x, ((double)ssh_time_measure_get(x, SSH_TIME_GRANULARITY_YEAR_ANOMALISTIC))))
 
+
+#if 0
 #ifdef HAVE_USLEEP
 #define USLEEP(x)                                                             \
     ((printf("sleep for %.12f seconds.\n", ((double)(x)) / 1000000.0)),       \
@@ -130,6 +175,15 @@ Test time measurement.
     ((printf("sleep for %.12f seconds.\n", ((double)((x)/1000000)))),         \
      (sleep(x / 1000000)))
 #endif /* HAVE_USLEEP */
+#endif /* 0 */
+
+/* New version using ssh_sleep() interface */
+#define USLEEP(x)                                                             \
+     do {                                                                     \
+       printf("sleep for %.12f seconds.\n", ((double)(x)) / 1000000.0);       \
+       ssh_sleep(x / 1000000, x % 1000000);                                   \
+     } while (0)
+
 
 int main()
 {
