@@ -8,27 +8,36 @@
 /*
  *        Program: Urlparse
  *        $Source: /ssh/CVS/src/lib/sshutil/tests/t-url.c,v $
- *        $Author: kivinen $
+ *        $Author: ksr $
  *
  *        Creation          : 10:45 Jul 10 1998 kivinen
  *        Last Modification : 21:09 Jun 29 2000 kivinen
- *        Last check in     : $Date: 2000/06/30 15:07:04 $
- *        Revision number   : $Revision: 1.7 $
+ *        Last check in     : $Date: 2001/06/05 14:52:24 $
+ *        Revision number   : $Revision: 1.12 $
  *        State             : $State: Exp $
  *        Version           : 1.186
+ *        
  *
  *        Description       : Test program for library to parse urls
  */
 /*
- * $Id: t-url.c,v 1.7 2000/06/30 15:07:04 kivinen Exp $
+ * $Id: t-url.c,v 1.12 2001/06/05 14:52:24 ksr Exp $
  * $EndLog$
  */
 
 
+/* #undef SSH_URL_PARSE_USE_RELAXED */
+#define SSH_URL_PARSE_USE_RELAXED
+
 #include "sshincludes.h"
 #include "sshbuffer.h"
 #include "sshurl.h"
 
+#define FUNC_O "ssh_url_parse"
+#define FUNC_M "ssh_url_parse_relaxed"
+
+static char *parsefunc;
+
 typedef struct TestUrlRec {
   const char *url;
   const char *scheme;
@@ -38,88 +47,118 @@ typedef struct TestUrlRec {
   const char *port;
   const char *path;
   Boolean ok;
+  Boolean gok;
 } *TestUrl;
 
 struct TestUrlRec tests[] = {
   { "http://www.ssh.fi/testing/host",
-    "http", "www.ssh.fi", NULL, NULL, NULL, "testing/host", TRUE },
+    "http", "www.ssh.fi", NULL, NULL, NULL, "testing/host", TRUE, TRUE },
   { "http://[0012123123/testing/host",
-    "http", "[0012123123", NULL, NULL, NULL, "testing/host", TRUE },
+    "http", "[0012123123", NULL, NULL, NULL, "testing/host", TRUE, TRUE },
   { "http://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]/testing/host",
     "http", "[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]", NULL, NULL, NULL,
-    "testing/host", TRUE },
+    "testing/host", TRUE, TRUE },
   { "http://[fedc:ba98:7654:3210:fedc:ba98:7654:3210]/testing/host",
     "http", "[fedc:ba98:7654:3210:fedc:ba98:7654:3210]", NULL, NULL, NULL,
-    "testing/host", TRUE },
+    "testing/host", TRUE, TRUE },
   { "http://[1080:0:0:0:8:800:200C:4171]/testing/host",
     "http", "[1080:0:0:0:8:800:200C:4171]", NULL, NULL, NULL,
-    "testing/host", TRUE },
+    "testing/host", TRUE, TRUE },
   { "http://[3FFE:2A00:100:7031::1]/testing/host",
     "http", "[3FFE:2A00:100:7031::1]", NULL, NULL, NULL,
-    "testing/host", TRUE },
+    "testing/host", TRUE, TRUE },
   { "http://[1080::8:800:200C:417A]/testing/host",
     "http", "[1080::8:800:200C:417A]", NULL, NULL, NULL,
-    "testing/host", TRUE },
+    "testing/host", TRUE, TRUE },
   { "http://[::192.9.5.5]/testing/host",
     "http", "[::192.9.5.5]", NULL, NULL, NULL,
-    "testing/host", TRUE },
+    "testing/host", TRUE, TRUE },
   { "http://[::FFFF:129.144.52.38]/testing/host",
     "http", "[::FFFF:129.144.52.38]", NULL, NULL, NULL,
-    "testing/host", TRUE },
+    "testing/host", TRUE, TRUE },
   { "http://[2010:836B:4179::836B:4179]/testing/host",
     "http", "[2010:836B:4179::836B:4179]", NULL, NULL, NULL,
-    "testing/host", TRUE },
+    "testing/host", TRUE, TRUE },
   { "http:///testing/host",
-    "http", "", NULL, NULL, NULL, "testing/host", TRUE },
+    "http", "", NULL, NULL, NULL, "testing/host", TRUE, TRUE },
   { "scheme1:///scheme2:///scheme3://data",
-    "scheme1", "", NULL, NULL, NULL, "scheme2:///scheme3://data", TRUE},
+    "scheme1", "", NULL, NULL, NULL, "scheme2:///scheme3://data", TRUE, TRUE},
   { "ftp://kivinen:foobar@ftp.ssh.fi:21/hidden",
-    "ftp", "ftp.ssh.fi", "kivinen", "foobar", "21", "hidden", TRUE },
+    "ftp", "ftp.ssh.fi", "kivinen", "foobar", "21", "hidden", TRUE, TRUE },
   { "ftp://kivinen:foobar@[::10.0.0.1]:21/hidden",
-    "ftp", "[::10.0.0.1]", "kivinen", "foobar", "21", "hidden", TRUE },
+    "ftp", "[::10.0.0.1]", "kivinen", "foobar", "21", "hidden", TRUE, TRUE },
+  { "ftp://kivinen:@ftp.ssh.fi:21/hidden",
+    "ftp", "ftp.ssh.fi", "kivinen", "", "21", "hidden", TRUE, TRUE },
+  { "ftp://kivinen@ftp.ssh.fi:21/hidden",
+    "ftp", "ftp.ssh.fi", "kivinen", NULL, "21", "hidden", TRUE, TRUE },
+  { "ftp://kivinen@ftp.ssh.fi:21/hidden",
+    "ftp", "ftp.ssh.fi", "kivinen", NULL, "21", "hidden", TRUE, TRUE },
   { "scheme://username:password@host:2222/path",
-    "scheme", "host", "username", "password", "2222", "path", TRUE },
+    "scheme", "host", "username", "password", "2222", "path", TRUE, TRUE },
   { "scheme://username:password@[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]:2222/path",
     "scheme", "[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]", "username",
-    "password", "2222", "path", TRUE },
+    "password", "2222", "path", TRUE, TRUE },
   { "scheme://username:password@host/path",
-    "scheme", "host", "username", "password", NULL, "path", TRUE },
+    "scheme", "host", "username", "password", NULL, "path", TRUE, TRUE },
   { "scheme://username@host:2222/path",
-    "scheme", "host", "username", NULL, "2222", "path", TRUE },
+    "scheme", "host", "username", NULL, "2222", "path", TRUE, TRUE },
   { "scheme://username:@host:2222/path",
-    "scheme", "host", "username", "", "2222", "path", TRUE },
+    "scheme", "host", "username", "", "2222", "path", TRUE, TRUE },
   { "scheme://:@host:2222/path",
-    "scheme", "host", "", "", "2222", "path", TRUE },
+    "scheme", "host", "", "", "2222", "path", TRUE, TRUE },
   { "scheme://:password@host:2222/path",
-    "scheme", "host", "", "password", "2222", "path", TRUE },
+    "scheme", "host", "", "password", "2222", "path", TRUE, TRUE },
   { "scheme://host:2222/path",
-    "scheme", "host", NULL, NULL, "2222", "path", TRUE },
+    "scheme", "host", NULL, NULL, "2222", "path", TRUE, TRUE },
   { "//username:password@host:2222/path",
-    NULL, "host", "username", "password", "2222", "path", TRUE },
+    NULL, "host", "username", "password", "2222", "path", TRUE, TRUE },
   { "scheme://username:password@host:2222",
-    "scheme", "host", "username", "password", "2222", NULL, TRUE },
+    "scheme", "host", "username", "password", "2222", NULL, TRUE, TRUE },
   { "scheme://username:password@host",
-    "scheme", "host", "username", "password", NULL, NULL, TRUE },
+    "scheme", "host", "username", "password", NULL, NULL, TRUE, TRUE },
   { "scheme://username:password@host/",
-    "scheme", "host", "username", "password", NULL, "", TRUE },
+    "scheme", "host", "username", "password", NULL, "", TRUE, TRUE },
   { "scheme://host/path",
-    "scheme", "host", NULL, NULL, NULL, "path", TRUE },
+    "scheme", "host", NULL, NULL, NULL, "path", TRUE, TRUE },
   { "scheme://host",
-    "scheme", "host", NULL, NULL, NULL, NULL, TRUE },
+    "scheme", "host", NULL, NULL, NULL, NULL, TRUE, TRUE },
   { "//host",
-    NULL, "host", NULL, NULL, NULL, NULL, TRUE },
+    NULL, "host", NULL, NULL, NULL, NULL, TRUE, TRUE },
   { "host",
-    NULL, "host", NULL, NULL, NULL, NULL, FALSE },
+    NULL, "host", NULL, NULL, NULL, NULL, FALSE, TRUE },
   { "/path",
-    NULL, NULL, NULL, NULL, NULL, "path", TRUE },
+    NULL, NULL, NULL, NULL, NULL, "path", TRUE, TRUE },
   { "",
-    NULL, NULL, NULL, NULL, NULL, NULL, FALSE },
+    NULL, NULL, NULL, NULL, NULL, NULL, FALSE, FALSE },
   { "socks://muuri.ssh.fi:1080",
-    "socks", "muuri.ssh.fi", NULL, NULL, "1080", NULL, TRUE },
+    "socks", "muuri.ssh.fi", NULL, NULL, "1080", NULL, TRUE, TRUE },
   { "scheme://usernam%65:pas%73word@h%6Fst:2222/%70ath",
-    "scheme", "host", "username", "password", "2222", "path", TRUE },
+    "scheme", "host", "username", "password", "2222", "path", TRUE, TRUE },
   { "scheme://username%40host:pass%3aword@%68%6F%73%74:2222/%70ath",
-    "scheme", "host", "username@host", "pass:word", "2222", "path", TRUE }
+    "scheme", "host", "username@host", "pass:word", "2222", "path", TRUE, TRUE },
+  { "www.ssh.fi",
+    NULL, "www.ssh.fi", NULL, NULL, NULL, NULL, FALSE, TRUE },
+  { "//www.ssh.fi:389",
+    NULL, "www.ssh.fi", NULL, NULL, "389", NULL, TRUE, TRUE },
+  { "www.ssh.fi:389",
+    NULL, "www.ssh.fi", NULL, NULL, "389", NULL, FALSE, TRUE },
+  { "//usr@www.ssh.fi:389",
+    NULL, "www.ssh.fi", "usr", NULL, "389", NULL, TRUE, TRUE },
+  { "file://netbsd",
+    "file", "netbsd", NULL, NULL, NULL, NULL, TRUE, TRUE },
+  { "http:index.html",
+    "http", "index.html", NULL, NULL, NULL, NULL, FALSE, FALSE },
+  { "mailto:kivinen@ssh.com",
+    "mailto", "ssh.com", "kivinen", NULL, NULL, NULL, FALSE, TRUE },
+  { "mailto:kivinen@ssh.com?subject=Subscribe&body=send%20current-issue", 
+    "mailto", "ssh.com", "kivinen", NULL, NULL, "?subject=Subscribe&body=send current-issue", FALSE, TRUE },
+  { "ldap://parketti.hel.fi.ssh.com:389",
+    "ldap", "parketti.hel.fi.ssh.com", NULL, NULL, "389", NULL, TRUE, TRUE },
+  { "ldap://parketti.hel.fi.ssh.com:389/duh",
+    "ldap", "parketti.hel.fi.ssh.com", NULL, NULL, "389", "duh", TRUE, TRUE },
+  { "www.google.com/search?q=\"body=send%20\"",
+    NULL, "www.google.com", NULL, NULL, NULL, "search?q=\"body=send \"", FALSE, TRUE }
+
 };
 
 typedef struct TestFormItemRec {
@@ -214,52 +253,140 @@ int main(int argc, char **argv)
   char *scheme, *host, *port, *username, *password, *path, *key, *value;
   size_t path_length, key_len, value_len;
   SshMapping mapping;
+  Boolean want_nulls;
 
   for(i = 0; i < sizeof(tests) / sizeof(*tests); i++)
     {
+      want_nulls = FALSE;
+      parsefunc = FUNC_O;
+
       if (ssh_url_parse_and_decode(tests[i].url, &scheme, &host,
                                    &port, &username, &password, &path))
         {
           if (!tests[i].ok)
-            ssh_fatal("ssh_url_parse returned true, even if it should have failed, url = %s", tests[i].url);
+            ssh_fatal("%s returned true, even if it should have " \
+                      "failed,\nurl = %s", parsefunc, tests[i].url);
+
+#ifdef SSH_URL_PARSE_USE_RELAXED
+          want_nulls = FALSE;
+          parsefunc = FUNC_M;
+          
+          if (ssh_url_parse_relaxed_and_decode(tests[i].url, &scheme, &host,
+                                               &port, &username, &password,
+                                               &path))
+            {
+              if (!tests[i].gok)
+                ssh_fatal("%s returned true, even if it " \
+                          "should have failed, url = %s", parsefunc, tests[i].url);
+            }
+          else
+            {
+              if (tests[i].gok)
+                ssh_fatal("%s returned false, even if it " \
+                          "should have succeeded, url = %s", parsefunc, tests[i].url);
+              want_nulls = TRUE;
+            }
+#endif
         }
       else
         {
+          /* We are about to fail, be prepared */
+          want_nulls = TRUE;
+
           if (tests[i].ok)
-            ssh_fatal("ssh_url_parse returned false, even if it should have succeeded, url = %s", tests[i].url);
+            {
+              ssh_fatal("%s returned false, even if it should have " \
+                        "succeeded,\nurl = %s", parsefunc, tests[i].url);
+            }
+#ifdef SSH_URL_PARSE_USE_RELAXED
+          want_nulls = FALSE;
+          parsefunc = FUNC_M;
+
+          if (ssh_url_parse_relaxed_and_decode(tests[i].url, &scheme, &host,
+                                               &port, &username, &password,
+                                               &path))
+            {
+              if (!tests[i].gok)
+                ssh_fatal("%s returned true, even if it " \
+                          "should have failed, url = %s", parsefunc, tests[i].url);
+            }
+          else
+            {
+              if (tests[i].gok)
+                ssh_fatal("%s returned false, even if it " \
+                          "should have succeeded, url = %s", parsefunc, tests[i].url);
+              want_nulls = TRUE;
+            }
+#endif
         }
+
+      if (want_nulls)
+        {
 #define CHECK(s) \
-      if (s == NULL && tests[i].s != NULL) \
-        ssh_fatal("ssh_url_parse returned NULL for %s, it should have returned %s for url = %s", #s, tests[i].s, tests[i].url); \
-      if (s != NULL && tests[i].s == NULL) \
-        ssh_fatal("ssh_url_parse returned %s for %s, it should have returned NULL for url = %s", s, #s, tests[i].url); \
-      if (s != NULL && strcmp(s, tests[i].s) != 0) \
-        ssh_fatal("ssh_url_parse returned %s for %s, it should have returned %s for url = %s", s, #s, tests[i].s, tests[i].url);
-      CHECK(scheme);
-      CHECK(host);
-      CHECK(port);
-      CHECK(username);
-      CHECK(password);
-      CHECK(path);
+         if (s != NULL) \
+          { \
+            ssh_fatal("%s did not return NULL for %s, url = %s", \
+                      parsefunc, s, #s, tests[i].s, tests[i].url); \
+          }
+
+          CHECK(scheme);
+          CHECK(host);
+          CHECK(port);
+          CHECK(username);
+          CHECK(password);
+          CHECK(path);
+#undef CHECK
+        }
+      else
+        {
+#define CHECK(s) \
+          if (s == NULL && tests[i].s != NULL) \
+          { \
+            fprintf(stderr, "%s\n\tschema:%s, host:%s, port:%s, username:%s,\n\tpassword:%s, path:%s\n", \
+                    tests[i].url, scheme, host, port, username, password, path); \
+            ssh_fatal("%s returned NULL for %s, it should have returned\n'%s' for url = %s", parsefunc, #s, tests[i].s, tests[i].url); \
+          } \
+          if (s != NULL && tests[i].s == NULL) \
+          { \
+            fprintf(stderr, "%s\n\tschema:%s, host:%s, port:%s, username:%s,\n\tpassword:%s, path:%s\n", \
+                    tests[i].url, scheme, host, port, username, password, path); \
+            ssh_fatal("%s returned '%s' for %s, it should have returned\nNULL for url = '%s'", parsefunc, s, #s, tests[i].url); \
+          } \
+          if (s != NULL && strcmp(s, tests[i].s) != 0) \
+          { \
+            fprintf(stderr, "%s\n\tschema:%s, host:%s, port:%s, username:%s,\n\tpassword:%s, path:%s\n", \
+                    tests[i].url, scheme, host, port, username, password, path); \
+            ssh_fatal("%s returned '%s' for %s, it should have returned\n'%s' for url = %s", parsefunc, s, #s, tests[i].s, tests[i].url); \
+          } 
+    
+          CHECK(scheme);
+          CHECK(host);
+          CHECK(port);
+          CHECK(username);
+          CHECK(password);
+          CHECK(path);
+#undef CHECK
+        }
     }
+
   for(i = 0; i < sizeof(form_tests) / sizeof(*form_tests); i++)
     {
       if (ssh_url_parse_form(form_tests[i].url, &path, &path_length,
                              &mapping))
         {
           if (!form_tests[i].ok)
-            ssh_fatal("ssh_url_parse_form returned TRUE, should have returned FALSE, url = %s",
-                      form_tests[i].url);
+            ssh_fatal("ssh_url_parse_form returned TRUE, should have returned " \
+                      "FALSE, url = %s",form_tests[i].url);
         }
       else
         {
           if (form_tests[i].ok)
-            ssh_fatal("ssh_url_parse_form returned FALSE, should have returned TRUE, url = %s",
-                      form_tests[i].url);
+            ssh_fatal("ssh_url_parse_form returned FALSE, should have returned " \
+                      "TRUE, url = %s", form_tests[i].url);
         }
       if (strcmp(form_tests[i].path, path) != 0)
-        ssh_fatal("ssh_url_parse_form path check failed, url = %s, path = %s, should be %s",
-                  form_tests[i].url, path, form_tests[i].path);
+        ssh_fatal("ssh_url_parse_form path check failed, url = %s, path = %s, " \
+                  "should be %s", form_tests[i].url, path, form_tests[i].path);
       for(j = 0; j < 10; j++)
         {
           if (form_tests[i].table[j].key == NULL ||
@@ -270,13 +397,16 @@ int main(int argc, char **argv)
                                   (void *) &value, &value_len))
             {
               mapping_print(mapping);
-              ssh_fatal("ssh_url_parse_form mapping check failed, cannot find key %s from the mapping, url = %s",
+              ssh_fatal("ssh_url_parse_form mapping check failed, cannot find " \
+                        "key %s from the mapping, url = %s",
                         form_tests[i].table[j].key,
                         form_tests[i].url);
             }
           if (strcmp(value, form_tests[i].table[j].value) != 0)
             {
-              ssh_fatal("ssh_url_parse_form mapping check failed, value for key %s from the mapping does not match, is = %s, should be = %s, url = %s",
+              ssh_fatal("ssh_url_parse_form mapping check failed, value for key " \
+                        "%s from the mapping does not match, is = %s, should be " \
+                        "= %s, url = %s",
                         form_tests[i].table[j].key,
                         value,
                         form_tests[i].table[j].value,
@@ -299,7 +429,8 @@ int main(int argc, char **argv)
           if (j == 10)
             {
               mapping_print(mapping);
-              ssh_fatal("ssh_url_parse_form mapping check failed, found key %s, with value %s that should not exists, url = %s",
+              ssh_fatal("ssh_url_parse_form mapping check failed, found key %s" \
+                        ", with value %s that should not exists, url = %s",
                         key, value, form_tests[i].url);
             }
         }
