@@ -1,10 +1,11 @@
 /* zutil.c -- target dependent utility functions for the compression library
  * Copyright (C) 1995-1998 Jean-loup Gailly.
- * For conditions of distribution and use, see copyright notice in zlib.h 
+ * For conditions of distribution and use, see copyright notice in zlib.h
  */
 
-/* @(#) $Id: zutil.c,v 1.7 2000/05/15 14:01:14 santtu Exp $ */
+/* @(#) $Id: zutil.c,v 1.8 2001/01/15 14:30:00 mtr Exp $ */
 
+#include "sshincludes.h"
 #include "zutil.h"
 
 struct internal_state      {int dummy;}; /* for buggy compilers */
@@ -206,9 +207,6 @@ extern voidp  calloc OF((uInt items, uInt size));
 extern void   free   OF((voidpf ptr));
 #endif
 
-void *ssh_xcalloc(unsigned long nitems, unsigned long size);
-void ssh_xfree(void *ptr);
-
 voidpf zcalloc (opaque, items, size)
     voidpf opaque;
     unsigned items;
