@@ -8,18 +8,15 @@
    subject to change. Applications should only use zlib.h.
  */
 
-/* @(#) $Id: zutil.h,v 1.11 2000/09/22 09:50:40 tunskitj Exp $ */
+/* @(#) $Id: zutil.h,v 1.12 2001/08/27 12:46:11 rochford Exp $ */
 
 #ifndef _Z_UTIL_H
 #define _Z_UTIL_H
 
 #include "zlib.h"
-
-#ifdef _KERNEL
-/* The sshincludes.h will detect _KERNEL and include appropriate, platform dependent,
-   kernel include file. */
 #include "sshincludes.h"
-#else /* _KERNEL */
+
+#ifndef _KERNEL
 #ifdef STDC
 #  include <stddef.h>
 #  include <string.h>
